#ifndef LINEFORMAT_H
#define LINEFORMAT_H
enum EPrintLineFormat
{
    LF_LEFT        = 0x01000000,
    LF_RIGHT       = 0x02000000,
    LF_CENTER      = 0x04000000,
    LF_BOLD        = 0x08000000,
    LF_ITALIC      = 0x10000000,
    LF_INVERT      = 0x20000000
};

class LineFormat
{
public:
    LineFormat()noexcept
        : format_(0){}
    LineFormat(const LineFormat &other)noexcept
        : format_(other.format_){}
    LineFormat(LineFormat &&other)noexcept
        : format_(other.format_){}
    ~LineFormat() {}

    qint32 format() const {return format_;}
    void setFormat(qint32 format) {format_ = format;}

    LineFormat & operator = (const LineFormat &other) {format_ = other.format_; return *this;}
    LineFormat &operator =(LineFormat &&other) = default;
    bool operator == (const LineFormat &other) const {return format_ == other.format_;}
    bool operator != (const LineFormat &other) const {return format_ != other.format_;}

    qint32 printGray() const {return format_ & 0x00001FFF;}
    void setPrintGray(qint32 val) {format_ = (format_ & 0xFFFFE000) | (val & 0x00001FFF);}

    qint32 font() const {return (format_ & 0x00006000) >> 13;}
    void setFont(qint32 font) {format_ = (format_ & 0xFFFF9FFF) | ((font & 0x3) << 13);}

    qint32 lineSpacing() const {return (format_ & 0x000F1000) >> 15;}
    void setLineSpacing(qint32 ls) {format_ = (format_ & 0xFFF07FFF) | ((ls & 0x1F) << 15);}

    qint32 symbolsPerLine() const {return font() < 1 ? 48 : (font() < 3 ? 32 : 24);}


    bool isLeft() const {return format_ & LF_LEFT;}
    bool isRight() const {return format_ & LF_RIGHT;}
    bool isCenter() const {return format_ & LF_CENTER;}
    bool isBold() const {return format_ & LF_BOLD;}
    bool isItalic() const {return format_ & LF_ITALIC;}
    bool isInvert() const {return format_ & LF_INVERT;}

    void setFlags(qint32 flags) {format_ = (format_ & 0x00FFFFFF) | (flags & 0xFF000000);}


private:
    qint32 format_;
};

#endif //LINEFORMAT_H
