#include "fsapi.h"

#include "logmanip.h"
#include <QtAndroidExtras/QAndroidJniObject>
#include <QtAndroidExtras/QtAndroid>
#include <QAndroidJniEnvironment>
#include <QAndroidIntent>
#include <QCryptographicHash>

#include <QSerialPort>
#include <QDataStream>
#include <QFileInfo>

FsApi::FsApi(QObject *parent)
    : QObject(parent)
//    , port_(new QSerialPort("/dev/ttyHSL1",this))
//    , port_(new QSerialPort("/dev/ttyS0",this))
    , port_(nullptr)
{
    if(QFileInfo::exists("/dev/ttyHSL1")) port_ = new QSerialPort("/dev/ttyHSL1",this);
    else port_ = new QSerialPort("/dev/ttyS0",this);
}

FsApi::~FsApi()
{
    port_->close();
}

bool FsApi::open()
{
    bool res = port_->open(QIODevice::ReadWrite);
    if(res)
    {
        port_->clear();
        port_->clearError();
        port_->setBaudRate(115200);
        port_->setDataBits(QSerialPort::Data8);
        port_->setStopBits(QSerialPort::OneStop);
        port_->setParity(QSerialPort::NoParity);
        port_->setFlowControl(QSerialPort::NoFlowControl);
        port_->setReadBufferSize(0);
        port_->clear();
        port_->clearError();
        lmWarning() << port_->portName() << res;
    }
    else
    {
        lmWarning() << port_->portName() << port_->errorString();
    }
    return res;
}

bool FsApi::isOpen() const
{
    return port_->isOpen();
}

void FsApi::close()
{
    port_->close();
}
/*
    private byte[] exec(byte[] cmd, int timeout)
*/

bool FsApi::exec(const FsReq &req, FsRep &rep, FsRwTimeout timeout)
{
    bool done = false;
    rep = FsRep();
    if(req.isValid() && port_ && port_->isOpen())
    {
        port_->clearError();
        port_->clear();
        //Если нужно посмотреть логи обмена с ФН, то
        //Раз ,
        //lmWarning() << logbinary(cmd.rawData());
        port_->write(req.rawData());
        port_->waitForBytesWritten(timeout.second);
        port_->waitForReadyRead(timeout.first);
        quint8 prefix = 0;
        quint16 len = 0;
        QByteArray buf;
        char byte = 0;
        while(port_->read(&byte, 1) > 0)
        {
            if(prefix == 0)
            {
                if(static_cast<quint8>(byte) == FsMsg::MSG_START)
                {
                    prefix = FsMsg::MSG_START;
                    buf.push_back(byte);
                }
                else continue;
            }
            else
            {
                buf.push_back(byte);
                if(buf.size() == 3)
                {
                    QDataStream ds(buf.mid(1));
                    ds.setByteOrder(QDataStream::LittleEndian);
                    ds >> len;
                }
                else
                {
                    if(buf.size() == len + 5)
                    {
                        done = true;
                        break;
                    }
                }
            }
            if(!port_->bytesAvailable())port_->waitForReadyRead(100);
        }
        //Два
//        lmWarning() << logbinary(buf) << done << (int )req.code();
        if(done)
        {
            rep.setRawData(buf);
            //lmDebug() << (quint32)answer.setRawData(buf) << answer.code() << (int) cmd.code();
        }
        lmWarning() << logbinary(req.rawData()) << logtab  << logbinary(buf);
    }
    return done;
}
