#include "hwmodeldatastorage.h"
#include "logmanip.h"
#include "crc.h"
#include "androidapihelper.h"
#include "devicestatus.h"
#include "cryptedsettingsstorage.h"
#include "modeldatastorage.h"

#include <QAndroidJniEnvironment>
#include <QAndroidJniExceptionCleaner>
#include <QThread>
#include <QDataStream>
#include <QTextCodec>
#include <QFile>
#include <QDir>

#	define FR_SERIAL_ADDRESS	0x0FF0u
#	define FR_SERIAL_LENGTH		8u
#	define FR_SERIAL_ADDRESS2	0x0FECu
#	define FR_SERIAL_LENGTH2	12u
#	define FR_SERIAL_FLAGS_ADDRESS	0x0F80u
#	define FR_SERIAL_FLAGS_LENGTH	4u
#	define FR_MODEL_BYTE_ADDRESS	0x0FF8u
#	define FR_MODEL_BYTE_LENGTH		2u
#	define FR_MODEL_STRING_ADDRESS	0x0FA0u
#	define FR_MODEL_STRING_LENGTH	64u
#	define FR_EEPROM_STRING_ENCODING "CP866"


#if ALLOW_HW_MODEL_DATA_STORAGE

class OldDataReader
{
public:
    static void restoreOldData();
private:
    static bool _notCheckedYet;
    static QMutex _restorMutex;
    OldDataReader(){}
    ~OldDataReader(){}
    static bool checkAndRestoreOldData();
    static bool getSerialAddress(quint16 &addr, quint8 &len, HwModelDataStorage *se);
    static quint32 calcSerialHash(qint64 serial, int slot);
};

HwModelDataStorage::HwModelDataStorage()
{
//    QByteArray buf;
//    qint32 sz = readNvRam(0, 4096, buf);
//    lmWarning() << sz << logbinary(buf);

}

HwModelDataStorage::~HwModelDataStorage()
{

}

ModelData HwModelDataStorage::getModelData()
{
    auto readData = []() ->ModelData {
        ModelData res1, res2;
        ModelDataStorage mds;
        CryptedSettingsStorage main(ModelDataStorage::MAIN_FILE, nullptr);
        CryptedSettingsStorage backup(ModelDataStorage::BACK_FILE, nullptr);
        mds.getModelData(&main, res1);
        mds.getModelData(&backup, res2);
        if(res1 != res2)
        {
            if(res1.isValid())
            {
                mds.setModelData(&backup, res1);
            }
            else if(res2.isValid())
            {
                mds.setModelData(&main, res2);
                res1 = res2;
            }
        }
        return res1;
    };
    ModelData res = readData();
    lmWarning() << logvariant(res.toMap());
    if(!res.isValid())
    {
        OldDataReader::restoreOldData();
    }
    res = readData();
    lmWarning() << logvariant(res.toMap());
    return res;
}

void HwModelDataStorage::setModelData(const ModelData &md)
{
    ModelDataStorage mds;
    CryptedSettingsStorage main(ModelDataStorage::MAIN_FILE, nullptr);
    CryptedSettingsStorage backup(ModelDataStorage::BACK_FILE, nullptr);
    mds.setModelData(&main, md);
    mds.setModelData(&backup, md);
}

void HwModelDataStorage::cleanAll() const
{
    QDir dir("/storage/emulated/0/armax/frcore/");
    QFileInfoList fil = dir.entryInfoList(QDir::Files);
    std::for_each(fil.begin(), fil.end(), [](const QFileInfo &f){QFile::remove(f.absoluteFilePath());});
}

bool HwModelDataStorage::readNvRam(quint16 addr, quint16 count, QByteArray &buf)
{
    if(!QFile::exists("/storage/emulated/0/armax/frcore/eeprom.emul")) return false;
    QFile f ("/storage/emulated/0/armax/frcore/eeprom.emul");
    if(!f.open(QIODevice::ReadOnly)) return false;
    if(addr + count > 4096) return false;
    f.seek(addr);
    buf = f.read(count);
    f.close();
    return buf.size() == count;
}

bool OldDataReader::_notCheckedYet = true;
QMutex OldDataReader::_restorMutex;
void OldDataReader::restoreOldData()
{
#if ALLOW_OLD_MODEL_DATA_READ
    _restorMutex.lock();
    if(_notCheckedYet)
    {
        lmWarning() << "START RESTORE";
        checkAndRestoreOldData();
        _notCheckedYet = false;
    }
    _restorMutex.unlock();
#endif
}



bool OldDataReader::checkAndRestoreOldData()
{
    HwModelDataStorage se;
    quint16 addr = FR_SERIAL_ADDRESS;
    quint8 len = FR_SERIAL_LENGTH;
    lmWarning() << "GET_SERIAL_ADDRESS";
    if(!getSerialAddress(addr, len, &se))
    {
        return false;
    }

    Q_ASSERT(len > 0);
    QByteArray buf;
    qint64 serial = 0;
    quint32  hash = 0, calcHash0 = 0, calcHash1 = 0;
    lmWarning() << "READ SERIAL";
    if(!se.readNvRam(addr, len, buf) || buf.isEmpty() || buf == QByteArray(len, '\xFF'))
    {
        lmWarning() << logbinary(buf);
        return false;
    }
    lmWarning() << addr << len << FR_SERIAL_ADDRESS << FR_SERIAL_ADDRESS2<< logbinary(buf);
    QDataStream ds(buf);
    ds.setByteOrder(QDataStream::BigEndian);
    if(addr == FR_SERIAL_ADDRESS2)
    {
        ds >> serial >> hash;
    }
    else
    {
        quint32 sbuf = 0;
        ds >>sbuf >> hash;
        serial = sbuf;
    }
    lmWarning() << serial << hash;
    calcHash0 = calcSerialHash(serial, 0);
    calcHash1 = calcSerialHash(serial, 1);
    lmWarning() << calcHash0 << calcHash1 << hash << serial;
    if((hash != calcHash0 && hash != calcHash1) || hash == 0)
    {
        return false;
    }
    lmWarning() << QString::number(serial, 16) << serial << calcHash0 << calcHash1 << hash;

    quint16 model = 0;
    QString modelName;
    if(!se.readNvRam(FR_MODEL_BYTE_ADDRESS, FR_MODEL_BYTE_LENGTH, buf) || buf.isEmpty() || buf == QByteArray(len, '\xFF'))
    {
        lmWarning() << logbinary(buf);
        return false;
    }
    QDataStream mds(buf);
    mds.setByteOrder(QDataStream::BigEndian);
    mds >> model;
    lmWarning() << model << serial;
    if(!se.readNvRam(FR_MODEL_STRING_ADDRESS, FR_MODEL_STRING_LENGTH, buf) || buf.isEmpty() || buf == QByteArray(len, '\xFF'))
    {
        lmWarning() << logbinary(buf);
        return false;
    }
    QByteArray temp(buf.size(), '\xFF');
    if(temp != buf)
    {
        int idx = buf.indexOf('\x00');
        if(idx > 0) buf = buf.mid(0, idx);
        QTextDecoder dec(QTextCodec::codecForName(FR_EEPROM_STRING_ENCODING));
        modelName = dec.toUnicode(buf);
    }
    lmWarning() << model << modelName << serial;
    if(model == 0 || modelName.isEmpty() || serial <= 0ll) return false;

    ModelData md;
    md.setModel(model);
    md.setModelName(modelName);
    md.setSerial(QString::number(serial));
    md.signData(AndroidApiHelper().getHardwareHash());
    lmWarning() << logvariant(md.toMap());
    if(md.isValid())
    {
        se.setModelData(md);
        return true;
    }
    return false;
}

bool OldDataReader::getSerialAddress(quint16 &addr, quint8 &len, HwModelDataStorage *se)
{
    addr = 0;
    len = 0;

    QByteArray buf;
    if(!se->readNvRam(FR_SERIAL_FLAGS_ADDRESS, FR_SERIAL_FLAGS_LENGTH, buf)) return false;

    QDataStream ds(buf);
    ds.setByteOrder(QDataStream::BigEndian);
    quint8 crc = 0;
    ds >> addr >> len >> crc;
    if(crc == Crc8Dallas()(buf.mid(0, 3)) && addr + len < 4096)
    {
        lmWarning() << addr << len << logbinary(buf);
        return true;
    }
    lmWarning() << addr << len << logbinary(buf);
    return false;
}

quint32 OldDataReader::calcSerialHash(qint64 serial, int slot)
{
#ifdef USE_CASHBOX_SERIAL_AS_PROC
    qint64 proc = serial;
    QString ps = QString::number(proc);
#else
    QString ps = AndroidApiHelper().getImei(slot);
    quint32 proc = ps.toULongLong();
#endif
    QByteArray data = QString::number(serial).toUtf8() +
            ps.toUtf8() +
            QString::number(serial ^ proc).toUtf8();
    return Crc32()(data);
}


#endif
