#ifndef CASHBOXSERVICEAPI_H
#define CASHBOXSERVICEAPI_H

#include "coreapiconst.h"
#include "coretransaction.h"
#include "coreapiresult.h"
#include "cashboxstatus.h"

#include <QObject>
#include <QAndroidServiceConnection>
#include <QAndroidIntent>
#include <QAndroidBinder>
#include <QtAndroid>
#include <QPointer>
#include <QMutex>

class CashboxServiceApi;

class CashboxServiceConnection : public QAndroidServiceConnection
{
public:
    explicit CashboxServiceConnection(CashboxServiceApi *parent)noexcept;
    explicit CashboxServiceConnection(CashboxServiceApi *parent, const QAndroidJniObject &serviceConnection)noexcept;
    ~CashboxServiceConnection() override;

    void onServiceConnected(const QString &name, const QAndroidBinder &serviceBinder) override;
    void onServiceDisconnected(const QString &name) override;
private:
    QPointer<CashboxServiceApi> api_;
};

//-------------------------------------------------------------------------------------------

class CashboxServiceApi : public QObject
{
    Q_OBJECT
    Q_DISABLE_COPY(CashboxServiceApi)
    friend class CashboxServiceConnection;
public:
    explicit CashboxServiceApi(QObject *parent = nullptr)noexcept;
    ~CashboxServiceApi() override;

    bool isReady() const;
    bool ping();

    CoreApiResult loadCashierNames();
    CoreApiResult exec(CoreTransaction trans);
    CoreApiResult writeSerial(CoreTransaction trans);

    CoreApiResult loadCashboxStatus(bool toExternal = false);
    ModelData loadModelData(bool toExternal = false);
    CoreApiResult loadReceiptData();

signals:
    void error(QString error);
protected:
    QSharedPointer<QAndroidBinder> binder_;
    CashboxServiceConnection connection_;

    void waitBinder(qint32 timeout);

    CoreApiResult transact(CoreApiConst::RemoteCommand code, QAndroidParcel &reply, QAndroidParcel *data = nullptr);

    virtual void initGlobalStatus(const CashboxStatus &status) = 0;
    virtual void initCatalogsData(const QVariantMap &data) = 0;
};

#endif // CASHBOXSERVICEAPI_H
