#include "receiptconfigdata.h"
#include "cashboxbuildconfig.h"

#include "formatutils.h"

const QString ReceiptConfigData::SECTION = "RECEIPT";
const QString ReceiptConfigData::MAX_RECEIPT_AMOUNT = "maxAmount";


ReceiptConfigData::ReceiptConfigData() noexcept
    : BaseCoreConfigData()
    , maxReceiptAmount_(DEFAULT_AMOUNT_MULT)
{

}

ReceiptConfigData::ReceiptConfigData(const ReceiptConfigData &other) noexcept
    : BaseCoreConfigData(other)
    , maxReceiptAmount_(other.maxReceiptAmount_)
{

}

ReceiptConfigData::ReceiptConfigData(ReceiptConfigData &&other) noexcept
    : BaseCoreConfigData(other)
    , maxReceiptAmount_(other.maxReceiptAmount_)
{

}

ReceiptConfigData::~ReceiptConfigData()
{

}

QString ReceiptConfigData::section() const
{
    return SECTION;
}

QStringList ReceiptConfigData::fields() const
{
    return QStringList{MAX_RECEIPT_AMOUNT};
}

QMap<QString, QString> ReceiptConfigData::toStringMap() const
{
    return QMap<QString, QString> {
        {MAX_RECEIPT_AMOUNT, maxReceiptAmount_.toString()}
    };
}

void ReceiptConfigData::parseStringMap(const QMap<QString, QString> &map)
{
    maxReceiptAmount_ = FixNumber(DEFAULT_AMOUNT_MULT);
    if(map.contains(MAX_RECEIPT_AMOUNT)) maxReceiptAmount_.setString(map[MAX_RECEIPT_AMOUNT]);
}

QVariantMap ReceiptConfigData::toMap() const
{
    return QVariantMap {
        {MAX_RECEIPT_AMOUNT, maxReceiptAmount_.value()}
    };
}

void ReceiptConfigData::parseMap(const QVariantMap &map)
{
    maxReceiptAmount_ = FixNumber(DEFAULT_AMOUNT_MULT);
    if(map.contains(MAX_RECEIPT_AMOUNT)) maxReceiptAmount_.setValue(map[MAX_RECEIPT_AMOUNT].toLongLong());
}

FixNumber ReceiptConfigData::maxReceiptAmount() const
{
    return maxReceiptAmount_;
}

void ReceiptConfigData::setMaxReceiptAmount(const FixNumber &newMaxReceiptAmount)
{
    maxReceiptAmount_ = newMaxReceiptAmount;
}

ReceiptConfigData &ReceiptConfigData::operator =(const ReceiptConfigData &other) noexcept
{
    maxReceiptAmount_ = other.maxReceiptAmount_;
    return *this;
}

ReceiptConfigData &ReceiptConfigData::operator =(ReceiptConfigData &&other) noexcept
{
    std::swap(maxReceiptAmount_, other.maxReceiptAmount_);
    return *this;
}


bool ReceiptConfigData::operator ==(const ReceiptConfigData &other) const noexcept
{
    return maxReceiptAmount_ == other.maxReceiptAmount_;
}

bool ReceiptConfigData::operator !=(const ReceiptConfigData &other) const noexcept
{
    return !(*this == other);
}
