#include "xreport.h"
#include "formatutils.h"

XReport::XReport(QObject *parent)
    : QObject(parent)
    , type_(fdf::DocType::Invalid)
    , dt_(QDateTime::currentDateTime())
    , model_()
    , regData_()
    , externalId_()
    , clientId_()
    , header_()
    , footer_()
    , counters_()
    , status_()
    , cashier_()
    , cashierInn_()
    , address_()
    , place_()
{

}

XReport::XReport(const FsCounters &counters, QObject *parent)
    : QObject(parent)
    , type_(counters.type() == FsTotalType::CycleTotal ?
                fdf::DocType::CycleReport :
                fdf::DocType::FsReport)
    , dt_(QDateTime::currentDateTime())
    , model_()
    , regData_()
    , externalId_()
    , clientId_()
    , header_()
    , footer_()
    , counters_(counters)
    , status_()
    , cashier_()
    , cashierInn_()
    , address_()
    , place_()
{

}

XReport::XReport(const fdf::DocType &type, QObject *parent)
    : QObject(parent)
    , type_(type)
    , dt_(QDateTime::currentDateTime())
    , model_()
    , regData_()
    , externalId_()
    , clientId_()
    , header_()
    , footer_()
    , counters_()
    , status_()
    , cashier_()
    , cashierInn_()
    , address_()
    , place_()
{

}

XReport::~XReport()
{

}

QString XReport::docName() const
{
    return type_ == fdf::DocType::CycleReport ?
               tr("Статистика за смену") : tr("Статистика ФН");
}

fdf::DocType XReport::type() const
{
return type_;
}


const QDateTime &XReport::dt() const
{
    return dt_;
}

void XReport::setDt(const QDateTime &newDt)
{
    dt_ = newDt;
}

const ModelData &XReport::model() const
{
    return model_;
}

void XReport::setModel(const ModelData &newModel)
{
    model_ = newModel;
}

const RegData &XReport::regData() const
{
    return regData_;
}

void XReport::setRegData(const RegData &newRegData)
{
    regData_ = newRegData;
}

const QString &XReport::externalId() const
{
    return externalId_;
}

void XReport::setExternalId(const QString &newExternalId)
{
    externalId_ = newExternalId;
}

const QString &XReport::clientId() const
{
    return clientId_;
}

void XReport::setClientId(const QString &newClientId)
{
    clientId_ = newClientId;
}

const QString &XReport::header() const
{
    return header_;
}

void XReport::setHeader(const QString &newHeader)
{
    header_ = newHeader;
}

const QString &XReport::footer() const
{
    return footer_;
}

void XReport::setFooter(const QString &newFooter)
{
    footer_ = newFooter;
}

const FsCounters &XReport::counters() const
{
    return counters_;
}

void XReport::setCounters(const FsCounters &newCounters)
{
    counters_ = newCounters;
}

const FsFullStatus &XReport::status() const
{
    return status_;
}

void XReport::setStatus(const FsFullStatus &newStatus)
{
    status_ = newStatus;
}

const QString &XReport::cashier() const
{
    return cashier_;
}

void XReport::setCashier(const QString &newCashier)
{
    cashier_ = newCashier;
}

const QString &XReport::cashierInn() const
{
    return cashierInn_;
}

void XReport::setCashierInn(const QString &newCashierInn)
{
    cashierInn_ = newCashierInn;
}

const QString &XReport::address() const
{
    return address_;
}

void XReport::setAddress(const QString &newAddress)
{
    address_ = newAddress;
}

const QString &XReport::place() const
{
    return place_;
}

void XReport::setPlace(const QString &newPlace)
{
    place_ = newPlace;
}



void XReport::clean()
{
    type_ = fdf::DocType::Invalid;
    dt_ = QDateTime::currentDateTime();
    model_ = ModelData();
    regData_ = RegData();
    externalId_.clear();
    clientId_.clear();
    header_.clear();
    footer_.clear();
    counters_ = FsCounters();
    cashier_.clear();
    cashierInn_.clear();
    address_.clear();
    place_.clear();
}

QVariantMap XReport::toMap() const
{
    return {
        {"type", static_cast<qint32>(type_)  },
        {"model", model_.toMap()            },
        {"reg", regData_.toMap()        },
        {"dt", DT2STR_(dt_)                 },
        {"externalId",     externalId_      },
        {"clientId",       clientId_        },
        {"header", header_                  },
        {"footer", footer_                  },
        {"counters", counters_.toMap()      },
        {"status", status_.toMap()          },
        {"cashier",        cashier_         },
        {"cashierInn",     cashierInn_      },
        {"address",        address_         },
        {"place",     place_      }
    };
}

QVariantMap XReport::toExternalMap() const
{
    //NOTE: адрес и пр. + header и footer добавлены для поддержки POST запросов в будущем
    return {
        {"type", static_cast<qint32>(type_)},
        {"address",   address_},
        {"place",     place_},
//        {"ownerInn",   regData_.ownerInn() },
//        {"regNumber", regData_.regNumber()},
//        {"ownerName",  regData_.ownerName()},

        {"fsIsExpired", status_.status().warnings().testFlag(fs::Warning::NeedFsChange)},
        {"fsIsNearExpired", status_.status().warnings().testFlag(fs::Warning::NeedFsChange30)},
        {"fsIsOwerFlow", status_.status().warnings().testFlag(fs::Warning::FsMemoryOverflow)},
        {"ofdTimedOut", status_.status().warnings().testFlag(fs::Warning::OfdTimedOut)},
        {"ofdLogicalError", status_.status().warnings().testFlag(fs::Warning::OfdDocumentsError)},
        {"ofdCheckAccount", status_.status().warnings().testFlag(fs::Warning::NeedKktConfiguration)},
        {"ofdIsClosed", status_.status().warnings().testFlag(fs::Warning::OfdIsClosed)},

        {"documentsCount", status_.status().lastFd()},
        {"receiptsCount", status_.cycle().receipt()},
        {"cycleIsOpen", status_.cycle().opened()},
        {"notSendedDocs", status_.transport().messagesCount()},
        {"notSendedNotifications", status_.labelsTransport().notificationCount()},
        {"firstNotSendedDocDt", status_.transport().firstDocDt().isValid() ?
                                    QVariant( status_.transport().firstDocDt().toString(Qt::ISODate)) : QVariant()},

        {"model", model_.toExternalMap()},
        {"reg", regData_.toExternalMap()},
        {"dt", DT2STR_(dt_)},
        {"fsNumber", status_.status().fsNumber().trimmed()},
        {"externalId",     externalId_      },
        {"clientId",       clientId_        },
        {"header", header_},
        {"footer", footer_},
        {type_ == fdf::DocType::CycleReport ? QStringLiteral("cycleCounters") : QStringLiteral("fsCounters"), counters_.toExternalMap()},
        {"cashier",        cashier_.isEmpty() ? QVariant() : QVariant(cashier_)},
        {"cashierInn",     cashierInn_.isEmpty() ? QVariant() : QVariant(cashierInn_)}
    };
}

QString XReport::toHtml() const
{
    //TODO: Добавить адрес?
    QStringList res;
    res << "<html><head>"
        << "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">"
        << "</head><body>" ;
    res.append(model_.toHtml().split("\n"));
    res.append(regData_.toHtml().split("\n"));
    res.append(QStringLiteral("<h3 align=\"center\"><b>%1</b></h3>").arg(docName()));
    res.append(QStringLiteral("<p align=\"right\">%1</p>").arg(dt_.toString("dd.MM.yyyy hh:mm")));
    res.append(counters_.toHtml());
    res	<< "</body></html>";
    //    lmWarning() << loglist(res);
    return res.join("\n");
}

void XReport::parseMap(const QVariantMap &map)
{
    fdf::DocType tp = static_cast<fdf::DocType>(map["type"].toInt());
    if(tp == fdf::DocType::CycleReport || tp == fdf::DocType::FsReport)
    {
        type_= tp;
        model_.parseMap(map["model"].toMap());
        regData_.parseMap(map["reg"].toMap());
        dt_ = STR2DT_(map["dt"].toString());
        externalId_ = map["externalId"].toString().trimmed();
        clientId_ = map["clientId"].toString().trimmed();
        header_ = map["header"].toString();
        footer_ = map["footer"].toString();
        counters_.parseMap(map["counters"].toMap());
        status_.parseMap(map["status"].toMap());
        cashier_ = map["cashier"].toString();
        cashierInn_ = map["cashierInn"].toString();
        address_ = map["address"].toString();
        place_ = map["place"].toString();
    }
}

