#ifndef BBCODEPARSER_H
#define BBCODEPARSER_H

#include "textprinterdocument.h"
#include "BBDocument.h"
#include <QObject>

using namespace bbcpp;



class BbCodeParser : public QObject
{
    Q_OBJECT
    Q_DISABLE_COPY(BbCodeParser)
public:
    static const QMap<QString, DocumentBlock::EBarcodeType> BARCODE_TYPES;
    explicit BbCodeParser(QObject *parent = nullptr);
    virtual ~BbCodeParser();

    void parseText(QString text);
    void parseChildren(const BBNode& parent);

    QList<DocumentBlock> blocks() const;
    TextPrinterDocument document() const;

    quint8 symbolsPerLine() const;
    void setSymbolsPerLine(const quint8 &symbolsPerLine);

    QString imagesPath() const;
    void setImagesPath(const QString &imagesPath);

private:
    enum ETextAlignment
    {
        ALIGN_LEFT              = 0,
        ALIGN_CENTER            = 1,
        ALIGN_RIGHT             = 2,
        ALIGN_JUSTIFY           = 3

    };
    enum ECharFormat : quint8
    {
        CF_NONE					= 0x00,
        CF_BOLD					= 0x01,
        CF_ITALIC				= 0x02,
        CF_INVERSE				= 0x04,
        CF_UNDERLINE			= 0x08,
        CF_DOUBLE_WIDTH			= 0x10,
        CF_DOUBLE_HEIGHT		= 0x20,
        CF_USER_CHARSET			= 0x40//?
    };
    struct Field
    {
        quint8 width_;
        ETextAlignment alignment_;
        quint8 zerofill_;
        Field():width_(1), alignment_(ALIGN_LEFT), zerofill_(0){}
    };
    struct Column
    {
        quint8 width_;
        ETextAlignment alignment_;
        Column() :width_(1), alignment_(ALIGN_LEFT){}
    };
    struct Hr
    {
        QChar symbol_;
        Hr() : symbol_(QLatin1Char('_')){}
    };
    Field *curField_;
    Column *curColumn_;
    Hr *curHr_;
    MonochromeImage *curImage_;
    QString text_;
    QByteArray textFont_;
    QByteArray textFormat_;
    quint8 symbolsPerLine_;
    quint8 curFont_;
    quint8 curFormat_;
    ETextAlignment curAlignment_;
    DocumentBlock::EBlockType blockType_;
    DocumentBlock::EBarcodeType barcodeType_;
    QList<DocumentBlock> blocks_;
    QString imagesPath_;

    QStringList test;

    void openElement(BBElementPtr elem);
    void processParams(BBElementPtr elem);
    void closeElement(BBElementPtr elem);
    void loadImage(const QString &name);

    DocumentBlocks createCurrentBlock() const;
    void processCurentBlock(bool addEmpty = false);
//    DocumentBlock createEmptyBlock(int alignment);


};

#endif // BBCODEPARSER_H
