#include "printdocumentformatter.h"
#include "appdirectories.h"
#include "logmanip.h"
#include "cashboxbuildconfig.h"
#include "androidapihelper.h"
#include "bbcodeparser.h"


#include <QDateTime>

#include <QStorageInfo>
#include <QNetworkInterface>
#include <QFileInfo>

PrintDocumentFormatter::PrintDocumentFormatter(const RegData &rd, QObject *parent)
    : QObject(parent)
    , regData_(rd)
{

}

PrintDocumentFormatter::~PrintDocumentFormatter()
{

}

TextPrinterDocument PrintDocumentFormatter::welcome(const CashboxStatus &st) const
{
    DocumentBlocks blocks;
    QStringList text;
    LineFormat f;
    f.setPrintGray(5000);

//        text << QStringLiteral("01234567890123456789012345678901234567890123456789");
//        for(int i = 0; i < 4; ++i)
//        {
//            f.setFont(i);
//            f.setFlags(LF_LEFT | LF_BOLD);
//            blocks << DocumentBlock::createSimpleBlock(text, f);
//            f.setFlags(LF_LEFT | LF_INVERT);
//            blocks << DocumentBlock::createSimpleBlock(text, f);
//            f.setFlags(LF_LEFT | LF_ITALIC);
//            blocks << DocumentBlock::createSimpleBlock(text, f);
//        }
//        text.clear();

    if(!QFile::exists(APATH.logo()) || QFileInfo(":/icons/logo.png").size() != QFileInfo(APATH.logo()).size())
    {
        QFile in(":/icons/logo.png");
        QFile out(APATH.logo());
        if(in.open(QIODevice::ReadOnly) && out.open(QIODevice::WriteOnly))
        {
            out.write(in.readAll());
            out.flush();
        }
        lmWarning() << in.errorString() << logtab << out.errorString();
        in.close();
        out.close();
    }


    MonochromeImage img;
    img.setFileName(APATH.logo());
    DocumentBlock imgBl;
    imgBl.setImg(img);
    imgBl.setType(DocumentBlock::Image);
    f.setFlags(LF_CENTER);
    imgBl.setFormat(f);

    blocks <<imgBl;

    f.setFont(2);
    f.setFlags(LF_CENTER | LF_BOLD);

    text << QString(f.symbolsPerLine(), QLatin1Char('*'))
         << (st.modelData().modelName() + " " + st.modelData().version());
    if(!st.modelData().serial().isEmpty())
    {
        text << QStringLiteral("%1\t%2").arg(tr("ЗН"), st.modelData().serial());
    }

    text << QDateTime::currentDateTime().toString("dd.MM.yyyy\thh:mm:ss");
    text << text.first();
    blocks << DocumentBlock::createSimpleBlock(text, f);
    text.clear();

    if(st.isFiscalized())
    {
        f.setFlags(LF_CENTER);
        RegData rd = st.regData();
        text << rd.ownerName() << rd.address() << rd.place()
             << QStringLiteral("ИНН\t%1").arg(rd.ownerInn())
             << QStringLiteral("РНМ\t%1").arg(rd.regNumber());



    }

    f.setFont(1);
    QStringList ips = st.dev().ipAddresses();
    for(const QString &ip :ips)
    {
        if(!ip.trimmed().isEmpty())text << QStringLiteral("IP\t%1").arg(ip);
    }
    blocks << DocumentBlock::createSimpleBlock(text, f);
    text.clear();


    return TextPrinterDocument (blocks);
}

TextPrinterDocument PrintDocumentFormatter::test(const CashboxStatus &st) const
{
    DocumentBlocks blocks = welcome(st).blocks();

    QStringList text;
    LineFormat f;
    f.setPrintGray(5000);


    f.setFont(2);
    f.setFlags(LF_CENTER);

    QList<QNetworkInterface> interfaces = QNetworkInterface::allInterfaces();
    QString inet = "Нет связи с Интернет";
    for(const QNetworkInterface &i :interfaces)
    {
        if(i.flags().testFlag(QNetworkInterface::IsLoopBack) ||
                !i.flags().testFlag(QNetworkInterface::IsUp)) continue;
        QString ifaceTypeName;
        switch (i.type()) {
        case QNetworkInterface::Virtual     : ifaceTypeName = "виртуальный"; break;
        case QNetworkInterface::Ethernet    : ifaceTypeName = "ethernet"; break;
        case QNetworkInterface::Slip        : ifaceTypeName = "SLIP"; break;
        case QNetworkInterface::CanBus      : ifaceTypeName = "CanBas"; break;
        case QNetworkInterface::Ppp         : ifaceTypeName = "PPP"; break;
        case QNetworkInterface::Fddi        : ifaceTypeName = "FDDI"; break;
        case QNetworkInterface::Wifi        : ifaceTypeName = "WiFi"; break;
        case QNetworkInterface::Phonet      : ifaceTypeName = "Модем"; break;
        case QNetworkInterface::Ieee802154  : ifaceTypeName = "WPAN"; break;
        case QNetworkInterface::SixLoWPAN   : ifaceTypeName = "WPAN"; break;
        case QNetworkInterface::Ieee80216   : ifaceTypeName = "WiMAX"; break;
        case QNetworkInterface::Ieee1394    : ifaceTypeName = "FireWire"; break;
        default:
            ifaceTypeName = AndroidApiHelper().currentNetworkInterfaceName();
            break;
        }
        if(!ifaceTypeName.isEmpty()) ifaceTypeName = " (" + ifaceTypeName + ")";
        inet = QStringLiteral("Интернет:\t%1%2").arg(i.humanReadableName())
                .arg(ifaceTypeName);
    }
    text << inet;
    QStorageInfo si(APATH.workDir());
    text << QStringLiteral("Доступно %1/%2 байт").arg(si.bytesFree()).arg(si.bytesTotal());
    blocks << DocumentBlock::createSimpleBlock(text, f);
    text.clear();
    DocumentBlock qr;
    qr.setQrCode(st.modelData().modelName().toUtf8() + "/" + st.modelData().version().toUtf8() +
                 "/" + st.modelData().serial().toUtf8());
    qr.setFormat(f);
    blocks << qr;

    DocumentBlock barcode;
    barcode.setBarCodeType(DocumentBlock::CODE_128);
    barcode.setBarCode(st.modelData().serial().toLatin1());
    barcode.setFormat(f);
    blocks << barcode;

    f.setFont(3);
    f.setFlags(LF_CENTER);
    blocks.push_front(DocumentBlock::createSimpleBlock(tr("ТЕСТИРОВАНИЕ ПО"), f));
    blocks.push_back(DocumentBlock::createSimpleBlock(tr("УСПЕШНО!"), f));
    return TextPrinterDocument (blocks);
}

TextPrinterDocument PrintDocumentFormatter::writeSerial(const CashboxStatus &st) const
{
    TextPrinterDocument doc = welcome(st);
    LineFormat f;
    f.setPrintGray(2000);
    f.setFont(2);
    f.setFlags(LF_CENTER | LF_BOLD | LF_INVERT);
    DocumentBlock b = DocumentBlock::createSimpleBlock(tr("Заводской номер успешно записан"), f);
    doc.appendBlock(b);
    return doc;
}

TextPrinterDocument PrintDocumentFormatter::document(BaseDoc *doc, bool brief, bool fromFs) const
{
    if(!doc) return TextPrinterDocument();
    switch (doc->type()) {
    case fdf::DocType::Registration:
    case fdf::DocType::ReRegistration: return fiscalization(dynamic_cast<BaseRegDoc*>(doc));
    case fdf::DocType::CycleOpen: return openCycle(dynamic_cast<CycleOpenDoc*>(doc));
    case fdf::DocType::CycleClose: return closeCycle(dynamic_cast<CycleCloseDoc*>(doc), brief);
    case fdf::DocType::CalcReport: return calcReport(dynamic_cast<CalcReportDoc*>(doc), brief);
    case fdf::DocType::FsClose: return closeFs(dynamic_cast<FsCloseDoc*>(doc));
    case fdf::DocType::Receipt:
    case fdf::DocType::Fosa:
    case fdf::DocType::Correction:
    case fdf::DocType::FosaCorrection: return receipt(dynamic_cast<BaseReceiptDoc*>(doc), fromFs);

    default: return TextPrinterDocument();
    }
}

TextPrinterDocument PrintDocumentFormatter::preprintFiscalization(const IncomingReg &reg,
                                                                  const CashboxStatus &st) const
{
    DocumentBlocks blocks;
    QStringList text;
    LineFormat f;
    f.setPrintGray(5000);
    f.setFont(2);
    f.setFlags(LF_CENTER);

    text << reg.ownerName() << tr("ИНН %1").arg(reg.ownerInn()) << reg.address();
    if(!reg.place().isEmpty()) text << tr("МЕСТО РАСЧЕТОВ %1").arg(reg.place());
    if(!reg.cashier().isEmpty())
    {
        QString prefix;
        if(!reg.cashier().toLower().contains("кассир"))prefix = "КАССИР ";
        text << tr("%1%2").arg(prefix).arg(reg.cashier());
    }
    QString name = (reg.forFiscalize() && !st.fs().isFiscalized()) ?
                tr("Отчет о рег.") : tr("Отчет о перерег.");
    text << name << ("(Предварительная печать)");
    blocks << DocumentBlock::createSimpleBlock(text, f);
    text.clear();
    f.setFlags(LF_RIGHT);
    text << QDateTime::currentDateTime().toString("dd.MM.yyyy hh:mm");

    if(reg.getAutoMode())
    {
        text << tr("АВТ.РЕЖ");
    }
    if(reg.getOfflineMode())
    {
        text << tr("АВТОНОМНО");
    }
    if(reg.getEncriptionMode())
    {
        text << tr("ШФД");
    }
    blocks << DocumentBlock::createSimpleBlock(text, f);
    text.clear();
    f.setFlags(LF_LEFT);
    if(!reg.getOfflineMode())
    {
        text << tr("ИНН ОФД\t%1").arg(reg.ofdInn())
             << tr("ОФД\t%1").arg(reg.ofdName())
             << tr("САЙТ ФНС\t%1").arg(reg.ftsSite())
             << tr("ЭЛ.АДР.ОТПРАВИТЕЛЯ\t%1").arg(reg.email());
    }
    if(reg.getAutoMode() && !reg.boxId().isEmpty())
    {
        text << tr("АВТОМАТ\t%1").arg(reg.boxId());
    }
    if(static_cast<qint32>(reg.taxes()) != 0)
    {
        text << tr("СНО\t%1").arg(fdf::taxesToString(reg.taxes()));
    }
    blocks << DocumentBlock::createSimpleBlock(text, f);
    text.clear();
    if(reg.extFlags().toUInt() != 0u)
    {
        f.setFlags(LF_RIGHT);
        blocks << DocumentBlock::createSimpleBlock(fdf::regFlag1290ToStrings(reg.extFlags().flags()), f);
    }
    f.setFlags(LF_RIGHT);
    if(reg.forChangeFs() || reg.forRefiscalize())
    {
        text << QStringLiteral("ИЗМ. СВЕД. О ККТ\t%1").arg(fdf::reasonsToString(reg.reasons().flags()));
        blocks << DocumentBlock::createSimpleBlock(text, f);
        text.clear();
    }
    text << QStringLiteral("ФФД ККТ\t%1").arg(fs::ffd2String(fs::FFD::FFD1_2))
         << QStringLiteral("ФФД ФН\t%1").arg(fs::ffd2String(st.fs().supportedFfd()))
         << QStringLiteral("Номер версии формата ФД\t%1").arg(fs::ffd2String(fs::FFD::FFD1_2))
         << QStringLiteral("ВЕР.ККТ\t%1").arg(st.modelData().version())
         << QStringLiteral("РН ККТ\t%1").arg(reg.regNumber())
         << QStringLiteral("ЗН ККТ\t%1").arg(st.modelData().serial())
         << QStringLiteral("ФД %1\tФН %2").arg(st.fs().status().lastFd() + 1).arg(st.fs().fsNumber());
    blocks << DocumentBlock::createSimpleBlock(text, f);
    text.clear();

    return TextPrinterDocument (blocks);
}

TextPrinterDocument PrintDocumentFormatter::fiscalization(BaseRegDoc *doc) const
{
    DocumentBlocks blocks;
    QStringList text;
    LineFormat f;
    f.setPrintGray(5000);
    f.setFont(2);
    f.setFlags(LF_CENTER);

    CoreRegData reg = doc->data();

    text << reg.ownerName() << tr("ИНН %1").arg(reg.ownerInn()) << reg.address();
    if(!reg.place().isEmpty()) text << tr("МЕСТО РАСЧЕТОВ %1").arg(reg.place());
    if(!reg.cashier().isEmpty())
    {
        QString prefix;
        if(!reg.cashier().toLower().contains("кассир"))prefix = "КАССИР ";
        text << tr("%1%2").arg(prefix).arg(reg.cashier());
    }
    text << doc->docShortName();
    blocks << DocumentBlock::createSimpleBlock(text, f);
    text.clear();
    f.setFlags(LF_RIGHT);
    text << doc->dt().toString("dd.MM.yyyy hh:mm");

    if(reg.getAutoMode())
    {
        text << tr("АВТ.РЕЖ");
    }
    if(reg.getOfflineMode())
    {
        text << tr("АВТОНОМНО");
    }
    if(reg.getEncriptionMode())
    {
        text << tr("ШФД");
    }
    blocks << DocumentBlock::createSimpleBlock(text, f);
    text.clear();
    f.setFlags(LF_LEFT);
    if(!reg.getOfflineMode())
    {
        text << tr("ИНН ОФД\t%1").arg(reg.ofdInn())
             << tr("ОФД\t%1").arg(reg.ofdName())
             << tr("САЙТ ФНС\t%1").arg(reg.ftsSite())
             << tr("ЭЛ.АДР.ОТПРАВИТЕЛЯ\t%1").arg(reg.email());
    }
    if(reg.getAutoMode() && !reg.boxId().isEmpty())
    {
        text << tr("АВТОМАТ\t%1").arg(reg.boxId());
    }
    if(static_cast<qint32>(reg.taxes()) != 0)
    {
        text << tr("СНО\t%1").arg(fdf::taxesToString(reg.taxes()));
    }
    blocks << DocumentBlock::createSimpleBlock(text, f);
    text.clear();
    if(reg.extFlags().toUInt() != 0u)
    {
        f.setFlags(LF_RIGHT);
        blocks << DocumentBlock::createSimpleBlock(fdf::regFlag1290ToStrings(reg.extFlags().flags()), f);
    }
    f.setFlags(LF_RIGHT);
    if(reg.forChangeFs() || reg.forRefiscalize())
    {
        text << QStringLiteral("ИЗМ. СВЕД. О ККТ\t%1").arg(fdf::reasonsToString(reg.reasons().flags()));
        blocks << DocumentBlock::createSimpleBlock(text, f);
        text.clear();
    }

    ReRegDoc *rereg = dynamic_cast<ReRegDoc *>(doc);
//    lmWarning() << rereg;
    if(rereg && !reg.forFiscalize())
    {
        blocks.append(fsCounters(rereg->fsCounters(), false));
    }

    text << QStringLiteral("ФФД ККТ\t%1").arg(fs::ffd2String(reg.cashboxFfd()))
         << QStringLiteral("ФФД ФН\t%1").arg(fs::ffd2String(reg.fsFfd()))
         << QStringLiteral("Номер версии формата ФД\t%1").arg(fs::ffd2String(reg.ffd()))
         << QStringLiteral("ВЕР.ККТ\t%1").arg(reg.cashboxVersion())
         << QStringLiteral("РН ККТ\t%1").arg(reg.regNumber())
         << QStringLiteral("ЗН ККТ\t%1").arg(reg.serial())
         << QStringLiteral("РЕСУРС КЛЮЧЕЙ\t%1").arg(reg.keysResource())
         << QStringLiteral("ФД %1\tФН %2").arg(doc->fd()).arg(doc->fsNumber());
    blocks << DocumentBlock::createSimpleBlock(text, f);
    text.clear();
    f.setFlags(LF_CENTER | LF_BOLD);
    text << QStringLiteral("ФП %1").arg(doc->fiscalCode(), 10, 10, QLatin1Char('0') );
    blocks << DocumentBlock::createSimpleBlock(text, f);
    text.clear();
    return TextPrinterDocument (blocks);
}

TextPrinterDocument PrintDocumentFormatter::calcReport(CalcReportDoc *doc, bool brief) const
{
    QStringList text;
    LineFormat f;
    f.setPrintGray(5000);
    f.setFont(2);
    DocumentBlocks blocks = fiscalHeader(doc);
    if(doc->cycle() > 0)
    {
        f.setFlags(LF_LEFT);
        text << QStringLiteral("СМЕНА %1\t%2").arg(doc->cycle()).arg(doc->dt().toString("dd.MM.yyyy hh:mm"));
    }
    else
    {
        f.setFlags(LF_RIGHT);
        text << doc->dt().toString("dd.MM.yyyy hh:mm");

    }
    blocks << DocumentBlock::createSimpleBlock(text, f);
    text.clear();
    if(doc->offlineMode())
    {
        f.setFlags(LF_RIGHT);
        blocks << DocumentBlock::createSimpleBlock(tr("АВТОНОМНО"), f);
    }
    else
    {
        f.setFlags(LF_LEFT);
        text << QStringLiteral("НЕПЕРЕДАННЫХ ФД\t%1").arg(doc->notSendedDocs())
             << QStringLiteral("ПЕРВЫЙ НЕПЕРЕДАННЫЙ ФД\t%1").arg(doc->firstNotSended());
        if(doc->notSendedDocs() > 0) text << QStringLiteral("ФД НЕ ПЕРЕДАНЫ С\t%1").arg(doc->firstNotSendedDocDt().toString("dd.MM.yyyy"));
        blocks << DocumentBlock::createSimpleBlock(text, f);
        text.clear();
    }
    f.setFlags(LF_LEFT);
    text << QStringLiteral("РЕСУРС КЛЮЧЕЙ\t%1").arg(doc->keysResource());
    if(doc->notSendedNotifications() > 0)
    {
        text << QStringLiteral("НЕПЕРЕДАНО УВЕДОМЛЕНИЙ\t%1").arg(doc->notSendedNotifications());
    }
    blocks << DocumentBlock::createSimpleBlock(text, f);
    text.clear();

    blocks.append(fsCounters(doc->fsCounters(), brief));
    blocks.append(shortFsCounters(doc->notSendedCounters(), brief));

    blocks.append(fiscalFooter(doc));
    return TextPrinterDocument (blocks);
}

TextPrinterDocument PrintDocumentFormatter::openCycle(CycleOpenDoc *doc) const
{
    QStringList text;
    LineFormat f;
    f.setPrintGray(5000);
    f.setFont(2);
    f.setFlags(LF_LEFT);
    DocumentBlocks blocks = fiscalHeader(doc);
    if(!doc->cashier().isEmpty())
    {
        f.setFlags(LF_LEFT);
        QString prefix;
        if(!doc->cashier().toLower().contains("кассир"))prefix = "КАССИР ";
        text << tr("%1%2").arg(prefix).arg(doc->cashier());
        blocks << DocumentBlock::createSimpleBlock(text, f);
        text.clear();
    }
    f.setFlags(LF_LEFT);
    text << QStringLiteral("СМЕНА %1\t%2").arg(doc->cycle()).arg(doc->dt().toString("dd.MM.yyyy hh:mm"));
    blocks << DocumentBlock::createSimpleBlock(text, f);
    text.clear();
    f.setFlags(LF_RIGHT);
    text = fs::warningsToString(doc->warnings());
    if(!text.isEmpty())
    {
        blocks << DocumentBlock::createSimpleBlock(text, f);
        text.clear();
    }
    f.setFlags(LF_LEFT);
    text << fdf::ofdWarningsToString(doc->ofdWarnings()).trimmed();
    if(text.back().isEmpty()) text.clear();
    text << QStringLiteral("ВЕР. ККТ %1\tФФД ККТ %2")
            .arg(doc->cashboxVersion()).arg(fs::ffd2String(doc->cashboxFFD()));
    blocks << DocumentBlock::createSimpleBlock(text, f);
    text.clear();
    blocks.append(fiscalFooter(doc));
    return TextPrinterDocument (blocks);
}

TextPrinterDocument PrintDocumentFormatter::closeCycle(CycleCloseDoc *doc, bool brief) const
{
    QStringList text;
    LineFormat f;
    f.setPrintGray(5000);
    f.setFont(2);
    DocumentBlocks blocks = fiscalHeader(doc);
    if(!doc->cashier().isEmpty())
    {
        f.setFlags(LF_LEFT);
        QString prefix;
        if(!doc->cashier().toLower().contains("кассир"))prefix = "КАССИР ";
        text << tr("%1%2").arg(prefix).arg(doc->cashier());
        blocks << DocumentBlock::createSimpleBlock(text, f);
        text.clear();
    }
    f.setFlags(LF_LEFT);
    text << QStringLiteral("СМЕНА %1\t%2").arg(doc->cycle()).arg(doc->dt().toString("dd.MM.yyyy hh:mm"));
    blocks << DocumentBlock::createSimpleBlock(text, f);
    text.clear();
    f.setFlags(LF_LEFT);
    text << QStringLiteral("ФД ЗА СМЕНУ\t%1").arg(doc->documentsCount())
         << QStringLiteral("%1\t%2")
            .arg(regData_.extFlags().getFosaCashbox() ?"БСО ЗА СМЕНУ" :"ЧЕКОВ ЗА СМЕНУ")
            .arg(doc->receiptsCount())
            << QStringLiteral("РЕСУРС КЛЮЧЕЙ\t%1").arg(doc->keysResource())
         << QStringLiteral("НЕПЕРЕДАННЫХ ФД\t%1").arg(doc->notSendedDocs());
    if(doc->notSendedDocs() > 0)
    {
        text<< QStringLiteral("ФД НЕ ПЕРЕДАНЫ С\t%1")
               .arg(doc->firstNotSendedDocDt().toString("dd.MM.yyyy"));
    }
    if(doc->notSendedNotifications() > 0)
    {
        text << QStringLiteral("НЕПЕРЕДАНО УВЕДОМЛЕНИЙ\t%1").arg(doc->notSendedNotifications());
    }
    blocks << DocumentBlock::createSimpleBlock(text, f);
    text.clear();

    blocks.append(fsCounters(doc->fsCounters(), brief));
    blocks.append(fsCounters(doc->cycleCounters(), brief));

    f.setFlags(LF_RIGHT);
    text = fs::warningsToString(doc->warnings());
    text.append(fdf::incorrectLabelsFlagToString(doc->incorrectLabelsFlags()));
    text.append(fdf::incorrectNotificationsFlagToString(doc->incorrectNotificationsFlags()));
    if(!text.isEmpty())
    {
        blocks << DocumentBlock::createSimpleBlock(text, f);
        text.clear();
    }
    f.setFlags(LF_LEFT);
    text << fdf::ofdWarningsToString(doc->ofdWarnings()).trimmed();

    if(text.back().isEmpty()) text.clear();
    else
    {
        blocks << DocumentBlock::createSimpleBlock(text, f);
        text.clear();
    }

    blocks.append(fiscalFooter(doc));
    return TextPrinterDocument (blocks);
}

TextPrinterDocument PrintDocumentFormatter::closeFs(FsCloseDoc *doc) const
{
    QStringList text;
    LineFormat f;
    f.setPrintGray(5000);
    f.setFont(2);
    DocumentBlocks blocks = fiscalHeader(doc);
    if(!doc->cashier().isEmpty())
    {
        f.setFlags(LF_LEFT);
        QString prefix;
        if(!doc->cashier().toLower().contains("кассир"))prefix = "КАССИР ";
        text << tr("%1%2").arg(prefix).arg(doc->cashier());
        blocks << DocumentBlock::createSimpleBlock(text, f);
        text.clear();
    }
    f.setFlags(LF_LEFT);
    text << QStringLiteral("\t%1").arg(doc->dt().toString("dd.MM.yyyy hh:mm"));
    blocks << DocumentBlock::createSimpleBlock(text, f);
    text.clear();
    f.setFlags(LF_LEFT);
    blocks.append(fsCounters(doc->fsCounters(), false));
    blocks.append(fiscalFooter(doc));
    return TextPrinterDocument (blocks);
}

TextPrinterDocument PrintDocumentFormatter::receipt(BaseReceiptDoc *doc, bool fromFs) const
{
    QStringList text;
    LineFormat f;
    f.setPrintGray(5000);
    f.setFont(2);
    DocumentBlocks blocks = fiscalHeader(doc);
    if(!doc->cashier().isEmpty())
    {
        f.setFlags(LF_LEFT);
        QString prefix;
        if(!doc->cashier().toLower().contains("кассир"))prefix = "КАССИР\t";
        text << tr("%1%2").arg(prefix).arg(doc->cashier());
        blocks << DocumentBlock::createSimpleBlock(text, f);
        text.clear();
    }
    f.setFlags(LF_LEFT);
    text << QStringLiteral("ЧЕК %1\t%2").arg(doc->receiptNumber())
            .arg(doc->dt().toString("dd.MM.yyyy hh:mm"))
         << QStringLiteral("%1\tСМЕНА %2")
            .arg(fdf::paymentTypeToString(doc->paymentAttr()))
            .arg(doc->cycle());
    if(!doc->buyerPhone().isEmpty())
    {
        text << QStringLiteral("%1\t%2")
                .arg(doc->buyerPhone().contains("@") ? "ЭЛ. АДР. ПОКУПАТЕЛЯ" : "ТЕЛ. ПОКУПАТЕЛЯ")
                .arg(doc->buyerPhone());
    }
    if(doc->buyerInfo().has_value())
    {
        text << QStringLiteral("ПОКУПАТЕЛЬ\t%1").arg(doc->buyerInfo().value().name());
        if(!doc->buyerInfo().value().address().isEmpty())
        {
            text << QStringLiteral("АДРЕС КЛИЕНТА\t%1").arg(doc->buyerInfo().value().address());
        }
    }
    if(!doc->boxId().isEmpty())
    {
        text << QStringLiteral("АВТОМАТ\t%1").arg(doc->boxId());
    }
    text << QStringLiteral("СНО %1\t%2").arg(fdf::taxToString(doc->tax()))
                                             .arg(doc->additionalParam()).trimmed();
    text << QStringLiteral("САЙТ ФНС\t%1").arg(doc->fnsSite());

    CorrectionDoc *corrdoc = dynamic_cast<CorrectionDoc*>(doc);
    if(corrdoc )
    {
        text << QStringLiteral("ОСН. ДЛЯ КОРР.\t%1")
                .arg(corrdoc->correctionReason().date().toString("dd.MM.yyyy"));
    }

    blocks << DocumentBlock::createSimpleBlock(text, f);
    text.clear();
    PSimpleReceiptOperations ops = doc->operations();
    for(const PSimpleReceiptOperation &o :ops)
    {
        if(o.dynamicCast<ReceiptOperation>()) blocks.append(receiptOperation(*o.dynamicCast<ReceiptOperation>()));
        else if(o) blocks.append(simpleReceiptOperation(*o));
    }
    f.setFont(3);
    f.setFlags(LF_LEFT | LF_BOLD);
    text << QStringLiteral("ИТОГ\t%1").arg(doc->amount().toString('.'));
    blocks << DocumentBlock::createSimpleBlock(text, f);
    text.clear();
    f.setFont(2);
    f.setFlags(LF_LEFT);
    if(doc->change().has_value() && doc->change().value().value() > 0)
    {
        FixNumber total = doc->cash().value_or(FixNumber(DEFAULT_AMOUNT_MULT, 0ll)) + doc->change().value();
        text << QStringLiteral("ПОЛУЧЕНО НАЛ.\t%1").arg(total.toString('.'));
        text << QStringLiteral("СДАЧА\t%1").arg(doc->change().value().toString('.'));
    }
    if(doc->cash().has_value())
    {
        text << QStringLiteral("НАЛ.\t%1").arg(doc->cash().value().toString('.'));
    }
    if(doc->card().has_value())
    {
        text << QStringLiteral("БЕЗНАЛ.\t%1").arg(doc->card().value().toString('.'));
    }
    if(doc->prepay().has_value())
    {
        text << QStringLiteral("ПРЕДВАРИТЕЛЬНАЯ ОПЛАТА (АВАНС)\t%1").arg(doc->prepay().value().toString('.'));
    }
    if(doc->postpay().has_value())
    {
        text << QStringLiteral("ПОСЛЕДУЮЩАЯ ОПЛАТА (КРЕДИТ)\t%1").arg(doc->postpay().value().toString('.'));
    }
    if(doc->barter().has_value())
    {
        text << QStringLiteral("ИНАЯ ФОРМА ОПЛАТЫ\t%1").arg(doc->barter().value().toString('.'));
    }
    if(doc->vat20().has_value())
    {
        text << QStringLiteral("СУММА НДС 20%\t%1").arg(doc->vat20().value().toString('.'));
    }
    if(doc->vat10().has_value())
    {
        text << QStringLiteral("СУММА НДС 10%\t%1").arg(doc->vat10().value().toString('.'));
    }
    if(doc->vat20_120().has_value())
    {
        text << QStringLiteral("СУММА НДС 20/120\t%1").arg(doc->vat20_120().value().toString('.'));
    }
    if(doc->vat10_110().has_value())
    {
        text << QStringLiteral("СУММА НДС 10/110\t%1").arg(doc->vat10_110().value().toString('.'));
    }
    if(doc->vat0().has_value())
    {
        text << QStringLiteral("СУММА НДС 0%\t%1").arg(doc->vat0().value().toString('.'));
    }
    if(doc->vatNone().has_value())
    {
        text << QStringLiteral("СУММА БЕЗ НДС \t%1").arg(doc->vatNone().value().toString('.'));
    }
    if(doc->t2107() == 1)
    {
        text << "\tКМ?";
    }
    if(doc->userParam().has_value())
    {
        text << doc->userParam().value().name() << doc->userParam().value().value();
    }


    blocks << DocumentBlock::createSimpleBlock(text, f);
    text.clear();
    if(!fromFs)
    {
        f.setFlags(LF_CENTER);
        DocumentBlock qr;
        qr.setQrCode(doc->qrText().toUtf8());
        qr.setFormat(f);
        blocks << qr;
    }

    f.setFlags(LF_LEFT);
    blocks.append(fiscalFooter(doc));
    return TextPrinterDocument (blocks);
}

TextPrinterDocument PrintDocumentFormatter::xreport(XReport *rep, bool brief) const
{
    QStringList text;
    LineFormat f;
    f.setPrintGray(5000);
    f.setFont(2);
    DocumentBlocks blocks;

    if(!rep->header().isEmpty())
    {
        BbCodeParser bcp;
        bcp.parseText(rep->header());
        blocks.append(bcp.blocks());
    }

    f.setFlags(LF_CENTER);

    text << rep->regData().ownerName() << tr("ИНН %1").arg(rep->regData().ownerInn())
         << rep->address();
    if(!rep->place().isEmpty()) text << tr("МЕСТО РАСЧЕТОВ %1").arg(rep->regData().place());
                                                text << rep->docName() << rep->dt().toString("dd.MM.yyyy hh:mm");
    blocks << DocumentBlock::createSimpleBlock(text, f);
    text.clear();

    if(!rep->cashier().isEmpty())
    {
        f.setFlags(LF_LEFT);
        QString prefix;
        if(!rep->cashier().toLower().contains("кассир"))prefix = "КАССИР ";
        text << tr("%1%2").arg(prefix).arg(rep->cashier());
        blocks << DocumentBlock::createSimpleBlock(text, f);
        text.clear();
    }
    f.setFlags(LF_LEFT);
    if(rep->type() == fdf::DocType::CycleReport)
    {
        text << QStringLiteral("СМЕНА %1\tОТКРЫТА ").arg(rep->counters().cycle()).arg(rep->status().cycleOpenDt().toString("dd.MM.yyyy hh:mm"));
        if(!rep->status().cycleIsOpen())
        {
            QString s = QStringLiteral("ЗАКРЫТА %1").arg(rep->status().cycleOpenDt().toString("dd.MM.yyyy hh:mm"));
            if(text.last().size() + s.size() + 1 <= f.symbolsPerLine())
            {
                text.last() += " " + s;
            }
            else
            {
                text << ("\t" + s);
            }
        }
    }
    else
    {
        text << QStringLiteral("ЗАКРЫТО СМЕН\t%1").arg(rep->counters().cycle());
    }
    blocks << DocumentBlock::createSimpleBlock(text, f);
    text.clear();
    f.setFlags(LF_LEFT);
    text << QStringLiteral("ВСЕГО ФД\t%1").arg(rep->status().status().lastFd());
    if(rep->type() == fdf::DocType::CycleReport)
    {
        text << QStringLiteral("%1\t%2")
                    .arg(rep->regData().extFlags().getFosaCashbox() ?"БСО ЗА СМЕНУ" :"ЧЕКОВ ЗА СМЕНУ")
                    .arg(rep->status().cycle().receipt());

    }
    text << QStringLiteral("НЕПЕРЕДАННЫХ ФД\t%1").arg(rep->status().transport().messagesCount());
    if(rep->status().transport().messagesCount() > 0)
    {
        text<< QStringLiteral("ФД НЕ ПЕРЕДАНЫ С\t%1")
               .arg(rep->status().transport().firstDocDt().toString("dd.MM.yyyy"));
    }
    if(rep->status().labelsTransport().notificationCount() > 0)
    {
        text << QStringLiteral("НЕПЕРЕДАНО УВЕДОМЛЕНИЙ\t%1")
                    .arg(rep->status().labelsTransport().notificationCount());
    }
    blocks << DocumentBlock::createSimpleBlock(text, f);
    text.clear();

    blocks.append(fsCounters(rep->counters(), brief));

    f.setFlags(LF_RIGHT);
    text = fs::warningsToString(rep->status().status().warnings());
    if(!text.isEmpty())
    {
        blocks << DocumentBlock::createSimpleBlock(text, f);
        text.clear();
    }
    f.setFlags(LF_LEFT);
    if(!text.isEmpty() && text.back().isEmpty()) text.clear();
    else
    {
        blocks << DocumentBlock::createSimpleBlock(text, f);
        text.clear();
    }
    f.setFont(2);
    f.setFlags(LF_LEFT);
    text.clear();
    text << QStringLiteral("РН ККТ\t%1").arg(rep->regData().regNumber())
         << QStringLiteral("ФН\t%1").arg(rep->status().status().fsNumber());
    blocks << DocumentBlock::createSimpleBlock(text, f);
    text.clear();
    if(!rep->footer().isEmpty())
    {
        BbCodeParser bcp;
        bcp.parseText(rep->footer());
        blocks.append(bcp.blocks());
    }
    return TextPrinterDocument (blocks);
}



DocumentBlocks PrintDocumentFormatter::fiscalHeader(BaseWorkDoc *doc) const
{
    DocumentBlocks blocks;


    if(!doc->header().isEmpty())
    {
        BbCodeParser bcp;
        bcp.parseText(doc->header());
        blocks.append(bcp.blocks());
    }

    QStringList text;
    LineFormat f;
    f.setPrintGray(5000);
    f.setFont(2);
    f.setFlags(LF_CENTER);

    text << doc->ownerName() << tr("ИНН %1").arg(doc->ownerInn()) << doc->address();
    if(!doc->place().isEmpty()) text << tr("МЕСТО РАСЧЕТОВ %1").arg(doc->place());
    text << doc->docShortName();
    blocks << DocumentBlock::createSimpleBlock(text, f);
    text.clear();
    return blocks;
}

DocumentBlocks PrintDocumentFormatter::fiscalFooter(BaseWorkDoc *doc) const
{
    DocumentBlocks blocks;
    QStringList text;
    LineFormat f;
    f.setPrintGray(5000);
    f.setFont(2);
    f.setFlags(LF_LEFT);
    text << QStringLiteral("РН ККТ\t%1").arg(doc->regNumber())
         << QStringLiteral("ФД %1\tФН %2").arg(doc->fd()).arg(doc->fsNumber());
    blocks << DocumentBlock::createSimpleBlock(text, f);
    text.clear();
    f.setFlags(LF_CENTER | LF_BOLD);
    text << QStringLiteral("ФП %1").arg(doc->fiscalCode(), 10, 10, QLatin1Char('0') );
    blocks << DocumentBlock::createSimpleBlock(text, f);
    text.clear();
    if(!doc->footer().isEmpty())
    {
        BbCodeParser bcp;
        bcp.parseText(doc->footer());
        blocks.append(bcp.blocks());
    }
    return blocks;
}

DocumentBlocks PrintDocumentFormatter::shortFsCounters(const ShortTaggedFsCounters &counters, bool brief,
                                                      const QString &underLine) const
{
    DocumentBlocks blocks;
    LineFormat f;
    f.setPrintGray(5000);
    f.setFont(2);
    f.setFlags(LF_CENTER);
    blocks << DocumentBlock::createSimpleBlock(counters.title(), f)
           << DocumentBlock::createSimpleBlock(QStringLiteral("ЧЕКОВ(БСО)\t%1").arg(counters.count()), f);
    if(counters.debit().count() > 0)blocks << shortOperationsCounters(counters.debit(), tr("ПРИХ."), brief);
    if(counters.debitRefund().count() > 0)blocks << shortOperationsCounters(counters.debitRefund(), tr("ВОЗВР. ПРИХ."), brief);
    if(counters.credit().count() > 0)blocks << shortOperationsCounters(counters.credit(), tr("РАСХ."), brief);
    if(counters.creditRefund().count() > 0)blocks << shortOperationsCounters(counters.creditRefund(), tr("ВОЗВР. РАСХ."), brief);
    if(!underLine.isEmpty()) blocks << DocumentBlock::createSimpleBlock(underLine, f);
    return blocks;
}

DocumentBlocks PrintDocumentFormatter::correctionCounters(const FsCorrectionCounters &counters, bool brief) const
{
    DocumentBlocks blocks;
    LineFormat f;
    f.setPrintGray(5000);
    f.setFont(2);
    f.setFlags(LF_CENTER);
    blocks << DocumentBlock::createSimpleBlock("ЧЕКИ (БСО) КОРРЕКЦИИ", f)
           << DocumentBlock::createSimpleBlock(QStringLiteral("ЧЕКОВ(БСО)\t%1").arg(counters.count()), f);
    if(counters.debit().count() > 0)blocks << correctionCounter(counters.debit(), tr("ПРИХ."), brief);
    if(counters.debitRefund().count() > 0)blocks << correctionCounter(counters.debitRefund(), tr("ВОЗВР. ПРИХ."), brief);
    if(counters.credit().count() > 0)blocks << correctionCounter(counters.credit(), tr("РАСХ."), brief);
    if(counters.creditRefund().count() > 0)blocks << correctionCounter(counters.creditRefund(), tr("ВОЗВР. РАСХ."), brief);
//    if(!underLine.isEmpty()) blocks << DocumentBlock::createSimpleBlock(underLine, f);
    return blocks;
}

DocumentBlock PrintDocumentFormatter::shortOperationsCounters(const FsShortOperationCounters &counters,
                                                              const QString &typeTitle, bool brief) const
{
    LineFormat f;
    f.setPrintGray(5000);
    f.setFont(2);
    f.setFlags(LF_LEFT);
    QStringList text;
    text << tr("%1 %2\t%3").arg(regData_.extFlags().getFosaCashbox() ? "КОЛ.БСО" : "КОЛ. ЧЕКОВ")
                .arg(typeTitle).arg(counters.count());
    if(!brief || counters.totalAmount().value() > 0)
    {
        text << tr("СУММА %1 всего\t%2").arg(typeTitle).arg(counters.totalAmount().toString(QLatin1Char('.')));
    }
    if(counters.totalAmount().value() > 0ll)
    {
         if(!brief || counters.cash().value() > 0ll)
            text << tr("СУММА %1 НАЛ.\t%2").arg(typeTitle).arg(counters.cash().toString(QLatin1Char('.')));
         if(!brief || counters.card().value() > 0ll)
             text << tr("СУММА %1 БЕЗНАЛ.\t%2").arg(typeTitle).arg(counters.card().toString(QLatin1Char('.')));
         if(!brief || counters.prepay().value() > 0ll)
             text << tr("СУММА ПРЕДВАРИТЕЛЬНЫХ ОПЛАТ (АВАНСОВ)\t%1")
                         .arg(counters.prepay().toString(QLatin1Char('.')));
         if(!brief || counters.postpay().value() > 0ll)
             text << tr("СУММА ПОСЛЕДУЮЩИХ ОПЛАТ (КРЕДИТОВ)\t%1")
                         .arg(counters.postpay().toString(QLatin1Char('.')));
         if(!brief || counters.barter().value() > 0ll)
             text << tr("СУММА ИНОЙ ФОРМОЙ ОПЛАТЫ\t%1").arg(counters.barter().toString(QLatin1Char('.')));
    }
    return DocumentBlock::createSimpleBlock(text, f);

}

DocumentBlock PrintDocumentFormatter::correctionCounter(const FsCorrectionCounter &counters,
                                                        const QString &typeTitle, bool brief) const
{
    LineFormat f;
    f.setPrintGray(5000);
    f.setFont(2);
    f.setFlags(LF_LEFT);
    QStringList text;
    text << tr("%1 %2\t%3").arg(regData_.extFlags().getFosaCashbox() ? "КОЛ.БСО" : "КОЛ. ЧЕКОВ")
                .arg(typeTitle).arg(counters.count());
    if(!brief || counters.amount().value() > 0)
    {
        text << tr("СУММА %1 всего\t%2").arg(typeTitle).arg(counters.amount().toString(QLatin1Char('.')));
    }
    return DocumentBlock::createSimpleBlock(text, f);
}

DocumentBlocks PrintDocumentFormatter::opperationsCounters(const FsOperationCounters &counters,
                                                           const QString &typeTitle, bool brief) const
{
    DocumentBlocks blocks;
    blocks << shortOperationsCounters(counters, typeTitle, brief);
    LineFormat f;
    f.setPrintGray(5000);
    f.setFont(2);
    f.setFlags(LF_LEFT);
    QStringList text;

    if(counters.totalAmount().value() )
    {
        if(!brief || counters.vat20().value() > 0ll)
            text << tr("СУММА НДС 20% %1\t%2").arg(typeTitle).arg(counters.vat20().toString(QLatin1Char('.')));
        if(!brief || counters.vat10().value() > 0ll)
            text << tr("СУММА НДС 10% %1\t%2").arg(typeTitle).arg(counters.vat10().toString(QLatin1Char('.')));
        if(!brief || counters.vat20_120().value() > 0ll)
            text << tr("СУМ. НДС РАСЧ. 20/120 %1\t%2")
                        .arg(typeTitle).arg(counters.vat20_120().toString(QLatin1Char('.')));
        if(!brief || counters.vat10_110().value() > 0ll)
            text << tr("СУМ. НДС РАСЧ. 10/110 %1\t%2")
                        .arg(typeTitle).arg(counters.vat10_110().toString(QLatin1Char('.')));
        if(!brief || counters.vat0().value() > 0ll)
            text << tr("ОБОРОТ С НДС 0% %1\t%2").arg(typeTitle).arg(counters.vat0().toString(QLatin1Char('.')));
        if(!brief || counters.vatNone().value() > 0ll)
            text << tr("ОБОРОТ БЕЗ НДС %1\t%2").arg(typeTitle).arg(counters.vatNone().toString(QLatin1Char('.')));
    }

    if(!text.isEmpty()) blocks <<  DocumentBlock::createSimpleBlock(text, f);
    return blocks;
}

DocumentBlocks PrintDocumentFormatter::fsCounters(const TaggedFsCounters &counters, bool brief) const
{
    DocumentBlocks blocks;
    LineFormat f;
    f.setPrintGray(5000);
    f.setFont(2);
    f.setFlags(LF_CENTER);
    blocks << DocumentBlock::createSimpleBlock(counters.title(), f);
    f.setFlags(LF_LEFT);
    QStringList text;
    text << text << tr("%1\t%2").arg(regData_.extFlags().getFosaCashbox() ? "ВСЕГО БСО" : "ВСЕГО ЧЕКОВ")
            .arg(counters.count());
    blocks << DocumentBlock::createSimpleBlock(text, f);
    blocks.append(opperationsCounters(counters.debit(), tr("ПРИХ."), brief));
    blocks.append(opperationsCounters(counters.debitRefund(), tr("ВОЗВР. ПРИХ."), brief));
    blocks.append(opperationsCounters(counters.credit(), tr("РАСХ."), brief));
    blocks.append(opperationsCounters(counters.creditRefund(), tr("ВОЗВР. РАСХ."), brief));
    blocks.append(shortFsCounters(counters.corrections(), brief, QString()));
    DocumentBlock::createSimpleBlock(QStringLiteral("------------------------"), f);
    return blocks;
}

DocumentBlocks PrintDocumentFormatter::fsCounters(const FsCounters &counters, bool brief) const
{
    DocumentBlocks blocks;
    LineFormat f;
    f.setPrintGray(5000);
    f.setFont(2);
    f.setFlags(LF_CENTER);
    blocks << DocumentBlock::createSimpleBlock(counters.title(), f);
    f.setFlags(LF_LEFT);
    QStringList text;
    text << text << tr("%1\t%2").arg(regData_.extFlags().getFosaCashbox() ? "ВСЕГО БСО" : "ВСЕГО ЧЕКОВ")
            .arg(counters.count());
    blocks << DocumentBlock::createSimpleBlock(text, f);
    blocks.append(opperationsCounters(counters.debit(), tr("ПРИХ."), brief));
    blocks.append(opperationsCounters(counters.debitRefund(), tr("ВОЗВР. ПРИХ."), brief));
    blocks.append(opperationsCounters(counters.credit(), tr("РАСХ."), brief));
    blocks.append(opperationsCounters(counters.creditRefund(), tr("ВОЗВР. РАСХ."), brief));
    blocks.append(correctionCounters(counters.corrections(), brief));
    DocumentBlock::createSimpleBlock(QStringLiteral("------------------------"), f);
    return blocks;
}

DocumentBlocks PrintDocumentFormatter::simpleReceiptOperation(const SimpleReceiptOperation &op) const
{
    DocumentBlocks res;

    LineFormat f;
    f.setPrintGray(5000);
    f.setFont(2);
    f.setFlags(LF_LEFT);
    QStringList text;
    text << QStringLiteral("%1\t%2").arg(fdf::opTypePrint(op.type()))
            .arg(fdf::itemPaymentTypeToString(op.paymentType()));
    if(op.paymentType()  != fdf::ItemPaymentType::Advance)
    {
        text << op.name();
    }

//    text << QStringLiteral("%1x%2\t=%3").arg(op.price().toString('.'))
//            .arg(op.quantity().toString('.'))
//            .arg(op.cost().toString('.'));
    if(op.unit().has_value())
    {
        text << QStringLiteral("%1x%2 %3\t=%4").arg(op.price().toString('.'))
                .arg(op.quantity().toString('.'))
                .arg(fdf::unitText(op.unit().value()))
                .arg(op.cost().toString('.'));
    }
    else
    {
        text << QStringLiteral("%1x%2\t=%3").arg(op.price().toString('.'))
                .arg(op.quantity().toString('.'))
                .arg(op.cost().toString('.'));
    }
    text << QStringLiteral("%1\t%2").arg(fdf::vatLabel(op.vatRate())).arg(op.printedCostVat());
    if(!op.additionalParam().isEmpty()) text << op.additionalParam();
    //NOTE: В связи с изменениями в ФФД. возможно нужно будет отключать по настройке. а как тогда с апи?
//    if(fdf::itemPayAgentFlagIsValid(op.agentFlag()))
//    {
//        text << fdf::itemPayAgentFlagToString(op.agentFlag())
//             << QStringLiteral("ИНН ПОСТАВЩИКА\t%1").arg(op.providerInn())
//             << QStringLiteral("ТЛФ. ПОСТ.\t%1")
//                .arg(op.providerData().value_or(ProviderData()).phones().join(","))
//             << op.providerData().value_or(ProviderData()).name();
//        if(op.agentData().has_value())
//        {
//            if(op.agentFlag() == fdf::ItemPayAgentFlag::BankAgent ||
//                    op.agentFlag() == fdf::ItemPayAgentFlag::BankSubAgent)
//            {
//                text << QStringLiteral("ОПЕРАТОР ПЕРЕВОДА\t%1").arg(op.agentData().value().operatorName())
//                     << QStringLiteral("АДР. ОП. ПЕРЕВОДА\t%1").arg(op.agentData().value().operatorAddress())
//                     << QStringLiteral("ИНН ОП. ПЕРЕВОДА\t%1").arg(op.agentData().value().operatorInn())
//                     << QStringLiteral("ОП. АГЕНТА\t%1").arg(op.agentData().value().bankOperation())
//                     << QStringLiteral("ТЛФ. ОП. ПЕРЕВОДА\t%1")
//                        .arg(op.agentData().value().operatorPhones().join(","));
//            }
//            else
//            {
//                text << QStringLiteral("ТЛФ. ПЛ. АГЕНТА\t%1")
//                        .arg(op.agentData().value().agentPhones().join(","))
//                     << QStringLiteral("ТЛФ. ОП. ПР. ПЛАТЕЖА\t%1")
//                        .arg(op.agentData().value().paysOperatorPhones().join(","));
//            }
//        }
//    }
    if(op.exciseTax().has_value())
    {
        text << QStringLiteral("АКЦИЗ\t%1").arg(op.exciseTax().value().toString('.'));
    }
    if(!op.countryCode().isEmpty() && !op.declarationNumber().isEmpty())
    {
        text << QStringLiteral("КОД СТРАНЫ\t%1").arg(op.countryCode())
             << QStringLiteral("ДЕКЛАРАЦИЯ\t%1").arg(op.declarationNumber());
    }


    res << DocumentBlock::createSimpleBlock(text, f);
    text.clear();


    return res;
}

DocumentBlocks PrintDocumentFormatter::receiptOperation(const ReceiptOperation &op) const
{
    if(!op.isLabled()) return simpleReceiptOperation(op);
    DocumentBlocks res;

    LineFormat f;
    f.setPrintGray(5000);
    f.setFont(2);
    f.setFlags(LF_LEFT);
    QStringList text;
    text << QStringLiteral("%1\t%2").arg(fdf::opTypePrint(op.type()))
            .arg(fdf::itemPaymentTypeToString(op.paymentType()));
    if(op.paymentType()  != fdf::ItemPaymentType::Advance)
    {
        text << op.name();
    }

    if(op.fraction().has_value()  && op.fraction().value().isValid())
    {
        text << QStringLiteral("%1x%2 шт.\t=%3").arg(op.price().toString('.'))
                .arg(op.quantity().toString('.'))
                .arg(op.cost().toString('.'));


        lmWarning() << fdf::checkLabelFlagToString(op.checkLabelFlags());
        if(!fdf::checkLabelFlagToString(op.checkLabelFlags()).trimmed().isEmpty())
        {
            QString m = QStringLiteral("%1").arg(fdf::checkLabelFlagToString(op.checkLabelFlags()));
            lmWarning() << m;
            if(!op.controlCode().trimmed().isEmpty())
            {
                m += QStringLiteral(" %1").arg(op.controlCode());
            }
            lmWarning() << m;
            text << QStringLiteral("ДРОБЬ %1/%2\t%3")
                    .arg(op.fraction().value().nominator())
                    .arg(op.fraction().value().denominator())
                    .arg(m);
        }
        else
        {
            text << QStringLiteral("ДРОБЬ\t%1/%2")
                    .arg(op.fraction().value().nominator())
                    .arg(op.fraction().value().denominator());
        }

    }
    else
    {
        if(op.unit().has_value())
        {
            text << QStringLiteral("%1x%2 %3\t=%4").arg(op.price().toString('.'))
                    .arg(op.quantity().toString('.'))
                    .arg(fdf::unitText(op.unit().value()))
                    .arg(op.cost().toString('.'));
        }
        else
        {
            text << QStringLiteral("%1x%2\t=%3").arg(op.price().toString('.'))
                    .arg(op.quantity().toString('.'))
                    .arg(op.cost().toString('.'));
        }
        lmWarning() << fdf::checkLabelFlagToString(op.checkLabelFlags());
        if(!fdf::checkLabelFlagToString(op.checkLabelFlags()).trimmed().isEmpty())
        {
            QString m = QStringLiteral("%1").arg(fdf::checkLabelFlagToString(op.checkLabelFlags()));
            lmWarning() << m;
            if(!op.controlCode().trimmed().isEmpty())
            {
                m += QStringLiteral(" %1").arg(op.controlCode());
            }
            lmWarning() << m;
            text << QStringLiteral("\t%1").arg(m);
        }
    }

//    lmWarning() << loglist(text);
    text << QStringLiteral("%1\t%2").arg(fdf::vatLabel(op.vatRate())).arg(op.printedCostVat());
    if(!op.additionalParam().isEmpty()) text << op.additionalParam();
    //NOTE: В связи с изменениями в ФФД. возможно нужно будет отключать по настройке. а как тогда с апи?
//    if(fdf::itemPayAgentFlagIsValid(op.agentFlag()))
//    {
//        text << fdf::itemPayAgentFlagToString(op.agentFlag())
//             << QStringLiteral("ИНН ПОСТАВЩИКА\t%1").arg(op.providerInn())
//             << QStringLiteral("ТЛФ. ПОСТ.\t%1")
//                .arg(op.providerData().value_or(ProviderData()).phones().join(","))
//             << op.providerData().value_or(ProviderData()).name();
//        if(op.agentData().has_value())
//        {
//            if(op.agentFlag() == fdf::ItemPayAgentFlag::BankAgent ||
//                    op.agentFlag() == fdf::ItemPayAgentFlag::BankSubAgent)
//            {
//                text << QStringLiteral("ОПЕРАТОР ПЕРЕВОДА\t%1").arg(op.agentData().value().operatorName())
//                     << QStringLiteral("АДР. ОП. ПЕРЕВОДА\t%1").arg(op.agentData().value().operatorAddress())
//                     << QStringLiteral("ИНН ОП. ПЕРЕВОДА\t%1").arg(op.agentData().value().operatorInn())
//                     << QStringLiteral("ОП. АГЕНТА\t%1").arg(op.agentData().value().bankOperation())
//                     << QStringLiteral("ТЛФ. ОП. ПЕРЕВОДА\t%1")
//                        .arg(op.agentData().value().operatorPhones().join(","));
//            }
//            else
//            {
//                text << QStringLiteral("ТЛФ. ПЛ. АГЕНТА\t%1")
//                        .arg(op.agentData().value().agentPhones().join(","))
//                     << QStringLiteral("ТЛФ. ОП. ПР. ПЛАТЕЖА\t%1")
//                        .arg(op.agentData().value().paysOperatorPhones().join(","));
//            }
//        }
//    }
    if(op.exciseTax().has_value())
    {
        text << QStringLiteral("АКЦИЗ\t%1").arg(op.exciseTax().value().toString('.'));
    }
    if(!op.countryCode().isEmpty() && !op.declarationNumber().isEmpty())
    {
        text << QStringLiteral("КОД СТРАНЫ\t%1").arg(op.countryCode())
             << QStringLiteral("ДЕКЛАРАЦИЯ\t%1").arg(op.declarationNumber());
    }


    res << DocumentBlock::createSimpleBlock(text, f);
    text.clear();


    return res;
    //TODO: После того, как доделаю маркировку
    return DocumentBlocks();
}
