#ifndef PRINTDOCUMENTFORMATTER_H
#define PRINTDOCUMENTFORMATTER_H

#include "textprinterdocument.h"
#include "cashboxstatus.h"
#include "incomingreg.h"
#include "regdoc.h"
#include "cycleopendoc.h"
#include "calcreportdoc.h"
#include "cycleclosedoc.h"
#include "fsclosedoc.h"
#include "receiptdoc.h"
#include "xreport.h"

#include <QObject>

class PrintDocumentFormatter : public QObject
{
    Q_OBJECT
    Q_DISABLE_COPY(PrintDocumentFormatter)
public:
    explicit PrintDocumentFormatter(const RegData &rd, QObject *parent = nullptr);
    ~PrintDocumentFormatter() override;

    TextPrinterDocument welcome(const CashboxStatus &st) const;
    TextPrinterDocument test(const CashboxStatus &st) const;
    TextPrinterDocument writeSerial(const CashboxStatus &st) const;
    TextPrinterDocument document(BaseDoc *doc, bool brief, bool fromFs) const;
    TextPrinterDocument preprintFiscalization(const IncomingReg & reg, const CashboxStatus &st) const;
    TextPrinterDocument fiscalization(BaseRegDoc *doc) const;
    TextPrinterDocument calcReport(CalcReportDoc *doc, bool brief) const;
    TextPrinterDocument openCycle(CycleOpenDoc *doc) const;
    TextPrinterDocument closeCycle(CycleCloseDoc *doc, bool brief) const;
    TextPrinterDocument closeFs(FsCloseDoc *doc) const;
    TextPrinterDocument receipt(BaseReceiptDoc *doc, bool fromFs = false) const;
    TextPrinterDocument xreport(XReport *rep, bool brief) const;


private:
    RegData regData_;
    //TODO: Для переререгистрации печать причин перерегистрации и счетчиков фн


    DocumentBlocks fiscalHeader(BaseWorkDoc *doc) const;
    DocumentBlocks fiscalFooter(BaseWorkDoc *doc) const;

    DocumentBlocks shortFsCounters(const ShortTaggedFsCounters &counters, bool brief,
                                  const QString &underLine = QStringLiteral("------------------------")) const;
    DocumentBlocks correctionCounters(const FsCorrectionCounters &counters, bool brief) const;
    DocumentBlock shortOperationsCounters(const FsShortOperationCounters &counters, const QString &typeTitle, bool brief) const;
    DocumentBlock correctionCounter(const FsCorrectionCounter &counters, const QString &typeTitle, bool brief) const;
    DocumentBlocks opperationsCounters(const FsOperationCounters &counters, const QString &typeTitle, bool brief) const;
    DocumentBlocks fsCounters(const TaggedFsCounters &counters, bool brief) const;
    DocumentBlocks fsCounters(const FsCounters &counters, bool brief) const;


    DocumentBlocks simpleReceiptOperation(const SimpleReceiptOperation &op) const;
    DocumentBlocks receiptOperation(const ReceiptOperation &op) const;

};

#endif // PRINTDOCUMENTFORMATTER_H
