#include "androidapihelper.h"

#include "cashboxbuildconfig.h"
#include "logmanip.h"
#include "appdirectories.h"

#include <QDebug>
#include <QtAndroidExtras/QAndroidJniObject>
#include <QtAndroidExtras/QtAndroid>
#include <QAndroidJniEnvironment>
#include <QAndroidIntent>
#include <QCryptographicHash>
#include <QtAndroid>

QMutex AndroidApiHelper::_cashDataMutex;

AndroidApiHelper::AndroidApiHelper(QObject *parent) noexcept
    : QObject(parent)
    , obj_(new QAndroidJniObject(
          "ru/armax/cashbox/AndroidApiHelper",
          "(Landroid/content/Context;)V",
          QtAndroid::androidContext().object()))
{
}

AndroidApiHelper::~AndroidApiHelper()
{
    delete obj_;
}

void AndroidApiHelper::showDateTimeSettings()
{
    if(obj_)
    {
        obj_->callMethod<void>("showDateTimeSettings");
    }
}


QString AndroidApiHelper::getDsn()
{
    static QString lastDsn;
    QString res;
    _cashDataMutex.lock();
    res = lastDsn;
    _cashDataMutex.unlock();

    if(obj_ && res.isEmpty())
    {
        QAndroidJniExceptionCleaner cleaner(QAndroidJniExceptionCleaner::OutputMode::Silent);
        Q_UNUSED(cleaner)
        QAndroidJniObject obj = obj_->callObjectMethod("getDsn", "()Ljava/lang/String;");
        jstring jstr = obj.object<jstring>();
        QAndroidJniEnvironment env;
        QString qstr(env->GetStringUTFChars(jstr, 0));
        res = qstr.trimmed();
        if(!res.isEmpty())
        {
            _cashDataMutex.lock();
            lastDsn = res;
            _cashDataMutex.unlock();
        }
    }
    return res;
}

QString AndroidApiHelper::getHardwareHash()
{
    QByteArray res = QCryptographicHash::hash(getDsn().toLatin1(), QCryptographicHash::Sha3_256).toHex();
    return QString::fromLatin1(res);
}

QString AndroidApiHelper::getSerial()
{
    return getDsn();
}
QString AndroidApiHelper::packageName() const
{
    static QString lastName;
    QString res;
    _cashDataMutex.lock();
    res = lastName;
    _cashDataMutex.unlock();
    if(obj_ && res.isEmpty())
    {
        QAndroidJniExceptionCleaner cleaner(QAndroidJniExceptionCleaner::OutputMode::Silent);
        Q_UNUSED(cleaner)
        QAndroidJniObject obj = obj_->callObjectMethod("packageName", "()Ljava/lang/String;");
        jstring jstr = obj.object<jstring>();
        QAndroidJniEnvironment env;
        QString qstr(env->GetStringUTFChars(jstr, 0));
        res = qstr.trimmed();
        _cashDataMutex.lock();
        lastName = res;
        _cashDataMutex.unlock();
    }
    return res;
}

QString AndroidApiHelper::packagePath() const
{
    static QString lastName;
    QString res;
    _cashDataMutex.lock();
    res = lastName;
    _cashDataMutex.unlock();
    if(obj_ && res.isEmpty())
    {
        QAndroidJniExceptionCleaner cleaner(QAndroidJniExceptionCleaner::OutputMode::Silent);
        Q_UNUSED(cleaner)
        QAndroidJniObject obj = obj_->callObjectMethod("packagePath", "()Ljava/lang/String;");
        jstring jstr = obj.object<jstring>();
        QAndroidJniEnvironment env;
        QString qstr(env->GetStringUTFChars(jstr, 0));
        res = qstr.trimmed();
        _cashDataMutex.lock();
        lastName = res;
        _cashDataMutex.unlock();
    }
    return res;
}

QString AndroidApiHelper::versionName() const
{
    static QString lastVersion;
    QString res;
    _cashDataMutex.lock();
    res = lastVersion;
    _cashDataMutex.unlock();
    if(obj_ && res.isEmpty())
    {
        QAndroidJniExceptionCleaner cleaner(QAndroidJniExceptionCleaner::OutputMode::Silent);
        Q_UNUSED(cleaner)
        QAndroidJniObject obj = obj_->callObjectMethod("versionName", "()Ljava/lang/String;");
        jstring jstr = obj.object<jstring>();
        QAndroidJniEnvironment env;
        QString qstr(env->GetStringUTFChars(jstr, 0));
        res = qstr.trimmed();
        _cashDataMutex.lock();
        lastVersion = res;
        _cashDataMutex.unlock();
    }
    return res;
}

qint64 AndroidApiHelper::versionCode() const
{
    static jlong lastVersion = -1;
    jlong res = -1;
    _cashDataMutex.lock();
    res = lastVersion;
    _cashDataMutex.unlock();
    if(obj_ && res < 0)
    {
        QAndroidJniExceptionCleaner cleaner(QAndroidJniExceptionCleaner::OutputMode::Silent);
        Q_UNUSED(cleaner)
        res = obj_->callMethod<jlong>("versionCode", "()J");
        _cashDataMutex.lock();
        lastVersion = res;
        _cashDataMutex.unlock();

    }
    return static_cast<qint64>(res);
}

bool AndroidApiHelper::installPackage(const QString &path) const
{
    QAndroidJniExceptionCleaner cleaner(QAndroidJniExceptionCleaner::OutputMode::Silent);
    Q_UNUSED(cleaner)
    jlong res = -1;
    QAndroidJniObject param = QAndroidJniObject::fromString(path);
    res = obj_->callMethod<jboolean>("installPackage", "(Ljava/lang/String;)Z",
                                  param.object<jstring>());
    return res;
}

bool AndroidApiHelper::uninstallPackage(const QString &packageName) const
{
    QAndroidJniExceptionCleaner cleaner(QAndroidJniExceptionCleaner::OutputMode::Silent);
    Q_UNUSED(cleaner)
    jlong res = -1;
    QAndroidJniObject param = QAndroidJniObject::fromString(packageName);
    res = obj_->callMethod<jboolean>("uninstallPackage", "(Ljava/lang/String;)Z",
                                  param.object<jstring>());
    return res;
}

QString AndroidApiHelper::sdcardStorage() const
{
    static QString lastName;
    QString res;
    _cashDataMutex.lock();
    res = lastName;
    _cashDataMutex.unlock();
    if(obj_ && res.isEmpty())
    {
        QAndroidJniExceptionCleaner cleaner(QAndroidJniExceptionCleaner::OutputMode::Silent);
        Q_UNUSED(cleaner)
        QAndroidJniObject obj = obj_->callObjectMethod("sdcardStorage", "()Ljava/lang/String;");
        jstring jstr = obj.object<jstring>();
        QAndroidJniEnvironment env;
        QString qstr(env->GetStringUTFChars(jstr, 0));
        res = qstr.trimmed();
        _cashDataMutex.lock();
        lastName = res;
        _cashDataMutex.unlock();
    }
    return res;
}

QString AndroidApiHelper::filePackageName(const QString &path) const
{
    QString res;
    QAndroidJniExceptionCleaner cleaner(QAndroidJniExceptionCleaner::OutputMode::Silent);
    Q_UNUSED(cleaner)
    QAndroidJniObject param = QAndroidJniObject::fromString(path);
    QAndroidJniObject obj = obj_->callObjectMethod("filePackageName",
                                                   "(Ljava/lang/String;)Ljava/lang/String;",
                                                   param.object<jstring>());
    jstring jstr = obj.object<jstring>();
    QAndroidJniEnvironment env;
    QString qstr(env->GetStringUTFChars(jstr, 0));
    res = qstr.trimmed();
    return res;
}

QString AndroidApiHelper::fileVersionName(const QString &path) const
{
    QString res;
    QAndroidJniExceptionCleaner cleaner(QAndroidJniExceptionCleaner::OutputMode::Silent);
    Q_UNUSED(cleaner)
    QAndroidJniObject param = QAndroidJniObject::fromString(path);
    QAndroidJniObject obj = obj_->callObjectMethod("fileVersionName",
                                                   "(Ljava/lang/String;)Ljava/lang/String;",
                                                   param.object<jstring>());
    jstring jstr = obj.object<jstring>();
    QAndroidJniEnvironment env;
    QString qstr(env->GetStringUTFChars(jstr, 0));
    res = qstr.trimmed();
    return res;
}

qint64 AndroidApiHelper::fileVersionCode(const QString &path) const
{
    QAndroidJniExceptionCleaner cleaner(QAndroidJniExceptionCleaner::OutputMode::Silent);
    Q_UNUSED(cleaner)
    jlong res = -1;
    QAndroidJniObject param = QAndroidJniObject::fromString(path);
    res = obj_->callMethod<jlong>("fileVersionCode", "(Ljava/lang/String;)J",
                                  param.object<jstring>());
    return res;
}

QString AndroidApiHelper::currentNetworkInterfaceName() const
{
    static QString lastVersion;
    QString res;
    _cashDataMutex.lock();
    res = lastVersion;
    _cashDataMutex.unlock();
    if(obj_ && res.isEmpty())
    {
        QAndroidJniExceptionCleaner cleaner(QAndroidJniExceptionCleaner::OutputMode::Silent);
        Q_UNUSED(cleaner)
        QAndroidJniObject obj = obj_->callObjectMethod("currentNetworkInterfaceName", "()Ljava/lang/String;");
        jstring jstr = obj.object<jstring>();
        QAndroidJniEnvironment env;
        QString qstr(env->GetStringUTFChars(jstr, 0));
        res = qstr.trimmed();
        _cashDataMutex.lock();
        lastVersion = res;
        _cashDataMutex.unlock();
    }
    return res;
}

FixNumber AndroidApiHelper::getAccumLevel() const
{
    QAndroidJniExceptionCleaner cleaner(QAndroidJniExceptionCleaner::OutputMode::Silent);
    Q_UNUSED(cleaner)
    jfloat res = obj_->callMethod<jfloat>("getAccumLevel", "()F");
    return FixNumber(10, static_cast<double>(res));
}


void AndroidApiHelper::showPackageActivity(const QString &pckg)
{
    lmWarning() << pckg;
    if(!pckg.isEmpty() && pckg.toLower().trimmed() != "ignore")
    {
        QAndroidJniExceptionCleaner cleaner(QAndroidJniExceptionCleaner::OutputMode::Silent);
        Q_UNUSED(cleaner)
        QAndroidJniObject param = QAndroidJniObject::fromString(pckg);
        obj_->callMethod<void>("showPackageActivity", "(Ljava/lang/String;)V", param.object<jstring>());
    }
}

void AndroidApiHelper::initApp()
{
    QAndroidJniExceptionCleaner cleaner(QAndroidJniExceptionCleaner::OutputMode::Silent);
    Q_UNUSED(cleaner)
    QAndroidJniObject param = QAndroidJniObject::fromString(QDir::fromNativeSeparators(APATH.workDir()));
    obj_->callMethod<void>("initApp", "(Ljava/lang/String;)V", param.object<jstring>());
}
void AndroidApiHelper::releaseApp()
{
    QAndroidJniExceptionCleaner cleaner(QAndroidJniExceptionCleaner::OutputMode::Silent);
    Q_UNUSED(cleaner)
    obj_->callMethod<void>("releaseApp");
}

QString AndroidApiHelper::getImei(qint32 slot)
{
    QAndroidJniExceptionCleaner cleaner(QAndroidJniExceptionCleaner::OutputMode::Silent);
    Q_UNUSED(cleaner)
    QAndroidJniEnvironment env;

    QAndroidJniObject obj = obj_->callObjectMethod("getIMEIForSlot", "(I)Ljava/lang/String;", slot);
    jstring jstr = obj.object<jstring>();
    QString qstr(env->GetStringUTFChars(jstr, 0));
    QString res = qstr.trimmed();
    lmWarning() << slot << res;
    return res;


}

QString AndroidApiHelper::parseBarcode(const QByteArray &imageBytes)
{
    QAndroidJniExceptionCleaner cleaner(QAndroidJniExceptionCleaner::OutputMode::Silent);
    Q_UNUSED(cleaner)
    auto qByteArray2jbyteArray =[] (QByteArray buf) -> jbyteArray{
        QAndroidJniEnvironment env;
        jbyteArray array = env->NewByteArray(buf.length());
        env->SetByteArrayRegion (array, 0, buf.length(), reinterpret_cast<jbyte*>(buf.data()));
        return array;
    };
    jbyteArray jarray = qByteArray2jbyteArray(imageBytes);

    QAndroidJniObject param = QAndroidJniObject::fromLocalRef(jarray);

    QAndroidJniObject res = obj_->callObjectMethod("scanBarcode", "([B)Ljava/lang/String;",
                                                       param.object<jbyteArray>());
    QAndroidJniEnvironment env;
    QString qstr(env->GetStringUTFChars(res.object<jstring>(), 0));
    return qstr;
}

void AndroidApiHelper::showDateTimeConfig()
{
    QAndroidJniExceptionCleaner cleaner(QAndroidJniExceptionCleaner::OutputMode::Silent);
    Q_UNUSED(cleaner)
    obj_->callMethod<void>("showDateTimeConfig");
}

void AndroidApiHelper::showWifiConfig()
{
    QAndroidJniExceptionCleaner cleaner(QAndroidJniExceptionCleaner::OutputMode::Silent);
    Q_UNUSED(cleaner)
     obj_->callMethod<void>("showWifiConfig");
}

void AndroidApiHelper::showGsmConfig()
{
    QAndroidJniExceptionCleaner cleaner(QAndroidJniExceptionCleaner::OutputMode::Silent);
    Q_UNUSED(cleaner)
    obj_->callMethod<void>("showGsmConfig");
}

void AndroidApiHelper::showMobileDataConfig()
{
    QAndroidJniExceptionCleaner cleaner(QAndroidJniExceptionCleaner::OutputMode::Silent);
    Q_UNUSED(cleaner)
    obj_->callMethod<void>("showMobileDataConfig");
}

void AndroidApiHelper::initForeGround()
{
    if(QtAndroid::androidService().isValid())
    {
        QAndroidJniExceptionCleaner cleaner(QAndroidJniExceptionCleaner::OutputMode::Silent);
        Q_UNUSED(cleaner)
        obj_->callMethod<void>("initForeGround", "(Landroid/app/Service;)V", QtAndroid::androidService().object());
    }
}

bool AndroidApiHelper::sendAboutToLouncher(const QString &serial, const QString &modelName, int modelCode) const
{
    QAndroidJniExceptionCleaner cleaner(QAndroidJniExceptionCleaner::OutputMode::Silent);
    Q_UNUSED(cleaner)
    QAndroidJniObject ser = QAndroidJniObject::fromString(serial);
    QAndroidJniObject m = QAndroidJniObject::fromString(modelName);
    return obj_->callMethod<jboolean>("sendAboutToLouncher",
                                      "(Ljava/lang/String;Ljava/lang/String;I)Z",
                                      ser.object<jstring>(), m.object<jstring>(), modelCode);
}

void AndroidApiHelper::sendBroadCast(const QString &action, const QString &key, const QVariant &data)
{
    QAndroidJniExceptionCleaner cleaner(QAndroidJniExceptionCleaner::OutputMode::Silent);
    Q_UNUSED(cleaner)
    QAndroidIntent intent(action);
    intent.putExtra(key, data);
    if(intent.handle().object())
    {
        QtAndroid::androidContext().callMethod<void>("sendBroadcast", "(Landroid/content/Intent;)V", intent.handle().object());
    }
}

bool AndroidApiHelper::isPackageIstalled(const QString &pkg) const
{
    QAndroidJniExceptionCleaner cleaner(QAndroidJniExceptionCleaner::OutputMode::Silent);
    Q_UNUSED(cleaner)
    QAndroidJniObject p = QAndroidJniObject::fromString(pkg);
    return obj_->callMethod<jboolean>("isPackageInstalled",
                                      "(Ljava/lang/String;)Z",
                                      p.object<jstring>());
}

bool AndroidApiHelper::cashbox105Installed() const
{
    static const QStringList PACKAGES = {
        "ru.limon.bank",
        "ru.armax.cachboxtouchneva570",
        "ru.armax.cachboxtoucha90paymob",
        "ru.armax.cachboxrmk390",
        "ru.armax.cachboxrmkneva",
        "ru.armax.cachboxtouchmhr02f",
        "ru.armax.cachboxtouchmhr03f",
        "ru.armax.cachboxtouchneva",
        "ru.armax.cachboxtouchpardus570",
        "ru.armax.cashboxtouch390",
        "ru.armax.cashboxtouch570",
        "ru.armax.cashboxtouchaposa9",
        "ru.armax.cashboxtouchb06",
        "ru.armax.cashboxtouchbyb06",
        "ru.armax.cashboxtouchcs10",
        "ru.armax.cashboxtouchlmob",
        "ru.armax.cashboxtouchpaymobM1",
        "ru.armax.cashboxtouchpikassa",
        "ru.armax.cashboxtouchplatasfcs10",
        "ru.armax.cashboxtouchtalphar2",
        "ru.armax.cashboxtouchtalpha",
        "ru.armax.cashboxtouchtzpro",
        "ru.armax.cashboxtouchxac"
    };
    return std::any_of(PACKAGES.begin(), PACKAGES.end(),
                       [this](const QString &p){return isPackageIstalled(p);});
}
