#ifndef BASESQLITEDBCONNECTOR_H
#define BASESQLITEDBCONNECTOR_H


#include <QObject>
#include <QSqlDatabase>
#include <QPointer>
#include <QThreadStorage>


class BaseSqliteDbConnector: public QObject
{
    Q_OBJECT
    Q_DISABLE_COPY(BaseSqliteDbConnector)
public:
    enum ConnectionMode
    {
        CM_READ_ONLY,
        CM_READ_WRITE
    };

    enum EDbVersionState: quint32
    {
        DATABASE_IS_VALID		= 0,
        DATABASE_CHECKING_ERROR	= 1,
        DATABASE_IS_OLD			= 2,
        DATABASE_IS_NEW			= 3,
        DATABASE_IS_NOT_EXISTS	= 4
    };


    virtual ~BaseSqliteDbConnector();

    virtual QSqlDatabase &db() {
        return db_;
    }
    virtual QSqlDatabase db() const {
        return db_;
    }

    virtual void clearAllConnections();
    virtual bool vacuum() const;

    virtual bool resetDb();

    virtual EDbVersionState checkVersion(qint32 &dbVersion, QString &msg, bool updateDb);
    virtual EDbVersionState checkVersion(qint32 &dbVersion, QString &msg); //Только проверка

protected:

    typedef QThreadStorage<qint32> ConnectionsCounter;
    enum EMetadataState
    {
        MDS_VALID,
        MDS_NO_TABLE,
        MDS_NO_RECORD,
        MDS_UNCNOWN_ERROR
    };

    QSqlDatabase db_;
    bool readOnly_;
    QString prefix_;
    ConnectionsCounter *opened_;


    BaseSqliteDbConnector(const QString &connectionPrefix, ConnectionsCounter *opened,
                                   QObject *parent = nullptr)noexcept;//READWRITE
    BaseSqliteDbConnector(const QString &connectionPrefix, ConnectionMode mode,
                          ConnectionsCounter *opened, QObject *parent = nullptr)noexcept;

    virtual void init();
    virtual void release();
    virtual void attacheDatabases();

    virtual QString databaseFileName() const = 0;
    virtual QString connectionName() const = 0;


    virtual QString driver() const;
    virtual QString host() const;
    virtual qint16 port() const;
    virtual QString user() const;
    virtual QString password() const;

    virtual bool resetDbTables(const QStringList &tables);

    virtual bool getMetaData(qint32 &metadataVersion, EMetadataState &state) = 0;
    virtual qint32 actualMetadataVersion() const = 0;
    virtual bool updateDatabase(qint32 from) = 0;
    virtual bool processMetadataState(EMetadataState state, QString &msg);
};



#endif
