#include "localizationmgr.h"
#include "formatutils.h"

#include <QMutexLocker>

#include <QFile>
#include <QJsonDocument>
#include <QJsonParseError>


QSharedPointer<LocalizationManager> LocalizationManager::_manager;
QMutex LocalizationManager::_mutex;

LocalizationManager::~LocalizationManager()
{

}



QStringList LocalizationManager::localisations()
{
    _mutex.lock();
    QSharedPointer<LocalizationManager> man = manager();
    QStringList res;
    if(man) res = man->localisations_.values();
    _mutex.unlock();
    return res;
}

void LocalizationManager::addLocalisations(const QStringList &localisations)
{
    _mutex.lock();
    QSharedPointer<LocalizationManager> man = manager();
    if(man)
    {
        man->localisations_ = man->localisations_.unite(QSet<QString>(localisations.begin(), localisations.end()));
    }
    _mutex.unlock();
}

void LocalizationManager::addLocalisation(const QString &localisation)
{
    _mutex.lock();
    QSharedPointer<LocalizationManager> man = manager();
    if(man)
    {
        man->localisations_.insert(localisation);
    }
    _mutex.unlock();

}

LocalizationManager::LocalizationManager() noexcept:
    localisations_()
{

}

QSharedPointer<LocalizationManager> LocalizationManager::manager()
{
    if(_manager.isNull())
    {
        _manager = QSharedPointer<LocalizationManager>(new LocalizationManager());
    }
    return _manager;
}
