#ifndef APPVERSIONMANAGER_H
#define APPVERSIONMANAGER_H


#include <QSharedPointer>
#include <QMutex>
#include <QSet>
#include <QStringList>


class LocalizationManager
{
public:
    ~LocalizationManager();

    static QStringList localisations();
    static void addLocalisations(const QStringList &localisations);
    static void addLocalisation(const QString &localisation);
    static void saveCurrentVersions(const QString &path, const QString &appName);
    static LocalizationManager loadVersions(const QString &path, const QString &appName);

private:
    static QSharedPointer<LocalizationManager> _manager;
    static QMutex _mutex;
    QSet<QString> localisations_;
    LocalizationManager() noexcept;
    static QSharedPointer<LocalizationManager> manager();
};

#ifndef ADD_LOCALISATION_FILE
#define ADD_LOCALISATION_FILE(file) \
    LocalizationManager::addLocalisation(file)
#define ADD_LOCALISATION_FILES(files) \
    LocalizationManager::addLocalisation(files)
#endif

#endif // APPVERSIONMANAGER_H
