#ifndef LOGGERMANAGER_H
#define LOGGERMANAGER_H

#include "logconsts.h"

#include <QPointer>
#include <QTextStream>
#include <QStringList>
#include <QDebug>
#include <QMutex>
#include <QFile>
#include <functional>



class QTimer;

typedef std::function<void(QtMsgType , const QMessageLogContext &, const QString&)> LoggerMessageOutput;

class QT5EXT_LOGGER_EXPORT LoggerManager
{
    Q_ENUMS(EQtDebugOut)
    static const qint64 MAX_FILE_SIZE = 64 * 1024 * 1024;
public:
    virtual ~LoggerManager();
    enum EQtDebugOut
    {
        QDO_NO			= 0,
        QDO_FILE		= 1,
        QDO_STDERR		= 2,
        QDO_BOTH		= QDO_FILE | QDO_STDERR
    };
    Q_DECLARE_FLAGS(EQtDebugOuts, EQtDebugOut)

    static bool initEmptyLog();
    static bool initLog(const QString &path, quint32 keepingDays, QtMsgType minimumLevel = QtDebugMsg);
    static bool initLog(const QString &path, quint32 keepingDays, bool rotateLogs, QtMsgType minimumLevel = QtDebugMsg);
    static void releaseLog();

    static void logMessageOutput(QtMsgType type, const QMessageLogContext &context, const QString &msg);
    static void ignoreLogs(QtMsgType type, const QMessageLogContext &context, const QString &msg);

    static EQtDebugOuts debugOuts();
    static void setDebugOuts(const EQtDebugOuts &debugOuts);
    static QString logsPath();


    static QMap<QString, LoggerMessageOutput> outputs();
    static void setOutputs(const QMap<QString, LoggerMessageOutput> &outputs);
    static void setOutputs(const QString &category, LoggerMessageOutput output);
    static void removeOutput(const QString &category);

    static bool rotateLogs();

    static bool keepLogHandlers();
    static void setKeepLogHandlers(bool keepLogHandlers);

    static QString defaultSuffix();
    static void setDefaultSuffix(const QString &defaultSuffix);

    static QStringList additionalEraseFilters();
    static void setAdditionalEraseFilters(const QStringList &additionalEraseFilters);

    static bool cleanAnyOldLogs();
    static void setCleanAnyOldLogs(bool cleanAnyOldLogs);

    static quint32 keepingDays();

private:
    QString suffix_;
    QFile *logFile_;
    QTextStream *stream_;
    static QMap<QString, LoggerMessageOutput> _outputs;

    Q_DISABLE_COPY(LoggerManager)

    static QString _defaultSuffix;
    static EQtDebugOuts _debugOuts;
    static QString _logPath;
    static quint32 _keepingDays;
    static QtMsgType _minimumLogLevel;
    static QMutex _logMutex;
    static QTimer *_eraceTimer;

    static bool _rotateLogs;
    static bool _keepLogHandlers;
    static QStringList _additionalEraseFilters;
    static QMap<QString, QSharedPointer<LoggerManager> > _loggers;
    static QAtomicInt _loggersCounter;
    static bool _cleanAnyOldLogs;

    LoggerManager(const QString &suffix);


    void open();
    void close();

    QTextStream &stream();

};
Q_DECLARE_OPERATORS_FOR_FLAGS(LoggerManager::EQtDebugOuts)


#endif // LOGGERMANAGER_H

