#ifndef LOGMANIP_H
#define LOGMANIP_H

#include "logconsts.h"

#include <QTextStream>
#include <QStringList>
#include <QDebug>

#if (QT_VERSION >= QT_VERSION_CHECK(5, 5, 0))
#	define lmDebug QMessageLogger(QT_MESSAGELOG_FILE, QT_MESSAGELOG_LINE, QT_MESSAGELOG_FUNC).debug().noquote
#	define lmInfo QMessageLogger(QT_MESSAGELOG_FILE, QT_MESSAGELOG_LINE, QT_MESSAGELOG_FUNC).info().noquote
#	define lmWarning QMessageLogger(QT_MESSAGELOG_FILE, QT_MESSAGELOG_LINE, QT_MESSAGELOG_FUNC).warning().noquote
#	define lmCritical QMessageLogger(QT_MESSAGELOG_FILE, QT_MESSAGELOG_LINE, QT_MESSAGELOG_FUNC).critical().noquote
#	define lmFatal QMessageLogger(QT_MESSAGELOG_FILE, QT_MESSAGELOG_LINE, QT_MESSAGELOG_FUNC).fatal().noquote
#	define lmCDebug(category, ...) \
        for (bool qt_category_enabled = category().isDebugEnabled(); qt_category_enabled; qt_category_enabled = false) \
            QMessageLogger(QT_MESSAGELOG_FILE, QT_MESSAGELOG_LINE, QT_MESSAGELOG_FUNC, category().categoryName()).debug(__VA_ARGS__).noquote()
#	define lmCInfo(category, ...) \
        for (bool qt_category_enabled = category().isInfoEnabled(); qt_category_enabled; qt_category_enabled = false) \
            QMessageLogger(QT_MESSAGELOG_FILE, QT_MESSAGELOG_LINE, QT_MESSAGELOG_FUNC, category().categoryName()).info(__VA_ARGS__).noquote()
#	define lmCWarning(category, ...) \
        for (bool qt_category_enabled = category().isWarningEnabled(); qt_category_enabled; qt_category_enabled = false) \
            QMessageLogger(QT_MESSAGELOG_FILE, QT_MESSAGELOG_LINE, QT_MESSAGELOG_FUNC, category().categoryName()).warning(__VA_ARGS__).noquote()
#	define lmCCritical(category, ...) \
        for (bool qt_category_enabled = category().isCriticalEnabled(); qt_category_enabled; qt_category_enabled = false) \
            QMessageLogger(QT_MESSAGELOG_FILE, QT_MESSAGELOG_LINE, QT_MESSAGELOG_FUNC, category().categoryName()).critical(__VA_ARGS__).noquote()
#else
#	define lmDebug QMessageLogger(__FILE__, __LINE__, Q_FUNC_INFO).debug
#	define lmInfoQMessageLogger(__FILE__, __LINE__, Q_FUNC_INFO).debug
#	define lmWarning QMessageLogger(__FILE__, __LINE__, Q_FUNC_INFO).warning
#	define lmCritical QMessageLogger(__FILE__, __LINE__, Q_FUNC_INFO).critical
#	define lmFatal QMessageLogger(__FILE__, __LINE__, Q_FUNC_INFO).fatal
#	define lmCDebug(category, ...) \
     for (bool qt_category_enabled = category().isDebugEnabled(); qt_category_enabled; qt_category_enabled = false) \
         QMessageLogger(__FILE__, __LINE__, Q_FUNC_INFO, category().categoryName()).debug(__VA_ARGS__)
#	define lmCInfo(category, ...) \
     for (bool qt_category_enabled = category().isDebugEnabled(); qt_category_enabled; qt_category_enabled = false) \
         QMessageLogger(__FILE__, __LINE__, Q_FUNC_INFO, category().categoryName()).debug(__VA_ARGS__)
#	define lmCWarning(category, ...) \
     for (bool qt_category_enabled = category().isWarningEnabled(); qt_category_enabled; qt_category_enabled = false) \
         QMessageLogger(__FILE__, __LINE__, Q_FUNC_INFO, category().categoryName()).warning(__VA_ARGS__)
#	define lmCCritical(category, ...) \
     for (bool qt_category_enabled = category().isCriticalEnabled(); qt_category_enabled; qt_category_enabled = false) \
          QMessageLogger(__FILE__, __LINE__, Q_FUNC_INFO, category().categoryName()).critical(__VA_ARGS__)

#endif

extern QT5EXT_LOGGER_EXPORT QTextStream &logtab(QTextStream &stream);
//--------------------------------------------------------------------------------------------------

class QT5EXT_LOGGER_EXPORT logbinary
{
    Q_DISABLE_COPY(logbinary)
public:
    explicit logbinary(const QByteArray &data, const char *enc = SRC_ENCODING);
    logbinary(logbinary &&other)noexcept;
    ~logbinary();

    friend QT5EXT_LOGGER_EXPORT QTextStream &operator << (QTextStream &stream, const logbinary& manip);
    friend QT5EXT_LOGGER_EXPORT QDebug operator << (QDebug debug, const logbinary &manip);
    logbinary &operator=(logbinary &&other);
private:
    QByteArray data_;
    QByteArray encoding_;
};

//--------------------------------------------------------------------------------------------------

class QT5EXT_LOGGER_EXPORT logencoded
{
    Q_DISABLE_COPY(logencoded)
public:
    explicit logencoded(const QByteArray &data, const char *enc = SRC_ENCODING);
    explicit logencoded(const char *data, const char *enc = SRC_ENCODING);
    logencoded(logencoded &&other)noexcept;
    ~logencoded();

    friend QT5EXT_LOGGER_EXPORT QTextStream &operator << (QTextStream &stream, const logencoded& manip);
    friend QT5EXT_LOGGER_EXPORT QDebug operator << (QDebug debug, const logencoded &manip);
    logencoded &operator=(logencoded &&other);
private:
    QByteArray data_;
    QByteArray encoding_;
};

//--------------------------------------------------------------------------------------------------

class QT5EXT_LOGGER_EXPORT loglist
{
    Q_DISABLE_COPY(loglist)
public:
    explicit loglist(const QStringList &data);
    loglist(loglist &&other)noexcept;
    ~loglist();

    friend QT5EXT_LOGGER_EXPORT QTextStream &operator << (QTextStream &stream, const loglist& manip);
    friend QT5EXT_LOGGER_EXPORT QDebug operator << (QDebug debug, const loglist &manip);
    loglist &operator =(loglist &&other);
private:
    QStringList data_;
};


//--------------------------------------------------------------------------------------------------

class QT5EXT_LOGGER_EXPORT logvariant
{
    Q_DISABLE_COPY(logvariant)
public:
    explicit logvariant(const QVariant &data);
    logvariant(logvariant &&other)noexcept;
    ~logvariant();

    friend QT5EXT_LOGGER_EXPORT QTextStream &operator << (QTextStream &stream, const logvariant& manip);
    friend QT5EXT_LOGGER_EXPORT QDebug operator << (QDebug debug, const logvariant &manip);
    logvariant &operator =(logvariant &&other);
private:
    QVariant data_;
};



#endif // LOGMANIP_H

