#include "cashboxservicebinder.h"

//#include "merchantconfigmanager.h"
//#include "qrworkerthread.h"
//#include "selectmodeactivitywrapper.h"
//#include "posworkerthread.h"

#include "logmanip.h"
#include "coreapiresult.h"


#include <QAndroidJniObject>
#include <QAndroidParcel>
#include <QEventLoop>
#include <QTimer>

CashboxServiceBinder::CashboxServiceBinder()
    : QObject(nullptr)
    , QAndroidBinder()
    , queue_(new ProcessingQueue)
{
    setObjectName("BINDER." + QString::number(reinterpret_cast<quint64>(this), 16));
//    lmWarning() << objectName();
}

CashboxServiceBinder::CashboxServiceBinder(const QAndroidJniObject &binder)
    : QObject(nullptr)
    , QAndroidBinder(binder)
    , queue_(new ProcessingQueue)
{
    setObjectName("BINDER." + QString::number(reinterpret_cast<quint64>(this), 16));
//    lmWarning() << objectName();
}

CashboxServiceBinder::~CashboxServiceBinder()
{
    delete queue_;
//    lmWarning() << objectName();
}

bool CashboxServiceBinder::onTransact(int code, const QAndroidParcel &data, const QAndroidParcel &reply, QAndroidBinder::CallType flags)
{
    Q_UNUSED(flags)
    QVariantMap map = data.readVariant().toMap();
//    lmWarning() << objectName() << code << static_cast<qint32>(flags) << logvariant(map);
    CoreTransaction transaction(static_cast<CoreApiConst::RemoteCommand>(code), map);
//    lmWarning() << logvariant(transaction.toMap());
    if(transaction.uid().trimmed().isEmpty()) transaction.genUid();
    QEventLoop loop;
    auto resultWaiter = [&loop] () ->void {
        QTimer::singleShot(5, &loop, SLOT(quit()));
    };
    bool savedToQueue =  queue_->enqueue(transaction, resultWaiter);
//    lmWarning() << savedToQueue;
    CoreApiResult res;
    if(savedToQueue)
    {
        loop.exec();
        res = queue_->takeResult(transaction.uid());
    }
    else
    {
        res = CoreApiResult(CoreApiConst::ErrorCode::InvalidTransactionType, tr("Дублирование ид транзакции"));
    }
    reply.writeVariant(res.toMap());
    //TODO: Нужно отследить время жизни. позаботиться о том, чтобы не вызывались сигналы, когда возможно удаление
    blockSignals(true);
    lmWarning() << transaction.uid()
                << QString::number(static_cast<qint32>(res.code()), 16)
                << res.descr();

    return true;
}

QVariantMap CashboxServiceBinder::formatResult(CoreApiConst::ErrorCode code, const QString &msg, const QVariantMap &data)
{
    CoreApiResult res(code, msg, data);
    return res.toMap();
}

