#ifndef CORECONFIGDB_H
#define CORECONFIGDB_H


#include "basesqlitedbconnector.h"

#include "cashier.h"
#include "ofdsettings.h"

class CoreConfigDB: public BaseSqliteDbConnector
{
    Q_OBJECT
    Q_DISABLE_COPY(CoreConfigDB)
    static const qint32 DB_METADATA_VERSION = 1;
public:
    explicit CoreConfigDB(QObject *parent = nullptr);//READWRITE
    explicit CoreConfigDB(ConnectionMode mode, QObject *parent = nullptr);
    ~CoreConfigDB() override;
    bool loadCashierNames(QVariantList &cashiers);
    bool loadCashiers(bool active, bool local, bool server, Cashiers &cashiers);
    bool loadCashiers(bool active, bool local, bool server, QVariantList &cashiers);
    bool loadCashiers(qint32 idFrom, bool active, bool local, bool server, QVariantList &cashiers);
    bool saveCashier(const Cashier &c, QVariantMap &out);
    bool removeCashier(int id);


    bool loadSupportedOfd(QVariantList &ofd);
    bool loadSupportedOfd(QList<OfdSettings> &ofd);
    bool loadCurrentOfd(const QString &inn, OfdSettings &ofd);
    bool saveCurrentOfd(const OfdSettings &ofd, bool forceReset);


    bool resetDb() override;

private:
    static ConnectionsCounter _rwopened;
    static ConnectionsCounter _roopened;


    QString databaseFileName() const override;
    QString connectionName() const override;

    bool getMetaData(qint32 &metadataVersion, EMetadataState &state) override;
    qint32 actualMetadataVersion() const override;
    bool updateDatabase(qint32 from) override;

    bool loadCashiers(QString filter, QMap<qint32, Cashier> &cashiers);
    bool loadCashiers(QString filter, Cashiers &cashiers);
    bool loadCashiers(QString filter, QVariantList &cashiers);
};

//-----------------------------------------------------------------------------------------

class CoreConfigDBDbInitializer : public QObject
{
    Q_OBJECT
    Q_DISABLE_COPY(CoreConfigDBDbInitializer)
    friend class CoreConfigDB;
public:
    explicit CoreConfigDBDbInitializer(QObject *parent = nullptr);
    ~CoreConfigDBDbInitializer() override;

    static bool initializeDatabase();
    bool createDatabase(QSqlDatabase *const db, const QString &fileName);
    bool updateDatabase(QSqlDatabase *const db, qint32 toMetaData);
private:
    QString getScript(const QString &fileName);
    bool applyScript(QSqlDatabase *const db, const QString &sql);

    static bool initCashiers(QSqlDatabase *db);
    static bool initOfd(QSqlDatabase *db);
};


#endif // CORECONFIGDB_H

