#include "coreconfigstore.h"
#include "appdirectories.h"

const QSet<QString> CoreConfigStore::SUPPORTED_SECTIONS = {
    ReceiptConfigData::SECTION
};

CoreConfigStore::CoreConfigStore(QObject *parent)
    : BaseIniConfigStore(APATH.configDir() + "/core.conf", parent)
    , receiptConfig_()
{

}

CoreConfigStore::~CoreConfigStore()
{

}

ReceiptConfigData CoreConfigStore::receiptConfig() const
{
    if(receiptConfig_.has_value()) return receiptConfig_.value();
    ReceiptConfigData res;
    load(&res);
    receiptConfig_ = res;
    return res;
}

void CoreConfigStore::setReceiptConfig(const ReceiptConfigData &rec)
{
    save(&rec);
    receiptConfig_ = rec;
}

CoreApiResult CoreConfigStore::processTransaction(const CoreTransaction &trans)
{
    QString section = trans.params().value("section").toString();
    switch (trans.operation()) {
    case CoreTransaction::Operation::LOAD_CORE_SETTINGS:
    {
        BaseCoreConfigData *res = createObject(section);
        if(!res) return CoreApiResult(CoreApiConst::ErrorCode::InvalidSettingsSection,
                                      tr("Указан некорректный раздел настроек: %1").arg(section));
        load(res);
        CoreApiResult ans = res->toSuccessResult();
        delete res;
        return ans;
    }break;
    case CoreTransaction::Operation::SAVE_CORE_SETTINGS:
    {
        BaseCoreConfigData *res = createObject(section);
        if(!res) return CoreApiResult(CoreApiConst::ErrorCode::InvalidSettingsSection,
                                      tr("Указан некорректный раздел настроек: %1").arg(section));
        res->parseMap(trans.params().value("value").toMap());
        save(res);
        load(res);
        CoreApiResult ans = res->toSuccessResult();
        delete res;
        resetObject(section);
        return ans;
    }break;
    default:
        return CoreApiResult(CoreApiConst::ErrorCode::InvalidTransactionType);
        break;
    }
}

BaseCoreConfigData *CoreConfigStore::createObject(const QString &name)
{
    BaseCoreConfigData *res = nullptr;
    if(name == ReceiptConfigData::SECTION)
    {
        res = new ReceiptConfigData;
    }
    return res;
}

void CoreConfigStore::resetObject(const QString &name)
{
    if(name == ReceiptConfigData::SECTION)
    {
        receiptConfig_.reset();
    }
}
