#include "basedocdbsaver.h"
#include "formatutils.h"
#include "cycleopendoc.h"
#include "cycleclosedoc.h"
#include "receiptdoc.h"
#include "fsclosedoc.h"
#include "regdoc.h"
#include "calcreportdoc.h"

BaseDocDbSaver::BaseDocDbSaver(BaseDoc *doc) noexcept
    : type_(fdf::DocType::Invalid)
    , fd_(-1)
    , fiscalCode_(0)
    , dt_()
    , fsNumber_()
    , externalId_()
    , clientId_()
    , header_()
    , footer_()
    , ffd_(fs::FFD::FFD1_2)
    , cycle_(0)
    , address_()
    , ownerInn_()
    , regNumber_()
    , ownerName_()
    , place_()
    , fields_()
{
    parseDoc(doc);
}

BaseDocDbSaver::BaseDocDbSaver(const BaseDocDbSaver &other) noexcept
    : type_(other.type_)
    , fd_(other.fd_)
    , fiscalCode_(other.fiscalCode_)
    , dt_(other.dt_)
    , fsNumber_(other.fsNumber_)
    , externalId_(other.externalId_)
    , clientId_(other.clientId_)
    , header_(other.header_)
    , footer_(other.footer_)
    , ffd_(other.ffd_)
    , cycle_(other.cycle_)
    , address_(other.address_)
    , ownerInn_(other.ownerInn_)
    , regNumber_(other.regNumber_)
    , ownerName_(other.ownerName_)
    , place_(other.place_)
    , fields_(other.fields_)
{

}

BaseDocDbSaver::BaseDocDbSaver(BaseDocDbSaver &&other) noexcept
    : type_(other.type_)
    , fd_(other.fd_)
    , fiscalCode_(other.fiscalCode_)
    , dt_()
    , fsNumber_()
    , externalId_()
    , clientId_()
    , header_()
    , footer_()
    , ffd_(other.ffd_)
    , cycle_(other.cycle_)
    , address_()
    , ownerInn_()
    , regNumber_()
    , ownerName_()
    , place_()
    , fields_()
{
    dt_.swap(other.dt_);
    fsNumber_.swap(other.fsNumber_);
    externalId_.swap(other.externalId_);
    clientId_.swap(other.clientId_);
    header_.swap(other.header_);
    footer_.swap(other.footer_);
    address_.swap(other.address_);
    ownerInn_.swap(other.ownerInn_);
    regNumber_.swap(other.regNumber_);
    ownerName_.swap(other.ownerName_);
    place_.swap(other.place_);
    fields_.swap(other.fields_);
}

BaseDocDbSaver::~BaseDocDbSaver()
{

}

fdf::DocType BaseDocDbSaver::type() const
{
    return type_;
}

void BaseDocDbSaver::setType(fdf::DocType newType)
{
    type_ = newType;
}

qint32 BaseDocDbSaver::fd() const
{
    return fd_;
}

void BaseDocDbSaver::setFd(qint32 newFd)
{
    fd_ = newFd;
}

quint32 BaseDocDbSaver::fiscalCode() const
{
    return fiscalCode_;
}

void BaseDocDbSaver::setFiscalCode(quint32 newFiscalCode)
{
    fiscalCode_ = newFiscalCode;
}

const QDateTime &BaseDocDbSaver::dt() const
{
    return dt_;
}

void BaseDocDbSaver::setDt(const QDateTime &newDt)
{
    dt_ = newDt;
}

const QString &BaseDocDbSaver::fsNumber() const
{
    return fsNumber_;
}

void BaseDocDbSaver::setFsNumber(const QString &newFsNumber)
{
    fsNumber_ = newFsNumber;
}

const QString &BaseDocDbSaver::externalId() const
{
    return externalId_;
}

void BaseDocDbSaver::setExternalId(const QString &newExternalId)
{
    externalId_ = newExternalId;
}

const QString &BaseDocDbSaver::clientId() const
{
    return clientId_;
}

void BaseDocDbSaver::setClientId(const QString &newClientId)
{
    clientId_ = newClientId;
}

const QString &BaseDocDbSaver::header() const
{
    return header_;
}

void BaseDocDbSaver::setHeader(const QString &newHeader)
{
    header_ = newHeader;
}

const QString &BaseDocDbSaver::footer() const
{
    return footer_;
}

void BaseDocDbSaver::setFooter(const QString &newFooter)
{
    footer_ = newFooter;
}

fs::FFD BaseDocDbSaver::ffd() const
{
    return ffd_;
}

void BaseDocDbSaver::setFfd(fs::FFD newFfd)
{
    ffd_ = newFfd;
}

quint16 BaseDocDbSaver::cycle() const
{
    return cycle_;
}

void BaseDocDbSaver::setCycle(quint16 newCycle)
{
    cycle_ = newCycle;
}

const QString &BaseDocDbSaver::address() const
{
    return address_;
}

void BaseDocDbSaver::setAddress(const QString &newAddress)
{
    address_ = newAddress;
}

const QString &BaseDocDbSaver::ownerInn() const
{
    return ownerInn_;
}

void BaseDocDbSaver::setOwnerInn(const QString &newOwnerInn)
{
    ownerInn_ = newOwnerInn;
}

const QString &BaseDocDbSaver::regNumber() const
{
    return regNumber_;
}

void BaseDocDbSaver::setRegNumber(const QString &newRegNumber)
{
    regNumber_ = newRegNumber;
}

const QString &BaseDocDbSaver::ownerName() const
{
    return ownerName_;
}

void BaseDocDbSaver::setOwnerName(const QString &newOwnerName)
{
    ownerName_ = newOwnerName;
}

const QString &BaseDocDbSaver::place() const
{
    return place_;
}

void BaseDocDbSaver::setPlace(const QString &newPlace)
{
    place_ = newPlace;
}

const QVariantList &BaseDocDbSaver::fields() const
{
    return fields_;
}

void BaseDocDbSaver::setFields(const QVariantList &newFields)
{
    fields_ = newFields;
}

QVariantMap BaseDocDbSaver::toMap() const
{
    return {
        {"type",       static_cast<qint32>(type_)},
        {"docNumber",  fd_},
        {"fiscalCode", QString::number(fiscalCode_)},
        {"dt",         DT2STR_(dt_)},
        {"fsNumber",   fsNumber_},
        {"externalId", externalId_},
        {"clientId",   clientId_},
        {"header",     header_},
        {"footer",     footer_},
        {"ffd",        static_cast<qint32>(ffd_)},
        {"cycle",      cycle_},
        {"address",    address_},
        {"ownerInn",   ownerInn_},
        {"regNumber",  regNumber_},
        {"ownerName",  ownerName_},
        {"place",      place_},
        {"fields",     fields_}
    };
}

BaseDoc *BaseDocDbSaver::create(QObject *parent) const
{
    BaseDoc *doc = nullptr;
    switch (static_cast<fdf::DocType>(type_)) {
    case fdf::DocType::Registration: doc = new RegDoc(parent);break;
    case fdf::DocType::ReRegistration: doc = new ReRegDoc(parent);break;
    case fdf::DocType::CycleOpen: doc = new CycleOpenDoc(parent);break;
    case fdf::DocType::CycleClose: doc = new CycleCloseDoc(parent);break;
    case fdf::DocType::CalcReport: doc = new CalcReportDoc(parent);break;
    case fdf::DocType::FsClose: doc = new FsCloseDoc(parent);break;
    case fdf::DocType::Receipt: doc = new ReceiptDoc(parent);break;
    case fdf::DocType::Fosa: doc = new FosaDoc(parent);break;
    case fdf::DocType::Correction: doc = new CorrectionDoc(parent);break;
    case fdf::DocType::FosaCorrection: doc = new CorrectionFosaDoc(parent);break;
    default: return nullptr;
    }
    doc->setFd(fd_);
    doc->setFiscalCode(fiscalCode_);
    doc->setDt(dt_);
    doc->setFsNumber(fsNumber_);
    doc->setExternalId(externalId_);
    doc->setClientId(clientId_);
    doc->setHeader(header_);
    doc->setFooter(footer_);
    doc->setSavedFields(fields_);
    BaseWorkDoc *wd = qobject_cast<BaseWorkDoc *>(doc);
    if(wd)
    {
        wd->setFfd(ffd_);
        wd->setCycle(cycle_);
        wd->setAddress(address_);
        wd->setOwnerInn(ownerInn_);
        wd->setRegNumber(regNumber_);
        wd->setOwnerName(ownerName_);
        wd->setPlace(place_);
    }
    BaseRegDoc *rdd = dynamic_cast<BaseRegDoc *>(doc);
    if(rdd)
    {
        CoreRegData rd = rdd->data();
        rd.setFfd(ffd_);
        rd.setAddress(address_);
        rd.setOwnerInn(ownerInn_);
        rd.setRegNumber(regNumber_);
        rd.setOwnerName(ownerName_);
        rd.setPlace(place_);
        rdd->setData(rd);
    }
    return doc;
}

BaseDocDbSaver &BaseDocDbSaver::operator =(const BaseDocDbSaver &other) noexcept
{
    type_ = other.type_;
    fd_ = other.fd_;
    fiscalCode_ = other.fiscalCode_;
    dt_ = other.dt_;
    fsNumber_ = other.fsNumber_;
    externalId_ = other.externalId_;
    clientId_ = other.clientId_;
    header_ = other.header_;
    footer_ = other.footer_;
    ffd_ = other.ffd_;
    cycle_ = other.cycle_;
    address_ = other.address_;
    ownerInn_ = other.ownerInn_;
    regNumber_ = other.regNumber_;
    ownerName_ = other.ownerName_;
    place_ = other.place_;
    fields_ = other.fields_;
    return *this;
}
BaseDocDbSaver &BaseDocDbSaver::operator =(BaseDocDbSaver &&other) noexcept
{
    type_ = other.type_;
    fd_ = other.fd_;
    fiscalCode_ = other.fiscalCode_;
    dt_.swap(other.dt_);
    fsNumber_.swap(other.fsNumber_);
    externalId_.swap(other.externalId_);
    clientId_.swap(other.clientId_);
    header_.swap(other.header_);
    footer_.swap(other.footer_);
    ffd_ = other.ffd_;
    cycle_ = other.cycle_;
    address_.swap(other.address_);
    ownerInn_.swap(other.ownerInn_);
    regNumber_.swap(other.regNumber_);
    ownerName_.swap(other.ownerName_);
    place_.swap(other.place_);
    fields_.swap(other.fields_);
    return *this;
}
bool BaseDocDbSaver::operator ==(const BaseDocDbSaver &other) const noexcept
{
    return type_ == other.type_ &&
           fd_ == other.fd_ &&
           fiscalCode_ == other.fiscalCode_ &&
           dt_ == other.dt_ &&
           fsNumber_ == other.fsNumber_ &&
           externalId_ == other.externalId_ &&
           clientId_ == other.clientId_ &&
           header_ == other.header_ &&
           footer_ == other.footer_ &&
           ffd_ == other.ffd_ &&
           cycle_ == other.cycle_ &&
           address_ == other.address_ &&
           ownerInn_ == other.ownerInn_ &&
           regNumber_ == other.regNumber_ &&
           ownerName_ == other.ownerName_ &&
           place_ == other.place_ &&
           fields_ == other.fields_;
}

bool BaseDocDbSaver::operator !=(const BaseDocDbSaver &other) const noexcept
{
    return !(*this == other);
}

void BaseDocDbSaver::parseDoc(BaseDoc *doc)
{
    if(doc)
    {
        type_ = doc->type();
        fd_ = doc->fd();
        fiscalCode_ = doc->fiscalCode();
        dt_ = doc->dt();
        fsNumber_ = doc->fsNumber();
        externalId_ = doc->externalId();
        clientId_ = doc->clientId();
        header_ = doc->header();
        footer_ = doc->footer();
        fields_ = doc->fieldsForSaving();
        if(type_ == fdf::DocType::Registration ||
            type_ == fdf::DocType::ReRegistration)
        {
            parseRegDoc(qobject_cast<BaseRegDoc*>(doc));
        }
        else
        {
            parseWorkDoc(qobject_cast<BaseWorkDoc*>(doc));
        }
    }
}

void BaseDocDbSaver::parseRegDoc(BaseRegDoc *doc)
{
    if(doc)
    {
        RegData rd = doc->regData();
        ffd_ = rd.ffd();
        cycle_ = 0;
        address_ = rd.address();
        ownerInn_ = rd.ownerInn();
        regNumber_ = rd.regNumber();
        ownerName_ = rd.ownerName();
        place_ = rd.place();
    }
}

void BaseDocDbSaver::parseWorkDoc(BaseWorkDoc *doc)
{
    if(doc)
    {
        ffd_ = doc->ffd();
        cycle_ = doc->cycle();
        address_ = doc->address();
        ownerInn_ = doc->ownerInn();
        regNumber_ = doc->regNumber();
        ownerName_ = doc->ownerName();
        place_ = doc->place();
    }
}
