#ifndef BASEDOCDBSAVER_H
#define BASEDOCDBSAVER_H

#include "basedoc.h"
#include "regdoc.h"

class BaseDocDbSaver
{
public:
    enum DocTextType
    {
        Unknown             = 0,
        Header              = 1,
        Footer              = 2
    };
    explicit BaseDocDbSaver(BaseDoc *doc) noexcept;
    BaseDocDbSaver(const BaseDocDbSaver &other) noexcept;
    BaseDocDbSaver(BaseDocDbSaver &&other) noexcept;
    ~BaseDocDbSaver();

    fdf::DocType type() const;
    void setType(fdf::DocType newType);

    qint32 fd() const;
    void setFd(qint32 newFd);

    quint32 fiscalCode() const;
    void setFiscalCode(quint32 newFiscalCode);

    const QDateTime &dt() const;
    void setDt(const QDateTime &newDt);

    const QString &fsNumber() const;
    void setFsNumber(const QString &newFsNumber);

    const QString &externalId() const;
    void setExternalId(const QString &newExternalId);

    const QString &clientId() const;
    void setClientId(const QString &newClientId);

    const QString &header() const;
    void setHeader(const QString &newHeader);

    const QString &footer() const;
    void setFooter(const QString &newFooter);

    fs::FFD ffd() const;
    void setFfd(fs::FFD newFfd);

    quint16 cycle() const;
    void setCycle(quint16 newCycle);

    const QString &address() const;
    void setAddress(const QString &newAddress);

    const QString &ownerInn() const;
    void setOwnerInn(const QString &newOwnerInn);

    const QString &regNumber() const;
    void setRegNumber(const QString &newRegNumber);

    const QString &ownerName() const;
    void setOwnerName(const QString &newOwnerName);

    const QString &place() const;
    void setPlace(const QString &newPlace);

    const QVariantList &fields() const;
    void setFields(const QVariantList &newFields);

    QVariantMap toMap() const;

    BaseDoc *create(QObject *parent = nullptr) const;

    BaseDocDbSaver &operator =(const BaseDocDbSaver &other) noexcept;
    BaseDocDbSaver &operator =(BaseDocDbSaver &&other) noexcept;
    bool operator ==(const BaseDocDbSaver &other) const noexcept;
    bool operator !=(const BaseDocDbSaver &other) const noexcept;

private:
    fdf::DocType type_;
    qint32 fd_;
    quint32 fiscalCode_;
    QDateTime dt_;
    QString fsNumber_;
    QString externalId_;
    QString clientId_;
    QString header_;
    QString footer_;
    fs::FFD ffd_;
    quint16 cycle_;
    QString address_;
    QString ownerInn_;
    QString regNumber_;
    QString ownerName_;
    QString place_;
    QVariantList fields_;

    void parseDoc(BaseDoc *doc);
    void parseRegDoc(BaseRegDoc *doc);
    void parseWorkDoc(BaseWorkDoc *doc);
};

#endif // BASEDOCDBSAVER_H
