#ifndef DOCSDB_H
#define DOCSDB_H

#include "basesqlitedbconnector.h"
#include "basedocdbsaver.h"

//TODO: При ошибках сохранения складываать документ в файлик с описанием ошибок date_time_fs_fd.json

class QSqlError;

class DocsDb : public BaseSqliteDbConnector
{
    Q_OBJECT
    Q_DISABLE_COPY(DocsDb)
    static const qint32 DB_METADATA_VERSION = 1;
    static const qint32 RECEND_INTERVAL     = -60;
public:
    explicit DocsDb(const QString &regNumber, QObject *parent = nullptr);
    explicit DocsDb(const QString &regNumber, ConnectionMode mode, QObject *parent = nullptr);
    ~DocsDb() override;

    bool resetDb() override;

    bool saveDocument(const BaseDocDbSaver &doc);
    BaseDoc * loadDocument(quint32 fd);
    BaseDoc * loadDocument(const QString &extId);

private:
    static ConnectionsCounter _rwopened;
    static ConnectionsCounter _roopened;
    QString cashboxRegNumber_;

    QString databaseFileName() const override;
    QString connectionName() const override;

    bool getMetaData(qint32 &metadataVersion, EMetadataState &state) override;
    qint32 actualMetadataVersion() const override;
    bool updateDatabase(qint32 from) override;

    bool loadBaseDocDbSaver(BaseDocDbSaver &bds, const QString &filter);//NOTE: Для выбора одного документа
    bool loadDocText(BaseDocDbSaver &bds, const QVariant &id);
    bool loadDocRegData(BaseDocDbSaver &doc, const QVariant &id);
    bool loadDocPlace(BaseDocDbSaver &doc, const QVariant &id);
    bool loadFields(BaseDocDbSaver &doc, const QVariant &id);

    bool saveDocText(qint32 docTextType, const QString &text, QVariant &id);
    bool saveDocRegData(const BaseDocDbSaver &doc, QVariant &id);
    bool saveDocPlace(const BaseDocDbSaver &doc, QVariant &id);
    void saveError(const BaseDocDbSaver &doc, const QSqlError &error,
                   const QVariantMap &bounds, const QString &sql, const QString &descr = QString());
    void saveError(const BaseDocDbSaver &doc, const QSqlError &error, const QString &descr = QString());
};

//--------------------------------------------------------------------------------------------------

class DocsDBDbInitializer : public QObject
{
    Q_OBJECT
    Q_DISABLE_COPY(DocsDBDbInitializer)
    friend class DocsDB;
public:
    explicit DocsDBDbInitializer(QObject *parent = nullptr);
    ~DocsDBDbInitializer() override;

    static bool initializeDatabase(const QString &regNumber);
    bool createDatabase(QSqlDatabase *const db, const QString &fileName);
    bool updateDatabase(QSqlDatabase *const db, qint32 toMetaData);
private:
    QString getScript(const QString &fileName);
    bool applyScript(QSqlDatabase *const db, const QString &sql);

};

#endif // DOCSDB_H
