#include "keysupdater.h"
#include "internalobjectsorage.h"
#include "logmanip.h"
#include "ofdmsgcontainer.h"
#include "fsfullstatus.h"
#include "watcherdb.h"

KeysUpdater::KeysUpdater(FsWrapper *fs, ProcessingDataWrapper *pdw, QObject *parent)
    : QObject(parent)
    , fs_(fs)
    , pdw_(pdw)
    , socket_(new SyncSocketWrapper("osp",this))
    , lastErroCode_(CoreApiConst::ErrorCode::Ok)
    , lastErrorMsg_()
{

}

KeysUpdater::~KeysUpdater()
{

}

CoreApiConst::ErrorCode KeysUpdater::lastErroCode() const
{
    return lastErroCode_;
}

const QString &KeysUpdater::lastErrorMsg() const
{
    return lastErrorMsg_;
}

CoreApiConst::ErrorCode KeysUpdater::exec(DocumentBlock &text)
{
    text = DocumentBlock();
    lastErroCode_ = CoreApiConst::ErrorCode::Ok;
    lastErrorMsg_.clear();

    fs::KeysStatus ks = fs::KeysStatus::KeysUpdated, buf = fs::KeysStatus::KeysUpdated;
    QString addr;
    CoreApiConst::ErrorCode err = fs_->getKeysAddress(true, ks, addr);
    lmWarning() << CoreApiConst::defaultErrorMsg(err);
//Это для отладки вместо МГМ без d7
//    err = CoreApiConst::ErrorCode::UnknownFsCmd;
//TEST
    if(err == CoreApiConst::ErrorCode::UnknownFsCmd)
    {

        WhatcherDb dbc(WhatcherDb::CM_READ_ONLY);
        if(!dbc.getKeysStatus(IOSG::getFs().fsNumber().trimmed(), ks))
        {
            return CoreApiConst::ErrorCode::KeysStatusLoadingError;
        }
        FsFullStatus st = pdw_->getFsFullStatus();
        lmWarning() << static_cast<qint32>(ks) << logvariant(st.toMap());

        if(st.cycle().cycle() > 0 && ks == fs::KeysStatus::KeysUpdated) return CoreApiConst::ErrorCode::Ok;
        OfdSettings ofdsts = pdw_->getOfd();
        addr = ofdsts.keysAddress().trimmed() + ":" + QString::number(ofdsts.keysPort());
        lmWarning() << static_cast <qint32>(ks) << addr;
    }
    else
    {
        lmWarning() << static_cast <qint32>(ks);
//        ks = fs::KeysStatus::Keys15; //Для тестов
        if(ks == fs::KeysStatus::KeysUpdated) return CoreApiConst::ErrorCode::Ok;
        err = fs_->getKeysAddress(false, buf, addr);
        lmWarning() << static_cast <qint32>(ks) << addr;
        if(err != CoreApiConst::ErrorCode::Ok) return err;
    }
    quint8 a20 = 0;
    quint16 srvErr = 0;
    QString srvMsg;
    err = update(addr, a20, srvErr, srvMsg);
    //TEST
//    err = CoreApiConst::ErrorCode::LabledProductsNotSupported;
    //TEST
    lmWarning() << CoreApiConst::defaultErrorMsg(err);
    LineFormat f;
    f.setPrintGray(5000);
    f.setFont(2);
    f.setFlags(LF_CENTER | LF_BOLD);
    QStringList txt;
    if(err == CoreApiConst::ErrorCode::Ok)
    {
        //NOTE: Павлов сказал добавить
        txt << "Обновление ключей проверки КМ прошло успешно!";
        text = DocumentBlock::createSimpleBlock(txt, f);
        WhatcherDb dbc(WhatcherDb::CM_READ_WRITE);
        dbc.updateKeysStatus(IOSG::getFs().fsNumber().trimmed());
        lmWarning() << loglist(txt);
        return err;
    }
    switch (ks) {
    case fs::KeysStatus::KeysUpdated:
        txt << "Нет необходимости в проведении процедуры обновления ключей проверки КМ";
        break;
    case fs::KeysStatus::Keys15:
        txt << "Обновление ключей проверки не выполнено.";
        break;
    case fs::KeysStatus::Keys60:
        txt << "Обновление ключей проверки не выполнено. Обратитесь в службу технической поддержки.";
        break;
    default:
        break;
    }
    if(err == CoreApiConst::ErrorCode::KeysServerError)
    {
        txt << tr("ОШИБКА СВЯЗИ С СЕРВЕРОМ ОКП");
    }
    else if(err == CoreApiConst::ErrorCode::OfdMsgError)
    {
        switch (a20) {
        case 1: txt << "Неверный фискальныйпризнак";break;
        case 2: txt << "Неверный формат ответа";break;
        case 3: txt << "Неверный номер запроса в ответе";break;
        case 4: txt << "Неверный номер ФН";break;
        case 5: txt << "Неверный CRC блока данных";break;
        case 7: txt << "Неверная длина ответа";break;
        default:
            break;
        }
    }
    else if(err == CoreApiConst::ErrorCode::KeysServerFailure)
    {
        txt << tr("Ошибка сервиса обновления ключей проверки КМ")
            << tr("Код ответа сервиса\t%1").arg(srvErr)
            << srvMsg;
    }
    else if(err == CoreApiConst::ErrorCode::KeysServerUnknownError)
    {
        txt << "Неизвестный ответ сервиса обновления ключей проверки";
    }
    else
    {
        txt << CoreApiConst::defaultErrorMsg(err);
    }
    text = DocumentBlock::createSimpleBlock(txt, f);
    lmWarning() << CoreApiConst::defaultErrorMsg(err) << loglist(txt);
    return err;
}

CoreApiConst::ErrorCode KeysUpdater::update(const QString &addr, quint8 &a20,
                                            quint16 &srvAns, QString &srvMsg)
{
    QString host = addr;
    if(host.contains("//")) host = host.mid(host.indexOf("//") + 2);
    lmWarning() << host.mid(host.lastIndexOf(":") + 1).trimmed();
    quint16 port = host.mid(host.lastIndexOf(":") + 1).trimmed().toUInt();
    host = host.mid(0, host.lastIndexOf(":"));

    QByteArray data;
    CoreApiConst::ErrorCode err = CoreApiConst::ErrorCode::Ok;
    bool needUpdate = true;
    ReadFunc func = [this](QTcpSocket *socket) -> QByteArray {
        return readMsg(socket);
    };
    lmWarning() << host << port << addr;
    while(needUpdate)
    {
        lmWarning() << "START UPDATE LOOP";
        err = getKeysData(data);
        lmWarning() << logbinary(data);
        if(err != CoreApiConst::ErrorCode::Ok) return err;
        if(!socket_->connectedToHost())
        {
            if(!socket_->connectToHost(host, port)) return CoreApiConst::ErrorCode::KeysServerError;
        }
        data = socket_->sendMsg(data, func);
        lmWarning() << logbinary(data);
        if(data.isEmpty()) return CoreApiConst::ErrorCode::KeysServerError;
        err = writeKeysData(data, a20, srvAns, srvMsg);
        needUpdate = (err == CoreApiConst::ErrorCode::RetryKeysUpdate);
        lmWarning() << CoreApiConst::defaultErrorMsg(err) << needUpdate;
    }
    lmWarning() << "BEFORE DISCONNECT";
    socket_->setExchangeTimeout(1000);
    socket_->disconnectFromHost();
    lmWarning() << "DISCONNECTED";
    return err;
}

QByteArray KeysUpdater::readMsg(QTcpSocket *socket) const
{
    if(!socket) return QByteArray();
    QByteArray res;
    qint32 len = 30;
    QElapsedTimer readTimer;
    readTimer.start();
    while(socket && res.size() < len && readTimer.elapsed() < 10000)
    {
        res.append(socket->read(len - res.size()));
        if(len == 30 && res.size() >= 30)
        {
            QDataStream ds(res.mid(24, 2));
            ds.setByteOrder(QDataStream::LittleEndian);
            quint16 buf = 0;
            ds >> buf;
            len += buf;
        }
    }
    lmWarning() << logbinary(res);
    return res;
}

CoreApiConst::ErrorCode KeysUpdater::getKeysData(QByteArray &data)
{
    CoreApiConst::ErrorCode err = fs_->getKeysData(QDateTime::currentDateTime(), data);
    lmWarning() << CoreApiConst::defaultErrorMsg(err) << logbinary(data);
    if(err != CoreApiConst::ErrorCode::Ok) return err;
    lmWarning() << logbinary(data);
    FsFullStatus fs = IOSG::getFs();
    KeysMsgContainer c(0x0016u, fs.fsNumber().trimmed(), data);
    data = c.serialize();
    lmWarning() << logbinary(data);
    return CoreApiConst::ErrorCode::Ok;
}

CoreApiConst::ErrorCode KeysUpdater::writeKeysData(const QByteArray &data, quint8 &fsAnswer,
                                                   quint16 &srvAns, QString &srvMsg)
{
    FsFullStatus fs = IOSG::getFs();
    KeysMsgContainer c(fs.fsNumber(), data);
    return  fs_->writeKeysData(c.container(), fsAnswer, srvAns, srvMsg);
}

