#ifndef PRINTDOCWATCHER_H
#define PRINTDOCWATCHER_H

#include "fswrapper.h"
#include "coretransaction.h"
#include "printdocumentformatter.h"
#include "watcherdb.h"
#include <QObject>

/*Восстановление ненапечатанных документов
 * Буду удалять невосстановленные, если они не последние?
 * Где брать данные для обработанных? вместо кэша?
 *
 *
 *Что  с formatToExternal?
 */

class ProcessingDataWrapper;

class PrintDocWatcher : public QObject
{
    Q_OBJECT
    Q_DISABLE_COPY(PrintDocWatcher)
    friend class ProcessingDataWrapper;
    static const QSet<qint32> CHECKABLE_TRANSACTIONS;
public:
    enum Status
    {
        Initial                 = 0x00,
        BeforeFsFinalize        = 0x01,
        BeforePrint             = 0x02,
        Final                   = 0x03,
        FinalSuccess            = 0x04
    };

    ~PrintDocWatcher() override;

    //С самого начала //сохранить еще и тип транзакции
    bool init(const CoreTransaction &trans, qint32 docNumber);
    //Перед закрытием документа в фн
    bool startFs(const QString &externalId);
    //После закрытия документа в фн
    bool update(const QString &externalId, qint32 docNumber, const QVariantMap &doc,
                const TextPrinterDocument &printable);
    //После печати
    bool finalize(const QString &externalId, CoreApiConst::ErrorCode result);
    //Проверка наличия ненапечатанного документа. для статуса
    bool hasNotPrinted();
    //в frworkingthread будет проверяться конкретная транзакция. если таковая есть - восстанавливаться и печататься
    bool checkNotPrinted(const CoreTransaction &trans);
    //Ищем и возвращаем, чтобы не было повторов
    bool find(const CoreTransaction &trans, CoreApiResult &result, TextPrinterDocument &pdoc);
    bool find(const CoreTransaction &trans, CoreApiResult &result);
    //Удаляю, если не удалось зафискалить
    bool remove(const QString &externalId);
    //Восстанавливаю при старте. без печати. на случай падения или выключения
    bool restoreLast(QString &externalId);
    //Восстанавливаю при старте. и печатаю по команде
    bool restoreLast(QString &externalId, CoreApiResult &result, TextPrinterDocument &pdoc);

    bool restore(const CoreTransaction &trans, CoreApiResult &result, TextPrinterDocument &pdoc);

    void clean();

    CoreApiConst::ErrorCode lastError() const;

    const QString &lastErrorMsg() const;

signals:

private:
    FsWrapper *fs_;
    WhatcherDb *dbc_;
    CoreApiConst::ErrorCode lastError_;
    QString lastErrorMsg_;
    bool formatResultToExternal_;


    explicit PrintDocWatcher(ProcessingDataWrapper *parent, FsWrapper *fs);

    ProcessingDataWrapper *pdw();

    bool restore(const QString &externalId, CoreApiResult &result, TextPrinterDocument &pdoc);

    bool finishRestore(const WhatcherDb::Data &incoming, CoreApiResult &result, TextPrinterDocument &pdoc);

    bool needRestore(const WhatcherDb::Data &incoming, const PFsArchiveReportBase &docSummary) const;
    bool checkReceiptAmount(const FixNumber &amount, const QVariantMap &incoming) const;

    bool restoreReceipt(const WhatcherDb::Data &incoming, const PFsArchiveReportBase &docSummary,
                        CoreApiResult &result, TextPrinterDocument &pdoc);
};





#endif // PRINTDOCWATCHER_H
