#include "umkaauth.h"

#include "logmanip.h"

#include <QTimer>
#include <QUrl>
#include <QUrlQuery>
#include <QMetaObject>
#include <QEventLoop>

UmkaAuth::UmkaAuth(QObject *parent)
    : QObject(parent)
    , progressTimer_(new QTimer(this))
    , http_(new QNetworkAccessManager(this))
    , lastData_()
    , lastError_()
{
    connect(progressTimer_, SIGNAL(timeout()), this, SLOT(timeout()));
    progressTimer_->setSingleShot(true);


    connect(http_, &QNetworkAccessManager::authenticationRequired            , this, &UmkaAuth::authenticationRequired            );
    connect(http_, &QNetworkAccessManager::encrypted                         , this, &UmkaAuth::encrypted                         );
    connect(http_, &QNetworkAccessManager::finished                          , this, &UmkaAuth::finished                          );
    connect(http_, &QNetworkAccessManager::networkAccessibleChanged          , this, &UmkaAuth::networkAccessibleChanged          );
    connect(http_, &QNetworkAccessManager::preSharedKeyAuthenticationRequired, this, &UmkaAuth::preSharedKeyAuthenticationRequired);
    connect(http_, &QNetworkAccessManager::proxyAuthenticationRequired       , this, &UmkaAuth::proxyAuthenticationRequired       );
    connect(http_, &QNetworkAccessManager::sslErrors                         , this, &UmkaAuth::sslErrors                         );
}

UmkaAuth::~UmkaAuth()
{

}

void UmkaAuth::login(const QString &userName, const QString &password)
{
    lmWarning() << userName << password;
    progressTimer_->start(60000);
    lastError_.clear();
    lastData_.clean();
    QUrl url("https://umka365.ru/kkm-trade/api/LoginSession");
    QUrlQuery uq;
    uq.addQueryItem("login", userName);
    uq.addQueryItem("password", password);
    url.setQuery(uq);
    QNetworkRequest req(url);
    http_->get(req);
}

const umka365::SessionData &UmkaAuth::lastData() const
{
    return lastData_;
}

const QString &UmkaAuth::lastError() const
{
    return lastError_;
}

bool UmkaAuth::syncLogin(const QString &userName, const QString &password)
{
    bool success = false;
    QEventLoop loop;
    auto sfinished = [&success, &loop] (bool res) ->void {
        success = res;
        loop.quit();
    };
    QMetaObject::Connection conn = connect(this, &UmkaAuth::loginFinished, sfinished);
    login(userName, password);
    loop.exec();
    progressTimer_->stop();
    disconnect(conn);
    lmWarning() << lastError_ << logtab << logvariant(lastData_.toMap());
    return success;
}


void UmkaAuth::loginResult(bool success)
{
    lmWarning() << progressTimer_->isActive();
    progressTimer_->stop();
    if(!success || !lastData_.isValid())
    {
        lastError_ = tr("Доступ запрещен");
        emit loginFinished(false);
    }
    else
    {
        emit loginFinished(true);
    }
}

void UmkaAuth::authenticationRequired(QNetworkReply *reply, QAuthenticator *authenticator)
{
    Q_UNUSED(reply)
    Q_UNUSED(authenticator)
}

void UmkaAuth::encrypted(QNetworkReply *reply)
{
    Q_UNUSED(reply)
}

void UmkaAuth::finished(QNetworkReply *reply)
{
    if(!reply)return;
    lmWarning() << progressTimer_->isActive();
    if(reply->error() != QNetworkReply::NoError)
    {
        lmWarning() << reply->error() << reply->errorString();
        progressTimer_->stop();
        lastError_ = tr("Сетевая ошибка: ") + reply->errorString();
        emit loginFinished(false);
    }
    else if(progressTimer_->isActive())
    {

//        bool loginned = lastData_.isValid();
        bool parsed = lastData_.parseData(reply->readAll());
        loginResult(parsed);
    }
}

void UmkaAuth::networkAccessibleChanged(QNetworkAccessManager::NetworkAccessibility accessible)
{
    Q_UNUSED(accessible)
}

void UmkaAuth::preSharedKeyAuthenticationRequired(QNetworkReply *reply, QSslPreSharedKeyAuthenticator *authenticator)
{
    Q_UNUSED(reply)
    Q_UNUSED(authenticator)

}

void UmkaAuth::proxyAuthenticationRequired(const QNetworkProxy &proxy, QAuthenticator *authenticator)
{
    Q_UNUSED(proxy)
    Q_UNUSED(authenticator)
}

void UmkaAuth::sslErrors(QNetworkReply *reply, const QList<QSslError> &errors)
{
    if(reply)
    {
        reply->ignoreSslErrors(errors);
    }
}

void UmkaAuth::timeout()
{
    progressTimer_->stop();
    lastError_ = tr("Превышен интервал ожидания запроса от сервера");
    emit loginFinished(false);
}
