#ifndef UMKAAUTH_H
#define UMKAAUTH_H

#include "umka365SessionData.h"

#include <QObject>

#include <QNetworkAccessManager>
#include <QNetworkRequest>
#include <QNetworkReply>
#include <QAuthenticator>

class QTimer;

class UmkaAuth : public QObject
{
    Q_OBJECT
    Q_DISABLE_COPY(UmkaAuth)
public:
    explicit UmkaAuth(QObject *parent = nullptr);
    ~UmkaAuth() override;

    const QString &lastError() const;

    bool syncLogin(const QString &userName, const QString &password);

    const umka365::SessionData &lastData() const;

public slots:
    void login(const QString &userName, const QString &password);
signals:
    void loginFinished(bool success);
private:
    QTimer *progressTimer_;
    QNetworkAccessManager *http_;
    umka365::SessionData lastData_;
    QString lastError_;

private slots:
    void loginResult(bool success);

    void authenticationRequired(QNetworkReply *reply, QAuthenticator *authenticator);
    void encrypted(QNetworkReply *reply);
    void finished(QNetworkReply *reply);
    void networkAccessibleChanged(QNetworkAccessManager::NetworkAccessibility accessible);
    void preSharedKeyAuthenticationRequired(QNetworkReply *reply, QSslPreSharedKeyAuthenticator *authenticator);
    void proxyAuthenticationRequired(const QNetworkProxy &proxy, QAuthenticator *authenticator);
    void sslErrors(QNetworkReply *reply, const QList<QSslError> &errors);
    void timeout();
};

#endif // UMKAAUTH_H
