#include "watcherator.h"

#include "coreapiresult.h"

Watcherator::Watcherator(PrintDocWatcher *watcher, const CoreTransaction &trans,
                         qint32 docNumber)
    : watcher_(watcher)
    , transaction_(trans)
    , docNumber_(docNumber)
    , needRemove_(true)
{

}

Watcherator::~Watcherator()
{
    if(needRemove_)remove();
}

bool Watcherator::init()
{
    bool res = watcher_->init(transaction_, docNumber_);
    lastError_ = watcher_->lastError();
    lastErrorMsg_ = watcher_->lastErrorMsg();
    return res;
}

bool Watcherator::startFs()
{
    bool res =  watcher_->startFs(transaction_.externalId());
    lastError_ = watcher_->lastError();
    lastErrorMsg_ = watcher_->lastErrorMsg();
    needRemove_ = false;
    return res;
}

bool Watcherator::update(qint32 docNumber, const QVariantMap &doc,
                         const TextPrinterDocument &printable)
{
    bool res = watcher_->update(transaction_.externalId(), docNumber, doc, printable);
    docNumber_ = docNumber;
    lastError_ = watcher_->lastError();
    lastErrorMsg_ = watcher_->lastErrorMsg();
    needRemove_ = false;
    return res;
}

bool Watcherator::finalize(CoreApiConst::ErrorCode result)
{
    bool res = watcher_->finalize(transaction_.externalId(), result);
    lastError_ = watcher_->lastError();
    lastErrorMsg_ = watcher_->lastErrorMsg();
    needRemove_ = false;
    return res;
}

bool Watcherator::remove()
{
    bool res = watcher_->remove(transaction_.externalId());
    lastError_ = watcher_->lastError();
    lastErrorMsg_ = watcher_->lastErrorMsg();
    return res;
}

CoreApiResult Watcherator::toErrorResult()
{
    CoreApiResult res = CoreApiResult(lastError_, lastErrorMsg_);
    remove();
    return res;
}

qint32 Watcherator::docNumber() const
{
    return docNumber_;
}

void Watcherator::clean()
{
    if(watcher_)watcher_->clean();
}
