#ifndef WATCHERDB_H
#define WATCHERDB_H


#include "basesqlitedbconnector.h"

#include "coretransaction.h"
#include "printdocumentformatter.h"
#include "coreapiresult.h"
#include "fsmsg.h"

#include "cashier.h"
#include "ofdsettings.h"

class WhatcherDb: public BaseSqliteDbConnector
{
    Q_OBJECT
    Q_DISABLE_COPY(WhatcherDb)
    static const qint32 DB_METADATA_VERSION = 1;
public:
    struct Data
    {
        qint64 id;
        QString externalId;
        CoreTransaction::Operation operation;
        QString clientId;
        qint32 docNumber;
        QVariantMap incoming;
        QDateTime dt;
        qint32 status;
        Data()
            : id(-1ll)
            , externalId()
            , operation(CoreTransaction::Operation::INVALID)
            , clientId()
            , docNumber(-1)
            , incoming()
            , dt()
            , status(0) {}
    };
    explicit WhatcherDb(QObject *parent = nullptr);//READWRITE
    explicit WhatcherDb(ConnectionMode mode, QObject *parent = nullptr);
    ~WhatcherDb() override;

    //С самого начала //сохранить еще и тип транзакции
    bool initDocument(const CoreTransaction &trans, qint32 docNumber);
    //Перед закрытием документа в фн
    bool startFs(const QString &externalId);
    //После закрытия документа в фн
    bool update(const QString &externalId, qint32 docNumber, const QVariantMap &doc,
                const TextPrinterDocument &printable);

    //После печати
    bool finalize(const QString &externalId, CoreApiConst::ErrorCode result);
    //Проверка наличия ненапечатанного документа. для статуса
    bool hasNotPrinted();
    //в frworkingthread будет проверяться конкретная транзакция. если таковая есть - восстанавливаться и печататься
    bool checkNotPrinted(const CoreTransaction &trans);//Возвращает фолс и в случае ошибки. но тогда ошибка вернется и на этапе формирования документа
    //Ищем и возвращаем, чтобы не было повторов
    bool find(const CoreTransaction &trans, CoreApiResult &result, TextPrinterDocument &printable, int &status);
    bool find(const QString &externalId, CoreApiResult &result, TextPrinterDocument &printable, int &status);
    //Удаляю, если не удалось зафискалить
    bool remove(const QString &externalId);
    bool clean();
    bool getIncoming(const QString &externalId, Data &data);

    bool getNotPrinterExternalId(QString &externalId);

    bool update2112(fdf::IncorrectLabelsFlag flag, qint32 cycle);
    bool update2113(fdf::IncorrectNotificationsFlag flag, qint32 cycle);

    bool get2112And2113(qint32 cycle, fdf::IncorrectLabelsFlags &f2112,
                        fdf::IncorrectNotificationsFlags &f2113);

    bool resetDb() override;

    bool updateKeysStatus(const QString &fsNumber);
    bool getKeysStatus(const QString &fsNumber, fs::KeysStatus &ks);


    CoreApiConst::ErrorCode lastError() const;

    const QString &lastErrorMsg() const;

private:
    static ConnectionsCounter _rwopened;
    static ConnectionsCounter _roopened;

    CoreApiConst::ErrorCode lastError_;
    QString lastErrorMsg_;


    QString databaseFileName() const override;
    QString connectionName() const override;

    bool getMetaData(qint32 &metadataVersion, EMetadataState &state) override;
    qint32 actualMetadataVersion() const override;
    bool updateDatabase(qint32 from) override;

    void cleanErrors();
};

//-----------------------------------------------------------------------------------------

class WhatcherDbDbInitializer : public QObject
{
    Q_OBJECT
    Q_DISABLE_COPY(WhatcherDbDbInitializer)
    friend class WhatcherDb;
public:
    explicit WhatcherDbDbInitializer(QObject *parent = nullptr);
    ~WhatcherDbDbInitializer() override;

    static bool initializeDatabase();
    bool createDatabase(QSqlDatabase *const db, const QString &fileName);
    bool updateDatabase(QSqlDatabase *const db, qint32 toMetaData);
private:
    QString getScript(const QString &fileName);
    bool applyScript(QSqlDatabase *const db, const QString &sql);

};


#endif // WATCHERDB_H

