#ifndef PRINTERSWRAPPER_H
#define PRINTERSWRAPPER_H

#include "textprinterdocument.h"
#include "coreapiconst.h"
#include <QObject>
#include <QImage>

class QAndroidJniObject;

class PrintersWrapper : public QObject
{
    Q_OBJECT
    Q_DISABLE_COPY(PrintersWrapper)
public:
    explicit PrintersWrapper(QObject *parent = nullptr);
    ~PrintersWrapper() override;

    bool printBlock(const DocumentBlock &block);
    bool printBlocks(const DocumentBlocks &blocks);
    bool hasPaper();
    bool feed();
    bool printTextDocument(const TextPrinterDocument &doc);
    CoreApiConst::ErrorCode test();

    CoreApiConst::ErrorCode lastErrorCode();


signals:
private:
    QAndroidJniObject *obj_;

    qint32 paperWidth();
    qint32 imgBlockSize();

    bool printLine(qint32 format, const QString &line);
    bool printBarCode(qint32 format, qint32 type, const QString &line);
    bool printQrCode(qint32 format, const QString &line);
    bool printBitMap(const QByteArray &data, qint32 format);

    bool printTextBlock(const DocumentBlock &block);
    bool printFormattedImage(const MonochromeImage &img, qint32 rawword,
                             const LineFormat &f);
    bool printQImage(const QImage &img, const LineFormat &f);

    bool startDocument();
    bool finishDocument();
};

#endif // PRINTERSWRAPPER_H
