import QtQuick 2.12
import QtQuick.Controls 2.12
import Qt.labs.platform 1.1
import ru.armax.cashbox 1.0
//import ru.armax.cashbox.agentdatawrapper 1.0
//import ru.armax.cashbox.innvalidator 1.0

Page {
    width: parent.width
    height: parent.height
    signal closed()
    title: qsTr("Данные агента")
    id: agentDataPage
    objectName: "agentDataPage"
    property alias ownFooter: footer
    property alias ownAgent: agent

    AgentDataWrapper {
        id: agent
        onError: function(msg){
            msgError.text = msg
            msgError.open()
        }
        onFinish: {
            agentDataPage.closed()
        }
    }

    BusyIndicator {
        id: busyIndicator
        anchors.horizontalCenter: parent.horizontalCenter
        anchors.verticalCenter: parent.verticalCenter
        running: false
        z: 10
    }
    MessageDialog {
        id: msgError
        buttons: MessageDialog.Close
        text: ""
    }
    ScrollView {
        id: infoScrollView
        anchors.fill: parent
        anchors.margins: 0
        ScrollBar.vertical.policy: ScrollBar.AlwaysOff
        ScrollBar.horizontal.policy: ScrollBar.AlwaysOff
        width: agentDataPage.width
        height: agentDataPage.height
        contentHeight: agentDataPage.height * 6
        contentWidth: parent.width - anchors.margins
        Item {
            anchors.fill: parent
            height:  agentDataPage.height * 6
            width: agentDataPage.width - agentDataPage.anchors.margins
            ComboBox {
                id: cbAgent
                anchors.top: parent.top
                anchors.margins: 5
                anchors.left: parent.left
                anchors.right: parent.right
                model: agent.agentFlagModel
                textRole: "name"
                currentIndex: agent.agentFlagIndex
                onCurrentIndexChanged:  {
                    agent.agentFlagIndex = cbAgent.currentIndex
                }
            }
            Label
            {
                id: lProviderInn
                text: qsTr("ИНН поставщика")
                anchors.left: parent.left
                anchors.verticalCenter: leProviderInn.verticalCenter
                anchors.margins: 5
                enabled: agent.agentFlag != AgentDataWrapper.NotAgent
            }
            TextField
            {
                id: leProviderInn
                anchors.top: cbAgent.bottom
                anchors.margins: 5
                anchors.right: parent.right
                validator: InnValidator{}
                maximumLength: 10
                inputMethodHints: Qt.ImhDigitsOnly
                text: agent.providerInn
                enabled: agent.agentFlag != AgentDataWrapper.NotAgent
                color: acceptableInput ? "black" : "red"
            }
            Label {
                id: lProviderName
                anchors.left:  parent.left
                anchors.right: parent.right
                anchors.top: leProviderInn.bottom
                anchors.leftMargin: 5
                anchors.rightMargin: 5
                width: parent.width  -10

                horizontalAlignment: Label.AlignRight
                enabled: agent.agentFlag != AgentDataWrapper.NotAgent
                text: "Наименование поставщика"
            }
            TextField {
                id: leProviderName
                anchors.left:  parent.left
                anchors.right: parent.right
                anchors.top: lProviderName.bottom
                anchors.margins: 5
                enabled: agent.agentFlag != AgentDataWrapper.NotAgent
                text: agent.providerName
            }
            Label {
                id: lProviderPhones
                anchors.left:  parent.left
                anchors.right: parent.right
                anchors.top: leProviderName.bottom
                anchors.leftMargin: 5
                anchors.rightMargin: 5
                width: parent.width  -10

                horizontalAlignment: Label.AlignRight
                enabled: agent.agentFlag != AgentDataWrapper.NotAgent
                text: "Телефоны поставщика через запятую"
            }
            TextField {
                id: leProviderPhones
                anchors.left:  parent.left
                anchors.right: parent.right
                anchors.top: lProviderPhones.bottom
                anchors.margins: 5
                enabled: agent.agentFlag != AgentDataWrapper.NotAgent
                text: agent.providerPhones
            }
            Label {
                id: lOperatorName
                anchors.left:  parent.left
                anchors.right: parent.right
                anchors.top: leProviderPhones.bottom
                anchors.leftMargin: 5
                anchors.rightMargin: 5
                width: parent.width  -10

                horizontalAlignment: Label.AlignRight
                enabled: agent.isBankAgent
                text: "Наименование оператора перевода"
            }
            TextField {
                id: leOperatorName
                anchors.left:  parent.left
                anchors.right: parent.right
                anchors.top: lOperatorName.bottom
                anchors.margins: 5
                enabled: agent.isBankAgent
                text: agent.operatorName
            }
            Label {
                id: lOperatorAddress
                anchors.left:  parent.left
                anchors.right: parent.right
                anchors.top: leOperatorName.bottom
                anchors.leftMargin: 5
                anchors.rightMargin: 5
                width: parent.width  -10

                horizontalAlignment: Label.AlignRight
                enabled: agent.isBankAgent
                text: "Адрес оператора перевода"
            }
            TextField {
                id: leOperatorAddress
                anchors.left:  parent.left
                anchors.right: parent.right
                anchors.top: lOperatorAddress.bottom

                anchors.margins: 5
                enabled: agent.isBankAgent
                text: agent.operatorAddress
            }
            Label
            {
                id: lOperatorInn
                text: qsTr("ИНН оператора")
                anchors.left: parent.left
                anchors.verticalCenter: leOperatorInn.verticalCenter
                anchors.margins: 5
                enabled: agent.isBankAgent
            }
            TextField
            {
                id: leOperatorInn
                anchors.top: leOperatorAddress.bottom
                anchors.margins: 5
                anchors.right: parent.right
                validator: InnValidator{}
                maximumLength: 10
                inputMethodHints: Qt.ImhDigitsOnly
                text: agent.operatorInn
                enabled: agent.isBankAgent
                color: acceptableInput ? "black" : "red"
            }
            Label {
                id: lOperatorPhones
                anchors.left:  parent.left
                anchors.right: parent.right
                anchors.top: leOperatorInn.bottom
                anchors.leftMargin: 5
                anchors.rightMargin: 5
                width: parent.width  -10

                horizontalAlignment: Label.AlignRight
                enabled: agent.isBankAgent
                text: "Телефоны оператора перевода через запятую"
            }
            TextField {
                id: leOperatorPhones
                anchors.left:  parent.left
                anchors.right: parent.right
                anchors.top: lOperatorPhones.bottom
                anchors.margins: 5
                enabled: agent.isBankAgent
                text: agent.operatorPhones
            }
            Label {
                id: lOperation
                anchors.left:  parent.left
                anchors.right: parent.right
                anchors.top: leOperatorPhones.bottom
                anchors.leftMargin: 5
                anchors.rightMargin: 5
                width: parent.width  -10

                horizontalAlignment: Label.AlignRight
                enabled: agent.isBankAgent
                text: "Операция банковского платежного агента"
            }
            TextField {
                id: leOperation
                anchors.left:  parent.left
                anchors.right: parent.right
                anchors.top: lOperation.bottom
                anchors.margins: 5
                enabled: agent.isBankAgent
                text: agent.bankOperation
                maximumLength: 24
            }
            Label {
                id: lAgentPhones
                anchors.left:  parent.left
                anchors.right: parent.right
                anchors.top: leOperation.bottom
                anchors.leftMargin: 5
                anchors.rightMargin: 5
                width: parent.width  -10

                horizontalAlignment: Label.AlignRight
                enabled: agent.isPayAgent || agent.isBankAgent
                text: "Телефоны пл. агента через запятую"
            }
            TextField {
                id: leAgentPhones
                anchors.left:  parent.left
                anchors.right: parent.right
                anchors.top: lAgentPhones.bottom
                anchors.margins: 5
                enabled: agent.isPayAgent || agent.isBankAgent
                text: agent.agentPhones
            }
            Label {
                id: lPayOperatorPhones
                anchors.left:  parent.left
                anchors.right: parent.right
                anchors.top: leAgentPhones.bottom
                anchors.leftMargin: 5
                anchors.rightMargin: 5
                width: parent.width  -10
                horizontalAlignment: Label.AlignRight
                enabled: agent.isPayAgent
                wrapMode: Label.WordWrap

                text: "Телефоны оператора по приему платежей через запятую"
            }
            TextField {
                id: lePayOperatorrPhones
                anchors.left:  parent.left
                anchors.right: parent.right
                anchors.top: lPayOperatorPhones.bottom
                anchors.margins: 5
                enabled: agent.isPayAgent
                text: agent.payOperatorPhones
            }

        }

    }
    ToolBar {
        id: footer
        height: 70
        ToolButton {
            id: tbBack
            width: 64
            height: 64
            anchors.left: parent.left
            anchors.verticalCenter: parent.verticalCenter
            anchors.leftMargin: 10
            anchors.rightMargin: 10
            anchors.topMargin: 5
            anchors.bottomMargin: 5
            icon.source: "qrc:/icons/back_64.png"
            icon.color: "transparent"
            icon.width: 60
            icon.height: 60
            onClicked: {
                agentDataPage.closed()
            }
        }
        ToolButton {
            id: tbReset
            width: 64
            height: 64
            anchors.horizontalCenter: parent.horizontalCenter
            anchors.verticalCenter: parent.verticalCenter
            anchors.leftMargin: 10
            anchors.rightMargin: 10
            anchors.topMargin: 5
            anchors.bottomMargin: 5
            icon.source: "qrc:/icons/reset_64.png"
            icon.color: "transparent"
            icon.width: 64
            icon.height: 64
            onClicked: {
                resetData()
            }
        }
        ToolButton {
            id: tbSave
            width: 64
            height: 64
            anchors.right: parent.right
            anchors.verticalCenter: parent.verticalCenter
            anchors.leftMargin: 10
            anchors.rightMargin: 10
            anchors.topMargin: 5
            anchors.bottomMargin: 5
            icon.source: "qrc:/icons/apply_64.png"
            icon.color: "transparent"
            icon.width: 64
            icon.height: 64
            onClicked: {
                commit()
            }
        }
    }


    function resetData() {
        agent.clean()
    }
    function commit() {
        agent.agentFlagIndex    = cbAgent.currentIndex
        agent.providerInn       = leProviderInn.text
        agent.providerName      = leProviderName.text
        agent.providerPhones    = leProviderPhones.text
        agent.operatorName      = leOperatorName.text
        agent.operatorInn       = leOperatorInn.text
        agent.operatorAddress   = leOperatorAddress.text
        agent.agentPhones       = leAgentPhones.text
        agent.payOperatorPhones = lePayOperatorrPhones.text
        agent.operatorPhones    = leOperatorPhones.text
        agent.bankOperation     = leOperation.text
        agent.save()
    }

    Component.onCompleted: {
        width = parent.width
        height = parent.height
    }
}
