import QtQuick 2.12
import QtQuick.Controls 2.12
import Qt.labs.platform 1.1
import QtMultimedia 5.12

import ru.armax.cashbox 1.0

//import ru.armax.cashbox.sbarcodefilter 1.0

Page {
    width: parent.width
    height: parent.height
    signal closed()
    signal newBarcode(string text)
    title: qsTr("Сканирование ШК/QR")
    id: barcodePage
    objectName: "cyclePage"
    property alias ownFooter: footer

    Camera {
        id: camera
        captureMode: Camera.CaptureStillImage
        viewfinder.resolution: "640x480"

        flash.mode: Camera.FlashAuto
        focus {
            focusMode: Camera.FocusContinuous
            focusPointMode: Camera.FocusPointCenter
//            customFocusPoint: Qt.point(0.2, 0.2) // Focus relative to top-left corner
        }
    }

    VideoOutput {
      id: videoOutput
      source: camera
      anchors.fill: parent
      autoOrientation: true
      focus: visible

      fillMode: VideoOutput.PreserveAspectCrop
      filters: [
        barcodeFilter
      ]
      Rectangle {
          border.width: 1
          border.color: "green"
          color: "transparent"
          anchors.verticalCenter: parent.verticalCenter
          anchors.horizontalCenter: parent.horizontalCenter
          width:  3 * parent.width / 4
          height: width
      }
    }

    SBarcodeFilter {
      id: barcodeFilter
      onBarcodeParsed: function (text) {
          barcodePage.newBarcode(text)
      }
    }
    Slider {
        id: zoomSlider
        anchors.horizontalCenter: parent.horizontalCenter
        anchors.bottom: parent.bottom
        from: 1.0
        to: camera.maximumDigitalZoom
        onMoved: {
            camera.digitalZoom= zoomSlider.value
        }
    }
    ToolBar {
        id: footer
        height: 70
        ToolButton {
            id: tbBack
            width: 64
            height: 64
            anchors.left: parent.left
            anchors.verticalCenter: parent.verticalCenter
            anchors.leftMargin: 10
            anchors.rightMargin: 10
            anchors.topMargin: 5
            anchors.bottomMargin: 5
            icon.source: "qrc:/icons/back_64.png"
            icon.color: "transparent"
            icon.width: 60
            icon.height: 60
            onClicked: {
                barcodePage.closed()
            }
        }
        ToolButton {
            id: tbLight
            width: 64
            height: 64
            anchors.right: parent.right
            anchors.verticalCenter: parent.verticalCenter
            anchors.leftMargin: 10
            anchors.rightMargin: 10
            anchors.topMargin: 5
            anchors.bottomMargin: 5
            visible: false
            icon.source: camera.flash.mode == Camera.FlashVideoLight ? "qrc:/icons/light_off_64.png" : "qrc:/icons/light_on_64.png"
            icon.color: "transparent"
            icon.width: 60
            icon.height: 60
            onClicked: {
                if(camera.flash.mode == Camera.FlashVideoLight) {
                    camera.flash.mode = Camera.FlashAuto
                }
                else {
                    camera.flash.mode = Camera.FlashVideoLight
                }

            }
        }
    }
    Component.onCompleted: {

//        console.log(camera.flash.supportedModes.toString())
//        console.log(camera.focus.supportedFocusModes)
//        console.log(camera.focus.supportedFocusPointModes)
    }
}
