import QtQuick 2.12
import QtQuick.Controls 2.12
import Qt.labs.platform 1.1

import ru.armax.cashbox 1.0
//import ru.armax.cashbox.calcreportincomingwrapper 1.0
//import ru.armax.cashbox.cashboxstatus 1.0
//import ru.armax.cashbox.hexstringvalidator 1.0
//import ru.armax.cashbox.reportscorehandler 1.0

Page {
    width: parent.width
    height: parent.height
    id: pgCalcReport
    title: qsTr("Отчет о расчетах")
    signal closed()
    objectName: "pgCalcReport"

    property alias ownFooter: footer

    ToolBar {
        id: footer
        height: 70
        ToolButton {
            id: tbBack
            width: 64
            height: 64
            anchors.left: parent.left
            anchors.verticalCenter: parent.verticalCenter
            anchors.leftMargin: 10
            anchors.rightMargin: 10
            anchors.topMargin: 5
            anchors.bottomMargin: 5
            icon.source: "qrc:/icons/back_64.png"
            icon.color: "transparent"
            icon.width: 60
            icon.height: 60
            enabled: cbStatus.fiscalized
            onClicked: {
                pgCalcReport.closed()
            }
        }
        ToolButton {
            id: tbCalcReport
            width: 64
            height: 64
            anchors.right: parent.right
            anchors.verticalCenter: parent.verticalCenter
            anchors.leftMargin: 10
            anchors.rightMargin: 10
            anchors.topMargin: 5
            anchors.bottomMargin: 5
            icon.source: "qrc:/icons/print_64.png"
            icon.color: "transparent"
            icon.width: 64
            icon.height: 64
            onClicked: {
                handler.calcReport(incoming.toMap(), cbBrief.checked)
            }
        }
    }
    CashboxStatus {
        id: cbStatus
    }
    CalcReportIncomingWrapper {
        id: incoming
        onError: function (txt) {
            msgError.text = txt
            msgError.open()
        }
    }

    ReportsCoreHandler {
        id: handler
        onShowBusy: function(busy) {
            busyIndicator.running = busy
            pgCalcReport.enabled = !busy
            window.enableAll(!busy)
        }
        onError: function(txt) {
            msgError.text = txt
            busyIndicator.running = false
            pgCalcReport.enabled = true
            window.enableAll(true)
            msgError.open()
        }
        onCalcReportReady: {
            pgCalcReport.closed()
        }
    }
    BusyIndicator {
        id: busyIndicator
        anchors.horizontalCenter: parent.horizontalCenter
        anchors.verticalCenter: parent.verticalCenter
        running: false
        z: 10
    }
    MessageDialog {
        id: msgError
        buttons: MessageDialog.Close
        text: ""
    }
    GroupBox {
        id: gbAdditionalParameter
        anchors.margins: 5
        anchors.left: parent.left
        anchors.right: parent.right
        anchors.top: parent.top
        title: qsTr("Дополнительный реквизит ОТР")
        height:  anchors.margins * 2  + 1.5 * (leAdditionalParam.height  + leAdditionalParamVal.height)
        contentWidth: width
        TextField {
            id: leAdditionalParam
            anchors.margins: 5
            anchors.left: parent.left
            anchors.right: parent.right
            anchors.top: parent.top
            placeholderText: qsTr("Дополнительный реквизит")
            text: incoming.additionalParam
            maximumLength:  32
            onTextEdited: {
                incoming.additionalParam = leAdditionalParam.text
            }
            onPressed: {
                forceActiveFocus(Qt.MouseFocusReason)
                Qt.inputMethod.show()
            }

        }
        TextField {
            id: leAdditionalParamVal
            anchors.margins: 5
            anchors.left: parent.left
            anchors.right: parent.right
            anchors.top: leAdditionalParam.bottom
            placeholderText: qsTr("Значение дополнительного реквизита")
            text: incoming.additionalData
            maximumLength:  64
            validator: HexStringValidator{
                maxSize: 64
                canBeEmpty: true
                fixedSize: false
            }
            onTextEdited: {
                if(leAdditionalParamVal.acceptableInput) {
                    incoming.additionalData = leAdditionalParamVal.text
                }
            }
            onPressed: {
                forceActiveFocus(Qt.MouseFocusReason)
                Qt.inputMethod.show()
            }
        }
    }
    TextField
    {
        id: leUser
        anchors.margins: 5
        anchors.left: parent.left
        anchors.right: parent.right
        anchors.top: gbAdditionalParameter.bottom
        placeholderText: qsTr("Наименование пользователя")
        text: incoming.ownerName
        maximumLength:  256
        visible: incoming.canChangeOwnerName
        onTextEdited: {
            incoming.ownerName = leUser.text
        }
        onPressed: {
            forceActiveFocus(Qt.MouseFocusReason)
            Qt.inputMethod.show()
        }

    }
    TextField
    {
        id: leAddress
        anchors.margins: 5
        anchors.left: parent.left
        anchors.right: parent.right
        anchors.top: leUser.bottom
        placeholderText: qsTr("Адрес расчетов")
        text: incoming.address
        visible: incoming.canChangeAddress
        width: parent.width - anchors.margins * 2
        wrapMode: TextField.WordWrap
        onTextEdited: {
            incoming.address = leAddress.text
        }
        onPressed: {
            forceActiveFocus(Qt.MouseFocusReason)
            Qt.inputMethod.show()
        }

    }
    TextField
    {
        id: lePlace
        anchors.margins: 5
        anchors.left: parent.left
        anchors.right: parent.right
        anchors.top: leAddress.bottom
        placeholderText: qsTr("Место расчетов")
        text: incoming.place
        maximumLength:  256
        visible: incoming.canChangeAddress
        onTextEdited: {
            incoming.place = lePlace.text
        }
        onPressed: {
            forceActiveFocus(Qt.MouseFocusReason)
            Qt.inputMethod.show()
        }

    }
    CheckBox {
        id: cbBrief
        text: "Печатать в сокращенной форме"
        anchors.margins: 5
        anchors.left: parent.left
        anchors.right: parent.right
        anchors.top: lePlace.bottom
    }
    Component.onCompleted: {


    }
}
