import QtQuick 2.12
import QtQuick.Controls 2.12
import Qt.labs.platform 1.1

import ru.armax.cashbox 1.0
//import ru.armax.cashbox.cashierwrapper 1.0
//import ru.armax.cashbox.innvalidator 1.0

Page {
    width: parent.width
    height: parent.height
    id: cashierEditPage
    signal closed()
    title: qsTr("Данные кассира")
    objectName: "cashierEditPage"
    property alias eCashier: cashier
    property alias ownFooter: footer

    ToolBar {
        id: footer
        height: 70
        ToolButton {
            id: tbBack
            width: 64
            height: 64
            anchors.left: parent.left
            anchors.verticalCenter: parent.verticalCenter
            anchors.leftMargin: 10
            anchors.rightMargin: 10
            anchors.topMargin: 5
            anchors.bottomMargin: 5
            icon.source: "qrc:/icons/back_64.png"
            icon.color: "transparent"
            icon.width: 64
            icon.height: 64
            onClicked: {
                cashierEditPage.closed()
            }
        }
        ToolButton {
            id: tbSave
            width: 64
            height: 64
            anchors.right: parent.right
            anchors.verticalCenter: parent.verticalCenter
            anchors.leftMargin: 10
            anchors.rightMargin: 10
            anchors.topMargin: 5
            anchors.bottomMargin: 5
            icon.source: "qrc:/icons/save_64.png"
            icon.color: "transparent"
            icon.width: 64
            icon.height: 64
            visible: cashier.cashierIsEnabled
            onClicked: {
                cashier.save()
            }
        }
    }

    CashierWrapper {
        id: cashier
        onShowBusy: function(busy) {
            busyIndicator.running = busy
            cashierEditPage.enabled = !busy
            window.enableAll(!busy)
        }
        onError: function(txt) {
            msgError.text = txt
            busyIndicator.running = false
            cashierEditPage.enabled = true
            window.enableAll(true)
            msgError.open()
        }
        onSaved: {
            cashierEditPage.closed()
        }
    }

    BusyIndicator {
        id: busyIndicator
        anchors.horizontalCenter: parent.horizontalCenter
        anchors.verticalCenter: parent.verticalCenter
        running: false
        z: 10
    }
    MessageDialog {
        id: msgError
        buttons: MessageDialog.Close
        text: ""
    }
    ScrollView {
        id: infoScrollView
        anchors.fill: parent
        anchors.margins: 0
        ScrollBar.vertical.policy: ScrollBar.AlwaysOff
        ScrollBar.horizontal.policy: ScrollBar.AlwaysOff
        width: cashierEditPage.width
        height: cashierEditPage.height
        contentHeight: cashierEditPage.height * 3
        contentWidth: parent.width - anchors.margins
        Item {
            anchors.fill: parent
            height:  cashierEditPage.height * 3
            width: cashierEditPage.width - cashierEditPage.anchors.margins
            Label {
                id: lName
                text: qsTr("ФИО кассира")
                font.pixelSize: 16
                font.bold: true
                verticalAlignment: Text.AlignVCenter
                anchors.right: parent.right
                anchors.top: parent.top
                enabled: cashier.cashierIsEnabled
                anchors.margins: 10
            }
            TextField {
                id: txtName;
                anchors.top: lName.bottom
                anchors.right: parent.right
                anchors.left: parent.left
                text: cashier.name
                font.pixelSize: 16
                anchors.leftMargin: 10
                anchors.rightMargin: 10
                anchors.bottomMargin: 10
                enabled: cashier.cashierIsEnabled
                z: 1
                onTextEdited: function () {
                    cashier.name = txtName.text
                }
//                onPressed: {
//                    forceActiveFocus(Qt.MouseFocusReason)
//                    Qt.inputMethod.show()
//                }
            }
            Label {
                id: lLogin
                text: qsTr("Логин кассира")
                font.pixelSize: 16
                font.bold: true
                verticalAlignment: Text.AlignVCenter
                anchors.right: parent.right
                anchors.top: txtName.bottom
                enabled: cashier.cashierIsEnabled && !cashier.isSu
                anchors.margins: 10
            }
            TextField {
                id: txtLogin
                anchors.top: lLogin.bottom
                anchors.right: parent.right
                anchors.left: parent.left
                text: cashier.login
                font.pixelSize: 16
                anchors.leftMargin: 10
                anchors.rightMargin: 10
                anchors.bottomMargin: 10
                enabled: cashier.cashierIsEnabled && !cashier.isSu
                z: 1
                onTextEdited: function () {
                    cashier.login = txtLogin.text
                }
//                onPressed: {
//                    forceActiveFocus(Qt.MouseFocusReason)
//                    Qt.inputMethod.show()
//                }

            }
            Label {
                id: lPassword
                text: qsTr("Пароль кассира")
                font.pixelSize: 16
                font.bold: true
                verticalAlignment: Text.AlignVCenter
                anchors.right: parent.right
                anchors.top: txtLogin.bottom
                enabled: cashier.cashierIsEnabled
                anchors.margins: 10
            }
            TextField {
                id: txtPassword
                anchors.top: lPassword.bottom
                anchors.right: parent.right
                anchors.left: parent.left
                text: cashier.password
                font.pixelSize: 16
                anchors.leftMargin: 10
                anchors.rightMargin: 10
                anchors.bottomMargin: 10
                enabled: cashier.cashierIsEnabled
                z: 1
                onTextEdited: function () {
                    cashier.password = txtPassword.text
                }
                onPressed: {
                    forceActiveFocus(Qt.MouseFocusReason)
                    Qt.inputMethod.show()
                }

            }
            Label {
                id: lCashierInn
                anchors.left: parent.left
                anchors.margins: 5
                text: qsTr("ИНН Кассира")
                enabled: cashier.cashierIsEnabled
                anchors.verticalCenter:  leCashierInn.verticalCenter
            }
            TextField {
                id: leCashierInn
                anchors.margins: 5
                anchors.top: txtPassword.bottom
                anchors.right: parent.right
                text: cashier.inn
                maximumLength: 12
                enabled: cashier.cashierIsEnabled
                validator: InnValidator{}
                color: acceptableInput ? "black" : "red"
                onTextEdited: {
                    cashier.inn = leCashierInn.text
                }
                onPressed: {
//                        forceActiveFocus(Qt.MouseFocusReason)
//                        Qt.inputMethod.show()
                }

            }
            CheckBox {
                id: cbAllowReceipts
                text: qsTr("Чеки прихода/расхода")
                checked: cashier.permReceipts
                width: parent.width - 2 * anchors.margins
                anchors.margins: 10
                anchors.left: parent.left
                anchors.top:leCashierInn.bottom
                enabled: !cashier.isSu && cashier.cashierIsEnabled
                onReleased: {
                    cashier.permReceipts = cbAllowReceipts.checked
                }
            }
            CheckBox {
                id: cbAllowReturnReceipts
                width: parent.width - 2 * anchors.margins
                text: qsTr("Чеки возврата прихода/расхода")
                checked: cashier.permReturnReceipts
                anchors.margins: 10
                anchors.left: parent.left
                anchors.top:cbAllowReceipts.bottom
                enabled: !cashier.isSu && cashier.cashierIsEnabled
                onReleased: {
                    cashier.permReturnReceipts = cbAllowReturnReceipts.checked
                }
            }
            CheckBox {
                id: cbAllowCorrections
                width: parent.width - 2 * anchors.margins
                text: qsTr("Коррекции прихода/расхода")
                checked: cashier.permCorrections
                anchors.margins: 10
                anchors.left: parent.left
                anchors.top:cbAllowReturnReceipts.bottom
                enabled: !cashier.isSu && cashier.cashierIsEnabled
                onReleased: {
                    cashier.permCorrections = cbAllowCorrections.checked
                }
            }
            CheckBox {
                id: cbAllowReturnCorrections
                width: parent.width - 2 * anchors.margins
                text: qsTr("Коррекции возврата прихода/расхода")
                checked: cashier.permReturnCorrections
                anchors.margins: 10
                anchors.left: parent.left
                anchors.top:cbAllowCorrections.bottom
                enabled: !cashier.isSu && cashier.cashierIsEnabled
                onReleased: {
                    cashier.permReturnCorrections = cbAllowReturnCorrections.checked
                }
            }
            CheckBox {
                id: cbAllowCycleOpen
                width: parent.width - 2 * anchors.margins
                text: qsTr("Открытие смены")
                checked: cashier.permCycleOpen
                anchors.margins: 10
                anchors.left: parent.left
                anchors.top: cbAllowReturnCorrections.bottom
                enabled: !cashier.isSu && cashier.cashierIsEnabled
                onReleased: {
                    cashier.permCycleOpen = cbAllowCycleOpen.checked
                }
            }
            CheckBox {
                id: cbAllowCycleClose
                width: parent.width - 2 * anchors.margins
                text: qsTr("Закрытие смены")
                checked: cashier.permCycleClose
                anchors.margins: 10
                anchors.left: parent.left
                anchors.top: cbAllowCycleOpen.bottom
                enabled: !cashier.isSu && cashier.cashierIsEnabled
                onReleased: {
                    cashier.permCycleClose = cbAllowCycleClose.checked
                }
            }
            CheckBox {
                id: cbAllowReports
                width: parent.width - 2 * anchors.margins
                text: qsTr("Отчеты")
                checked: cashier.permReports
                anchors.margins: 10
                anchors.left: parent.left
                anchors.top: cbAllowCycleClose.bottom
                enabled: !cashier.isSu && cashier.cashierIsEnabled
                onReleased: {
                    cashier.permReports = cbAllowReports.checked
                }
            }
            CheckBox {
                id: cbAllowSettings
                width: parent.width - 2 * anchors.margins
                text: qsTr("Настройки, справочники")
                checked: cashier.permSettings
                anchors.margins: 10
                anchors.left: parent.left
                anchors.top: cbAllowReports.bottom
                enabled: !cashier.isSu && cashier.cashierIsEnabled
                onReleased: {
                    cashier.permSettings = cbAllowSettings.checked
                }
            }
            CheckBox {
                id: cbAllowFiscalization
                width: parent.width - 2 * anchors.margins
                text: qsTr("Регистрация, перерегистрация, закрытие архива")
                checked: cashier.permFiscalization
                anchors.margins: 10
                anchors.left: parent.left
                anchors.top: cbAllowSettings.bottom
                enabled: !cashier.isSu && cashier.cashierIsEnabled
                contentItem: Label {
                    text: cbAllowFiscalization.text
                    font: cbAllowFiscalization.font
                    horizontalAlignment: Text.AlignLeft
                    verticalAlignment: Text.AlignVCenter
                    leftPadding: cbAllowFiscalization.indicator.width + cbAllowFiscalization.spacing
                    wrapMode: Label.WordWrap
                }
                onReleased: {
                    cashier.permFiscalization = cbAllowFiscalization.checked
                }
            }
            CheckBox {
                id: cbAllowOfflineNotifications
                width: parent.width - 2 * anchors.margins
                text: qsTr("Выгрузка уведомлений в автономном режиме")
                checked: cashier.permOfflineNotifications
                anchors.margins: 10
                anchors.left: parent.left
                anchors.top: cbAllowFiscalization.bottom
                enabled: !cashier.isSu && cashier.cashierIsEnabled
                contentItem: Label {
                    text: cbAllowOfflineNotifications.text
                    font: cbAllowOfflineNotifications.font
                    horizontalAlignment: Text.AlignLeft
                    verticalAlignment: Text.AlignVCenter
                    leftPadding: cbAllowOfflineNotifications.indicator.width + cbAllowOfflineNotifications.spacing
                    wrapMode: Label.WordWrap
                }
                onReleased: {
                    cashier.permOfflineNotifications = cbAllowOfflineNotifications.checked
                }
            }
        }
    }

}
