import QtQuick 2.12
import QtQuick.Controls 2.12
import Qt.labs.platform 1.1   as QtLabs1

import ru.armax.cashbox 1.0
//import ru.armax.cashbox.cashiersmodel 1.0

Page {
    width: parent.width
    height: parent.height
    signal closed()
    id: cashiersView
    objectName: "cashiersView"
    title: qsTr("Кассиры")


    property alias ownFooter: footer

    CashiersModel {
        id: cashiersModel
        onShowBusy: function(busy) {
            busyIndicator.running = busy
            cashiersView.enabled = !busy
            window.enableAll(!busy)
        }
        onError: function(txt) {
            msgError.text = txt
            busyIndicator.running = false
            cashiersView.enabled = true
            window.enableAll(true)
            msgError.open()
        }
    }

    ToolBar {
        id: footer
        height: 74
        ToolButton {
            id: tbBack
            width: 64
            height: 64
            anchors.left: parent.left
            anchors.verticalCenter: parent.verticalCenter
            anchors.leftMargin: 10
            anchors.rightMargin: 10
            anchors.topMargin: 5
            anchors.bottomMargin: 5
            icon.source: "qrc:/icons/back_64.png"
            icon.color: "transparent"
            icon.width: 64
            icon.height: 64
            onClicked: {
                cashiersView.closed()
            }
        }

        ToolButton {
            id: tbAdd
            width: 64
            height: 64
            anchors.right: parent.right
            anchors.verticalCenter: parent.verticalCenter
            anchors.leftMargin: 10
            anchors.rightMargin: 10
            anchors.topMargin: 5
            anchors.bottomMargin: 5
            icon.source: "qrc:/icons/add_cashier_64.png"
            icon.color: "transparent"
            icon.width: 64
            icon.height: 64
            onClicked: {
                var page = window.showPage("CashierEditPage.qml");
                page.eCashier.setModel(cashiersModel)
            }
        }
    }


    BusyIndicator {
        id: busyIndicator
        anchors.horizontalCenter: parent.horizontalCenter
        anchors.verticalCenter: parent.verticalCenter
        running: false
        z: 10
    }
    QtLabs1.MessageDialog {
        id: msgError
        buttons: QtLabs1.MessageDialog.Close
        text: ""
    }
    ScrollView {
        id: usersView
        anchors.margins: 10
        anchors.fill: parent
        ScrollBar.vertical.policy: ScrollBar.AlwaysOff
        ScrollBar.horizontal.policy: ScrollBar.AlwaysOff
        z: 1
        ListView {
            id: lvUsers
            spacing: 10
            interactive: true
            orientation: Qt.Vertical
            highlightFollowsCurrentItem: true
            model: cashiersModel
            z: 2
            delegate: Item {
                height: 80
                width: parent.width
                Image {
                    id: imgUser
                    source: "qrc:/icons/cashier_64.png"
                    anchors.left: parent.left
                    anchors.verticalCenter: parent.verticalCenter;
                    width: 64
                    height: 64
                    fillMode: Image.PreserveAspectFit
                    z: 4
                    Rectangle {
                        color: cpermissions > 0 ? "lightgreen" : "gray"
                        width: 16
                        height: 16
                        radius: 16
                        anchors.bottom: parent.bottom
                        anchors.horizontalCenter: parent.horizontalCenter
                        z: 5
                    }
                }
                Label {
                    id: lUserName
                    text: "[" + clogin +"] " + cname
                    anchors.left: imgUser.right
                    anchors.margins: 10
                    anchors.right: parent.right
                    anchors.verticalCenter: parent.verticalCenter
                    verticalAlignment: Label.AlignVCenter
                    wrapMode: Label.WordWrap
                    z: 4
                }

                MouseArea {
                    anchors.fill: parent
                    onClicked: {
                        lvUsers.currentIndex = index
                    }
                    onPressAndHold: {
                        lvUsers.currentIndex = index

                        menuOp.y = mouse.y
                        if(menuOp.y + menuOp.heigh > usersView.height) {
                            menuOp.y = menuOp.y - menuOp.heigh
                        }
                        miRemoveCashier.visible = cashiersModel.canRemoveCashier(index)
                        menuOp.open()

                    }
                }
                Rectangle{
                    id: hl
                    anchors.fill: parent
                    color: lvUsers.currentIndex === index ? "lightsteelblue" : "white"
                }

                Menu {
                    id: menuOp

                    MenuItem {
                        text: qsTr("Редактировать")
                        onTriggered: {
                            var page = window.showPage("CashierEditPage.qml");
                            page.eCashier.setData(cashiersModel.getItem(index))
                            page.eCashier.setModel(cashiersModel)
                        }
                    }
                    MenuItem {
                        id :miRemoveCashier
                        text: qsTr("Удалить")
                        onTriggered: {
                            cashiersModel.removeCashier(lvUsers.currentIndex)
                        }
                    }
                }
            }
        }
    }
    Component.onCompleted: {
        cashiersModel.reload()
    }
}
