import QtQuick 2.12
import QtQuick.Controls 2.15
import Qt.labs.platform 1.1 as QtLabs
import Qt.labs.folderlistmodel 2.12 as Fm

import ru.armax.cashbox 1.0

//import ru.armax.cashbox.cashboxstatus 1.0
//import ru.armax.cashbox.approot 1.0

Page {
    width: parent.width
    height: parent.height
    signal closed()
    title: qsTr("Обновление прошивки")
    id: firmwareUpdatePage
    objectName: "firmwareUpdatePage"
    property alias ownFooter: footer

    property string firmwareFileName: ""


    AppRoot {
        id: appRoot
        onShowBusy: function(busy) {
            busyIndicator.running = busy
            firmwareUpdatePage.enabled = !busy
            window.enableAll(!busy)
        }
        onError: function(txt) {
            msgError.text = txt
            busyIndicator.running = false
            firmwareUpdatePage.enabled = true
            window.enableAll(true)
            lOut.text = appRoot.firmwareTestText()
            msgError.open()
        }

        onSuccessTest: {
            msgError.text = "Обновление не установлено!"
            busyIndicator.running = false
            firmwareUpdatePage.enabled = true
            window.enableAll(true)
            lOut.text = appRoot.firmwareTestText()
            msgError.open()
        }
    }
    BusyIndicator {
        id: busyIndicator
        anchors.horizontalCenter: parent.horizontalCenter
        anchors.verticalCenter: parent.verticalCenter
        running: false
        z: 10
    }
    QtLabs.MessageDialog {
        id: msgError
        buttons: QtLabs.MessageDialog.Close
        text: ""
    }

    Dialog {
        id: sdFiles
        modal: true
        width: parent.width
        height: parent.height
        standardButtons: Dialog.Close
        title: "Новая прошивка"
        ScrollView {
            id: sdScrollView
            anchors.margins: 10
            anchors.fill: parent
            ScrollBar.vertical.policy: ScrollBar.AlwaysOff
            ScrollBar.horizontal.policy: ScrollBar.AlwaysOff
            z: 1
            ListView {
                id:lvSdFiles
                interactive: true
                orientation: Qt.Vertical
                highlightFollowsCurrentItem: true
                z: 2
                spacing: 5
                model: Fm.FolderListModel {
                    id: sdFolderModel
                    nameFilters: ["*"]
                    showDirs: false
                    showDotAndDotDot : false
                }
                delegate : Item{
                    height: 40
                    width: parent.width
                    Text {
                        text: fileName
                        anchors.fill: parent
                        wrapMode: Text.WordWrap
                    }
                    MouseArea {
                        anchors.fill: parent
                        onClicked: {
                            if(fileIsDir) sdFolderModel.folder = sdFolderModel.folder + "/" + fileName
                            else {
                                firmwareUpdatePage.firmwareFileName = sdFolderModel.folder + "/" + fileName
                                lOut.text = ""
                                sdFiles.close()
                            }
                        }
                    }
                }

            }
        }
    }

    Label {
        id: lFirmware
        anchors.left: parent.left
        anchors.top: parent.top
        anchors.right: parent.right
        width: parent.width - 10
        anchors.margins: 5
        wrapMode: Label.WordWrap
        text: firmwareFileName.length > 0 ? "Новая прошивка: " + firmwareFileName : "Новая прошивка не выбрана"
    }
    Label {
        id: lOut
        anchors.left: parent.left
        anchors.top: lFirmware.bottom
        anchors.right: parent.right
        anchors.bottom: parent.bottom
        width: parent.width - 10
        anchors.margins: 5
        wrapMode: Label.WordWrap
        text: ""
    }


    RoundButton {
        id: rbFromSdCard
        anchors.margins: 10
        anchors.bottom: parent.bottom
        anchors.right: parent.right
        icon.source: "qrc:/icons/from_sdcard_64.png"
        icon.color: "transparent"
        icon.width: 50
        icon.height: 50
        z:100
        onPressed: {
            sdFolderModel.folder = appRoot.sdcardPath();
            sdFiles.open()
        }
    }
    ToolBar {
        id: footer
        height: 70
        ToolButton {
            id: tbBack
            width: 64
            height: 64
            anchors.left: parent.left
            anchors.verticalCenter: parent.verticalCenter
            anchors.leftMargin: 10
            anchors.rightMargin: 10
            anchors.topMargin: 5
            anchors.bottomMargin: 5
            icon.source: "qrc:/icons/home_64.png"
            icon.color: "transparent"
            icon.width: 60
            icon.height: 60
            onClicked: {
                firmwareUpdatePage.closed()
            }
        }
        ToolButton {
            id: tbCheck
            width: 64
            height: 64
            anchors.right: parent.right
            anchors.verticalCenter: parent.verticalCenter
            anchors.leftMargin: 10
            anchors.rightMargin: 10
            anchors.topMargin: 5
            anchors.bottomMargin: 5
            icon.source: "qrc:/icons/update_firmware_64.png"
            icon.color: "transparent"
            icon.width: 64
            icon.height: 64
            onClicked: {
                lOut.text = ""
                appRoot.updateFirmware(firmwareFileName)
            }
        }
    }
    Component.onCompleted: {
        width = parent.width
        height = parent.height
    }
}
