import QtQuick 2.12
import QtQuick.Controls 2.12
import Qt.labs.platform 1.1 as QtLabs1

import ru.armax.cashbox 1.0

//import ru.armax.cashbox.cashboxstatus 1.0
//import ru.armax.cashbox.fiscalizationhandler 1.0
//import ru.armax.cashbox.incomingregwrapper 1.0
//import ru.armax.cashbox.innvalidator 1.0
//import ru.armax.cashbox.regnumbervalidator 1.0
//import ru.armax.cashbox.hexstringvalidator 1.0

Page {
    width: parent.width
    height: parent.height
    id: pgFiscalize
    title: qsTr("Регистрация/Перерегистрация")
    signal closed()
    objectName: "pgFiscalize"

    property alias ownFooter: footer

    ToolBar {
        id: footer
        height: 70
        ToolButton {
            id: tbBack
            width: 64
            height: 64
            anchors.left: parent.left
            anchors.verticalCenter: parent.verticalCenter
            anchors.leftMargin: 10
            anchors.rightMargin: 10
            anchors.topMargin: 5
            anchors.bottomMargin: 5
            icon.source: "qrc:/icons/back_64.png"
            icon.color: "transparent"
            icon.width: 60
            icon.height: 60
            enabled: cbStatus.fiscalized
            onClicked: {
                pgFiscalize.closed()
            }
        }
        ToolButton {
            id: tbFiscalize
            width: 64
            height: 64
            anchors.right: parent.right
            anchors.verticalCenter: parent.verticalCenter
            anchors.leftMargin: 10
            anchors.rightMargin: 10
            anchors.topMargin: 5
            anchors.bottomMargin: 5
            icon.source: "qrc:/icons/print_64.png"
            icon.color: "transparent"
            icon.width: 64
            icon.height: 64
            onClicked: {
                //ЕНВД БОЛЬШЕ НЕТУ
                fiscData.allowSingleOnImputedProfit = false
                fiscData.preprint()
            }
            onPressAndHold: {
                menuFisc.y = tbFiscalize.y - menuFisc.height
                menuFisc.x = tbFiscalize.x - menuFisc.width - tbFiscalize.x
                menuFisc.open()
            }
            Menu {
                id: menuFisc

                MenuItem {
                    text: qsTr("Регистрация/\nПеререгистрация")
                    onTriggered: {
                        //ЕНВД БОЛЬШЕ НЕТУ
                         fiscData.allowSingleOnImputedProfit = false
                         fiscData.fiscalize()
                    }
                }
                MenuItem {
                    text: qsTr("Предварительная\nпечать")
                    onTriggered: {
                        //ЕНВД БОЛЬШЕ НЕТУ
                         fiscData.allowSingleOnImputedProfit = false
                         fiscData.preprint()
                    }
                }
            }
        }
    }


    CashboxStatus {
        id: cbStatus
    }
    IncomingRegWrapper {
        id: fiscData
        onError: function (txt) {
            msgError.text = txt
            msgError.open()
        }
        onNeedFiscalize: function(data) {
            fiscalizator.fiscalize(data)
        }

        onNeedPreprint:  function (data) {
            fiscalizator.preprint(data)
        }
    }

    FiscalizationHandler {
        id: fiscalizator
        onShowBusy: function(busy) {
            busyIndicator.running = busy
            pgFiscalize.enabled = !busy
            window.enableAll(!busy)
        }
        onError: function(txt) {
            msgError.text = txt
            busyIndicator.running = false
            window.enableAll(true)
            pgFiscalize.enabled = true
            msgError.open()
        }
        onFiscalized: {
            pgFiscalize.closed()
        }
        onSuccessPreprint: {
            dlgPreprint.text = 'Предварительная печать выполнена успешно. \nДля регистрации ККТ выполните \nДолгое нажатие на кнопку печати'
            busyIndicator.running = false
            pgFiscalize.enabled = true
            window.enableAll(true)
            dlgPreprint.open()
        }
    }
    BusyIndicator {
        id: busyIndicator
        anchors.horizontalCenter: parent.horizontalCenter
        anchors.verticalCenter: parent.verticalCenter
        running: false
        z: 10
    }
    QtLabs1.MessageDialog {
        id: msgError
        buttons: QtLabs1.MessageDialog.Close
        text: ""
    }
    QtLabs1.MessageDialog {
        id: dlgPreprint
        buttons: QtLabs1.MessageDialog.Close
        text: ""
    }


    ScrollView {
        id: scroll
        anchors.fill: parent
        ScrollBar.vertical.policy: ScrollBar.AlwaysOff
        ScrollBar.horizontal.policy: ScrollBar.AlwaysOff
        width: pgFiscalize.width
        height: pgFiscalize.height
        contentHeight: pgFiscalize.height * 7
        contentWidth: parent.width
        Item {
            anchors.fill: parent
            height:  pgFiscalize.height * 7
            ComboBox {
                id:cbOperation
                model: fiscData.operationsModel
                textRole: "name"
                anchors.margins: 5
                anchors.left: parent.left
                anchors.right: parent.right
                anchors.top: parent.top
                width: parent.width - 10
                onCurrentIndexChanged: {
                    gbReasons.visible = fiscData.codeOfOperation(cbOperation.currentIndex) !== 0
                    cbFsChange.checked = (fiscData.codeOfOperation(cbOperation.currentIndex) === 1)
                }
            }
            TextField
            {
                id: leUser
                anchors.margins: 5
                anchors.left: parent.left
                anchors.right: parent.right
                anchors.top: cbOperation.bottom
                placeholderText: qsTr("Наименование пользователя")
                text: fiscData.userName
                maximumLength:  256
                onTextEdited: {
                    fiscData.userName = leUser.text
                }
                onPressed: {
//                        forceActiveFocus(Qt.MouseFocusReason)
//                        Qt.inputMethod.show()
                }

            }
            Label
            {
                id: lUserInn
                text: qsTr("ИНН пользователя")
                anchors.left: parent.left
                anchors.verticalCenter: leUserInn.verticalCenter
                anchors.margins: 5
            }
            TextField
            {
                id:leUserInn
                anchors.top: leUser.bottom
                anchors.margins: 5
                anchors.right: parent.right
                validator: InnValidator{}
                maximumLength: 12
                inputMethodHints: Qt.ImhDigitsOnly
                text: fiscData.userInn
                color: acceptableInput ? "black" : "red"
                onTextEdited: {
                    fiscData.userInn = leUserInn.text
                    vRnm.inn = leUserInn.text
                }
                onPressed: {
//                        forceActiveFocus(Qt.MouseFocusReason)
//                        Qt.inputMethod.show()
                }

            }
            TextField
            {
                id: leAddress
                anchors.margins: 5
                anchors.left: parent.left
                anchors.right: parent.right
                anchors.top: leUserInn.bottom
                placeholderText: qsTr("Адрес расчетов")
                text: fiscData.address
                width: parent.width - anchors.margins * 2
                wrapMode: TextField.WordWrap

                onTextEdited: {
                    fiscData.address = leAddress.text
                }
                onPressed: {
//                        forceActiveFocus(Qt.MouseFocusReason)
//                        Qt.inputMethod.show()
                }

            }
            TextField
            {
                id: lePlace
                anchors.margins: 5
                anchors.left: parent.left
                anchors.right: parent.right
                anchors.top: leAddress.bottom
                placeholderText: qsTr("Место расчетов")
                text: fiscData.place
                maximumLength:  256
                onTextEdited: {
                    fiscData.place = lePlace.text
                }
                onPressed: {
//                        forceActiveFocus(Qt.MouseFocusReason)
//                        Qt.inputMethod.show()
                }

            }

            Label
            {
                id: lRnm
                text: qsTr("РНМ")
                anchors.margins: 5
                anchors.left: parent.left
                anchors.verticalCenter: leRnm.verticalCenter
            }
            RegNumberValidator {
                id: vRnm
                serial: cbStatus.serial
                inn: leUserInn.text
            }

            TextField
            {
                id: leRnm
                anchors.top: lePlace.bottom
                anchors.right: tbRnm.left
                anchors.left: lRnm.right
                anchors.margins: 5
                maximumLength: 16
                inputMethodHints: Qt.ImhDigitsOnly
                text: fiscData.regNumber
                validator: vRnm
                color: acceptableInput ? "black" : "red"
                horizontalAlignment: TextField.AlignHCenter
                onTextEdited: {
                    fiscData.regNumber = leRnm.text
                }
                onPressed: {
//                        forceActiveFocus(Qt.MouseFocusReason)
//                        Qt.inputMethod.show()
                }

            }
            ToolButton {
                id: tbRnm
                anchors.right: parent.right
                visible: cbStatus.isDebug && !cbStatus.fiscalized
                anchors.verticalCenter: leRnm.verticalCenter
                anchors.margins: 5

                width: 48
                height: 48
                icon.source: "qrc:/icons/star_48.png"
                icon.color: "transparent"
                icon.width: 48
                icon.height: 48
                onPressed: {
                    fiscData.genRegNumber()
                }
            }

            CheckBox {
                id:cbOffline
                anchors.left: parent.left
                anchors.top: leRnm.bottom
                text: qsTr("Автономно")
                checked: fiscData.offlineMode
                anchors.margins: 5
                onReleased: {
                    fiscData.offlineMode = cbOffline.checked
                    if(cbOffline.checked) {
                        leOfdInn.text = ""
                        leOfdName.text = ""
                        leEmail.text = ""
                        leFns.text = ""
                    }
                }
            }
            CheckBox {
                id:cbAuto
                anchors.left: cbOffline.right
                anchors.top: leRnm.bottom
                text: qsTr("В автомате")
                checked: fiscData.autoMode
                anchors.margins: 5
                onReleased: {
                    fiscData.autoMode = cbAuto.checked
                }
            }
            CheckBox {
                id:cbEnc
                anchors.right: parent.right
                anchors.top: leRnm.bottom
                text: qsTr("ШФД")
                checked: fiscData.encriptionMode
                anchors.margins: 5
                enabled: !cbOffline.checked
                onReleased: {
                    fiscData.encriptionMode = cbEnc.checked
                }
            }

            TextField
            {
                id: leOfdName
                anchors.margins: 5
                anchors.left: parent.left
                anchors.right: parent.right
                anchors.top: cbAuto.bottom
                placeholderText: qsTr("Наименование ОФД")
                text: fiscData.ofdName
                maximumLength:  256
                enabled: !cbOffline.checked
                width: parent.width - anchors.margins * 2
                wrapMode: TextField.WordWrap
                onTextEdited: {
                    fiscData.ofdName = leOfdName.text
                }
                onPressed: {
//                        forceActiveFocus(Qt.MouseFocusReason)
//                        Qt.inputMethod.show()
                }

            }
            Label
            {
                id: lOfdInn
                text: qsTr("ИНН ОФД")
                anchors.left: parent.left
                anchors.verticalCenter: leOfdInn.verticalCenter
                anchors.margins: 5
                enabled: !cbOffline.checked
            }
            TextField
            {
                id:leOfdInn
                anchors.top: leOfdName.bottom
                anchors.margins: 5
                anchors.right: parent.right
                validator: InnValidator{}
                maximumLength: 10
                inputMethodHints: Qt.ImhDigitsOnly
                text: fiscData.ofdInn
                enabled: !cbOffline.checked
                color: acceptableInput ? "black" : "red"
                onTextEdited: {
                    fiscData.ofdInn = leOfdInn.text
                }
                onPressed: {
//                        forceActiveFocus(Qt.MouseFocusReason)
//                        Qt.inputMethod.show()
                }

            }
            Label {
                id: lEmail
                anchors.left: parent.left
                anchors.margins: 5
                text: qsTr("Эл. почта")
                anchors.verticalCenter:  leEmail.verticalCenter
                enabled: !cbOffline.checked
            }
            TextField {
                id: leEmail
                anchors.margins: 5
                anchors.top: leOfdInn.bottom
                anchors.left: lEmail.right
                anchors.right: parent.right
                inputMethodHints: Qt.ImhEmailCharactersOnly
                text: fiscData.email
                maximumLength:  64
                enabled: !cbOffline.checked
                onTextEdited: {
                    fiscData.email = leEmail.text
                }
                onPressed: {
//                        forceActiveFocus(Qt.MouseFocusReason)
//                        Qt.inputMethod.show()
                }

            }
            Label {
                id: lFns
                anchors.left: parent.left
                anchors.margins: 5
                text: qsTr("Сайт ФНС")
                enabled: !cbOffline.checked
                anchors.verticalCenter:  leFns.verticalCenter
            }
            TextField {
                id: leFns
                anchors.margins: 5
                anchors.top: leEmail.bottom
                anchors.left: lFns.right
                anchors.right: parent.right
                text: fiscData.ftsSite
                maximumLength:  256
                enabled: !cbOffline.checked
                onTextEdited: {
                    fiscData.ftsSite = leFns.text
                }
            }
            Label {
                id: lBoxId
                anchors.left: parent.left
                anchors.margins: 5
                text: qsTr("Автомат")
                anchors.verticalCenter:  leBoxId.verticalCenter
            }
            TextField {
                id: leBoxId
                anchors.margins: 5
                anchors.top: leFns.bottom
                anchors.left: lBoxId.right
                anchors.right: parent.right
                maximumLength: 20
                text: fiscData.boxId
                enabled: cbAuto.checked
                onTextEdited: {
                    fiscData.boxId = leBoxId.text
                }
                onPressed: {
//                        forceActiveFocus(Qt.MouseFocusReason)
//                        Qt.inputMethod.show()
                }

            }
            Label {
                id: lCashier
                anchors.left: parent.left
                anchors.margins: 5
                text: qsTr("Кассир")
                anchors.verticalCenter:  leCashier.verticalCenter
            }
            TextField {
                id: leCashier
                anchors.margins: 5
                anchors.top: leBoxId.bottom
                anchors.left: lCashier.right
                anchors.right: parent.right
                text: fiscData.cashier
                maximumLength: 64
                enabled: !cbAuto.checked
                onTextEdited: {
                    fiscData.cashier = leCashier.text
                }
                onPressed: {
//                        forceActiveFocus(Qt.MouseFocusReason)
//                        Qt.inputMethod.show()
                }

            }
            Label {
                id: lCashierInn
                anchors.left: parent.left
                anchors.margins: 5
                text: qsTr("ИНН Кассира")
                anchors.verticalCenter:  leCashierInn.verticalCenter
            }
            TextField {
                id: leCashierInn
                anchors.margins: 5
                anchors.top: leCashier.bottom
                anchors.right: parent.right
                text: fiscData.cashierInn
                maximumLength: 12
                enabled: !cbAuto.checked
                onTextEdited: {
                    fiscData.cashierInn = leCashierInn.text
                }
                onPressed: {
//                        forceActiveFocus(Qt.MouseFocusReason)
//                        Qt.inputMethod.show()
                }

            }
            GroupBox {

                id: gbTaxes
                anchors.margins: 5
                anchors.left: parent.left
                anchors.right: parent.right
                anchors.top: leCashierInn.bottom
                title: qsTr("Системы налогообложения")
                height:  anchors.margins * 6  + cbBasic.height * 3
                contentWidth: width
                CheckBox {
                    id: cbBasic
                    text: "ОСН"
                    checked: fiscData.allowBasic
                    anchors.left:  parent.left
                    anchors.top: parent.top
                    anchors.bottomMargin: 5
                    onReleased: {
                        fiscData.allowBasic = cbBasic.checked
                    }
                }
                CheckBox {
                    id: cbSimpleProfit
                    text: "УСН доход"
                    checked: fiscData.allowSimpleProfit
                    anchors.left:  parent.horizontalCenter
                    anchors.top: parent.top
                    anchors.bottomMargin: 5
                    onReleased: {
                        fiscData.allowSimpleProfit= cbSimpleProfit.checked
                    }
                }
                CheckBox {
                    id: cbSimpleProfitExpense
                    text: "УСН дох.- расх."
                    checked: fiscData.allowSimpleProfitExpense
                    anchors.left:  parent.left
                    anchors.top: cbBasic.bottom
                    anchors.bottomMargin: 5
                    onReleased: {
                        fiscData.allowSimpleProfitExpense = cbSimpleProfitExpense.checked
                    }
                }
                CheckBox {
                    id: cbSingleOnImputedProfit
                    text: "ЕНВД"
                    checked: fiscData.allowSingleOnImputedProfit
                    anchors.left:  parent.horizontalCenter
                    anchors.top: cbBasic.bottom
                    anchors.bottomMargin: 5
                    enabled: false
                    onReleased: {
                        fiscData.allowSingleOnImputedProfit = cbSingleOnImputedProfit.checked
                    }
                }
                CheckBox {
                    id: cbSingleAgriculturalTax
                    text: "ЕСХН"
                    checked: fiscData.allowSingleAgriculturalTax
                    anchors.left:  parent.left
                    anchors.top: cbSimpleProfitExpense.bottom
                    anchors.bottomMargin: 5
                    onReleased: {
                        fiscData.allowSingleAgriculturalTax = cbSingleAgriculturalTax.checked
                    }
                }
                CheckBox {
                    id: cbPatentSystemOfTaxation
                    text: "ПАТЕНТ"
                    checked: fiscData.allowPatentSystemOfTaxation
                    anchors.left:  parent.horizontalCenter
                    anchors.top: cbSimpleProfitExpense.bottom
                    anchors.bottomMargin: 5
                    onReleased: {
                        fiscData.allowPatentSystemOfTaxation = cbPatentSystemOfTaxation.checked
                    }
                }
            }
            GroupBox {

                id: gbExtFlags
                anchors.margins: 5
                anchors.left: parent.left
                anchors.right: parent.right
                anchors.top: gbTaxes.bottom
                title: qsTr("Условия применения ККТ")
                height:  anchors.margins * 12  + cbExternalPrinter.height * 13
                contentWidth: width
                CheckBox {
                    id: cbExternalPrinter
                    text: "ПРИНТЕР В АВТОМАТЕ"
                    checked: fiscData.allowExternalPrinter
                    anchors.left:  parent.left
                    anchors.top: parent.top
                    anchors.bottomMargin: 5
                    onReleased: {
                        fiscData.allowExternalPrinter = cbExternalPrinter.checked
                    }
                }
                CheckBox {
                    id: cbFosaCashbox
                    text: "АС БСО"
                    checked: fiscData.allowFosaCashbox
                    anchors.left:  parent.left
                    anchors.top: cbExternalPrinter.bottom
                    anchors.bottomMargin: 5
                    onReleased: {
                        fiscData.allowFosaCashbox = cbFosaCashbox.checked
                    }
                }
                CheckBox {
                    id: cbInternetOnly
                    text: "ККТ ДЛЯ ИНТЕРНЕТ"
                    checked: fiscData.allowInternetOnly
                    anchors.left:  parent.left
                    anchors.top: cbFosaCashbox.bottom
                    anchors.bottomMargin: 5
                    onReleased: {
                        fiscData.allowInternetOnly = cbInternetOnly.checked
                    }
                }
                CheckBox {
                    id: cbExcisableProducts
                    text: "ПОДАКЦИЗНЫЕ ТОВАРЫ"
                    checked: fiscData.allowExcisableProducts
                    anchors.left:  parent.left
                    anchors.top: cbInternetOnly.bottom
                    anchors.bottomMargin: 5
                    onReleased: {
                        fiscData.allowExcisableProducts = cbExcisableProducts.checked
                    }
                }
                CheckBox {
                    id: cbLabledProducts
                    text: "ТМТ"
                    checked: fiscData.allowLabledProducts
                    anchors.left:  parent.left
                    anchors.top: cbExcisableProducts.bottom
                    anchors.bottomMargin: 5
                    onReleased: {
                        fiscData.allowLabledProducts = cbLabledProducts.checked
                    }
                }
                CheckBox {
                    id: cbServicesOnly
                    text: "ККТ ДЛЯ УСЛУГ"
                    checked: fiscData.allowServicesOnly
                    anchors.left:  parent.left
                    anchors.top: cbLabledProducts.bottom
                    anchors.bottomMargin: 5
                    enabled: !fiscData.allowFosaCashbox
                    onReleased: {
                        fiscData.allowServicesOnly = cbServicesOnly.checked
                    }
                }
                CheckBox {
                    id: cbGamesCashbox
                    text: "ПРОВЕДЕНИЕ АЗАРТНОЙ ИГРЫ"
                    checked: fiscData.allowGamesCashbox
                    anchors.left:  parent.left
                    anchors.top: cbServicesOnly.bottom
                    anchors.bottomMargin: 5
                    onReleased: {
                        fiscData.allowGamesCashbox = cbGamesCashbox.checked
                    }
                }
                CheckBox {
                    id: cbLotteryCashbox
                    text: "ПРОВЕДЕНИЕ ЛОТЕРЕИ"
                    checked: fiscData.allowLotteryCashbox
                    anchors.left:  parent.left
                    anchors.top: cbGamesCashbox.bottom
                    anchors.bottomMargin: 5
                    onReleased: {
                        fiscData.allowLotteryCashbox = cbLotteryCashbox.checked
                    }
                }
                CheckBox {
                    id: cbPawnshop
                    text: "ЛОМБАРД"
                    checked: fiscData.allowPawnshop
                    anchors.left:  parent.left
                    anchors.top: cbLotteryCashbox.bottom
                    anchors.bottomMargin: 5
                    onReleased: {
                        fiscData.allowPawnshop = cbPawnshop.checked
                    }
                }
                CheckBox {
                    id: cbInsurance
                    text: "СТРАХОВАНИЕ"
                    checked: fiscData.allowInsurance
                    anchors.left:  parent.left
                    anchors.top: cbPawnshop.bottom
                    anchors.bottomMargin: 5
                    onReleased: {
                        fiscData.allowInsurance = cbInsurance.checked
                    }
                }

                CheckBox {
                    id: cbAutoCashboxForLabled
                    text: "ККТ С ТОРГ. АВТОМАТОМ"
                    checked: fiscData.allowAutoCashboxForLabled
                    anchors.left:  parent.left
                    anchors.top: cbInsurance.bottom
                    anchors.bottomMargin: 5
                    onReleased: {
                        fiscData.allowAutoCashboxForLabled = cbAutoCashboxForLabled.checked
                    }
                }

                CheckBox {
                    id: cbRestaurant
                    text: "ККТ В ОБЩ. ПИТАНИИ"
                    checked: fiscData.allowRestaurant
                    anchors.left:  parent.left
                    anchors.top: cbAutoCashboxForLabled.bottom
                    anchors.bottomMargin: 5
                    onReleased: {
                        fiscData.allowRestaurant = cbRestaurant.checked
                    }
                }

                CheckBox {
                    id: cbWholesale
                    text: "ККТ В ОПТ. ТОРГОВЛЕ С ОРГ. И ИП"
                    checked: fiscData.allowWholesale
                    anchors.left:  parent.left
                    anchors.top: cbRestaurant.bottom
                    anchors.bottomMargin: 5
                    onReleased: {
                        fiscData.allowWholesale = cbWholesale.checked
                    }
                }
            }
            GroupBox {
                id: gbAdditionalParameter
                anchors.margins: 5
                anchors.left: parent.left
                anchors.right: parent.right
                anchors.top: gbExtFlags.bottom
                title: qsTr("Дополнительный реквизит ОР")
                height:  anchors.margins * 2  + 1.5 * (leAdditionalParam.height  + leAdditionalParamVal.height)
                contentWidth: width
                TextField {
                    id: leAdditionalParam
                    anchors.margins: 5
                    anchors.left: parent.left
                    anchors.right: parent.right
                    anchors.top: parent.top
                    placeholderText: qsTr("Дополнительный реквизит")
                    text: fiscData.additionalParam
                    maximumLength:  32
                    onTextEdited: {
                        fiscData.additionalParam = leAdditionalParam.text
                    }
                    onPressed: {
//                            forceActiveFocus(Qt.MouseFocusReason)
//                            Qt.inputMethod.show()
                    }

                }
                TextField {
                    id: leAdditionalParamVal
                    anchors.margins: 5
                    anchors.left: parent.left
                    anchors.right: parent.right
                    anchors.top: leAdditionalParam.bottom
                    placeholderText: qsTr("Значение дополнительного реквизита")
                    text: fiscData.additionalParamData
                    maximumLength:  64
                    validator: HexStringValidator{
                        maxSize: 64
                        canBeEmpty: true
                        fixedSize: false
                    }
                    onTextEdited: {
                        if(leAdditionalParamVal.acceptableInput) {
                            fiscData.additionalParamData = leAdditionalParamVal.text
                        }
                    }
                    onPressed: {
//                            forceActiveFocus(Qt.MouseFocusReason)
//                            Qt.inputMethod.show()
                    }
                }
            }

            GroupBox {

                id: gbReasons
                anchors.margins: 5
                anchors.left: parent.left
                anchors.right: parent.right
                anchors.top: gbAdditionalParameter.bottom
                title: qsTr("Причины перерегистрации")
                height:  anchors.margins * 40  + cbOfdChange.height * 21
                contentWidth: width

                CheckBox {
                    id: cbFsChange
                    text: "Замена ФН"
                    checked: fiscData.reasonFsChange
                    anchors.left:  parent.left
                    anchors.top: parent.top
                    anchors.bottomMargin: 2
                    enabled: false
                    onReleased: {
                        fiscData.reasonFsChange = cbFsChange.checked
                    }
                }
                CheckBox {
                    id: cbOfdChange
                    text: "Замена ОФД"
                    checked: fiscData.reasonOfdChange
                    anchors.left:  parent.left
                    anchors.top: cbFsChange.bottom
                    anchors.bottomMargin: 2
                    onReleased: {
                        fiscData.reasonOfdChange= cbOfdChange.checked
                    }
                }
                CheckBox {
                    id: cbUserNameChange
                    text: "Изменение наименования пользователя"
                    checked: fiscData.reasonUserNameChange
                    anchors.left:  parent.left
                    anchors.top: cbOfdChange.bottom
                    anchors.bottomMargin: 2
                    width: parent.width
                    contentItem: Label {
                        text: cbUserNameChange.text
                        font: cbUserNameChange.font
                        horizontalAlignment: Text.AlignLeft
                        verticalAlignment: Text.AlignVCenter
                        leftPadding: cbUserNameChange.indicator.width + cbUserNameChange.spacing
                        wrapMode: Label.WordWrap
                    }
                    onReleased: {
                        fiscData.reasonUserNameChange = cbUserNameChange.checked
                    }
                }
                CheckBox {
                    id: cbAddressChange
                    text: "Изменения адреса/места расчетов"
                    checked: fiscData.reasonAddressChange
                    anchors.left:  parent.left
                    anchors.top: cbUserNameChange.bottom
                    anchors.bottomMargin: 2
                    onReleased: {
                        fiscData.reasonAddressChange = cbAddressChange.checked
                    }
                }
                CheckBox {
                    id: cbOfflineToOnline
                    text: "Из автономного режима в режим передачи"
                    checked: fiscData.reasonOfflineToOnline
                    anchors.left:  parent.left
                    anchors.top: cbAddressChange.bottom
                    anchors.bottomMargin: 2
                    width: parent.width
                    contentItem: Label {
                        text: cbOfflineToOnline.text
                        font: cbOfflineToOnline.font
                        horizontalAlignment: Text.AlignLeft
                        verticalAlignment: Text.AlignVCenter
                        leftPadding: cbOfflineToOnline.indicator.width + cbOfflineToOnline.spacing
                        wrapMode: Label.WordWrap
                    }
                    onReleased: {
                        fiscData.reasonOfflineToOnline = cbOfflineToOnline.checked
                    }
                }
                CheckBox {
                    id: cbOnlineToOffline
                    text: "Из режима передачи в автономный"
                    checked: fiscData.reasonOnlineToOffline
                    anchors.left:  parent.left
                    anchors.top: cbOfflineToOnline.bottom
                    anchors.bottomMargin: 2
                    onReleased: {
                        fiscData.reasonOnlineToOffline = cbOnlineToOffline.checked
                    }
                }
                CheckBox {
                    id: cbCashboxVersionChange
                    text: "Изменение версии модели ККТ"
                    checked: fiscData.reasonCashboxVersionChange
                    anchors.left:  parent.left
                    anchors.top: cbOnlineToOffline.bottom
                    anchors.bottomMargin: 2
                    onReleased: {
                        fiscData.reasonCashboxVersionChange = cbCashboxVersionChange.checked
                    }
                }
                CheckBox {
                    id: cbTaxesChange
                    text: "Изменение налоговых систем"
                    checked: fiscData.reasonTaxesChange
                    anchors.left:  parent.left
                    anchors.top: cbCashboxVersionChange.bottom
                    anchors.bottomMargin: 2
                    onReleased: {
                        fiscData.reasonTaxesChange = cbTaxesChange.checked
                    }
                }
                CheckBox {
                    id: cbBoxIdChange
                    text: "Изменение ИД автомата"
                    checked: fiscData.reasonBoxIdChange
                    anchors.left:  parent.left
                    anchors.top: cbTaxesChange.bottom
                    anchors.bottomMargin: 2
                    onReleased: {
                        fiscData.reasonBoxIdChange = cbBoxIdChange.checked
                    }
                }
                CheckBox {
                    id: cbAutoToSingle
                    text: "Из автоматического режима в неавтоматический"
                    checked: fiscData.reasonAutoToSingle
                    anchors.left:  parent.left
                    anchors.top: cbBoxIdChange.bottom
                    anchors.bottomMargin: 2
                    width: parent.width
                    contentItem: Label {
                        text: cbAutoToSingle.text
                        font: cbAutoToSingle.font
                        horizontalAlignment: Text.AlignLeft
                        verticalAlignment: Text.AlignVCenter
                        leftPadding: cbAutoToSingle.indicator.width + cbAutoToSingle.spacing
                        wrapMode: Label.WordWrap
                    }
                    onReleased: {
                        fiscData.reasonAutoToSingle = cbAutoToSingle.checked
                    }
                }
                CheckBox {
                    id: cbSingleToAuto
                    text: "Из неавтоматического режима в автоматический"
                    checked: fiscData.reasonFsChange
                    anchors.left:  parent.left
                    anchors.top: cbAutoToSingle.bottom
                    anchors.bottomMargin: 2
                    width: parent.width
                    contentItem: Label {
                        text: cbSingleToAuto.text
                        font: cbSingleToAuto.font
                        horizontalAlignment: Text.AlignLeft
                        verticalAlignment: Text.AlignVCenter
                        leftPadding: cbSingleToAuto.indicator.width + cbSingleToAuto.spacing
                        wrapMode: Label.WordWrap
                    }
                    onReleased: {
                        fiscData.reasonSingleToAuto = cbSingleToAuto.checked
                    }
                }
                CheckBox {
                    id: cbBasicToFosa
                    text: "В режим БСО"
                    checked: fiscData.reasonBasicToFosa
                    anchors.left:  parent.left
                    anchors.top: cbSingleToAuto.bottom
                    anchors.bottomMargin: 2
                    onPressed: {
                        fiscData.reasonBasicToFosa = cbBasicToFosa.checked
                    }
                }
                CheckBox {
                    id: cbFosaToBasic
                    text: "Из режима БСО в режим без БСО"
                    checked: fiscData.reasonFosaToBasic
                    anchors.left:  parent.left
                    anchors.top: cbBasicToFosa.bottom
                    anchors.bottomMargin: 2
                    onReleased: {
                        fiscData.reasonFosaToBasic = cbFosaToBasic.checked
                    }
                }
                CheckBox {
                    id: cbInternetToBasic
                    text: "Перевод в режим для расчетов в Интернет"
                    checked: fiscData.reasonInternetToBasic
                    anchors.left:  parent.left
                    anchors.top: cbFosaToBasic.bottom
                    anchors.bottomMargin: 2
                    width: parent.width
                    contentItem: Label {
                        text: cbInternetToBasic.text
                        font: cbInternetToBasic.font
                        horizontalAlignment: Text.AlignLeft
                        verticalAlignment: Text.AlignVCenter
                        leftPadding: cbInternetToBasic.indicator.width + cbInternetToBasic.spacing
                        wrapMode: Label.WordWrap
                    }
                    onReleased: {
                        fiscData.reasonInternetToBasic = cbInternetToBasic.checked
                    }
                }
                CheckBox {
                    id: cbBasicToInternet
                    text: "Перевод из режима для расчетов в Интернет"
                    checked: fiscData.reasonBasicToInternet
                    anchors.left:  parent.left
                    anchors.top: cbInternetToBasic.bottom
                    anchors.bottomMargin: 2
                    width: parent.width
                    contentItem: Label {
                        text: cbBasicToInternet.text
                        font: cbBasicToInternet.font
                        horizontalAlignment: Text.AlignLeft
                        verticalAlignment: Text.AlignVCenter
                        leftPadding: cbBasicToInternet.indicator.width + cbBasicToInternet.spacing
                        wrapMode: Label.WordWrap
                    }
                    onReleased: {
                        fiscData.reasonBasicToInternet = cbBasicToInternet.checked
                    }
                }
                CheckBox {
                    id: cbGamesToBasic
                    text: "Разрешить применение для азартных игр"
                    checked: fiscData.reasonGamesToBasic
                    anchors.left:  parent.left
                    anchors.top: cbBasicToInternet.bottom
                    anchors.bottomMargin: 2
                    width: parent.width
                    contentItem: Label {
                        text: cbGamesToBasic.text
                        font: cbGamesToBasic.font
                        horizontalAlignment: Text.AlignLeft
                        verticalAlignment: Text.AlignVCenter
                        leftPadding: cbGamesToBasic.indicator.width + cbGamesToBasic.spacing
                        wrapMode: Label.WordWrap
                    }
                    onReleased: {
                        fiscData.reasonGamesToBasic = cbGamesToBasic.checked
                    }
                }
                CheckBox {
                    id: cbBasicToGames
                    text: "Запретить применение для азартных игр"
                    checked: fiscData.reasonBasicToGames
                    anchors.left:  parent.left
                    anchors.top: cbGamesToBasic.bottom
                    anchors.bottomMargin: 2
                    width: parent.width
                    contentItem: Label {
                        text: cbBasicToGames.text
                        font: cbBasicToGames.font
                        horizontalAlignment: Text.AlignLeft
                        verticalAlignment: Text.AlignVCenter
                        leftPadding: cbBasicToGames.indicator.width + cbBasicToGames.spacing
                        wrapMode: Label.WordWrap
                    }
                    onReleased: {
                        fiscData.reasonBasicToGames = cbBasicToGames.checked
                    }
                }
                CheckBox {
                    id: cbLotteryToBasic
                    text: "Разрешить применение для лотереи"
                    checked: fiscData.reasonLotteryToBasic
                    anchors.left:  parent.left
                    anchors.top: cbBasicToGames.bottom
                    anchors.bottomMargin: 2
                    onReleased: {
                        fiscData.reasonLotteryToBasic = cbLotteryToBasic.checked
                    }
                }
                CheckBox {
                    id: cbBasicToLottery
                    text: "Запретить применение для лотереии"
                    checked: fiscData.reasonBasicToLottery
                    anchors.left:  parent.left
                    anchors.top: cbLotteryToBasic.bottom
                    anchors.bottomMargin: 2
                    onReleased: {
                        fiscData.reasonBasicToLottery = cbBasicToLottery.checked
                    }
                }
                CheckBox {
                    id: cbFFDChange
                    text: "Изменение ФФД ККТ"
                    checked: fiscData.reasonFFDChange
                    anchors.left:  parent.left
                    anchors.top: cbBasicToLottery.bottom
                    anchors.bottomMargin: 2
//                    enabled: false
                    onReleased: {
                        fiscData.reasonFFDChange = cbFFDChange.checked
                    }
                }
                CheckBox {
                    id: cbAnotherReasons
                    text: "Иные причины"
                    checked: fiscData.reasonAnotherReasons
                    anchors.left:  parent.left
                    anchors.top: cbFFDChange.bottom
                    anchors.bottomMargin: 2
                    onReleased: {
                        fiscData.reasonAnotherReasons = cbAnotherReasons.checked
                    }
                }
            }
        }

    }




    Component.onCompleted: {
    }
}
