import QtQuick 2.12
import QtQuick.Controls 2.12
import Qt.labs.platform 1.1

import ru.armax.cashbox 1.0

//import ru.armax.cashbox.cashboxstatus 1.0
//import ru.armax.cashbox.hexstringvalidator 1.0
//import ru.armax.cashbox.reportscorehandler 1.0

Page {
    width: parent.width
    height: parent.height
    id: pgFsArchiveReport
    title: qsTr("Документы из архива ФН")
    signal closed()
    objectName: "pgFsArchiveReport"

    property alias ownFooter: footer

    ToolBar {
        id: footer
        height: 70
        ToolButton {
            id: tbBack
            width: 64
            height: 64
            anchors.left: parent.left
            anchors.verticalCenter: parent.verticalCenter
            anchors.leftMargin: 10
            anchors.rightMargin: 10
            anchors.topMargin: 5
            anchors.bottomMargin: 5
            icon.source: "qrc:/icons/back_64.png"
            icon.color: "transparent"
            icon.width: 60
            icon.height: 60
            enabled: cbStatus.fiscalized
            onClicked: {
                pgFsArchiveReport.closed()
            }
        }
        ToolButton {
            id: tbPrintReport
            width: 64
            height: 64
            anchors.right: parent.right
            anchors.verticalCenter: parent.verticalCenter
            anchors.leftMargin: 10
            anchors.rightMargin: 10
            anchors.topMargin: 5
            anchors.bottomMargin: 5
            icon.source: "qrc:/icons/print_64.png"
            icon.color: "transparent"
            icon.width: 64
            icon.height: 64
            onClicked: {
                    switch(cbReportType.currentIndex) {
                case 0: handler.findDoc(sbParam.value); break;
                case 1: handler.ofdTicket(sbParam.value); break;
                case 2: handler.tlvDoc(sbParam.value); break;
                case 3: handler.regDocFromFs(sbParam.value); break;
                case 4: handler.regTlvDocFromFs(sbParam.value); break;
                default: break;
                }
            }
        }
    }
    CashboxStatus {
        id: cbStatus
    }

    ReportsCoreHandler {
        id: handler
        onShowBusy: function(busy) {
            busyIndicator.running = busy
            pgFsArchiveReport.enabled = !busy
            window.enableAll(!busy)
        }
        onError: function(txt) {
            msgError.text = txt
            busyIndicator.running = false
            pgFsArchiveReport.enabled = true
            window.enableAll(true)
            msgError.open()
        }
        onReportIsReady: {
            pgFsArchiveReport.closed()
        }
    }
    BusyIndicator {
        id: busyIndicator
        anchors.horizontalCenter: parent.horizontalCenter
        anchors.verticalCenter: parent.verticalCenter
        running: false
        z: 10
    }
    MessageDialog {
        id: msgError
        buttons: MessageDialog.Close
        text: ""
    }

    ComboBox {
        id: cbReportType
        anchors.margins: 10
        anchors.top: parent.top
        anchors.left: parent.left
        anchors.right: parent.right
        textRole: 'name'
        model: ListModel {
            ListElement {name:"Документ по номеру" }
            ListElement {name:"Квитанция по номеру ФД" }
            ListElement {name:"Документ в TLV формате" }
            ListElement {name:"Итоги регистрации/перерегистрации" }
            ListElement {name:"Tlv документа регистрации/перерегистрации" }
        }
        onCurrentIndexChanged: {
            if(cbReportType.currentIndex > 2) {
                lParam.text = "Порядковый номер"
                sbParam.from = 0
                sbParam.to = cbStatus.regsCount
            }
            else {
                lParam.text = "ФД"
                sbParam.from = 1
                sbParam.to = cbStatus.lastDocNumber
            }
        }
    }
    Label {
        id: lParam
        text: "ФД"
        anchors.left: parent.left
        anchors.margins: 10
        anchors.verticalCenter: sbParam.verticalCenter
    }
    SpinBox {
        id:  sbParam
        anchors.margins: 10
        anchors.top:  cbReportType.bottom
        anchors.right: parent.right
        editable: true
    }

    Component.onCompleted: {


    }
}
