import QtQuick 2.12
import QtQuick.Controls 2.12
import Qt.labs.platform 1.1

import ru.armax.cashbox 1.0

//import ru.armax.cashbox.cashboxstatus 1.0
//import ru.armax.cashbox.approot 1.0

Page {
    width: parent.width
    height: parent.height
    signal closed()
    title: qsTr("Статистика по ФН")
    id: fsReportPage
    objectName: "fsReportPage"
    property alias ownFooter: footer


    CashboxStatus {
        id: cbStatus
    }

    ReportsCoreHandler {
        id: handler
        onShowBusy: function(busy) {
            busyIndicator.running = busy
            fsReportPage.enabled = !busy
            window.enableAll(!busy)
        }
        onError: function(txt) {
            msgError.text = txt
            busyIndicator.running = false
            fsReportPage.enabled = true
            window.enableAll(true)
            msgError.open()
        }
        onXReportSuccess: function(txt) {
            busyIndicator.running = false
            fsReportPage.enabled = true
            window.enableAll(true)
            resultText.text = txt
        }
    }
    BusyIndicator {
        id: busyIndicator
        anchors.horizontalCenter: parent.horizontalCenter
        anchors.verticalCenter: parent.verticalCenter
        running: false
        z: 10
    }
    MessageDialog {
        id: msgError
        buttons: MessageDialog.Close
        text: ""
    }
    ScrollView {
        id: infoScrollView
        anchors.fill: parent
        ScrollBar.vertical.policy: ScrollBar.AlwaysOff
        ScrollBar.horizontal.policy: ScrollBar.AlwaysOff
        z:1
        TextArea {
            id: resultText
            anchors.fill: parent
            width: fsReportPage.width * 0.9
            text: ""
            wrapMode: TextEdit.WordWrap
            textMargin: 10
            textFormat: TextEdit.RichText
            readOnly: true
            z:2
        }
    }
    ToolBar {
        id: footer
        height: 70
        ToolButton {
            id: tbBack
            width: 64
            height: 64
            anchors.left: parent.left
            anchors.verticalCenter: parent.verticalCenter
            anchors.leftMargin: 10
            anchors.rightMargin: 10
            anchors.topMargin: 5
            anchors.bottomMargin: 5
            icon.source: "qrc:/icons/home_64.png"
            icon.color: "transparent"
            icon.width: 60
            icon.height: 60
            onClicked: {
                fsReportPage.closed()
            }
        }
        ToolButton {
            id: tbPrint
            width: 64
            height: 64
            anchors.right: parent.right
            anchors.verticalCenter: parent.verticalCenter
            anchors.leftMargin: 10
            anchors.rightMargin: 10
            anchors.topMargin: 5
            anchors.bottomMargin: 5
            icon.source: "qrc:/icons/print_64.png"
            icon.color: "transparent"
            icon.width: 64
            icon.height: 64
            onClicked: {
                handler.fsReport()
            }
        }
    }
    Component.onCompleted: {
        width = parent.width
        height = parent.height
    }
}
