import QtQuick 2.12
import QtQuick.Controls 2.15

import Qt.labs.platform 1.1 as QtLabs

import ru.armax.cashbox 1.0

//import ru.armax.cashbox.cashboxstatus 1.0
//import ru.armax.cashbox.approot 1.0

Page {
    width: parent.width
    height: parent.height
    signal closed()
    title: qsTr("Тех. обнуление")
    id: fullClearPage
    objectName: "fullClearPage"
    property alias ownFooter: footer

    property string firmwareFileName: ""


    AppRoot {
        id: appRoot
        onShowBusy: function(busy) {
            busyIndicator.running = busy
            fullClearPage.enabled = !busy
        }
        onError: function(txt) {
            msgError.text = txt
            busyIndicator.running = false
            fullClearPage.enabled = true
            msgError.open()
        }

        onSuccessTotalClean: {
            appRoot.restart()
        }
    }
    BusyIndicator {
        id: busyIndicator
        anchors.horizontalCenter: parent.horizontalCenter
        anchors.verticalCenter: parent.verticalCenter
        running: false
        z: 10
    }
    QtLabs.MessageDialog {
        id: msgError
        buttons: QtLabs.MessageDialog.Close
        text: ""
    }

    QtLabs.MessageDialog {
        id: msgSuccess
        buttons: QtLabs.MessageDialog.Ok
        text: ""
        onOkClicked: {
            fullClearPage.closed()
        }
    }
    Item {
        id: loginItem
        anchors.left: parent.left
        anchors.right: parent.right
        anchors.margins: 10
        anchors.top: parent.top
        height: contentHeight
        width: parent.width
        TextField {
            id: leLogin
            width: parent.width
            height: 80
            inputMethodHints:  Qt.ImhDialableCharactersOnly
            z: 100
            font.pixelSize: 24
            padding: 10
            anchors.margins: 20
            anchors.top: parent.top
            horizontalAlignment: TextInput.AlignHCenter
            placeholderText: qsTr("Имя пользователя")
            onPressed: {
                forceActiveFocus(Qt.MouseFocusReason)
                Qt.inputMethod.show()
            }
        }
        TextField {
            id: lePassword
            anchors.top: leLogin.bottom
            width: parent.width
            horizontalAlignment: TextInput.AlignHCenter
            padding: 10
            z: 100
            font.pixelSize: 24
            echoMode: TextInput.Password
            anchors.margins: 20
            placeholderText: qsTr("Пароль")
            onPressed: {
                forceActiveFocus(Qt.MouseFocusReason)
                Qt.inputMethod.show()
            }
        }

    }
    Label {
        anchors.centerIn: parent
        width: parent.width - 20
        horizontalAlignment: Label.AlignHCenter
        verticalAlignment: Label.AlignVCenter
        color: "red"
        wrapMode: Label.WordWrap
        text: "Внимание!\nПосле технологического обнуления заводской номер кассы, регистрационные данные, локальные базы безвовзвратно исчезнут!\n"
    }

    ToolBar {
        id: footer
        height: 70
        ToolButton {
            id: tbBack
            width: 64
            height: 64
            anchors.left: parent.left
            anchors.verticalCenter: parent.verticalCenter
            anchors.leftMargin: 10
            anchors.rightMargin: 10
            anchors.topMargin: 5
            anchors.bottomMargin: 5
            icon.source: "qrc:/icons/home_64.png"
            icon.color: "transparent"
            icon.width: 60
            icon.height: 60
            onClicked: {
                fullClearPage.closed()
            }
        }
        ToolButton {
            id: tbCheck
            width: 64
            height: 64
            anchors.right: parent.right
            anchors.verticalCenter: parent.verticalCenter
            anchors.leftMargin: 10
            anchors.rightMargin: 10
            anchors.topMargin: 5
            anchors.bottomMargin: 5
            icon.source: "qrc:/icons/total_reset_64.png"
            icon.color: "transparent"
            icon.width: 64
            icon.height: 64
            onClicked: {
                appRoot.totalClean(leLogin.text, lePassword.text)
            }
        }
    }
    Component.onCompleted: {
        width = parent.width
        height = parent.height
    }
}
