import QtQuick 2.12
import QtQuick.Controls 2.12
import Qt.labs.platform 1.1 as QtLabs

import ru.armax.cashbox 1.0

//import ru.armax.cashbox.industrypropsmodel 1.0
//import ru.armax.cashbox.industrypropertywrapper 1.0

Page {
    width: parent.width
    height: parent.height
    signal closed()
    title: qsTr("Отраслевой реквизит")
    id: industryPropsPage
    objectName: "industryPropsPage"
    property alias ownFooter: footer
    property alias owmModel: ipModel

    BusyIndicator {
        id: busyIndicator
        anchors.horizontalCenter: parent.horizontalCenter
        anchors.verticalCenter: parent.verticalCenter
        running: false
        z: 10
    }
    QtLabs.MessageDialog {
        id: msgError
        buttons: QtLabs.MessageDialog.Close
        text: ""
    }
    IndustryPropsModel {
        id: ipModel
        onError: function(msg) {
            msgError.text = msg
            msgError.open()
        }
        onFinish: {
            industryPropsPage.closed()
        }
    }
    IndustryPropDialog {
        id: dlgEditProp
        width: industryPropsPage.width
        height: 420
        onAccepted: {
            if(ownProp.isValid) {
                console.log(curIdx.toString())
                ipModel.setItem(dlgEditProp.curIdx, dlgEditProp.editVal)
            }
        }
    }

    ScrollView {
        id: receiptScrollView
        anchors.fill: parent
        anchors.margins: 0
        ScrollBar.vertical.policy: ScrollBar.AlwaysOff
        ScrollBar.horizontal.policy: ScrollBar.AlwaysOff
        width: industryPropsPage.width
        height: industryPropsPage.height
        contentHeight: industryPropsPage.height * 6
        contentWidth: parent.width - anchors.margins

        ListView {
            id: lvProps
            spacing: 10
            interactive: true
            orientation: Qt.Vertical
            highlightFollowsCurrentItem: true
            model: ipModel
            z: 2
            delegate: Item {
                height: 80
                width: parent.width
                Label {
                    id: lFedId
                    text: fedId
                    anchors.left: parent.left
                    anchors.top: parent.top
                    anchors.margins: 10
                    verticalAlignment: Label.AlignVCenter
                    wrapMode: Label.WordWrap
                    z: 4
                }
                Label {
                    id: lDocDt
                    text: docDate
                    anchors.right: parent.right
                    anchors.top: parent.top
                    anchors.margins: 10
                    verticalAlignment: Label.AlignVCenter
                    z: 4
                }
                Label {
                    id: lDocNum
                    text: docNumber
                    anchors.left: parent.left
                    anchors.top: lDocDt.bottom
                    anchors.right: parent.right
                    anchors.margins: 10
                    verticalAlignment: Label.AlignVCenter
                    horizontalAlignment: Label.AlignHCenter
                    wrapMode: Label.WordWrap
                    z: 4
                }
                Label {
                    id: lVal
                    text: propValue
                    anchors.left: parent.left
                    anchors.top: lDocNum.bottom
                    anchors.bottom: parent.bottom
                    anchors.margins: 10
                    verticalAlignment: Label.AlignVCenter
                    z: 4
                }


                MouseArea {
                    anchors.fill: parent
                    onClicked: {
                        lvProps.currentIndex = index
                    }
                    onPressAndHold: {
                        lvProps.currentIndex = index
                        menuOp.y = mouse.y
                        if(menuOp.y + menuOp.heigh > receiptScrollView.height) {
                            menuOp.y = menuOp.y - menuOp.heigh
                        }

                        menuOp.open()
                    }
                }
                Rectangle{
                    id: hl
                    anchors.fill: parent
                    color: lvProps.currentIndex === index ? "lightsteelblue" : "white"
                }
                Menu {
                    id: menuOp

                    MenuItem {
                        text: qsTr("Редактировать")
                        onTriggered: {
                            dlgEditProp.editVal = ipModel.item(lvProps.currentIndex)
                            dlgEditProp.curIdx = lvProps.currentIndex
                            dlgEditProp.open()
                        }
                    }
                    MenuItem {
                        text: qsTr("Удалить")
                        onTriggered: {
                            ipModel.removeItem(lvProps.currentIndex)
                        }
                    }
                }
            }
        }
    }
    RoundButton {
        id: rbAddProp
        anchors.margins: 10
        anchors.bottom: parent.bottom
        anchors.right: parent.right
        icon.source: "qrc:/icons/add_prop_48.png"
        icon.color: "transparent"
        icon.width: 50
        icon.height: 50
        z:100
        onPressed: {
            dlgEditProp.editVal = ipModel.item(-1)
            dlgEditProp.curIdx = -1
            dlgEditProp.open()
        }
    }
    ToolBar {
        id: footer
        height: 70
        ToolButton {
            id: tbBack
            width: 64
            height: 64
            anchors.left: parent.left
            anchors.verticalCenter: parent.verticalCenter
            anchors.leftMargin: 10
            anchors.rightMargin: 10
            anchors.topMargin: 5
            anchors.bottomMargin: 5
            icon.source: "qrc:/icons/back_64.png"
            icon.color: "transparent"
            icon.width: 60
            icon.height: 60
            onClicked: {
                industryPropsPage.closed()
            }
        }
        ToolButton {
            id: tbReset
            width: 64
            height: 64
            anchors.horizontalCenter: parent.horizontalCenter
            anchors.verticalCenter: parent.verticalCenter
            anchors.leftMargin: 10
            anchors.rightMargin: 10
            anchors.topMargin: 5
            anchors.bottomMargin: 5
            icon.source: "qrc:/icons/reset_64.png"
            icon.color: "transparent"
            icon.width: 64
            icon.height: 64
            onClicked: {
                resetData()
            }
        }
        ToolButton {
            id: tbSave
            width: 64
            height: 64
            anchors.right: parent.right
            anchors.verticalCenter: parent.verticalCenter
            anchors.leftMargin: 10
            anchors.rightMargin: 10
            anchors.topMargin: 5
            anchors.bottomMargin: 5
            icon.source: "qrc:/icons/apply_64.png"
            icon.color: "transparent"
            icon.width: 64
            icon.height: 64
            onClicked: {
                commit()
            }
        }
    }


    function resetData() {
        ipModel.clean()
    }
    function commit() {
        ipModel.save()
    }

    Component.onCompleted: {
        width = parent.width
        height = parent.height
    }
}
