import QtQuick 2.12
import QtQuick.Controls 2.12
import Qt.labs.platform 1.1

import ru.armax.cashbox 1.0
//import ru.armax.cashbox.sendmailhelper 1.0

Item {
    width: parent.width
    height: parent.height
    signal closed()
    objectName: "logsSettingsPage"
    id: logsSettingsPage
    property bool clearResult : false
    property string logSizeStr : ""

    property alias ownFooter: footer
    property bool isSending : false



    SendMailHelper{
        id : sendMailHelper
    }


    Text{
        id : lbLogsSize
        anchors.top : parent.top
        anchors.margins: 5
        width: parent.width
        wrapMode: Text.WordWrap
        horizontalAlignment: Text.AlignHCenter
    }

    Item {
        id: iSendData
        anchors.top:lbLogsSize.bottom
        anchors.left: parent.left
        anchors.right: parent.right
        anchors.bottom: parent.bottom
        TextArea{
            id : taComment
            placeholderText: "введите комментарий"
            anchors.margins: 10
            anchors.top: parent.top
            anchors.left: parent.left
            anchors.right: parent.right
        }


        Text{
            id: tSend
            text : "Выполняется отправка логов. Не закрывайте данное окно в ходе выполнения процесса!!!"
            horizontalAlignment: Text.AlignHCenter
            wrapMode: Text.WordWrap
            color : "red"
            visible : isSending
            anchors.left: parent.left
            anchors.right: parent.right
            anchors.top:  taComment.bottom
            anchors.leftMargin: 10
            anchors.rightMargin: 10
            anchors.topMargin: 30
            anchors.bottomMargin: 30
        }

        ProgressBar{
            id : sendProgress
            visible : isSending
            width: parent.width - 10
            anchors.left: parent.left
            anchors.right: parent.right
            anchors.top:  tSend.bottom
            anchors.margins: 10
        }
    }

    function reloadSettingsList(){

        logSizeStr = sendMailHelper.logsSize()
        lbLogsSize.text = "Текущий размер логов: "+ (logSizeStr.length > 0 ? logSizeStr : "нет данных")

    }

    MessageDialog{
        id : msgClearLogs
        visible: false
        buttons: StandardButton.Yes | StandardButton.No
        text : "Вы действительно хотите удалить все логи?"
        title : "Подтверждение"
        onYesClicked: {
            logClearResult.text = sendMailHelper.clearLogs() ? "Логи очищены" :
                                                          "При очистке логов возникла ошибка";
            logClearResult.visible = true;
            reloadSettingsList()
        }
    }

    MessageDialog{
        id : logClearResult
        title : "Инфо"
    }
    ToolBar {
        id: footer
        height: 70
        ToolButton {
            id: tbBack
            width: 64
            height: 64
            anchors.left: parent.left
            anchors.verticalCenter: parent.verticalCenter
            anchors.leftMargin: 10
            anchors.rightMargin: 10
            anchors.topMargin: 5
            anchors.bottomMargin: 5
            icon.source: "qrc:/icons/home_64.png"
            icon.color: "transparent"
            icon.width: 64
            icon.height: 64
            onClicked: {
                logsSettingsPage.closed()
            }
        }
        ToolButton {
            id: tbClean
            width: 64
            height: 64
            anchors.horizontalCenter: parent.horizontalCenter
            anchors.verticalCenter: parent.verticalCenter
            anchors.leftMargin: 10
            anchors.rightMargin: 10
            anchors.topMargin: 5
            anchors.bottomMargin: 5
            icon.source: "qrc:/icons/clear_64.png"
            icon.color: "transparent"
            icon.width: 64
            icon.height: 64
            onClicked: {
                msgClearLogs.visible = true
                msgClearLogs.open()
            }
        }
        ToolButton {
            id: tbSend
            width: 64
            height: 64
            anchors.right: parent.right
            anchors.verticalCenter: parent.verticalCenter
            anchors.leftMargin: 10
            anchors.rightMargin: 10
            anchors.topMargin: 5
            anchors.bottomMargin: 5
            icon.source: "qrc:/icons/logs_64.png"
            icon.color: "transparent"
            icon.width: 64
            icon.height: 64
            onClicked: {
                isSending = true
                footer.enabled = false
                sendMailHelper.sendMail(false, taComment.text)
            }
        }
    }
    MessageDialog{
        id : msgSendResult
        title: "Инфо"
        onOkClicked: {
            isSending = false
            footer.enabled = true
            logsSettingsPage.closed()
        }
    }
    function sendResultHandler(res){
        msgSendResult.text = (res) ? "Логи успешно отправлены" :
                                        "При отправке логов возникла ошибка";
        msgSendResult.visible = true
    }

    function progresRangeHandler(from, to){
        sendProgress.from = from
        sendProgress.to = to
    }

    function progresValueHandler(val){
        sendProgress.value = val
    }
    Component.onCompleted: {

        reloadSettingsList()
        sendMailHelper.sendResult.connect(sendResultHandler);
        sendMailHelper.progressRegion.connect(progresRangeHandler);
        sendMailHelper.progressValue.connect(progresValueHandler);
    }
    Component.onDestruction: {
        sendMailHelper.sendResult.disconnect(sendResultHandler);
        sendMailHelper.progressRegion.disconnect(progresRangeHandler);
        sendMailHelper.progressValue.disconnect(progresValueHandler);
    }
}

