import QtQuick 2.12
import QtQuick.Controls 2.12
import Qt.labs.platform 1.1

import ru.armax.cashbox 1.0

//import ru.armax.cashbox.ofdsettingswrapper 1.0
//import ru.armax.cashbox.innvalidator 1.0

Page {
    width: parent.width
    height: parent.height
    signal closed()
    title: qsTr("Настройки ОФД")
    id: ofdSettingsPage
    objectName: "ofdSettingsPage"
    property alias ownFooter: footer

    OfdSettingsWrapper {
        id: ofd
        onShowBusy: function(busy) {
            busyIndicator.running = busy
            ofdSettingsPage.enabled = !busy
            ownFooter.enabled = !busy
            window.enableAll(!busy)
        }
        onError: function(txt) {
            msgError.text = txt
            busyIndicator.running = false
            ofdSettingsPage.enabled = true
            ownFooter.enabled = true
            window.enableAll(true)
            msgError.open()
        }
        onSuccess: {
            ofdSettingsPage.enabled = true
            ownFooter.enabled = true
            closed()
        }
    }

    BusyIndicator {
        id: busyIndicator
        anchors.horizontalCenter: parent.horizontalCenter
        anchors.verticalCenter: parent.verticalCenter
        running: false
        z: 10
    }
    MessageDialog {
        id: msgError
        buttons: MessageDialog.Close
        text: ""
    }
    ScrollView {
        id: infoScrollView
        anchors.fill: parent
        anchors.margins: 0
        ScrollBar.vertical.policy: ScrollBar.AlwaysOff
        ScrollBar.horizontal.policy: ScrollBar.AlwaysOff
        width: ofdSettingsPage.width
        height: ofdSettingsPage.height
        contentHeight: ofdSettingsPage.height * 6
        contentWidth: parent.width - anchors.margins
        Item {
            anchors.fill: parent
            height:  ofdSettingsPage.height * 6
            width: ofdSettingsPage.width - ofdSettingsPage.anchors.margins
            Label
            {
                id: lOfdInn
                text: qsTr("ИНН ОФД")
                anchors.left: parent.left
                anchors.verticalCenter: leOfdInn.verticalCenter
                anchors.margins: 5
            }
            TextField
            {
                id:leOfdInn
                anchors.top: parent.top
                anchors.margins: 5
                anchors.right: parent.right
                maximumLength: 10
                text: ofd.inn
                validator: InnValidator{}
                inputMethodHints: Qt.ImhDigitsOnly
                color: acceptableInput ? "black" : "red"
                enabled: ofd.innEnabled
                onTextEdited: {
                    ofd.inn = leOfdInn.text
                }
                onPressed: {
//                        forceActiveFocus(Qt.MouseFocusReason)
//                        Qt.inputMethod.show()
                }
            }
            GroupBox {
                id:gbOfd
                //NOTE: Может всплыть урл, который на чеке. тогда сюда добавить
                title: "Сервер ОФД"
                anchors.left: parent.left
                anchors.right: parent.right
                anchors.top: leOfdInn.bottom
                height: 200
                Label
                {
                    id: lOfdHost
                    text: qsTr("Адрес")
                    anchors.left: parent.left
                    anchors.verticalCenter: leOfdHost.verticalCenter
                }
                TextField
                {
                    id:leOfdHost
                    anchors.top: parent.top
                    anchors.right: parent.right
                    anchors.left: lOfdHost.right
                    text: ofd.address
                    onTextEdited: {
                        ofd.address = leOfdHost.text
                    }
                    onPressed: {
    //                        forceActiveFocus(Qt.MouseFocusReason)
    //                        Qt.inputMethod.show()
                    }
                }
                Label
                {
                    id: lOfdPort
                    text: qsTr("Порт")
                    anchors.left: parent.left
                    anchors.verticalCenter: sbOfdPort.verticalCenter
                }
                SpinBox
                {
                    id:sbOfdPort
                    anchors.top: leOfdHost.bottom
                    anchors.right: parent.right
                    value: ofd.port
                    from: 1
                    to: 65535
                    editable: true
                    onValueChanged: {
                        ofd.port = sbOfdPort.value
                    }
                }
                Label
                {
                    id: lOfdTout
                    text: qsTr("Таймаут")
                    anchors.left: parent.left
                    anchors.verticalCenter: sbOfdTout.verticalCenter
                }
                SpinBox
                {
                    id:sbOfdTout
                    anchors.top: sbOfdPort.bottom
                    anchors.right: parent.right
                    value: ofd.queryInterval
                    from: 1000
                    to: 300000
                    editable: true
                    onValueChanged: {
                        ofd.queryInterval = sbOfdTout.value
                    }
                }
            }
            GroupBox {
                id: gbOism
                //NOTE: Может всплыть урл, который на чеке. тогда сюда добавить
                title: "Сервер ОИСМ"
                anchors.left: parent.left
                anchors.right: parent.right
                anchors.top: gbOfd.bottom
                height: 200
                Label
                {
                    id: lOismHost
                    text: qsTr("Адрес")
                    anchors.left: parent.left
                    anchors.verticalCenter: leOismHost.verticalCenter
                }
                TextField
                {
                    id:leOismHost
                    anchors.top: parent.top
                    anchors.right: parent.right
                    anchors.left: lOismHost.right
                    text: ofd.labelsAddress
                    onTextEdited: {
                        ofd.labelsAddress = leOismHost.text
                    }
                    onPressed: {
    //                        forceActiveFocus(Qt.MouseFocusReason)
    //                        Qt.inputMethod.show()
                    }
                }
                Label
                {
                    id: lOismPort
                    text: qsTr("Порт")
                    anchors.left: parent.left
                    anchors.verticalCenter: sbOismPort.verticalCenter
                }
                SpinBox
                {
                    id:sbOismPort
                    anchors.top: leOismHost.bottom
                    anchors.right: parent.right
                    value: ofd.labelsPort
                    from: 1
                    to: 65535
                    editable: true
                    onValueChanged: {
                        ofd.labelsPort = sbOismPort.value
                    }
                }
                Label
                {
                    id: lOismTout
                    text: qsTr("Таймаут")
                    anchors.left: parent.left
                    anchors.verticalCenter: sbOismTout.verticalCenter
                }
                SpinBox
                {
                    id:sbOismTout
                    anchors.top: sbOismPort.bottom
                    anchors.right: parent.right
                    value: ofd.testTimeout
                    from: 1000
                    to: 30000
                    editable: true
                    onValueChanged: {
                        ofd.testTimeout= sbOismTout.value
                    }
                }
            }
            GroupBox {
                id: gbOkp
                title: "Сервер ОКП"
                anchors.left: parent.left
                anchors.right: parent.right
                anchors.top: gbOism.bottom
                height: 200
                Label
                {
                    id: lOkpHost
                    text: qsTr("Адрес")
                    anchors.left: parent.left
                    anchors.verticalCenter: leOkpHost.verticalCenter
                }
                TextField
                {
                    id:leOkpHost
                    anchors.top: parent.top
                    anchors.right: parent.right
                    anchors.left: lOkpHost.right
                    text: ofd.keysAddress
                    onTextEdited: {
                        ofd.keysAddress = leOkpHost.text
                    }
                    onPressed: {
    //                        forceActiveFocus(Qt.MouseFocusReason)
    //                        Qt.inputMethod.show()
                    }
                }
                Label
                {
                    id: lOkpPort
                    text: qsTr("Порт")
                    anchors.left: parent.left
                    anchors.verticalCenter: sbOkpPort.verticalCenter
                }
                SpinBox
                {
                    id:sbOkpPort
                    anchors.top: leOkpHost.bottom
                    anchors.right: parent.right
                    value: ofd.keysPort
                    from: 1
                    to: 65535
                    editable: true
                    onValueChanged: {
                        ofd.keysPort = sbOkpPort.value
                    }
                }
                Label
                {
                    id: lOkpTout
                    text: qsTr("Таймаут")
                    anchors.left: parent.left
                    anchors.verticalCenter: sbOkpTout.verticalCenter
                }
                SpinBox
                {
                    id:sbOkpTout
                    anchors.top: sbOkpPort.bottom
                    anchors.right: parent.right
                    value: ofd.keysTimeout
                    from: 1000
                    to: 300000
                    editable: true
                    onValueChanged: {
                        ofd.keysTimeout = sbOkpTout.value
                    }
                }
            }

        }
    }
    ToolBar {
        id: footer
        height: 70
        ToolButton {
            id: tbBack
            width: 64
            height: 64
            anchors.left: parent.left
            anchors.verticalCenter: parent.verticalCenter
            anchors.leftMargin: 10
            anchors.rightMargin: 10
            anchors.topMargin: 5
            anchors.bottomMargin: 5
            icon.source: "qrc:/icons/back_64.png"
            icon.color: "transparent"
            icon.width: 60
            icon.height: 60
            onClicked: {
                ofdSettingsPage.closed()
            }
        }
        ToolButton {
            id: tbReset
            width: 64
            height: 64
            anchors.horizontalCenter: parent.horizontalCenter
            anchors.verticalCenter: parent.verticalCenter
            anchors.leftMargin: 10
            anchors.rightMargin: 10
            anchors.topMargin: 5
            anchors.bottomMargin: 5
            icon.source: "qrc:/icons/reset_64.png"
            icon.color: "transparent"
            icon.width: 64
            icon.height: 64
            onClicked: {
                resetData()
            }
        }
        ToolButton {
            id: tbSave
            width: 64
            height: 64
            anchors.right: parent.right
            anchors.verticalCenter: parent.verticalCenter
            anchors.leftMargin: 10
            anchors.rightMargin: 10
            anchors.topMargin: 5
            anchors.bottomMargin: 5
            icon.source: "qrc:/icons/apply_64.png"
            icon.color: "transparent"
            icon.width: 64
            icon.height: 64
            onClicked: {
                ofd.save()
            }
        }
    }

    function resetData() {
        reload()
    }

    Component.onCompleted: {
        width = parent.width
        height = parent.height


    }
}
