import QtQuick 2.12
import QtQuick.Controls 2.15
import Qt.labs.platform 1.1 as QtLabs
import Qt.labs.folderlistmodel 2.12 as Fm

import ru.armax.cashbox 1.0

//import ru.armax.cashbox.cashboxstatus 1.0
//import ru.armax.cashbox.reportscorehandler 1.0
//import ru.armax.cashbox.approot 1.0

Page {
    width: parent.width
    height: parent.height
    signal closed()
    title: qsTr("Выгрузка уведомлений")
    id: offlineReportsPage
    objectName: "offlineReportsPage"
    property alias ownFooter: footer

    property string reportsFileName: ""

    ReportsCoreHandler {
        id: reportsHandler
        onError: function(txt){
            msgError.text = txt
            busyIndicator.running = false
            offlineReportsPage.enabled = true
            window.enableAll(true)
            msgError.open()
        }
        onOfflineReportsSaved: {
            msgSuccess.text = "Файл уведомлений скопирован на SD карту"
            busyIndicator.running = false
            offlineReportsPage.enabled = true
            window.enableAll(true)
            msgSuccess.open()
        }
    }

    QtLabs.MessageDialog {
        id: msgSuccess
        buttons: QtLabs.MessageDialog.Ok
        text: ""
        onOkClicked: {
            offlineReportsPage.closed();
        }
    }

    BusyIndicator {
        id: busyIndicator
        anchors.horizontalCenter: parent.horizontalCenter
        anchors.verticalCenter: parent.verticalCenter
        running: false
        z: 10
    }
    QtLabs.MessageDialog {
        id: msgError
        buttons: QtLabs.MessageDialog.Close
        text: ""
    }

    Dialog {
        id: sdFiles
        modal: true
        width: parent.width
        height: parent.height
        standardButtons: Dialog.Close
        title: "Файлы уведомлений"
        ScrollView {
            id: sdScrollView
            anchors.margins: 10
            anchors.fill: parent
            ScrollBar.vertical.policy: ScrollBar.AlwaysOff
            ScrollBar.horizontal.policy: ScrollBar.AlwaysOff
            z: 1
            ListView {
                id:lvSdFiles
                interactive: true
                orientation: Qt.Vertical
                highlightFollowsCurrentItem: true
                z: 2
                spacing: 5
                model: Fm.FolderListModel {
                    id: sdFolderModel
                    nameFilters: ["*.bin", "*.crpt", "*.fnm"]
                    showDirs: false
                    showDotAndDotDot : false
                }
                delegate : Item{
                    height: 40
                    width: parent.width
                    Text {
                        text: fileName
                        anchors.fill: parent
                        wrapMode: Text.WordWrap
                    }
                    MouseArea {
                        anchors.fill: parent
                        onClicked: {
                            if(fileIsDir) sdFolderModel.folder = sdFolderModel.folder + "/" + fileName
                            else {
                                offlineReportsPage.reportsFileName = sdFolderModel.folder + "/" + fileName
                                lOut.text = ""
                                sdFiles.close()
                                reportsHandler.saveOfflineReportToSdCard(offlineReportsPage.reportsFileName)
                            }
                        }
                    }
                }

            }
        }
    }

    Label {
        id: lFirmware
        anchors.left: parent.left
        anchors.top: parent.top
        anchors.right: parent.right
        width: parent.width - 10
        anchors.margins: 5
        wrapMode: Label.WordWrap
        text: reportsFileName.length > 0 ? "Выбран файл: " + reportsFileName : ""
    }
    Label {
        id: lOut
        anchors.left: parent.left
        anchors.top: lFirmware.bottom
        anchors.right: parent.right
        anchors.bottom: parent.bottom
        width: parent.width - 10
        anchors.margins: 5
        wrapMode: Label.WordWrap
        text: ""
    }


    RoundButton {
        id: rbToSdCard
        anchors.margins: 10
        anchors.bottom: parent.bottom
        anchors.right: parent.right
        icon.source: "qrc:/icons/from_sdcard_64.png"
        icon.color: "transparent"
        icon.width: 50
        icon.height: 50
        z:100
        onPressed: {
            console.log(reportsHandler.offlineReportsPath())
            sdFolderModel.folder = reportsHandler.offlineReportsPath()
            console.log(sdFolderModel.folder)
            sdFiles.open()
        }
    }
    ToolBar {
        id: footer
        height: 70
        ToolButton {
            id: tbBack
            width: 64
            height: 64
            anchors.left: parent.left
            anchors.verticalCenter: parent.verticalCenter
            anchors.leftMargin: 10
            anchors.rightMargin: 10
            anchors.topMargin: 5
            anchors.bottomMargin: 5
            icon.source: "qrc:/icons/home_64.png"
            icon.color: "transparent"
            icon.width: 60
            icon.height: 60
            onClicked: {
                offlineReportsPage.closed()
            }
        }
        ToolButton {
            id: tbCheck
            width: 64
            height: 64
            anchors.right: parent.right
            anchors.verticalCenter: parent.verticalCenter
            anchors.leftMargin: 10
            anchors.rightMargin: 10
            anchors.topMargin: 5
            anchors.bottomMargin: 5
            icon.source: "qrc:/icons/offline_report_64.png"
            icon.color: "transparent"
            icon.width: 64
            icon.height: 64
            onClicked: {
                lOut.text = ""
                reportsHandler.offlineReports()
            }
        }
    }
    Component.onCompleted: {
        width = parent.width
        height = parent.height
    }
}
