import QtQuick 2.12
import QtQuick.Controls 2.12
import Qt.labs.platform 1.1

import ru.armax.cashbox 1.0


Page {
    width: parent.width
    height: parent.height
    signal closed()
    title: qsTr("Настройки чека")
    id: receiptConfigPage
    objectName: "receiptConfigPage"
    property alias ownFooter: footer


    ReceiptConfigHandler {
        id: config
        onShowBusy: function(busy) {
            busyIndicator.running = busy
            receiptConfigPage.enabled = !busy
            window.enableAll(!busy)
        }
        onError: function(txt) {
            msgError.text = txt
            busyIndicator.running = false
            receiptConfigPage.enabled = true
            window.enableAll(true)
            msgError.open()
        }
        onLoaded: {
            dsbMaxAmount.value = config.maxReceiptAmount
            busyIndicator.running = false
            receiptConfigPage.enabled = true
            window.enableAll(true)
        }
        onSavedSuccess: {
            dsbMaxAmount.value = config.maxReceiptAmount
            busyIndicator.running = false
            receiptConfigPage.enabled = true
            window.enableAll(true)
        }
    }
    Label {
        id: lMaxAmount
        anchors.horizontalCenter: parent.horizontalCenter
        anchors.top: parent.top
        anchors.leftMargin: 10
        anchors.rightMargin:10

        horizontalAlignment: Text.AlignHCenter
        font.pixelSize: 16
        text: qsTr("Максимальная сумма чека")
    }
    DoubleSpinBox {
        id: dsbMaxAmount
        anchors.leftMargin: 10
        anchors.rightMargin:10
        anchors.top: lMaxAmount.bottom
        anchors.left: parent.left
        anchors.right: parent.right
        from: 0
        value: 0
        to: 2147483647
        stepSize: 100
        font.pixelSize: 20
        editable: true
        decimals: 2
        realValue: value / 100.0
        onValueChanged: {
            config.maxReceiptAmount = dsbMaxAmount.value
        }
    }

    BusyIndicator {
        id: busyIndicator
        anchors.horizontalCenter: parent.horizontalCenter
        anchors.verticalCenter: parent.verticalCenter
        running: false
        z: 10
    }
    MessageDialog {
        id: msgError
        buttons: MessageDialog.Close
        text: ""
    }

    ToolBar {
        id: footer
        height: 70
        ToolButton {
            id: tbBack
            width: 64
            height: 64
            anchors.left: parent.left
            anchors.verticalCenter: parent.verticalCenter
            anchors.leftMargin: 10
            anchors.rightMargin: 10
            anchors.topMargin: 5
            anchors.bottomMargin: 5
            icon.source: "qrc:/icons/home_64.png"
            icon.color: "transparent"
            icon.width: 60
            icon.height: 60
            onClicked: {
                receiptConfigPage.closed()
            }
        }
        ToolButton {
            id: tbSave
            width: 64
            height: 64
            anchors.right: parent.right
            anchors.verticalCenter: parent.verticalCenter
            anchors.leftMargin: 10
            anchors.rightMargin: 10
            anchors.topMargin: 5
            anchors.bottomMargin: 5
            icon.source: "qrc:/icons/save_64.png"
            icon.color: "transparent"
            icon.width: 64
            icon.height: 64
            onClicked: {
                config.save()
            }
        }
    }
    Component.onCompleted: {
        width = parent.width
        height = parent.height
        config.reload()
    }
}
