import QtQuick 2.15
import QtQuick.Controls 2.15
import Qt.labs.platform 1.1 as QtLabs

import ru.armax.cashbox 1.0

//import ru.armax.cashbox.receiptoperationwrapper 1.0

Page {
    width: parent.width
    height: parent.height
    id: opEditPage
    signal closed()
    title: qsTr("Предмет расчета")
    objectName: "opEditPage"
    property alias eOperation: operation
//    property alias ownFooter: footer
    property alias opIndex: operation.index
    property alias opData: operation.data
    property alias ownFooter: footer
    property alias isSale: operation.isSale

    QtLabs.MessageDialog {
        id: msgError
        buttons: QtLabs.MessageDialog.Close
        text: ""
    }
    ReceiptOperationWrapper {
        id: operation

        onError: function(err){
            msgError.text = err
            msgError.open()
        }
        onFinish: {
            opEditPage.closed()
        }
        onShowBusy: function(busy) {
            busyIndicator.running = busy
            opEditPage.enabled = !busy
            window.enableAll(!busy)
        }
        onHasFractialChanged: {
            if(hasFractial)
            {
                lPrice.text = "Цена за шт."
                lePrice.text = operation.priceForUnit
            }
            else
            {
                lPrice.text = "Цена"
                lePrice.text = operation.price
            }
        }
        onLabelChanged: {
            lLabel.text = operation.labelText()
        }
        onNeedCheckLabel: {
            lcfDialog.open()
        }
    }
    LabelCheckFlagsDialog {
        id: lcfDialog
        width: parent.width
        height: parent.height
        canSkipB5: operation.canSkipB5
        onAccepted: {
            operation.checkLabel(lcfDialog.flags)
        }
    }

    Dialog {
        id: dlgLabel
        title: "Введите марку"
        width: parent.width
        height: parent.height / 2
        standardButtons: Dialog.Ok | Dialog.Close
        TextEdit {
            id:teLabel
            text: ""
            anchors.fill: parent
            wrapMode: TextEdit.Wrap
        }
        onAccepted:  {
            operation.parseBarcode(teLabel.text)
        }
    }

    BusyIndicator {
        id: busyIndicator
        anchors.horizontalCenter: parent.horizontalCenter
        anchors.verticalCenter: parent.verticalCenter
        running: false
        z: 10
    }
    ScrollView {
        id: infoScrollView
        anchors.fill: parent
        anchors.margins: 0
        ScrollBar.vertical.policy: ScrollBar.AlwaysOff
        ScrollBar.horizontal.policy: ScrollBar.AlwaysOff
        width: opEditPage.width
        height: opEditPage.height
        contentHeight: opEditPage.height * 6
        contentWidth: parent.width - anchors.margins
        Item {
            anchors.fill: parent
            height:  opEditPage.height * 6
            width: opEditPage.width - opEditPage.anchors.margins

            ComboBox {
                id: cbType
                model: operation.opTypeModel
                textRole: "name"
                anchors.top: parent.top
                anchors.left:  parent.left
                anchors.right: parent.right
                anchors.margins: 5
                currentIndex: operation.opTypeIndex
                onCurrentIndexChanged: {
                    operation.opType = cbType.model[cbType.currentIndex].code
                }
            }
            TextField {
                id: txtName
                text: operation.name
                placeholderText: "Наименование товара/услуги"
                anchors.top: cbType.bottom
                anchors.left: parent.left
                anchors.right: parent.right
                anchors.margins: 5
                maximumLength: 128
                onTextEdited: {
                    operation.name = txtName.text
                }
            }
            ComboBox {
                id: cbVat
                model: operation.vatModel
                textRole: "name"
                anchors.top: txtName.bottom
                anchors.right: parent.right
                anchors.margins: 5
                currentIndex: operation.vatRate - 1
                onCurrentIndexChanged: {
                    operation.vatRate = cbVat.currentIndex + 1
                }
            }
            ComboBox {
                id: cbPayType
                model: operation.payTypeModel
                textRole: "name"
                anchors.top: txtName.bottom
                anchors.left:  parent.left
                anchors.right: cbVat.left
                anchors.margins: 5
                currentIndex: operation.payType - 1
                onCurrentIndexChanged: {
                    operation.payType = cbPayType.currentIndex + 1
                }
            }
            Label
            {
                id:lPrice
                anchors.top: cbPayType.bottom
                anchors.horizontalCenter: lePrice.horizontalCenter
                text: qsTr("Цена")
                font.pixelSize: 10
            }
            Label
            {
                id:lPromisStatus
                anchors.verticalCenter: lPrice.verticalCenter
                anchors.horizontalCenter: leQuantity.horizontalCenter
                text: qsTr("Планируемый статус")
                font.pixelSize: 10
            }
            TextField
            {
                id: lePrice
                anchors.margins: 5
                anchors.top: lPrice.bottom
                anchors.left: parent.left
                width: parent.width / 2 - 10
                inputMethodHints: Qt.ImhFormattedNumbersOnly
                text: operation.hasFractial ? operation.priceForUnit: operation.price
                validator: DoubleValidator{bottom: 0.0; top: 2147483647.0; decimals: 2}
                horizontalAlignment: TextField.AlignHCenter
                onEditingFinished: {
                    if(operation.hasFractial)
                    {
                        lPrice.text = "Цена за шт."
                        operation.priceForUnit = lePrice.text
                    }
                    else
                    {
                        lPrice.text = "Цена"
                        operation.price = lePrice.text
                    }
                }
        //        onPressed: {
        //            forceActiveFocus(Qt.MouseFocusReason)
        //            Qt.inputMethod.show()
        //        }
            }
            ComboBox
            {
                id: cbPromis
                anchors.margins: 5
                anchors.top: lPrice.bottom
                anchors.left: lePrice.right
                anchors.right: parent.right
                textRole: "name"
                model: ListModel{
                    ListElement {name: "Штучный"; code : ReceiptOperationWrapper.Piece}
                    ListElement {name: "Мерный"; code : ReceiptOperationWrapper.Measured}
                    ListElement {name: "Другой"; code : ReceiptOperationWrapper.OtherPromisStatus}
                }
                currentIndex:  operation.promisStatus
                onCurrentIndexChanged: {
                    operation.promisStatus = cbPromis.currentIndex
//                    var measured = (cbPromis.currentIndex == ReceiptOperationWrapper.Measured)
                    lUnit.visible = true//measured
                    cbUnit.visible =true// measured
                    lQUantity.visible = /*measured &&*/ !operation.hasFractial
                    leQuantity.visible = /*measured &&*/ !operation.hasFractial
                }
            }
            Label {
                id: lLabel
                text: ""
                anchors.top:  cbPromis.bottom
                anchors.left: parent.left
                anchors.right: parent.right
                width: parent.width - 10
                wrapMode: Label.WordWrap
                horizontalAlignment: Label.AlignHCenter
                anchors.margins: 5
            }
            Label {
                id: lUnit
                anchors.left: parent.left
                anchors.verticalCenter: cbUnit.verticalCenter
                anchors.margins: 5
                text: "Мера количества"
                visible: true
            }
            ComboBox
            {
                id: cbUnit
                anchors.margins: 5
                anchors.top: lLabel.bottom
                anchors.left: lUnit.right
                anchors.right: parent.right
                visible: true
                textRole: "name"
                model: operation.itemUnitModel
                currentIndex:  operation.itemUnit
                onCurrentIndexChanged: {
                    operation.itemUnit = cbUnit.currentIndex
                }
            }

            Label
            {
                id:lQUantity
                anchors.verticalCenter: leQuantity.verticalCenter
                anchors.left: parent.left
                text: qsTr("Количество")
                font.pixelSize: 10
                visible: /*cbPromis.currentIndex == ReceiptOperationWrapper.Measured &&*/ !operation.hasFractial
            }
            TextField
            {
                id: leQuantity
                anchors.margins: 5
                anchors.top: cbUnit.bottom
                anchors.left: lQUantity.right
                anchors.right: parent.right
                width: parent.width / 2 - 10
                inputMethodHints: Qt.ImhFormattedNumbersOnly
                text: operation.quantity
                validator: DoubleValidator{bottom: 0.0; top: 2147483647.0; decimals: 3}
                horizontalAlignment: TextField.AlignHCenter
                visible: /*cbPromis.currentIndex == ReceiptOperationWrapper.Measured &&*/ !operation.hasFractial
                onEditingFinished: {
                    operation.quantity = leQuantity.text
                }
        //        onPressed: {
        //            forceActiveFocus(Qt.MouseFocusReason)
        //            Qt.inputMethod.show()
        //        }
            }
            GroupBox
            {
                id: gbFractial
                height: 160
                anchors.top: leQuantity.bottom
                anchors.left: parent.left
                anchors.right: parent.right
                anchors.margins: 0
                visible: /*(cbPromis.currentIndex == ReceiptOperationWrapper.Measured) &&*/ cbUnit.currentIndex == 0
                label: CheckBox {
                    id: cbHasFractial
                    text: "Дробное количество"
                    checked: operation.hasFractial
                    onReleased: {
                        operation.hasFractial = cbHasFractial.checked
                        if(!operation.hasFractial && cbHasFractial.checked) {
                            cbHasFractial.checked = false
                        }
//                        var measured = (cbPromis.currentIndex == ReceiptOperationWrapper.Measured)
                        lQUantity.visible =/* measured &&*/ !operation.hasFractial
                        leQuantity.visible = /*measured &&*/ !operation.hasFractial
                    }
                }
                SpinBox {
                    id: sbNominator
                    anchors.left: parent.left
                    anchors.top:parent.top
                    anchors.margins: 0
                    value: operation.nominator
                    from: 1
                    to: sbDenominator.value - 1
                    enabled: operation.hasFractial
                    onValueChanged: {
                        operation.nominator = sbNominator.value
                    }
                }
                Label {
                    id: lnd
                    anchors.verticalCenter: sbNominator.verticalCenter
                    anchors.left: sbNominator.right
                    text: "/"
                    font.pixelSize: 24
                    enabled: operation.hasFractial
                }
                SpinBox {
                    id: sbDenominator
                    anchors.right: parent.right
                    anchors.left: lnd.right
                    anchors.top:parent.top
                    anchors.margins: 0
                    from: 1
                    to: 1000
                    enabled: operation.hasFractial
                    onValueChanged:  {
                        sbNominator.to = sbDenominator.value - 1
                        operation.denominator = sbDenominator.value
                    }
                }
            }

        }
    }

    RoundButton {
        id: rbAgent
        anchors.margins: 10
        anchors.bottom: parent.bottom
        anchors.left: parent.left
        icon.source: "qrc:/icons/agent_64.png"
        icon.color: "transparent"
        icon.width: 50
        icon.height: 50
        z:100
        onPressed: {
            var page = window.showPage("AgentDataPage.qml");
            page.ownAgent.setData(operation.agentData())
            page.ownAgent.saveData.connect(operation.setAgentData)
        }
    }
    RoundButton {
        id: rbOpProps
        anchors.margins: 10
        anchors.bottom: parent.bottom
        anchors.horizontalCenter: parent.horizontalCenter
        icon.source: "qrc:/icons/receipt_props_48.png"
        icon.color: "transparent"
        icon.width: 50
        icon.height: 50
        z:100
        onPressed: {
            var page = window.showPage("ReceiptOperationPropsPage.qml");
            page.operation = operation
            page.resetData()
        }
    }
    RoundButton {
        id: rbIndustryProps
        anchors.margins: 10
        anchors.bottom: parent.bottom
        anchors.right: parent.right
        icon.source: "qrc:/icons/industry_prop_48.png"
        icon.color: "transparent"
        icon.width: 50
        icon.height: 50
        z:100
        onPressed: {
            var page = window.showPage("IndustryPropsView.qml");
            page.owmModel.setProps(operation.industryProps)
            page.owmModel.saveData.connect(operation.setIndustryProps)
        }
    }
    ToolBar {
        id: footer
        height: 70
        ToolButton {
            id: tbBack
            width: 64
            height: 64
            anchors.left: parent.left
            anchors.verticalCenter: parent.verticalCenter
            anchors.margins: 5
            icon.source: "qrc:/icons/back_64.png"
            icon.color: "transparent"
            icon.width: 60
            icon.height: 60
            onClicked: {

                opEditPage.closed()
            }
        }
        ToolButton {
            id: tbScanBarcode
            width: 64
            height: 64
            anchors.verticalCenter: parent.verticalCenter
            anchors.horizontalCenter: parent.horizontalCenter
            anchors.margins: 5
            icon.source: "qrc:/icons/barcode_64.png"
            icon.color: "transparent"
            icon.width: 64
            icon.height: 64
            onClicked: scanBarcode()
            onPressAndHold: {
                dlgLabel.open()
            }
        }

        ToolButton {
            id: tbSave
            width: 64
            height: 64
            anchors.right: parent.right
            anchors.verticalCenter: parent.verticalCenter
            anchors.margins: 5
            icon.source: "qrc:/icons/submit_64.png"
            icon.color: "transparent"
            icon.width: 64
            icon.height: 64
            onClicked: {
                if(cbType.currentIndex >= 0) operation.opType = cbType.model[cbType.currentIndex].code
                operation.name = txtName.text
                operation.vatRate = cbVat.currentIndex + 1
                operation.payType = cbPayType.currentIndex + 1
                if(operation.hasFractial)
                {
                    lPrice.text = "Цена за шт."
                    operation.priceForUnit = lePrice.text
                }
                else
                {
                    lPrice.text = "Цена"
                     operation.price = lePrice.text
                }
                operation.quantity = leQuantity.text
                operation.save()
            }
        }
    }
    function processBarcode(text) {
        operation.parseBarcode(text)
        window.popPage()
    }

    function scanBarcode() {
        var page = window.showPage("BarcodePage.qml");
        page.newBarcode.connect(processBarcode)
    }

    function parseProduct(prod) {
        operation.parseProductMap(prod)
    }

    Component.onCompleted: {

    }
}
