import QtQuick 2.12
import QtQuick.Controls 2.12
import Qt.labs.platform 1.1

import ru.armax.cashbox 1.0

//import ru.armax.cashbox.cashboxstatus 1.0
//import ru.armax.cashbox.receiptoperationwrapper 1.0
//import ru.armax.cashbox.innvalidator 1.0


Page {
    width: parent.width
    height: parent.height
    signal closed()
    title: qsTr("Настройки ПР")
    id: receiptOpPropsView
    objectName: "receiptOpPropsView"
    property alias ownFooter: footer
    property ReceiptOperationWrapper operation

    CashboxStatus {
        id: cbStatus
    }

    BusyIndicator {
        id: busyIndicator
        anchors.horizontalCenter: parent.horizontalCenter
        anchors.verticalCenter: parent.verticalCenter
        running: false
        z: 10
    }
    MessageDialog {
        id: msgError
        buttons: MessageDialog.Close
        text: ""
    }
    ScrollView {
        id: infoScrollView
        anchors.fill: parent
        anchors.margins: 0
        ScrollBar.vertical.policy: ScrollBar.AlwaysOff
        ScrollBar.horizontal.policy: ScrollBar.AlwaysOff
        width: receiptOpPropsView.width
        height: receiptOpPropsView.height
        contentHeight: receiptOpPropsView.height * 6
        contentWidth: parent.width - anchors.margins
        Item {
            anchors.fill: parent
            height:  receiptOpPropsView.height * 6
            width: receiptOpPropsView.width - receiptOpPropsView.anchors.margins
            TextField {
                id: leAdditional
                text: operation.additionalParam
                placeholderText: "Дополнительный реквизит предмета расчета"
                anchors.top: parent.top
                anchors.left: parent.left
                anchors.right: parent.right
                anchors.margins: 5
                maximumLength: 64
            }
            Label {
                id: lCountryCode
                anchors.left: parent.left
                anchors.verticalCenter: leCountryCode.verticalCenter
                horizontalAlignment: Label.AlignLeft
                anchors.margins: 5
                text: qsTr("Код страны происхождения товара")
            }
            TextField {
                id: leCountryCode
                anchors.margins: 5
                anchors.top: leAdditional.bottom
                anchors.right: parent.right
                anchors.left: lCountryCode.right
                maximumLength: 3
                text: operation.countryCode
            }
            Label {
                id: lDeclNumber
                anchors.left: parent.left
                anchors.verticalCenter: leDeclNumber.verticalCenter
                horizontalAlignment: Label.AlignLeft
                anchors.margins: 5
                text: qsTr("Номер декларации")
            }
            TextField {
                id: leDeclNumber
                anchors.margins: 5
                anchors.top: leCountryCode.bottom
                anchors.right: parent.right
                anchors.left: lDeclNumber.right
                maximumLength: 32
                text: operation.declarationNumber
            }
            Label {
                id: lExciseTax
                anchors.left: parent.left
                anchors.verticalCenter: leExciseTax.verticalCenter
                horizontalAlignment: Label.AlignLeft
                anchors.margins: 5
                visible: operation.isExcise
                text: qsTr("Акциз")
            }
            TextField {
                id: leExciseTax
                anchors.margins: 5
                anchors.top: leDeclNumber.bottom
                anchors.right: parent.right
                anchors.left: lExciseTax.right
                inputMethodHints: Qt.ImhFormattedNumbersOnly
                text: operation.exciseTax
                validator: DoubleValidator{bottom: 0.0; top: 2147483647.0; decimals: 2}
                visible: operation.isExcise
                horizontalAlignment: TextField.AlignHCenter
            }
        }
    }
    ToolBar {
        id: footer
        height: 70
        ToolButton {
            id: tbBack
            width: 64
            height: 64
            anchors.left: parent.left
            anchors.verticalCenter: parent.verticalCenter
            anchors.leftMargin: 10
            anchors.rightMargin: 10
            anchors.topMargin: 5
            anchors.bottomMargin: 5
            icon.source: "qrc:/icons/back_64.png"
            icon.color: "transparent"
            icon.width: 60
            icon.height: 60
            onClicked: {
                receiptOpPropsView.closed()
            }
        }
        ToolButton {
            id: tbReset
            width: 64
            height: 64
            anchors.horizontalCenter: parent.horizontalCenter
            anchors.verticalCenter: parent.verticalCenter
            anchors.leftMargin: 10
            anchors.rightMargin: 10
            anchors.topMargin: 5
            anchors.bottomMargin: 5
            icon.source: "qrc:/icons/reset_64.png"
            icon.color: "transparent"
            icon.width: 64
            icon.height: 64
            onClicked: {
                resetData()
            }
        }
        ToolButton {
            id: tbSave
            width: 64
            height: 64
            anchors.right: parent.right
            anchors.verticalCenter: parent.verticalCenter
            anchors.leftMargin: 10
            anchors.rightMargin: 10
            anchors.topMargin: 5
            anchors.bottomMargin: 5
            icon.source: "qrc:/icons/apply_64.png"
            icon.color: "transparent"
            icon.width: 64
            icon.height: 64
            onClicked: {
                commit()
            }
        }
    }

    function resetData() {
        leAdditional.text = operation.additionalParam
        leCountryCode.text = operation.countryCode
        leDeclNumber.text = operation.declarationNumber
        leExciseTax.text = operation.exciseTax

    }
    function commit() {
        operation.additionalParam = leAdditional.text
        operation.countryCode = leCountryCode.text
        operation.declarationNumber = leDeclNumber.text
        operation.exciseTax = leExciseTax.text

        closed()
    }

    Component.onCompleted: {
        width = parent.width
        height = parent.height


    }
}
