import QtQuick 2.12
import QtQuick.Controls 2.12
import Qt.labs.platform 1.1

import ru.armax.cashbox 1.0

//import ru.armax.cashbox.cashboxstatus 1.0
//import ru.armax.cashbox.receiptmodel 1.0
//import ru.armax.cashbox.innvalidator 1.0

Page {
    width: parent.width
    height: parent.height
    signal closed()
    title: qsTr("Настройки чека")
    id: receiptPropsView
    objectName: "receiptPropsView"
    property alias ownFooter: footer
    property ReceiptModel rModel

    CashboxStatus {
        id: cbStatus
    }

    BusyIndicator {
        id: busyIndicator
        anchors.horizontalCenter: parent.horizontalCenter
        anchors.verticalCenter: parent.verticalCenter
        running: false
        z: 10
    }
    MessageDialog {
        id: msgError
        buttons: MessageDialog.Close
        text: ""
    }
    ScrollView {
        id: infoScrollView
        anchors.fill: parent
        anchors.margins: 0
        ScrollBar.vertical.policy: ScrollBar.AlwaysOff
        ScrollBar.horizontal.policy: ScrollBar.AlwaysOff
        width: receiptPropsView.width
        height: receiptPropsView.height
        contentHeight: receiptPropsView.height * 6
        contentWidth: parent.width - anchors.margins
        Item {
            anchors.fill: parent
            height:  receiptPropsView.height * 6
            width: receiptPropsView.width - receiptPropsView.anchors.margins
            Label {
                id: lTax
                anchors.left: parent.left
                anchors.margins: 5
                text: qsTr("СНО")
                anchors.verticalCenter:  cbTax.verticalCenter
            }
            ComboBox {
                id: cbTax
                model: rModel.taxSystemModel
                textRole: "name"
                anchors.top: parent.top
                anchors.left:  lTax.right
                anchors.right: parent.right
                anchors.margins: 5
                currentIndex: rModel.taxIndex
                onCurrentIndexChanged: {
//                    rModel.taxIndex = cbTax.currentIndex
                }

            }
            Label {
                id: lCashier
                anchors.left: parent.left
                anchors.margins: 5
                text: qsTr("Кассир")
                enabled: !cbStatus.inAutoMode
                anchors.verticalCenter:  leCashier.verticalCenter
            }
            TextField {
                id: leCashier
                anchors.margins: 5
                anchors.top: cbTax.bottom
                anchors.left: lCashier.right
                anchors.right: parent.right
                text: rModel.cashier
                enabled: !cbStatus.inAutoMode
                maximumLength: 64
                onTextEdited: {
//                    rModel.cashier = leCashier.text
                }


            }
            Label {
                id: lCashierInn
                anchors.left: parent.left
                anchors.margins: 5
                enabled: !cbStatus.inAutoMode
                text: qsTr("ИНН Кассира")
                anchors.verticalCenter:  leCashierInn.verticalCenter
            }
            TextField {
                id: leCashierInn
                anchors.margins: 5
                anchors.top: leCashier.bottom
                anchors.right: parent.right
                text: rModel.cashierInn
                maximumLength: 12
                enabled: !cbStatus.inAutoMode
                validator: InnValidator{}
                inputMethodHints: Qt.ImhDigitsOnly
                color: acceptableInput ? "black" : "red"
                onTextEdited: {
//                    rModel.cashierInn = leCashierInn.text
                }
            }
            //Возможно убрать это поле, чтоб не отсвечивало
            TextField
            {
                id: leUser
                anchors.margins: 5
                anchors.left: parent.left
                anchors.right: parent.right
                anchors.top: leCashierInn.bottom
                placeholderText: qsTr("Наименование пользователя")
                text: rModel.ownerName
                maximumLength:  256
                enabled: rModel.canChangeOwnerName
                onTextEdited: {
//                    rModel.ownerName = leUser.text
                }

            }
            TextField
            {
                id: leAddress
                anchors.margins: 5
                anchors.left: parent.left
                anchors.right: parent.right
                anchors.top: leUser.bottom
                placeholderText: qsTr("Адрес расчетов")
                text: rModel.address
                enabled: rModel.canChangeAddress
                width: parent.width - anchors.margins * 2
                wrapMode: TextField.WordWrap

                onTextEdited: {
//                    rModel.address = leAddress.text
                }
            }
            TextField
            {
                id: lePlace
                anchors.margins: 5
                anchors.left: parent.left
                anchors.right: parent.right
                anchors.top: leAddress.bottom
                placeholderText: qsTr("Место расчетов")
                text: rModel.place
                maximumLength:  256
                enabled: rModel.canChangeAddress
                onTextEdited: {
//                    rModel.place = lePlace.text
                }
            }
            ComboBox {
                id: cbBuyerPhone
                anchors.verticalCenter:  leBuyerPhone.verticalCenter
                anchors.left: parent.left
                anchors.margins: 5
                textRole: "name"
                model: ListModel {
                    ListElement {name : "Телефон\nпокупателя"}
                    ListElement {name : "Эл.адр.\nпокупателя"}
                    ListElement {name : "NONE"}
                }
                currentIndex: 0
                onCurrentIndexChanged: {
                    leBuyerPhone.readOnly = false
                    if(cbBuyerPhone.currentIndex == 0) {
                        leBuyerPhone.inputMethodHints = Qt.ImhDigitsOnly
                    }
                    else {
                        leBuyerPhone.inputMethodHints = Qt.ImhEmailCharactersOnly
                        if(cbBuyerPhone.currentIndex == 2) {
                            leBuyerPhone.readOnly = true
                            leBuyerPhone.text = "none"
                            rModel.buyerPhone = leBuyerPhone.text
                        }
                    }
                }
            }
            TextField {
                id: leBuyerPhone
                anchors.margins: 5
                anchors.top: lePlace.bottom
                anchors.left: cbBuyerPhone.right
                anchors.right: parent.right
                text: rModel.buyerPhone
                onTextEdited:  {
//                    rModel.buyerPhone = leBuyerPhone.text
                }
            }
            Label {
                id: lBoxId
                anchors.left: parent.left
                anchors.margins: 5
                text: qsTr("Автомат")
                enabled: cbStatus.inAutoMode && rModel.canChangeBoxId
                anchors.verticalCenter:  leBoxId.verticalCenter
            }
            TextField {
                id: leBoxId
                anchors.margins: 5
                anchors.top: leBuyerPhone.bottom
                anchors.left: lBoxId.right
                anchors.right: parent.right
                maximumLength: 20
                text: rModel.boxId
                enabled: cbStatus.inAutoMode && rModel.canChangeBoxId
                onTextEdited: {
//                    rModel.boxId = leBoxId.text
                }
            }
            Label {
                id: lAdditionalParam
                anchors.left: parent.left
                anchors.margins: 5
                text: qsTr("Доп. рекв. чека")
                anchors.verticalCenter:  leAdditionalParam.verticalCenter
            }
            TextField {
                id: leAdditionalParam
                anchors.margins: 5
                anchors.top: leBoxId.bottom
                anchors.left: lAdditionalParam.right
                anchors.right: parent.right
                maximumLength: 20
                text: rModel.additionalParam
                onTextEdited: {
//                    rModel.boxId = leBoxId.text
                }
            }

            GroupBox {
                id: gbOperationalParam
                label: CheckBox {
                    id: cbOperationalParam
                    text: "Операционный реквизит чека"
                    onReleased: {
                        if(!cbOperationalParam.checked) {
                            rModel.cleanOperationalParam()
                        }
                    }
                }

                anchors.margins: 5
                anchors.left: parent.left
                anchors.right: parent.right
                anchors.top: leAdditionalParam.bottom
                height: 240
                contentWidth: width
                Label {
                    id: lOperationalCode
                    anchors.left: parent.left
                    anchors.margins: 5
                    text: qsTr("ИД")
                    anchors.verticalCenter:  sbOperationalCode.verticalCenter
                    enabled: cbOperationalParam.checked
                }
                SpinBox {
                    id: sbOperationalCode
                    anchors.margins: 5
                    anchors.top: parent.top
                    anchors.right: parent.right
                    from: 0
                    to: 255
                    editable: true
                    value: rModel.operationCode
                    enabled: cbOperationalParam.checked
                }
                Label {
                    id: lOperationalDt
                    anchors.left: parent.left
                    anchors.margins: 5
                    text: qsTr("Дата")
                    anchors.verticalCenter:  leOperationalDt.verticalCenter
                    enabled: cbOperationalParam.checked
                }
                TextField {
                    id: leOperationalDt
                    anchors.margins: 5
                    anchors.top: sbOperationalCode.bottom
                    anchors.right: parent.right
                    inputMethodHints: Qt.ImhDate
                    maximumLength: 16
                    inputMask: "99.99.9999 99:99;-"
                    text: rModel.operationDt
                    enabled: cbOperationalParam.checked
                }
                Label {
                    id: lOperationalData
                    anchors.left: parent.left
                    anchors.margins: 5
                    text: qsTr("Данные")
                    enabled: cbOperationalParam.checked
                    anchors.verticalCenter:  leOperationalData.verticalCenter
                }
                TextField {
                    id: leOperationalData
                    anchors.margins: 5
                    anchors.top: leOperationalDt.bottom
                    anchors.right: parent.right
                    maximumLength: 64
                    enabled: cbOperationalParam.checked
                    text: rModel.operationParamData
                }
            }
            GroupBox {
                id: gbUserParam
                label: CheckBox {
                    id: cbUserParam
                    text: "Дополнительный реквизит пользователя"
                    onReleased: {
                        if(!cbUserParam.checked) {
                            rModel.cleanUserParam()
                        }
                    }
                }

                anchors.margins: 5
                anchors.left: parent.left
                anchors.right: parent.right
                anchors.top: gbOperationalParam.bottom
                height: 200
                contentWidth: width
                Label {
                    id: lUserParamName
                    anchors.left: parent.left
                    anchors.margins: 5
                    text: qsTr("Наименование")
                    anchors.verticalCenter:  leUserParamName.verticalCenter
                    enabled: cbUserParam.checked
                }
                TextField {
                    id: leUserParamName
                    anchors.margins: 5
                    anchors.top: parent.top
                    anchors.right: parent.right
                    maximumLength: 64
                    text: rModel.additionalUserParamName
                    enabled: cbUserParam.checked
                }
                Label {
                    id: lUserParamVal
                    anchors.left: parent.left
                    anchors.right: parent.right
                    anchors.top: leUserParamName.bottom
                    horizontalAlignment: Label.AlignRight
                    anchors.margins: 5
                    text: qsTr("Значение")
                    enabled: cbUserParam.checked
                }
                TextField {
                    id: leUserParamVal
                    anchors.margins: 5
                    anchors.top: lUserParamVal.bottom
                    anchors.right: parent.right
                    anchors.left: parent.left
                    maximumLength: 256
                    enabled: cbUserParam.checked
                    text: rModel.additionalUserParamVal
                }
            }
            GroupBox {
                id: gbBuyer
                label: CheckBox {
                    id: cbBuyer
                    text: "Сведения о покупателе"
                    onReleased: {
                        if(!cbBuyer.checked) {
                            rModel.cleanBuyerInfo()
                        }
                    }
                }

                anchors.margins: 5
                anchors.left: parent.left
                anchors.right: parent.right
                anchors.top: gbUserParam.bottom
                height: 600
                contentWidth: width
                Label {
                    id: lBuyerName
                    anchors.left: parent.left
                    anchors.right: parent.right
                    anchors.top: parent.top
                    horizontalAlignment: Label.AlignRight
                    anchors.margins: 5
                    text: qsTr("Наименование")
                    enabled: cbBuyer.checked
                }
                TextField {
                    id: leBuyerName
                    anchors.margins: 5
                    anchors.top: lBuyerName.bottom
                    anchors.right: parent.right
                    anchors.left: parent.left
                    maximumLength: 256
                    enabled: cbBuyer.checked
                    text: rModel.buyerName
                }
                Label {
                    id: lBuyerAddress
                    anchors.left: parent.left
                    anchors.right: parent.right
                    anchors.top: leBuyerName.bottom
                    horizontalAlignment: Label.AlignRight
                    anchors.margins: 5
                    text: qsTr("Адрес")
                    enabled: cbBuyer.checked
                }
                TextField {
                    id: leBuyerAddress
                    anchors.margins: 5
                    anchors.top: lBuyerAddress.bottom
                    anchors.right: parent.right
                    anchors.left: parent.left
                    maximumLength: 256
                    enabled: cbBuyer.checked
                    text: rModel.buyerAddress
                }
                Label {
                    id: lBuyerInn
                    anchors.left: parent.left
                    anchors.margins: 5
                    enabled: cbBuyer.checked
                    text: qsTr("ИНН покупателя")
                    anchors.verticalCenter:  leBuyerInn.verticalCenter
                }
                TextField {
                    id: leBuyerInn
                    anchors.margins: 5
                    anchors.top: leBuyerAddress.bottom
                    anchors.right: parent.right
                    text: rModel.buyerInn
                    validator: InnValidator{}
                    inputMethodHints: Qt.ImhDigitsOnly
                    color: acceptableInput ? "black" : "red"
                    maximumLength: 12
                    enabled: cbBuyer.checked
                }
                Label {
                    id: lBuyerBirthday
                    anchors.left: parent.left
                    anchors.margins: 5
                    text: qsTr("Дата рождения")
                    anchors.verticalCenter:  leBuyerBirthday.verticalCenter
                    enabled: cbBuyer.checked
                }
                TextField {
                    id: leBuyerBirthday
                    anchors.margins: 5
                    anchors.top: leBuyerInn.bottom
                    anchors.right: parent.right
                    inputMethodHints: Qt.ImhDate
                    maximumLength: 10
                    inputMask: "99.99.9999;-"
                    text: rModel.buyerBirthDay
                    enabled: cbBuyer.checked
                }
                Label {
                    id: lBuyerCitizenship
                    anchors.left: parent.left
                    anchors.margins: 5
                    text: qsTr("Гражданство")
                    anchors.verticalCenter:  leBuyerCitizenship.verticalCenter
                    enabled: cbBuyer.checked
                }
                TextField {
                    id: leBuyerCitizenship
                    anchors.margins: 5
                    anchors.top: leBuyerBirthday.bottom
                    anchors.right: parent.right
                    maximumLength: 3
                    text: rModel.buyerCitizenship
                    enabled: cbBuyer.checked
                }
                Label {
                    id: lBuyerDocType
                    anchors.left: parent.left
                    anchors.margins: 5
                    text: qsTr("Код документа")
                    anchors.verticalCenter:  cbBuyerDocType.verticalCenter
                    enabled: cbBuyer.checked
                }
                ComboBox {
                    id: cbBuyerDocType
                    anchors.margins: 5
                    anchors.top: leBuyerCitizenship.bottom
                    anchors.right: parent.right
                    textRole: "code"
                    model: ListModel {
                        ListElement{code : ""}
                        ListElement{code : "21"}
                        ListElement{code : "22"}
                        ListElement{code : "26"}
                        ListElement{code : "27"}
                        ListElement{code : "28"}
                        ListElement{code : "31"}
                        ListElement{code : "32"}
                        ListElement{code : "33"}
                        ListElement{code : "34"}
                        ListElement{code : "35"}
                        ListElement{code : "36"}
                        ListElement{code : "37"}
                        ListElement{code : "38"}
                        ListElement{code : "40"}
                    }
                    currentIndex: rModel.buyerDocCodeIndex
                    enabled: cbBuyer.checked
                }
                Label {
                    id: lBuyerDocData
                    anchors.left: parent.left
                    anchors.right: parent.right
                    anchors.top: cbBuyerDocType.bottom
                    horizontalAlignment: Label.AlignRight
                    anchors.margins: 5
                    text: qsTr("Данные документа")
                    enabled: cbBuyer.checked
                }
                TextField {
                    id: leBuyerDocData
                    anchors.margins: 5
                    anchors.top: lBuyerDocData.bottom
                    anchors.right: parent.right
                    anchors.left: parent.left
                    maximumLength: 64
                    enabled: cbBuyer.checked
                    text: rModel.buyerDocData
                }
            }

            GroupBox {
                id: gbCorrectionReason
                anchors.margins: 5
                anchors.left: parent.left
                anchors.right: parent.right
                anchors.top: gbBuyer.bottom
                height: 200
                visible: rModel.isCorrection
                contentWidth: width
                title: "Основание для коррекции"
                CheckBox {
                    id: cbCorrectionIsIndependent
                    anchors.left: parent.left
                    anchors.margins: 5
                    anchors.top: parent.top
                    anchors.right: parent.right
                    checked: rModel.correctionIsIndependent
                    text: qsTr("Самостоятельня коррекция")
                }

                Label {
                    id: lReasonDate
                    anchors.left: parent.left
                    anchors.margins: 5
                    text: qsTr("Дата корр. расч.")
                    anchors.verticalCenter:  leReasonDate.verticalCenter
                }
                TextField {
                    id: leReasonDate
                    anchors.margins: 5
                    anchors.top: cbCorrectionIsIndependent.bottom
                    anchors.right: parent.right
                    inputMethodHints: Qt.ImhDate
                    maximumLength: 10
                    inputMask: "99.99.9999;-"
                    text: rModel.correctionDate
                }
                Label {
                    id: lReasonDoc
                    anchors.left: parent.left
                    anchors.right: parent.right
                    anchors.top: lReasonDate.bottom
                    horizontalAlignment: Label.AlignRight
                    anchors.margins: 5
                    text: qsTr("Номер предписания налогового органа")
                }
                TextField {
                    id: leReasonDoc
                    anchors.margins: 5
                    anchors.top: lReasonDoc.bottom
                    anchors.right: parent.right
                    anchors.left: parent.left
                    maximumLength: 32
                    text: rModel.correctionDocNumber
                }
            }
        }
    }
    ToolBar {
        id: footer
        height: 70
        ToolButton {
            id: tbBack
            width: 64
            height: 64
            anchors.left: parent.left
            anchors.verticalCenter: parent.verticalCenter
            anchors.leftMargin: 10
            anchors.rightMargin: 10
            anchors.topMargin: 5
            anchors.bottomMargin: 5
            icon.source: "qrc:/icons/back_64.png"
            icon.color: "transparent"
            icon.width: 60
            icon.height: 60
            onClicked: {
                receiptPropsView.closed()
            }
        }
        ToolButton {
            id: tbReset
            width: 64
            height: 64
            anchors.horizontalCenter: parent.horizontalCenter
            anchors.verticalCenter: parent.verticalCenter
            anchors.leftMargin: 10
            anchors.rightMargin: 10
            anchors.topMargin: 5
            anchors.bottomMargin: 5
            icon.source: "qrc:/icons/reset_64.png"
            icon.color: "transparent"
            icon.width: 64
            icon.height: 64
            onClicked: {
                resetData()
            }
        }
        ToolButton {
            id: tbSave
            width: 64
            height: 64
            anchors.right: parent.right
            anchors.verticalCenter: parent.verticalCenter
            anchors.leftMargin: 10
            anchors.rightMargin: 10
            anchors.topMargin: 5
            anchors.bottomMargin: 5
            icon.source: "qrc:/icons/apply_64.png"
            icon.color: "transparent"
            icon.width: 64
            icon.height: 64
            onClicked: {
                commit()
            }
        }
    }

    function resetData() {
        cbTax.currentIndex = rModel.taxIndex
        leCashier.text     = rModel.cashier
        leCashierInn.text  = rModel.cashierInn
        leUser.text        = rModel.ownerName
        leAddress.text     = rModel.address
        lePlace.text       = rModel.place
        leBuyerPhone.text  = rModel.buyerPhone
        leBoxId.text       = rModel.boxId
        leAdditionalParam.text  = rModel.additionalParam
        rModel.cleanOperationalParam()
        rModel.cleanUserParam()
        rModel.cleanBuyerInfo()
    }
    function commit() {
        rModel.taxIndex = cbTax.currentIndex
        rModel.cashier = leCashier.text
        rModel.cashierInn = leCashierInn.text
        rModel.ownerName = leUser.text
        rModel.address = leAddress.text
        rModel.place = lePlace.text
        rModel.buyerPhone = leBuyerPhone.text
        rModel.boxId = leBoxId.text
        rModel.additionalParam = leAdditionalParam.text
        if(!cbOperationalParam.checked) rModel.cleanOperationalParam()
        else {
            rModel.operationCode = sbOperationalCode.value
            rModel.operationDt = leOperationalDt.text
            rModel.operationParamData = leOperationalData.text
        }
        if(!cbUserParam.checked)rModel.cleanUserParam()
        else {
            rModel.additionalUserParamName = leUserParamName.text
            rModel.additionalUserParamVal = leUserParamVal.text
        }
        if(!cbBuyer.checked)rModel.cleanBuyerInfo()
        else
        {
            rModel.buyerName        = leBuyerName.text
            rModel.buyerInn         = leBuyerInn.text
            rModel.buyerBirthDay    = leBuyerBirthday.text
            rModel.buyerCitizenship = leBuyerCitizenship.text
            rModel.buyerDocCode     = cbBuyerDocType.currentText
            rModel.buyerDocData     = leBuyerDocData.text
            rModel.buyerAddress     = leBuyerAddress.text
        }
        if(rModel.isCorrection) {
            rModel.correctionDate = leReasonDate.text
            rModel.correctionDocNumber = leReasonDoc.text
            rModel.correctionIsIndependent = cbCorrectionIsIndependent.checked
        }
        closed()
    }

    onRModelChanged: {
        if(rModel) {
            cbTax.model = rModel.taxSystemModel
            cbTax.currentIndex = rModel.taxIndex
        }
    }

    Component.onCompleted: {
        width = parent.width
        height = parent.height


    }
}
