import QtQuick 2.12
import QtQuick.Controls 2.12
import Qt.labs.platform 1.1  as QtLabs1

import ru.armax.cashbox 1.0

//import ru.armax.cashbox.cashboxstatus 1.0
//import ru.armax.cashbox.approot 1.0
//import ru.armax.cashbox.receiptmodel 1.0
//import ru.armax.cashbox.receipthandler 1.0

Page {
    signal closed()
    id: receiptView
    title: rModel.receiptName
    objectName: "receiptView"
    property alias ownFooter: footer
    property alias receiptType: rModel.receiptType
    property alias isCorrection: rModel.isCorrection

    CashboxStatus {
        id: cbStatus
    }

    ReceiptHandler {
        id: rHandler
        onShowBusy: function(busy) {
            busyIndicator.running = busy
            receiptView.enabled = !busy
            ownFooter.enabled = !busy
            window.enableAll(!busy)
        }
        onError: function(txt) {
            rbSkipPrint.checked = false
            rHandler.skipPrint = false
            msgError.text = txt
            busyIndicator.running = false
            receiptView.enabled = true
            ownFooter.enabled = true
            window.enableAll(true)
            msgError.open()
        }
        onSuccess: {
            rbSkipPrint.checked = false
            rHandler.skipPrint = false
            receiptView.enabled = true
            ownFooter.enabled = true
            rModel.clean()
        }
    }

    ReceiptModel {
        id: rModel
    }


    BusyIndicator {
        id: busyIndicator
        anchors.horizontalCenter: parent.horizontalCenter
        anchors.verticalCenter: parent.verticalCenter
        running: false
        z: 10
    }

    QtLabs1.MessageDialog {
        id: msgError
        buttons: QtLabs1.MessageDialog.Close
        text: ""
    }
    ScrollView {
        id: receiptScrollView
        anchors.margins: 10
        anchors.fill: parent
        ScrollBar.vertical.policy: ScrollBar.AlwaysOff
        ScrollBar.horizontal.policy: ScrollBar.AlwaysOff
        z: 1
        ListView {
            id: lvOperations
            spacing: 10
            interactive: true
            orientation: Qt.Vertical
            highlightFollowsCurrentItem: true
            model: rModel
            z: 2
            delegate: Item {
                height: 80
                width: parent.width
                Image {
                    id: imgOperation
                    source: ricon
                    anchors.left: parent.left
                    anchors.verticalCenter: parent.verticalCenter;
                    width: 64
                    height: 64
                    fillMode: Image.PreserveAspectFit
                    z: 4
                }
                Label {
                    id: lOpName
                    text: rname
                    anchors.left: imgOperation.right
                    anchors.right: btnAddQ.left
                    anchors.margins: 10
                    anchors.top: parent.top
                    verticalAlignment: Label.AlignVCenter
                    wrapMode: Label.WordWrap
                    z: 4
                }
                Label {
                    id: lOpCost
                    text: rprice + " x " + rquantity
                    anchors.right: btnAddQ.left
                    anchors.bottom: parent.bottom
                    anchors.left: imgOperation.right
                    anchors.margins: 10
                    verticalAlignment: Label.AlignVCenter
                    z: 4
                }
                ToolButton {
                    id: btnAddQ
                    icon.source:  "qrc:/icons/plus_32.png"
                    icon.color: "transparent"
                    anchors.top: parent.top
                    anchors.right: parent.right
                    width: 40
                    height: 40
                    z: 4
                    onPressed: {
                        lvOperations.currentIndex = index
                        rModel.addQuantity(index)
                    }
                }
                ToolButton {
                    id: btnSubQ
                    icon.source:  "qrc:/icons/minus_32.png"
                    icon.color: "transparent"
                    anchors.bottom: parent.bottom
                    anchors.right: parent.right
                    width: 40
                    height: 40
                    z: 4
                    onPressed: {
                        lvOperations.currentIndex = index
                        rModel.subQuantity(index)
                    }
                }

                MouseArea {
                    anchors.fill: parent
                    onClicked: {
                        lvOperations.currentIndex = index
                    }
                    onPressAndHold: {
                        lvOperations.currentIndex = index
                        menuOp.y = mouse.y
                        if(menuOp.y + menuOp.heigh > receiptScrollView.height) {
                            menuOp.y = menuOp.y - menuOp.heigh
                        }

                        menuOp.open()
                    }
                }
                Rectangle{
                    id: hl
                    anchors.fill: parent
                    color: (lvOperations.currentIndex === index) ? "lightsteelblue" : "white"
                }
                Menu {
                    id: menuOp

                    MenuItem {
                        text: qsTr("Редактировать")
                        onTriggered: {
                            if(rModel.operationIsLabled(lvOperations.currentIndex)) {
                                var page = window.showPage("ReceiptOperationPage.qml");
                                page.opIndex = lvOperations.currentIndex
                                page.opData = rModel.operationData(lvOperations.currentIndex)
                                page.isSale = (receiptType === ReceiptModel.Debit)
                                page.eOperation.saveData.connect(rModel.addLabledOperation)
                            }
                            else {
                                var p = window.showPage("SimpleReceiptOperationPage.qml");
                                p.opIndex = lvOperations.currentIndex
                                p.opData = rModel.operationData(lvOperations.currentIndex)
                                p.eOperation.saveData.connect(rModel.addOperation)
                            }
                        }
                    }
                    MenuItem {
                        text: qsTr("Удалить")
                        onTriggered: {
                            rModel.removeOperation(lvOperations.currentIndex)
                        }
                    }
                }
            }
        }
    }

    RoundButton {
        id: rbProperties
        anchors.margins: 10
        anchors.bottom: parent.bottom
        anchors.left: parent.left
        icon.source: "qrc:/icons/receipt_props_48.png"
        icon.color: "transparent"
        icon.width: 50
        icon.height: 50
        z:100
        onPressed: {
            var page = window.showPage("ReceiptPropsView.qml");
            page.rModel = rModel
            page.resetData()
        }
    }
    RoundButton {
        id: rbSkipPrint
        anchors.margins: 10
        anchors.bottom: parent.bottom
        anchors.horizontalCenter: parent.horizontalCenter
        checkable: true
        icon.source: "qrc:/icons/skip_print_48.png"
        icon.color: "transparent"
        icon.width: 50
        icon.height: 50        
        z:100
        visible: !cbStatus.inOfflineMode
    }
    RoundButton {
        id: rbIndustryProps
        anchors.margins: 10
        anchors.bottom: parent.bottom
        anchors.right: parent.right
        icon.source: "qrc:/icons/industry_prop_48.png"
        icon.color: "transparent"
        icon.width: 50
        icon.height: 50
        z:100
        enabled: true /*rModel.containsLabels*/
        onPressed: {
            var page = window.showPage("IndustryPropsView.qml");
            page.owmModel.setProps(rModel.industryProps)
            page.owmModel.saveData.connect(rModel.setIndustryProps)

        }
    }
    ToolBar {
        id: footer
        height: 80
        ToolButton {
            id: tbBack
            width: 74
            height: 74
            anchors.left: parent.left
            anchors.verticalCenter: parent.verticalCenter
            anchors.margins: 5
            icon.source: "qrc:/icons/new_product_64.png"
            icon.color: "transparent"
            icon.width: 60
            icon.height: 60
            onClicked: {
                var page = window.showPage("SimpleReceiptOperationPage.qml");
                page.opIndex = -1
                page.eOperation.saveData.connect(rModel.addOperation)
            }
            onPressAndHold: {
                menuOperation.y = y - menuOperation.height
                menuOperation.x = x
                menuOperation.open()
            }
        }
        Label {
            anchors.centerIn: parent
            anchors.bottom: parent.bottom
            font.pixelSize: 16
            text: rModel.totalAmount
        }

        ToolButton {
            id: tbSave
            width: 74
            height: 74
            anchors.right: parent.right
            anchors.verticalCenter: parent.verticalCenter
            anchors.margins: 5
            icon.source: "qrc:/icons/pay_64.png"
            icon.color: "transparent"
            icon.width: 64
            icon.height: 64
            onClicked: {

            }
            onPressAndHold: {
                menuReceipt.y = y - menuReceipt.height
                menuReceipt.x = x - menuReceipt.width
                miLabledProduct.enabled = rModel.supportLabels
                menuReceipt.open()
            }
        }
        Menu {
            id: menuOperation
            MenuItem {
                id: miLabledProduct
                text: qsTr("Товар с маркировкой")
                enabled: rModel.supportLabels
                onTriggered: {
                    var page = window.showPage("ReceiptOperationPage.qml");
                    page.opIndex = -1
                    page.isSale = (receiptType === ReceiptModel.Debit)
                    page.eOperation.saveData.connect(rModel.addLabledOperation)
                }
            }
            MenuItem {
                text: qsTr("Другое")
                onTriggered: {
                    var page = window.showPage("SimpleReceiptOperationPage.qml");
                    page.opIndex = -1
                    page.eOperation.saveData.connect(rModel.addOperation)
                }
            }
        }
        Menu {
            id: menuReceipt
            enabled: lvOperations.count > 0

            MenuItem {
                text: qsTr("Оплатить наличными")
                enabled:  lvOperations.count > 0 && rModel.receiptAmount() > 0.0
                onTriggered: {
                    rHandler.skipPrint = rbSkipPrint.checked
                    if(isCorrection) rHandler.correctionFast(receiptType, rModel.receiptData(), rModel.receiptAmount(), 0.0)
                    else rHandler.receiptFast(receiptType, rModel.receiptData(), rModel.receiptAmount(), 0.0)
                }
            }
            MenuItem {
                text: qsTr("Оплатить картой")
                enabled:  lvOperations.count > 0 && rModel.receiptAmount() > 0.0
                onTriggered: {
                    rHandler.skipPrint = rbSkipPrint.checked
                    if(isCorrection) rHandler.correctionFast(receiptType, rModel.receiptData(), 0.0, rModel.receiptAmount())
                    else rHandler.receiptFast(receiptType, rModel.receiptData(), 0.0, rModel.receiptAmount())
                }
            }
            MenuItem {
                text: qsTr("Смешанная оплата")
                enabled:  lvOperations.count > 0 && rModel.receiptAmount() > 0.0
                onTriggered: {
                    var page = window.showPage("ReceiptPaymentPage.qml");
                    page.accepted.connect(makePayment)
                    page.amount = rModel.receiptAmount()

                    page.isSale = ((receiptType === ReceiptModel.Debit) && !rModel.isCorrection)
                    page.calcChange()
                }
            }
            MenuItem {
                enabled: lvOperations.count > 0
                text: qsTr("Очистить")
                onTriggered: {
                    rModel.clean()
                }
            }
        }
    }
    function makePayment(cash, card, prepay, postpay, barter) {
        rHandler.skipPrint = rbSkipPrint.checked
        console.log(rHandler.skipPrint.toString() + " " + rbSkipPrint.checked.toString())
        if(isCorrection) rHandler.correction(receiptType, rModel.receiptData(),
                                             cash, card, prepay, postpay, barter)
        else rHandler.receipt(receiptType, rModel.receiptData(),
                              cash, card, prepay, postpay, barter)
    }
}
