import QtQuick 2.12
import QtQuick.Controls 2.12
import Qt.labs.platform 1.1

import ru.armax.cashbox 1.0

//import ru.armax.cashbox.cashboxstatus 1.0
//import ru.armax.cashbox.approot 1.0

Page {
    width: parent.width
    height: parent.height
    signal closed()
    title: qsTr("Тестирование ККТ")
    id: selfTestPage
    objectName: "selfTestPage"
    property alias ownFooter: footer




    AppRoot {
        id: appRoot
        onShowBusy: function(busy) {
            busyIndicator.running = busy
            selfTestPage.enabled = !busy
            window.enableAll(!busy)
        }
        onError: function(txt) {
            msgError.text = txt
            busyIndicator.running = false
            selfTestPage.enabled = true
            window.enableAll(true)
            msgError.open()
        }

        onSuccessTest: {
            statusText.text = appRoot.testText()
            msgError.text = "Тестирование прошло успешно"
            busyIndicator.running = false
            selfTestPage.enabled = true
            window.enableAll(true)
            msgError.open()
        }
    }
    BusyIndicator {
        id: busyIndicator
        anchors.horizontalCenter: parent.horizontalCenter
        anchors.verticalCenter: parent.verticalCenter
        running: false
        z: 10
    }
    MessageDialog {
        id: msgError
        buttons: MessageDialog.Close
        text: ""
    }
    ScrollView {
        id: infoScrollView
        anchors.fill: parent
        ScrollBar.vertical.policy: ScrollBar.AlwaysOff
        ScrollBar.horizontal.policy: ScrollBar.AlwaysOff
        z:1
        TextArea {
            id: statusText
            anchors.fill: parent
            width: selfTestPage.width * 0.9
            text: ""
            wrapMode: TextEdit.WordWrap
            textMargin: 10
            textFormat: TextEdit.RichText
            readOnly: true
            z:2
        }
    }
    ToolBar {
        id: footer
        height: 70
        ToolButton {
            id: tbBack
            width: 64
            height: 64
            anchors.left: parent.left
            anchors.verticalCenter: parent.verticalCenter
            anchors.leftMargin: 10
            anchors.rightMargin: 10
            anchors.topMargin: 5
            anchors.bottomMargin: 5
            icon.source: "qrc:/icons/home_64.png"
            icon.color: "transparent"
            icon.width: 60
            icon.height: 60
            onClicked: {
                selfTestPage.closed()
            }
        }
        ToolButton {
            id: tbSave
            width: 64
            height: 64
            anchors.right: parent.right
            anchors.verticalCenter: parent.verticalCenter
            anchors.leftMargin: 10
            anchors.rightMargin: 10
            anchors.topMargin: 5
            anchors.bottomMargin: 5
            icon.source: "qrc:/icons/test_64.png"
            icon.color: "transparent"
            icon.width: 64
            icon.height: 64
            onClicked: {
                statusText.clear()
                appRoot.selfTest()
            }
        }
    }
    Component.onCompleted: {
        width = parent.width
        height = parent.height
        appRoot.reloadStatus()
    }
}
