import QtQuick 2.15
import QtQuick.Controls 2.15
import Qt.labs.platform 1.1 as QtLabs

import ru.armax.cashbox 1.0

//import ru.armax.cashbox.simplereceiptoperationwrapper 1.0

Page {
    width: parent.width
    height: parent.height
    id: opEditPage
    signal closed()
    title: qsTr("Предмет расчета")
    objectName: "opEditPage"
    property alias eOperation: operation
//    property alias ownFooter: footer
    property alias opIndex: operation.index
    property alias opData: operation.data
    property alias ownFooter: footer

    QtLabs.MessageDialog {
        id: msgError
        buttons: QtLabs.MessageDialog.Close
        text: ""
    }
    SimpleReceiptOperationWrapper {
        id: operation

        onError: function(err){
            msgError.text = err
            msgError.open()
        }
        onFinish: {
            opEditPage.closed()
        }
        onShowBusy: function(busy) {
            busyIndicator.running = busy
            opEditPage.enabled = !busy
            window.enableAll(!busy)
        }
        onBarcodeChanged: {
            lBarcode.text = operation.labelText()
        }
    }
    BusyIndicator {
        id: busyIndicator
        anchors.horizontalCenter: parent.horizontalCenter
        anchors.verticalCenter: parent.verticalCenter
        running: false
        z: 10
    }

    Dialog {
        id: dlgLabel
        title: "Введите штрихкод"
        width: parent.width
        height: parent.height / 2
        standardButtons: Dialog.Ok | Dialog.Close
        TextEdit {
            id:teLabel
            text: ""
            anchors.fill: parent
            wrapMode: TextEdit.Wrap
        }
        onAccepted:  {
            operation.parseBarcode(teLabel.text)
        }
    }
    ScrollView {
        id: infoScrollView
        anchors.fill: parent
        anchors.margins: 0
        ScrollBar.vertical.policy: ScrollBar.AlwaysOff
        ScrollBar.horizontal.policy: ScrollBar.AlwaysOff
        width: opEditPage.width
        height: opEditPage.height
        contentHeight: opEditPage.height * 6
        contentWidth: parent.width - anchors.margins
        Item {
            anchors.fill: parent
            height:  opEditPage.height * 6
            width: opEditPage.width - opEditPage.anchors.margins
            ComboBox {
                id: cbType
                model: operation.opTypeModel
                textRole: "name"
                anchors.top: parent.top
                anchors.left:  parent.left
                anchors.right: parent.right
                anchors.margins: 5
                currentIndex: operation.opTypeIndex
                onCurrentIndexChanged: {
                    operation.opType = cbType.model[cbType.currentIndex].code
                }
            }
            TextField {
                id: txtName
                text: operation.name
                placeholderText: "Наименование товара/услуги"
                anchors.top: cbType.bottom
                anchors.left: parent.left
                anchors.right: parent.right
                anchors.margins: 5
                maximumLength: 128
                onTextEdited: {
                    operation.name = txtName.text
                }
            }
            ComboBox {
                id: cbVat
                model: operation.vatModel
                textRole: "name"
                anchors.top: txtName.bottom
                anchors.right: parent.right
                anchors.margins: 5
                currentIndex: operation.vatRate - 1
                onCurrentIndexChanged: {
                    operation.vatRate = cbVat.currentIndex + 1
                }
            }
            ComboBox {
                id: cbPayType
                model: operation.payTypeModel
                textRole: "name"
                anchors.top: txtName.bottom
                anchors.left:  parent.left
                anchors.right: cbVat.left
                anchors.margins: 5
                currentIndex: operation.payType - 1
                onCurrentIndexChanged: {
                    operation.payType = cbPayType.currentIndex + 1
                }
            }
            Label
            {
                id:lPrice
                anchors.top: cbPayType.bottom
                anchors.horizontalCenter: lePrice.horizontalCenter
                text: qsTr("Цена")
                font.pixelSize: 10
            }
            Label
            {
                id:lQUantity
                anchors.verticalCenter: lPrice.verticalCenter
                anchors.horizontalCenter: leQuantity.horizontalCenter
                text: qsTr("Количество")
                font.pixelSize: 10
            }
            TextField
            {
                id: lePrice
                anchors.margins: 5
                anchors.top: lPrice.bottom
                anchors.left: parent.left
                width: parent.width / 2 - 10
                inputMethodHints: Qt.ImhFormattedNumbersOnly
                text: operation.price
                validator: DoubleValidator{bottom: 0.0; top: 2147483647.0; decimals: 2}
                horizontalAlignment: TextField.AlignHCenter
                onEditingFinished: {
                    operation.price = lePrice.text
                }
        //        onPressed: {
        //            forceActiveFocus(Qt.MouseFocusReason)
        //            Qt.inputMethod.show()
        //        }
            }
            TextField
            {
                id: leQuantity
                anchors.margins: 5
                anchors.top: lPrice.bottom
                anchors.left: lePrice.right
                anchors.right: parent.right
                width: parent.width / 2 - 10
                inputMethodHints: Qt.ImhFormattedNumbersOnly
                text: operation.quantity
                validator: DoubleValidator{bottom: 0.0; top: 2147483647.0; decimals: 3}
                horizontalAlignment: TextField.AlignHCenter
                onEditingFinished: {
                    operation.quantity = leQuantity.text
                }
        //        onPressed: {
        //            forceActiveFocus(Qt.MouseFocusReason)
        //            Qt.inputMethod.show()
        //        }
            }
            Label {
                id: lUnit
                anchors.left: parent.left
                anchors.verticalCenter: cbUnit.verticalCenter
                anchors.margins: 5
                text: "Мера количества"
            }
            ComboBox
            {
                id: cbUnit
                anchors.margins: 5
                anchors.top: leQuantity.bottom
                anchors.left: lUnit.right
                anchors.right: parent.right
                textRole: "name"
                model: operation.itemUnitModel
                currentIndex:  operation.itemUnit
                onCurrentIndexChanged: {
                    operation.itemUnit = cbUnit.currentIndex
                }
            }
            TextField {
                id: leAdditional
                text: operation.additionalParam
                placeholderText: "Дополнительный реквизит предмета расчета"
                anchors.top: cbUnit.bottom
                anchors.left: parent.left
                anchors.right: parent.right
                anchors.margins: 5
                maximumLength: 64
                onTextEdited: {
                    operation.additionalParam = leAdditional.text
                }
            }
            Label {
                id: lCountryCode
                anchors.left: parent.left
                anchors.verticalCenter: leCountryCode.verticalCenter
                horizontalAlignment: Label.AlignLeft
                anchors.margins: 5
                text: qsTr("Код страны происхождения товара")
            }
            TextField {
                id: leCountryCode
                anchors.margins: 5
                anchors.top: leAdditional.bottom
                anchors.right: parent.right
                anchors.left: lCountryCode.right
                maximumLength: 3
                text: operation.countryCode
                onTextEdited: {
                    operation.countryCode = leCountryCode.text
                }
            }
            Label {
                id: lDeclNumber
                anchors.left: parent.left
                anchors.verticalCenter: leDeclNumber.verticalCenter
                horizontalAlignment: Label.AlignLeft
                anchors.margins: 5
                text: qsTr("Номер декларации")
            }
            TextField {
                id: leDeclNumber
                anchors.margins: 5
                anchors.top: leCountryCode.bottom
                anchors.right: parent.right
                anchors.left: lDeclNumber.right
                maximumLength: 32
                text: operation.declarationNumber
                onTextEdited: {
                    operation.declarationNumber = leDeclNumber.text
                }
            }
            Label {
                id: lBarcode
                text: ""
                anchors.top:  leDeclNumber.bottom
                anchors.left: parent.left
                anchors.right: parent.right
                width: parent.width - 10
                wrapMode: Label.WordWrap
                horizontalAlignment: Label.AlignHCenter
                anchors.margins: 5
            }
            Label {
                id: lExciseTax
                anchors.left: parent.left
                anchors.verticalCenter: leExciseTax.verticalCenter
                horizontalAlignment: Label.AlignLeft
                anchors.margins: 5
                visible: operation.opType == SimpleReceiptOperationWrapper.ExcisableProduct
                text: qsTr("Акциз")
            }
            TextField {
                id: leExciseTax
                anchors.margins: 5
                anchors.top: lBarcode.bottom
                anchors.right: parent.right
                anchors.left: lExciseTax.right
                inputMethodHints: Qt.ImhFormattedNumbersOnly
                text: operation.exciseTax
                visible: operation.opType == SimpleReceiptOperationWrapper.ExcisableProduct
                validator: DoubleValidator{bottom: 0.0; top: 2147483647.0; decimals: 2}
                horizontalAlignment: TextField.AlignHCenter
                onEditingFinished: {
                    operation.exciseTax = leExciseTax.text
                }
            }
        }
    }


    RoundButton {
        id: rbAgent
        anchors.margins: 10
        anchors.bottom: parent.bottom
        anchors.left: parent.left
        icon.source: "qrc:/icons/agent_64.png"
        icon.color: "transparent"
        icon.width: 50
        icon.height: 50
        z:100
        onPressed: {
            var page = window.showPage("AgentDataPage.qml");
            page.ownAgent.setData(operation.agentData())
            page.ownAgent.saveData.connect(operation.setAgentData)
        }
    }
    RoundButton {
        id: rbIndustryProps
        anchors.margins: 10
        anchors.bottom: parent.bottom
        anchors.right: parent.right
        icon.source: "qrc:/icons/industry_prop_48.png"
        icon.color: "transparent"
        icon.width: 50
        icon.height: 50
        z:100
        onPressed: {
            var page = window.showPage("IndustryPropsView.qml");
            page.owmModel.setProps(operation.industryProps)
            page.owmModel.saveData.connect(operation.setIndustryProps)
        }
    }

    ToolBar {
        id: footer
        height: 70
        ToolButton {
            id: tbBack
            width: 64
            height: 64
            anchors.left: parent.left
            anchors.verticalCenter: parent.verticalCenter
            anchors.margins: 5
            icon.source: "qrc:/icons/back_64.png"
            icon.color: "transparent"
            icon.width: 60
            icon.height: 60
            onClicked: {
                opEditPage.closed()
            }
        }
        ToolButton {
            id: tbScanBarcode
            width: 64
            height: 64
            anchors.verticalCenter: parent.verticalCenter
            anchors.horizontalCenter: parent.horizontalCenter
            anchors.margins: 5
            icon.source: "qrc:/icons/barcode_64.png"
            icon.color: "transparent"
            icon.width: 64
            icon.height: 64
            onClicked: scanBarcode()
            onPressAndHold: {
                dlgLabel.open()
            }
        }
        ToolButton {
            id: tbSave
            width: 64
            height: 64
            anchors.right: parent.right
            anchors.verticalCenter: parent.verticalCenter
            anchors.margins: 5
            icon.source: "qrc:/icons/submit_64.png"
            icon.color: "transparent"
            icon.width: 64
            icon.height: 64
            onClicked: {
                operation.opType = cbType.model[cbType.currentIndex].code
                operation.name = txtName.text
                operation.vatRate = cbVat.currentIndex + 1
                operation.payType = cbPayType.currentIndex + 1
                operation.price = lePrice.text
                operation.quantity = leQuantity.text
                operation.additionalParam = leAdditional.text
                operation.countryCode = leCountryCode.text
                operation.declarationNumber = leDeclNumber.text
                operation.exciseTax = leExciseTax.text
                operation.itemUnit = cbUnit.currentIndex
                operation.save()
            }
        }
    }
    function processBarcode(text) {
        operation.parseBarcode(text)

        window.popPage()
    }

    function scanBarcode() {
        var page = window.showPage("BarcodePage.qml");
        page.newBarcode.connect(processBarcode)
    }

    function parseProduct(prod) {
        operation.parseProductMap(prod)
    }

    Component.onCompleted: {

    }
}
