import QtQuick 2.12
import QtQuick.Controls 2.12
import Qt.labs.platform 1.1
import ru.armax.cashbox 1.0

Page {
    width: parent.width
    height: parent.height
    id: pgSerialWrite
    title: qsTr("Установка ЗН")
    signal closed()
    objectName: "writeSerialPage"

    CashboxStatus {
        id: cbStatus
    }
    FiscalizationHandler {
        id: fiscalizator
        onShowBusy: function(busy) {
            busyIndicator.running = busy
            pgSerialWrite.enabled = !busy
        }
        onError: function(txt) {
            msgError.text = txt
            busyIndicator.running = false
            pgSerialWrite.enabled = true
            msgError.open()

        }
        onSerialWrote: {
            pgSerialWrite.closed()
        }
    }
    BusyIndicator {
        id: busyIndicator
        anchors.horizontalCenter: parent.horizontalCenter
        anchors.verticalCenter: parent.verticalCenter
        running: false
        z: 10
    }
    MessageDialog {
        id: msgError
        buttons: MessageDialog.Close
        text: ""
    }
    Label {
        id: lSerial
        text: qsTr("Заводской номер")
        anchors.horizontalCenter: parent.horizontalCenter
        anchors.topMargin: 50
        anchors.top: parent.top
        anchors.bottomMargin: 10
        font.pixelSize: 32
        color: "blue"
        z: 1
    }
    TextField {
        id: txtSerial;
        text: ""
        anchors.horizontalCenter: parent.horizontalCenter
        anchors.top: lSerial.bottom
        inputMethodHints: Qt.ImhDigitsOnly
        maximumLength: 12
        inputMask:  "2\\0\\0599999999;*"
//        inputMask:  "99999999;*"
        font.pixelSize: 32
        width: contentWidth * 1.2
        horizontalAlignment: Text.AlignHCenter
        anchors.margins: 10
        z: 1
        onPressed: {
            forceActiveFocus(Qt.MouseFocusReason)
            Qt.inputMethod.show()
        }

    }
    Button {
        id: btnWrite
        width: txtSerial.width
        height: 80
        anchors.horizontalCenter: parent.horizontalCenter
        anchors.margins: 10
        anchors.top: txtSerial.bottom
        text: qsTr("Записать")
        onPressed: {
            fiscalizator.writeSerial(txtSerial.text)
        }
        z: 1
    }

    Component.onCompleted: {
        txtSerial.forceActiveFocus()
        txtSerial.focus = true
        txtSerial.cursorPosition = txtSerial.text.length
    }
}
