import QtQuick 2.15
import QtQuick.Controls 2.15
import QtQuick.Window 2.15
import ru.armax.cashbox 1.0
//import QtGraphicalEffects 1.15

//import ru.armax.cashbox.cashboxstatus 1.0
//import ru.armax.cashbox.sessionwrapper 1.0
//import ru.armax.cashbox.mainmenumodel 1.0

ApplicationWindow {
    id: window
    width: Screen.desktopAvailableWidth
    height: Screen.desktopAvailableHeight
    visible: true

    property alias mainStackView: stackView

    title: qsTr("")

    AppRoot {
        id: appRoot
        onPopPage: {
            window.popPage()
        }
        onSuccessNotPrinted: {
            checkNotPrinted()
        }
    }

    CashboxStatus {
        id: cbStatus
        onStatusChanged: {
            lCashier.text = cbSession.cashierName
            lOrg.text = cbStatus.orgName
            lAddr.text = cbStatus.orgAddress
            if(cbStatus.cycleIsOpen){
                imgCycle.source = "qrc:/icons/open_32.png"
                lCycleHours.visible = true
                lCycleHours.text = cbStatus.cycleOpenedHours
                lCycleNumber.visible = true
                lCycleNumber.text = "№ " + cbStatus.cycleNumber
            }
            else {
                imgCycle.source = "qrc:/icons/closed_32.png"
                lCycleHours.visible = false
                lCycleNumber.visible = false
            }
        }
    }
    SessionWrapper {
        id: cbSession
        onCashierChanged: {
            lCashier.text = cbSession.cashierName
        }
    }

    MainMenuModel {
        id: mmModel
        onShowPage: function(page) {
            window.showPage(page)
        }

        onShowFirstPage: function(page) {
            window.showFirstPage(page)
        }
    }

    header: ToolBar {
        contentHeight: toolButton.implicitHeight

        ToolButton {
            id: toolButton
            icon.source: "qrc:/icons/menu_48.png"
            icon.color: "transparent"
            icon.width: toolButton.width
            icon.height: toolButton.height
            font.pixelSize: Qt.application.font.pixelSize * 1.6
            visible: stackView.currentItem.objectName != "loginPage"
            onClicked: {
                drawer.open()
            }

        }

        Label {
            text: appRoot.appHeader
            anchors.horizontalCenter: parent.horizontalCenter
            anchors.topMargin: 5
            anchors.top: parent.top
            maximumLineCount: 2
            horizontalAlignment: Text.AlignHCenter
        }
        Label {
            id: lbTitle
            text: ""
            anchors.horizontalCenter: parent.horizontalCenter
            anchors.bottomMargin: 5
            anchors.bottom: parent.bottom
            maximumLineCount: 2
            horizontalAlignment: Text.AlignHCenter
        }
        ToolButton {
            id: tbPrintLast
            icon.source: "qrc:/icons/printer_error_48.png"
            icon.color: "transparent"
            icon.width: 48
            icon.height: 48
            font.pixelSize: Qt.application.font.pixelSize * 1.6
            visible: stackView.currentItem.objectName != "loginPage" && cbStatus.hasNotPrinted
            anchors.right: parent.right
            onClicked: {
                appRoot.printNotPrinted()
            }

        }
    }

    Drawer {
        id: drawer
        width: window.width * 0.90
        height: window.height
        ToolBar {
            id: menHeader
            contentHeight: Math.max(imgCashier.implicitHeight + lCashier.implicitHeight + lIp.implicitHeight,
                               lOrg.implicitHeight + lAddr.implicitHeight + lPlace.implicitHeight) + 20
//            height: parent.height /4
            anchors.top: parent.top
            anchors.left: parent.left
            anchors.right: parent.right
            Image {
                id: imgCashier
                source: "qrc:/icons/cashier_48.png"
                anchors.horizontalCenter: lCashier.horizontalCenter
                anchors.top: parent.top
                anchors.margins: 5
            }
            Label {
                id: lCashier
                anchors.margins: 5
                anchors.left: parent.left
                anchors.top: imgCashier.bottom
                width: parent.width / 4
                text: cbSession.cashierName
                wrapMode: Text.WordWrap
                fontSizeMode:Text.Fit
                maximumLineCount: 3
                horizontalAlignment: Text.AlignHCenter
            }
            Label {
                id: lIp
                anchors.margins: 5
                anchors.left: parent.left
                anchors.right: parent.right
                anchors.top: lCashier.bottom
                width: lCashier.width
                text: cbStatus.ipAddress
                wrapMode: Text.WordWrap
                fontSizeMode:Text.Fit
                maximumLineCount: 2
                horizontalAlignment: Text.AlignLeft
            }
            Label {
                id: lOrg
                anchors.margins: 5
                anchors.left: lCashier.right
                anchors.right: parent.right
//                anchors.top: parent.top
                anchors.verticalCenter: imgCashier.verticalCenter
                width: 3 * parent.width / 4
                text: cbStatus.orgName
                wrapMode: Text.WordWrap
                maximumLineCount: 2
                fontSizeMode:Text.Fit
                horizontalAlignment: Text.AlignHCenter
            }
            Label {
                id: lAddr
                anchors.margins: 5
                anchors.left: lCashier.right
                anchors.right: parent.right
//                anchors.top: lOrg.bottom
                anchors.verticalCenter: lCashier.verticalCenter
                width: lOrg.width
                text: cbStatus.orgAddress
                wrapMode: Text.WordWrap
                fontSizeMode:Text.Fit
                maximumLineCount: 2
                horizontalAlignment: Text.AlignHCenter
            }
            Label {
                id: lPlace
                anchors.margins: 5
                anchors.left: lCashier.right
                anchors.right: parent.right
//                anchors.top: lAddr.bottom
                anchors.verticalCenter: lIp.verticalCenter
                width: lOrg.width
                text: cbStatus.place
                wrapMode: Text.WordWrap
                fontSizeMode:Text.Fit
                maximumLineCount: 2
                horizontalAlignment: Text.AlignHCenter
            }

        }
        ToolBar {
            id:menFooter
            contentHeight: imgCycle.implicitHeight
            anchors.bottom: parent.bottom
            anchors.left: parent.left
            anchors.right: parent.right
            Image {
                id:imgCycle
                source: cbStatus.cycleIsOpen ? "qrc:/icons/open_32.png" :"qrc:/icons/closed_32.png"
                anchors.left: parent.left
                anchors.verticalCenter: parent.verticalCenter;
                anchors.rightMargin: 5
                width: 40
                height: 40
                fillMode: Image.PreserveAspectFit
            }
            Label {
                id: lCycleHours
                visible: cbStatus.cycleIsOpen
                text: cbStatus.cycleOpenedHours
                horizontalAlignment: Text.AlignHCenter
                verticalAlignment: Text.AlignVCenter
                anchors.top: parent.top
                anchors.left: imgCycle.right
                anchors.leftMargin: 5
            }
            Label {
                id: lCycleNumber
                visible: cbStatus.cycleIsOpen
                text: "№ " + cbStatus.cycleNumber
                horizontalAlignment: Text.AlignHCenter
                verticalAlignment: Text.AlignVCenter
                anchors.top: lCycleHours.bottom
                anchors.left: imgCycle.right
                anchors.leftMargin: 5
            }
            ToolButton {
                id:btnBack
                icon.source: mmModel.isSubMenu ? "qrc:/icons/back_40.png" :"qrc:/icons/exit_40.png"
                icon.color: "transparent"
                icon.width: 40
                icon.height: 40
                anchors.right: parent.right
                anchors.verticalCenter: parent.verticalCenter;
                anchors.margins: 2
                anchors.rightMargin: 5
                width: parent.height
                height: parent.height
                onPressed: {
                    mmModel.isSubMenu ? mmModel.goBack() : appRoot.restart()
                }
            }
        }
        GridView {
            anchors.top: menHeader.bottom
            anchors.bottom: menFooter.top
            anchors.left: parent.left
            anchors.right: parent.right
            leftMargin: 5
            topMargin: 5
            rightMargin: 5
            bottomMargin: 5
//            anchors.margins: 5

            cellWidth: (parent.width - 20)/ 3; cellHeight: (parent.width - 20) / 3
            focus: true
            model: mmModel

            highlight: Rectangle { width: parent.cellWidth; height: parent.cellHeight; color: "lightsteelblue"; radius: 5 }

            delegate: Item {
                width: parent == null ? 1 : (parent.width - 20) / 3;
                height: parent == null ? 1 : (parent.width - 20) / 3
                Rectangle {
                    id: rItem
                    anchors.margins: 5
                    anchors.fill: parent
                    border.width: 0
                    radius: 8
                    Image {
                        id: myIcon
                        y: 10;
                        anchors.horizontalCenter: parent.horizontalCenter
                        source: mmicon
                        width: rItem.width /2
                        height: rItem.width / 2
                    }
                    Label
                    {   id: lItem
                        text: mmlabel
                        anchors.top: myIcon.bottom
                        anchors.bottom: parent.bottom
                        anchors.left: parent.left
                        anchors.right: parent.right
                        horizontalAlignment: Text.AlignHCenter
                        verticalAlignment: Text.AlignBottom
                        lineHeight: 0.75
                        width: parent.width
                        anchors.leftMargin: 2
                        anchors.rightMargin: 2
                        fontSizeMode: Text.Fit
                    }
                }
//                DropShadow {
//                        id: dshItem
//                        anchors.fill: rItem
//                        horizontalOffset: 3
//                        verticalOffset: 3
//                        radius: 8.0
//                        samples: 17
//                        color: "#80000000"
//                        source: rItem
//                        visible: myIcon.source.length > 0 || lItem.text.length > 0
//                    }
                MouseArea {
                    anchors.fill: parent

                    onReleased:  {
//                        dshItem.visible = myIcon.source.length > 0 || lItem.text.length > 0
                        rItem.border.width = 0
                        parent.GridView.view.currentIndex = index
                        var idx = index
                        var d = mmModel.get(index)
                        mmModel.action(d.mmaction)
                    }
                    onPressed: {
//                        dshItem.visible = false
                        rItem.border.width = myIcon.source.length > 0 || lItem.text.length > 0 ? 1 : 0
                    }
                }
            }
        }
    }

    StackView {
        id: stackView
        initialItem: "LoginForm.qml"
        anchors.fill: parent
    }
    function showFirstPage(page) {
        stackView.clear()
        stackView.push(page)
        console.log(stackView.currentItem.toString())
        stackView.currentItem.closed.connect(popPage)
        drawer.close()
        if(stackView.currentItem.hasOwnProperty("ownFooter")) {
            footer = stackView.currentItem.ownFooter
            defaultFooter.visible = false;
        }
        else {
            footer = defaultFooter
        }
        footer.visible = true
        lbTitle.text = stackView.currentItem.hasOwnProperty("title") ? stackView.currentItem.title : ""
        return stackView.currentItem
    }
    Timer {
        id: timerHome
        interval: 100
        repeat: false
        triggeredOnStart: false
        onTriggered: {
            showFirstPage(appRoot.homePage)
        }
    }

    function goHome() {
        timerHome.start()
    }

    function showPage(page) {
        console.log(page.toString())
        stackView.push(page)
        stackView.currentItem.closed.connect(popPage)
        drawer.close()
        if(stackView.currentItem.hasOwnProperty("ownFooter")) {
            footer = stackView.currentItem.ownFooter
            defaultFooter.visible = false;
        }
        else {
            footer = defaultFooter
        }
        footer.visible = true

        lbTitle.text = stackView.currentItem.hasOwnProperty("title") ? stackView.currentItem.title : ""

        return stackView.currentItem
    }
    function popPage() {
        console.log(stackView.depth.toString())
        if(stackView.depth > 1) {
            stackView.pop()
            if(stackView.currentItem.hasOwnProperty("ownFooter")) {
                footer = stackView.currentItem.ownFooter
                defaultFooter.visible = false;
            }
            else {
                footer = defaultFooter
            }
            footer.visible = true
            lbTitle.text = stackView.currentItem.hasOwnProperty("title") ? stackView.currentItem.title : ""
        }
        else if(appRoot.loginned) {
            goHome()
        }
        else {
            showFirstPage("LoginForm.qml")
        }
    }
    function currentPageName() {
        if(stackView.depth === 0) return ""
        return stackView.currentItem.objectName
    }
    function currentPage() {
        return stackView.currentItem
    }
    function checkNotPrinted() {
        tbPrintLast.visible= stackView.depth > 0 &&
                stackView.currentItem.objectName != "loginPage" &&
                cbStatus.hasNotPrinted
    }

    ToolBar {
        id: defaultFooter
        visible: false
        Label {
            anchors.horizontalCenter: parent.horizontalCenter
            anchors.verticalCenter: parent.verticalCenter
            text: cbStatus.fiscalModuleSerial ? "" :cbStatus.fiscalModuleSerial
        }
    }
    footer: defaultFooter
    Keys.onReleased: {
        if (event.matches(StandardKey.Back)) {
            popPage()
            event.accepted = true;
        }
    }

    function enableAll(e) {
        footer.enabled = e
        header.enabled = e
        drawer.enabled = e
    }


//    Component.onCompleted: {
//        showFirstPage("LoginForm.qml")
//    }

}
