#include "cashboxroot.h"

#include "appdirectories.h"
#include "loggermanager.h"
#include "cashboxbuildconfig.h"
#include "localizationmgr.h"
#include "logmanip.h"
#include "androidapihelper.h"
#include "approothandler.h"
#include "cashboxstatuswrapper.h"
#include "incomingregwrapper.h"
#include "loginhandler.h"
#include "mainmenumodel.h"
#include "fiscalizationhandler.h"
#include "innvalidator.h"
#include "regnumbervalidator.h"
#include "hexstringvalidator.h"
#include "baseworkincomingwrapper.h"
#include "reportscorehandler.h"
#include "receiptmodel.h"
#include "receipthandler.h"
#include "receiptoperationwrapper.h"
#include "sbarcodefilter.h"
#include "agentdatawrapper.h"
#include "industrypropsmodel.h"
#include "simplereceiptoperationwrapper.h"
#include "ofdsettingswrapper.h"
#include "sendmailhelper.h"
#include "cashiersmodel.h"
#include "cashierwrapper.h"

#include <QQmlApplicationEngine>

#include <QtAndroidExtras/QAndroidJniObject>
#include <QtAndroidExtras/QtAndroid>
#include <QAndroidJniEnvironment>
#include <QEvent>
#include <QTextCodec>
#include <QTranslator>

#include <android/log.h>
#include <QDebug>
#include <QKeyEvent>



CashboxRoot::CashboxRoot(int &argc, char **argv, const char * applicationName)
    : BaseQmlAppRoot(argc, argv, applicationName)
{
    SIPRequestEater *sipRequestEater = new SIPRequestEater(this);
    connect(sipRequestEater, &SIPRequestEater::popPage, this, &CashboxRoot::popPage);
    qApp->installEventFilter(sipRequestEater);
}

CashboxRoot::~CashboxRoot()
{

}

bool CashboxRoot::beforeQmlLoading()
{
    ADD_LOCALISATION_FILE(APPNAME);
    LoggerManager::setDebugOuts(LoggerManager::QDO_BOTH);
    LoggerManager::setKeepLogHandlers(true);
    LoggerManager::initLog(APATH.logsPath(), 7, true);

    QStringList locales = LocalizationManager::localisations();
    locales.push_front("qtbase");
    locales.push_front("qt");
    locales.push_front("qtdeclarative");
    locales.push_front("qtmultimedia");
    locales.push_front("qtquickcontrols");
    QLocale::setDefault(QLocale::system());

    for(const QString &l :locales)
    {
        QTranslator* translator = new QTranslator(this);
        if(translator->load(QLocale::system(), l, "_", APATH.translations(), ".qm"))
        {
            qApp->installTranslator(translator);
        }
    }

    QTextCodec::setCodecForLocale(QTextCodec::codecForName("UTF-8"));

    qRegisterMetaType<BaseCatalogModel*>();
//    qmlRegisterType<AppRootHandler>("ru.armax.cashbox.approot", 1, 0, "AppRoot");
//    qmlRegisterType<LoginHandler>("ru.armax.cashbox.loginhandler", 1, 0, "LoginHandler");
//    qmlRegisterType<CashboxStatusWrapper>("ru.armax.cashbox.cashboxstatus", 1, 0, "CashboxStatus");
//    qmlRegisterType<FiscalizationHandler>("ru.armax.cashbox.fiscalizationhandler", 1, 0, "FiscalizationHandler");
//    qmlRegisterType<IncomingRegWrapper>("ru.armax.cashbox.incomingregwrapper", 1, 0, "IncomingRegWrapper");
//    qmlRegisterType<MainMenuModel>("ru.armax.cashbox.mainmenumodel", 1, 0, "MainMenuModel");
//    qmlRegisterType<InnValidator>("ru.armax.cashbox.innvalidator", 1, 0, "InnValidator");
//    qmlRegisterType<RegNumberValidator>("ru.armax.cashbox.regnumbervalidator", 1, 0, "RegNumberValidator");
//    qmlRegisterType<HexStringValidator>("ru.armax.cashbox.hexstringvalidator", 1, 0, "HexStringValidator");
//    qmlRegisterType<CycleOpenIncomingWrapper>("ru.armax.cashbox.cycleopenincomingwrapper", 1, 0, "CycleOpenIncomingWrapper");
//    qmlRegisterType<CycleCloseIncomingWrapper>("ru.armax.cashbox.cyclecloseincomingwrapper", 1, 0, "CycleCloseIncomingWrapper");
//    qmlRegisterType<CalcReportIncomingWrapper>("ru.armax.cashbox.calcreportincomingwrapper", 1, 0, "CalcReportIncomingWrapper");
//    qmlRegisterType<FsCloseIncomingWrapper>("ru.armax.cashbox.fscloseincomingwrapper", 1, 0, "FsCloseIncomingWrapper");
//    qmlRegisterType<ReportsCoreHandler>("ru.armax.cashbox.reportscorehandler", 1, 0, "ReportsCoreHandler");
//    qmlRegisterType<ReceiptModel>("ru.armax.cashbox.receiptmodel", 1, 0, "ReceiptModel");
//    qmlRegisterType<ReceiptHandler>("ru.armax.cashbox.receipthandler", 1, 0, "ReceiptHandler");
//    qmlRegisterType<ReceiptOperationWrapper>("ru.armax.cashbox.receiptoperationwrapper", 1, 0, "ReceiptOperationWrapper");
//    qmlRegisterType<SBarcodeFilter>("ru.armax.cashbox.sbarcodefilter", 1, 0, "SBarcodeFilter");
//    qmlRegisterType<AgentDataWrapper>("ru.armax.cashbox.agentdatawrapper", 1, 0, "AgentDataWrapper");
//    qmlRegisterType<IndustryPropsModel>("ru.armax.cashbox.industrypropsmodel", 1, 0, "IndustryPropsModel");
//    qmlRegisterType<IndustryPropertyWrapper>("ru.armax.cashbox.industrypropertywrapper", 1, 0, "IndustryPropertyWrapper");
//    qmlRegisterType<SimpleReceiptOperationWrapper>("ru.armax.cashbox.simplereceiptoperationwrapper", 1, 0, "SimpleReceiptOperationWrapper");
//    qmlRegisterType<OfdSettingsWrapper>("ru.armax.cashbox.ofdsettingswrapper", 1, 0, "OfdSettingsWrapper");
//    qmlRegisterType<SendMailHelper>("ru.armax.cashbox.sendmailhelper", 1, 0, "SendMailHelper");
//    qmlRegisterType<CashiersModel>("ru.armax.cashbox.cashiersmodel", 1, 0, "CashiersModel");
//    qmlRegisterType<CashierWrapper>("ru.armax.cashbox.cashierwrapper", 1, 0, "CashierWrapper");

    lmWarning() << qApp->applicationName() << qApp->organizationName()
                << qApp->organizationDomain() << logtab
                << qApp->applicationVersion() << QLocale() << AndroidApiHelper().getDsn();
    return true;
}

void CashboxRoot::release()
{
    LoggerManager::releaseLog();
}


//--------------------------------------------------------

SIPRequestEater::SIPRequestEater(QObject *parent)
    : QObject(parent)
{
}

SIPRequestEater::~SIPRequestEater()
{

}

bool SIPRequestEater::eventFilter(QObject *obj, QEvent *event)
{
    if(event->type() == QEvent::RequestSoftwareInputPanel || event->type() == QEvent::FocusIn)
    {
        // filter out RequestSoftwareInputPanel event
        return true;
    }
    else if(event->type() == QEvent::KeyPress)
    {
        QKeyEvent *e = dynamic_cast<QKeyEvent*>(event);
        if(e && e->key() == Qt::Key_Back)
        {
            lmWarning() << e->key() << e->modifiers() << e->text() << static_cast<qint32>(Qt::Key_Back);
            emit popPage();
            return true;
        }
        lmWarning() << e->nativeScanCode() << e->key() << e->modifiers() << e->text() << static_cast<qint32>(Qt::Key_HomePage)
            << static_cast<qint32>(Qt::Key_Home);
    }
    return QObject::eventFilter(obj, event);
}
