#ifndef APPROOTHANDLER_H
#define APPROOTHANDLER_H

#include "cashboxstatuswrapper.h"
#include "basecorehandler.h"


#include <QObject>

class AndroidApiHelper;

class AppRootHandler : public BaseCoreHandler
{
    Q_OBJECT
    QML_ELEMENT
    QML_NAMED_ELEMENT(AppRoot)
    Q_DISABLE_COPY(AppRootHandler)
    Q_PROPERTY(QString dsn READ getDsn NOTIFY dsnChanged)
    Q_PROPERTY(bool loginned READ loginned NOTIFY loginnedChanged)
    Q_PROPERTY(QString homePage READ homePage NOTIFY homePageChanged)
    Q_PROPERTY(QString appHeader READ appHeader NOTIFY appHeaderChanged)
public:
    explicit AppRootHandler(QObject *parent = nullptr);
    virtual ~AppRootHandler() override;

    QString getDsn() const;
    bool loginned() const;
    QString homePage() const;

    QString appHeader() const;

    Q_INVOKABLE QString testText() const;

    Q_INVOKABLE QString firmwareTestText() const;
    Q_INVOKABLE QString sdcardPath() const;

public slots:
    void reloadStatus();
    void selfTest();
    void restart();
    void testFirmware(const QString &fn);
    void updateFirmware(const QString &fn);
    void firmwareToSdcard();
    void printNotPrinted();
    void fsClean();
    void totalClean(const QString &login, const QString &password);
signals:
    void dsnChanged(QString);
    void loginnedChanged(bool loginned);
    void homePageChanged(QString);
    void appHeaderChanged(QString);
    void popPage();
    void successTest();
    void successNotPrinted();
    void successClean();
    void successTotalClean();
private:
    AndroidApiHelper *android_;
    CashboxStatusWrapper *status_;
    QString firmwareTestText_;

    void processResult(const CoreTransaction & source, const CoreApiResult &res) override;
private slots:
    void onStatusChanged();
};

#endif // APPROOTHANDLER_H
