#include "basecoreconfighandler.h"

BaseCoreConfigHandler::BaseCoreConfigHandler(QObject *parent)
    : BaseCoreHandler{parent}
{

}

BaseCoreConfigHandler::~BaseCoreConfigHandler()
{

}

void BaseCoreConfigHandler::reload()
{
    BaseCoreConfigData *o = object();
    if(!o) emit error(tr("Не корректное наименование раздела настроек"));
    else
    {
        emit showBusy(true);
        std::function<QPair<CoreTransaction, CoreApiResult>()> task =
                [this, o]() -> QPair<CoreTransaction, CoreApiResult> {
            Cashier c = session_->cashier();
            CoreTransaction trans = o->createTransactionForLoad(c.login(), c.password());
            trans.genUid();
            CoreApiResult res = api_->exec(trans);
            QPair<CoreTransaction, CoreApiResult>resp = qMakePair(trans, res);
            return resp;
        };
        execTask(task);
    }
}

void BaseCoreConfigHandler::save()
{
    BaseCoreConfigData *o = object();
    if(!o) emit error(tr("Не корректное наименование раздела настроек"));
    else
    {
        emit showBusy(true);
        std::function<QPair<CoreTransaction, CoreApiResult>()> task =
                [this, o]() -> QPair<CoreTransaction, CoreApiResult> {
            Cashier c = session_->cashier();
            CoreTransaction trans = o->createTransactionForSave(c.login(), c.password());
            trans.genUid();
            CoreApiResult res = api_->exec(trans);
            QPair<CoreTransaction, CoreApiResult>resp = qMakePair(trans, res);
            return resp;
        };
        execTask(task);
    }
}

void BaseCoreConfigHandler::processResult(const CoreTransaction &source, const CoreApiResult &res)
{

    if(!res.isOk()) emit error(res.descr());
    else
    {
        BaseCoreConfigData *o = object();
        if(o)o->parseMap(res.data().value("value").toMap());
        if(source.operation() == CoreTransaction::Operation::SAVE_CORE_SETTINGS) emit savedSuccess();
        else emit loaded();
    }
}
