#include "fiscalizationhandler.h"

#include "logmanip.h"
#include "androidapihelper.h"
#include "serialcheker.h"
//#include "fiscalizationdocument.h"

FiscalizationHandler::FiscalizationHandler(QObject *parent)
    : BaseCoreHandler(parent)
{

}

FiscalizationHandler::~FiscalizationHandler()
{

}

void FiscalizationHandler::writeSerial(const QString &serial)
{
    emit showBusy(true);

    SerialCheker *checker = new SerialCheker;
    connect(checker, &SerialCheker::serialIsValid, this, &FiscalizationHandler::writeCheckedSerial);
    connect(checker, &SerialCheker::serialChekingError, this, &FiscalizationHandler::error);
    checker->checkSerial(serial, CashboxStatusWrapper(this).status().modelData());
}

void FiscalizationHandler::fiscalize(const QVariantMap &map)
{
    std::function<QPair<CoreTransaction, CoreApiResult>()> task = [this, map]() -> QPair<CoreTransaction, CoreApiResult> {
        CoreTransaction transaction;
        transaction.setCashierLogin(session_->cashier().login());
        transaction.setCashierPassword(session_->cashier().password());
        transaction.setOperation(CoreTransaction::Operation::FISCALIZATION);
        transaction.setParams(map);
        transaction.genUid();
        lmWarning() << logvariant(transaction.toMap());
        CoreApiResult res = api_->exec(transaction);
        lmWarning() << logvariant(res.toMap());
        api_->loadCashboxStatus();
        return qMakePair(transaction, res);
    };
    execTask(task);
}

void FiscalizationHandler::preprint(const QVariantMap &map)
{
    std::function<QPair<CoreTransaction, CoreApiResult>()> task = [this, map]() -> QPair<CoreTransaction, CoreApiResult> {
        CoreTransaction transaction;
        transaction.setCashierLogin(session_->cashier().login());
        transaction.setCashierPassword(session_->cashier().password());
        transaction.setOperation(CoreTransaction::Operation::PREPRINT_FISCALIZATION);
        transaction.setParams(map);
        transaction.genUid();
        lmWarning() << logvariant(transaction.toMap());
        CoreApiResult res = api_->exec(transaction);
        lmWarning() << logvariant(res.toMap());
        api_->loadCashboxStatus();
        return qMakePair(transaction, res);
    };
    execTask(task);
}

//void FiscalizationHandler::fillData(FiscalizationData *data)
//{
//    if(data)
//    {
//        data->init(CashboxStatusWrapper(this).status());
//    }
//}

void FiscalizationHandler::processResult(const CoreTransaction &source, const CoreApiResult &res)
{
    lmWarning() << res.isOk();
    if(!res.isOk())emit error(res.descr());
    emit showBusy(false);
    switch (source.operation()) {
    case CoreApiConst::RemoteCommand::WRITE_SERIAL:
    {
        if(res.isOk() && CashboxStatusWrapper().status().modelData().isValid())
        {
            emit serialWrote();
        }
    }break;
    case CoreApiConst::RemoteCommand::FISCALIZATION:
    {
        lmWarning() << logvariant(res.data());
        if(res.isOk())
        {
            emit fiscalized();
        }
    }break;
    case CoreApiConst::RemoteCommand::PREPRINT_FISCALIZATION:
    {
        lmWarning() << logvariant(res.data());
        if(res.isOk())
        {
            emit successPreprint();
        }
    }break;

    default:
        break;
    }
}

void FiscalizationHandler::writeCheckedSerial(const QString &serial)
{
    if(sender())sender()->deleteLater();
    std::function<QPair<CoreTransaction, CoreApiResult>()> task = [this, serial]() -> QPair<CoreTransaction, CoreApiResult> {
        CoreTransaction transaction;
        transaction.setCashierLogin(session_->cashier().login());
        transaction.setCashierPassword(session_->cashier().password());
        transaction.setOperation(CoreTransaction::Operation::WRITE_SERIAL);
        QVariantMap data;
        data.insert("serial", serial);
        CashboxStatus st(api_->loadCashboxStatus().data());
        ModelData md = st.modelData();
        md.setSerial(serial);
        md.signData(AndroidApiHelper().getHardwareHash());
        data.insert("sign", md.sign());
        transaction.setParams(data);
        transaction.genUid();
        lmWarning() << logvariant(transaction.toMap());
        CoreApiResult res = api_->writeSerial(transaction);
        lmWarning() << logvariant(res.toMap());
        api_->loadCashboxStatus();
        return qMakePair(transaction, res);
    };
    execTask(task);
}

//void FiscalizationHandler::fiscalize(FiscalizationData *data)
//{
//    if(data)
//    {
//        emit showBusy(true);
//        QVariantMap dataMap = data->toMap();
//        std::function<QPair<CoreTransaction, CoreApiResult>()> task =
//                [this, dataMap]() -> QPair<CoreTransaction, CoreApiResult> {
//            CoreTransaction transaction;
//            transaction.setCashierNumber(session_->cashier().number());
//            transaction.setCashierPassword(session_->cashier().password());
//            transaction.setOperation(CoreTransaction::Operation::FISCALIZE);
//            transaction.setParams(dataMap);
//            transaction.genUid();
//            lmWarning() << logvariant(transaction.toMap());
//            CoreApiResult res = api_->exec(transaction);
//            lmWarning() << logvariant(res.toMap());
//            api_->loadCashboxStatus();
//            return qMakePair(transaction, res);
//        };
//        execTask(task);
//    }
//}
