#include "loginhandler.h"

#include "logmanip.h"
#include "cashboxbuildconfig.h"
#include "sessionwrapper.h"
#include "androidapihelper.h"

#include <QDataStream>
#include <QAndroidJniObject>
#include <QtAndroid>
#include <QAndroidJniExceptionCleaner>
#include <QNetworkReply>
#include <QUrl>
#include <QUrlQuery>
#include <QJsonParseError>
#include <QJsonDocument>
#include <QTimer>

const QString LoginHandler::LOGIN_PATH = "/kkm-trade/api/LoginSession";

LoginHandler::LoginHandler(QObject *parent)
    : BaseCoreHandler(parent)
    , cashiersModel_()
{
    QTimer::singleShot(200, this, SLOT(reloadCashiers()));
}

LoginHandler::~LoginHandler()
{

}

void LoginHandler::login(const QString &login, const QString &password)
{
    if(AndroidApiHelper().cashbox105Installed())
    {
        emit error(tr("На аппарате установлены две версии ККТ (для ФФД 1.05 и ФФД 1.2). Должна быть установлена только одна из версий!"));
        return;
    }
    emit showBusy(true);

    std::function<QPair<CoreTransaction, CoreApiResult>()> task = [this, login, password]() -> QPair<CoreTransaction, CoreApiResult> {
        CoreTransaction transaction;
        transaction.setOperation(CoreTransaction::Operation::LOGIN);
        transaction.genUid();
        transaction.setCashierLogin(login);
        transaction.setCashierPassword(password);
        lmWarning() << logvariant(transaction.toMap());
        CoreApiResult res = api_->exec(transaction);
        lmWarning() << logvariant(res.toMap());
        if(res.isOk()) {
            Cashier c(res.data());
            SessionWrapper::setGlobalCashier(c);
        }
        api_->loadCashboxStatus();
        api_->loadReceiptData();
        return qMakePair(transaction, res);
    };
    execTask(task);
}

QVariantList LoginHandler::cashiersModel() const
{
    return cashiersModel_;
}

void LoginHandler::processResult(const CoreTransaction &source, const CoreApiResult &res)
{
    lmWarning() << res.isOk();
    if(!res.isOk())emit error(res.descr());
    switch (source.operation()) {
    case CoreApiConst::RemoteCommand::GET_CASHIER_NAMES:
    {
        cashiersModel_ = res.data()["cashiers"].toList();
        emit cashiersModelChanged(cashiersModel_);
        if(cashiersModel_.isEmpty())
        {
            QTimer::singleShot(200, this, SLOT(reloadCashiers()));
        }
        else
        {
            emit showBusy(false);
        }
    }break;
    case CoreApiConst::RemoteCommand::LOGIN:
    {
        emit showBusy(false);
        Cashier c(res.data());
        lmWarning() << c.login() << c.isActive() << res.isOk() << logtab
                    << c.name() << logtab << logvariant(c.toMap());
        SessionWrapper::setGlobalCashier(c);
        if(res.isOk() && c.isValid()) emit loginned();
    }break;

    default:
        break;
    }
}

void LoginHandler::reloadCashiers()
{
    emit showBusy(true);

    std::function<QPair<CoreTransaction, CoreApiResult>()> task = [this]() -> QPair<CoreTransaction, CoreApiResult> {
        api_->loadCashboxStatus();
        CoreTransaction transaction;
        transaction.setOperation(CoreTransaction::Operation::GET_CASHIER_NAMES);
        transaction.genUid();
        lmWarning() << logvariant(transaction.toMap());
        CoreApiResult res = api_->exec(transaction);
        lmWarning() << logvariant(res.toMap());
        return qMakePair(transaction, res);
    };
    execTask(task);
}

