#include "reportscorehandler.h"
#include "logmanip.h"
#include "cashboxbuildconfig.h"
#include "androidapihelper.h"
#include "appdirectories.h"
#include "xreport.h"

#include <QtConcurrent>

ReportsCoreHandler::ReportsCoreHandler(QObject *parent)
    : BaseCoreHandler(parent)
    , status_(new CashboxStatusWrapper(this))
{

}

ReportsCoreHandler::~ReportsCoreHandler()
{

}

QString ReportsCoreHandler::offlineReportsPath() const
{
    return  "file://" + APATH.offlineNotifications() + "/";
}

void ReportsCoreHandler::openCycle(const QVariantMap &map)
{
    emit showBusy(true);
    std::function<QPair<CoreTransaction, CoreApiResult>()> task = [this, map]() -> QPair<CoreTransaction, CoreApiResult> {
        CoreTransaction transaction;
        transaction.setCashierLogin(session_->cashier().login());
        transaction.setCashierPassword(session_->cashier().password());
        transaction.setOperation(CoreTransaction::Operation::OPEN_CYCLE);
        transaction.setParams(map);
        transaction.setPrintResult(true);
        transaction.genUid();
        lmWarning() << logvariant(transaction.toMap());
        CoreApiResult res = api_->exec(transaction);
        lmWarning() << logvariant(res.toMap());
        api_->loadCashboxStatus();
        return qMakePair(transaction, res);
    };
    execTask(task);
}

void ReportsCoreHandler::openCycleSilent(const QVariantMap &map)
{
    emit showBusy(true);
    std::function<QPair<CoreTransaction, CoreApiResult>()> task = [this, map]() -> QPair<CoreTransaction, CoreApiResult> {
        CoreTransaction transaction;
        transaction.setCashierLogin(session_->cashier().login());
        transaction.setCashierPassword(session_->cashier().password());
        transaction.setOperation(CoreTransaction::Operation::OPEN_CYCLE);
        transaction.setParams(map);
        transaction.setPrintResult(false);
        transaction.genUid();
        lmWarning() << logvariant(transaction.toMap());
        CoreApiResult res = api_->exec(transaction);
        lmWarning() << logvariant(res.toMap());
        api_->loadCashboxStatus();
        return qMakePair(transaction, res);
    };
    execTask(task);
}

void ReportsCoreHandler::closeCycle(const QVariantMap &map, bool brief)
{
    emit showBusy(true);
    std::function<QPair<CoreTransaction, CoreApiResult>()> task = [this, map, brief]() -> QPair<CoreTransaction, CoreApiResult> {
        CoreTransaction transaction;
        transaction.setCashierLogin(session_->cashier().login());
        transaction.setCashierPassword(session_->cashier().password());
        transaction.setOperation(CoreTransaction::Operation::CLOSE_CYCLE);
        transaction.setParams(map);
        transaction.setBriefReceipt(brief);
        transaction.setPrintResult(true);
        transaction.genUid();
        lmWarning() << logvariant(transaction.toMap());
        CoreApiResult res = api_->exec(transaction);
        lmWarning() << logvariant(res.toMap());
        api_->loadCashboxStatus();
        return qMakePair(transaction, res);
    };
    execTask(task);
}

void ReportsCoreHandler::closeCycleSilent(const QVariantMap &map, bool brief)
{
    std::function<QPair<CoreTransaction, CoreApiResult>()> task = [this, map, brief]() -> QPair<CoreTransaction, CoreApiResult> {
        CoreTransaction transaction;
        transaction.setCashierLogin(session_->cashier().login());
        transaction.setCashierPassword(session_->cashier().password());
        transaction.setOperation(CoreTransaction::Operation::CLOSE_CYCLE);
        transaction.setParams(map);
        transaction.setBriefReceipt(brief);
        transaction.setPrintResult(false);
        transaction.genUid();
        lmWarning() << logvariant(transaction.toMap());
        CoreApiResult res = api_->exec(transaction);
        lmWarning() << logvariant(res.toMap());
        api_->loadCashboxStatus();
        return qMakePair(transaction, res);
    };
    execTask(task);
}

void ReportsCoreHandler::calcReport(const QVariantMap &map, bool brief)
{
    emit showBusy(true);
    std::function<QPair<CoreTransaction, CoreApiResult>()> task = [this, map, brief]() -> QPair<CoreTransaction, CoreApiResult> {
        CoreTransaction transaction;
        transaction.setCashierLogin(session_->cashier().login());
        transaction.setCashierPassword(session_->cashier().password());
        transaction.setOperation(CoreTransaction::Operation::CALC_REPORT);
        transaction.setParams(map);
        transaction.setBriefReceipt(brief);
        transaction.genUid();
        lmWarning() << logvariant(transaction.toMap());
        CoreApiResult res = api_->exec(transaction);
        lmWarning() << logvariant(res.toMap());
        api_->loadCashboxStatus();
        return qMakePair(transaction, res);
    };
    execTask(task);
}

void ReportsCoreHandler::closeFs(const QVariantMap &map)
{
    emit showBusy(true);
    std::function<QPair<CoreTransaction, CoreApiResult>()> task = [this, map]() -> QPair<CoreTransaction, CoreApiResult> {
        CoreTransaction transaction;
        transaction.setCashierLogin(session_->cashier().login());
        transaction.setCashierPassword(session_->cashier().password());
        transaction.setOperation(CoreTransaction::Operation::FS_CLOSE);
        transaction.setParams(map);
        transaction.genUid();
        lmWarning() << logvariant(transaction.toMap());
        CoreApiResult res = api_->exec(transaction);
        lmWarning() << logvariant(res.toMap());
        api_->loadCashboxStatus();
        return qMakePair(transaction, res);
    };
    execTask(task);
}

void ReportsCoreHandler::findDoc(qint32 fd)
{
    emit showBusy(true);
    std::function<QPair<CoreTransaction, CoreApiResult>()> task = [this, fd]() -> QPair<CoreTransaction, CoreApiResult> {
        CoreTransaction transaction;
        transaction.setCashierLogin(session_->cashier().login());
        transaction.setCashierPassword(session_->cashier().password());
        transaction.setOperation(CoreTransaction::Operation::FIND_FS_DOC);
        QVariantMap params;
        params["fd"] = fd;
        transaction.setParams(params);
        transaction.genUid();
        lmWarning() << logvariant(transaction.toMap());
        CoreApiResult res = api_->exec(transaction);
        lmWarning() << logvariant(res.toMap());
        api_->loadCashboxStatus();
        return qMakePair(transaction, res);
    };
    emit showBusy(true);
    execTask(task);
}

void ReportsCoreHandler::ofdTicket(qint32 fd)
{
    emit showBusy(true);
    std::function<QPair<CoreTransaction, CoreApiResult>()> task = [this, fd]() -> QPair<CoreTransaction, CoreApiResult> {
        CoreTransaction transaction;
        transaction.setCashierLogin(session_->cashier().login());
        transaction.setCashierPassword(session_->cashier().password());
        transaction.setOperation(CoreTransaction::Operation::FIND_OFD_TICKET);
        QVariantMap params;
        params["fd"] = fd;
        transaction.setParams(params);
        transaction.genUid();
        lmWarning() << logvariant(transaction.toMap());
        CoreApiResult res = api_->exec(transaction);
        lmWarning() << logvariant(res.toMap());
        api_->loadCashboxStatus();
        return qMakePair(transaction, res);
    };
    execTask(task);
}

void ReportsCoreHandler::tlvDoc(qint32 fd)
{
    emit showBusy(true);
    std::function<QPair<CoreTransaction, CoreApiResult>()> task = [this, fd]() -> QPair<CoreTransaction, CoreApiResult> {
        CoreTransaction transaction;
        transaction.setCashierLogin(session_->cashier().login());
        transaction.setCashierPassword(session_->cashier().password());
        transaction.setOperation(CoreTransaction::Operation::GET_TLV_DOC);
        QVariantMap params;
        params["fd"] = fd;
        transaction.setParams(params);
        transaction.genUid();
        lmWarning() << logvariant(transaction.toMap());
        CoreApiResult res = api_->exec(transaction);
        lmWarning() << logvariant(res.toMap());
        api_->loadCashboxStatus();
        return qMakePair(transaction, res);
    };
    execTask(task);
}

void ReportsCoreHandler::regDocFromFs(qint32 num)
{
    emit showBusy(true);
    std::function<QPair<CoreTransaction, CoreApiResult>()> task = [this, num]() -> QPair<CoreTransaction, CoreApiResult> {
        CoreTransaction transaction;
        transaction.setCashierLogin(session_->cashier().login());
        transaction.setCashierPassword(session_->cashier().password());
        transaction.setOperation(CoreTransaction::Operation::GET_REG_ARCHIVE_DOC);
        QVariantMap params;
        params["number"] = num;
        transaction.setParams(params);
        transaction.genUid();
        lmWarning() << logvariant(transaction.toMap());
        CoreApiResult res = api_->exec(transaction);
        lmWarning() << logvariant(res.toMap());
        api_->loadCashboxStatus();
        return qMakePair(transaction, res);
    };
    execTask(task);
}

void ReportsCoreHandler::regTlvDocFromFs(qint32 num)
{
    emit showBusy(true);
    std::function<QPair<CoreTransaction, CoreApiResult>()> task = [this, num]() -> QPair<CoreTransaction, CoreApiResult> {
        CoreTransaction transaction;
        transaction.setCashierLogin(session_->cashier().login());
        transaction.setCashierPassword(session_->cashier().password());
        transaction.setOperation(CoreTransaction::Operation::GET_REG_TLV_DOC);
        QVariantMap params;
        params["number"] = num;
        transaction.setParams(params);
        transaction.genUid();
        lmWarning() << logvariant(transaction.toMap());
        CoreApiResult res = api_->exec(transaction);
        lmWarning() << logvariant(res.toMap());
        api_->loadCashboxStatus();
        return qMakePair(transaction, res);
    };
    execTask(task);
}

void ReportsCoreHandler::offlineReports()
{
    emit showBusy(true);
    std::function<QPair<CoreTransaction, CoreApiResult>()> task = [this]() -> QPair<CoreTransaction, CoreApiResult> {
        CoreTransaction transaction;
        transaction.setCashierLogin(session_->cashier().login());
        transaction.setCashierPassword(session_->cashier().password());
        transaction.setOperation(CoreTransaction::Operation::GET_OFFLINE_NOTIFICATIONS);
        transaction.genUid();
        transaction.setExternalId(QStringLiteral("offline_%1_%2.bin")
                                  .arg(QDateTime::currentDateTime().toString("yyyyMMdd_hhmmsszzz"))
                                  .arg(status_->status().regData().serial()));
        lmWarning() << logvariant(transaction.toMap());
        CoreApiResult res = api_->exec(transaction);
        lmWarning() << logvariant(res.toMap());
        api_->loadCashboxStatus();
        return qMakePair(transaction, res);
    };
    execTask(task);
}

void ReportsCoreHandler::saveOfflineReportToSdCard(const QString &fileName, const QString &baseName)
{
    QString err;
    QEventLoop loop;
    emit showBusy(true);
    auto exec = [&err, fileName, baseName] () -> bool {
        err.clear();
        AndroidApiHelper ap(nullptr);
        QStringList pathes = ap.sdcardStorage().split(":::");
        QString path = SDCARD_PATH;

        for(const QString &s: pathes)
        {
            if(s.startsWith(SDCARD_PATH))
            {
                lmWarning() << s;
                path = s + "/";
                break;
            }
        }
        QString fn = fileName;
        if(fn.startsWith("file://")) fn = fn.remove("file://");
        QFileInfo fi(fn);
        if(!baseName.isEmpty()) path = path + baseName;
        else path = path + fi.fileName();
        lmWarning() << ap.sdcardStorage() << logtab  <<  path << logtab <<loglist(pathes);
        QFile f(fi.absoluteFilePath());
        if(!f.copy(path))
        {
            err =  tr("Ошибка копирования файла:") + f.errorString();
            return false;
        }
        return true;
    };

    QFutureWatcher<bool> *watcher = new QFutureWatcher<bool>();
    connect(watcher, &QFutureWatcher<bool>::finished, &loop, &QEventLoop::quit);
    QFuture<bool> res = QtConcurrent::run(exec);
    watcher->setFuture(res);
    loop.exec();
    watcher->deleteLater();
    lmWarning() << res.result() << err;
    if(res.result()) emit offlineReportsSaved();
    else emit error(err);
}

void ReportsCoreHandler::xReport(bool brief)
{
    emit showBusy(true);
    std::function<QPair<CoreTransaction, CoreApiResult>()> task = [this, brief]() -> QPair<CoreTransaction, CoreApiResult> {
        CoreTransaction transaction;
        transaction.setCashierLogin(session_->cashier().login());
        transaction.setCashierPassword(session_->cashier().password());
        transaction.setOperation(CoreTransaction::Operation::X_REPORT);
        transaction.setBriefReceipt(brief);
//        transaction.setParams(map);
        transaction.genUid();
        lmWarning() << logvariant(transaction.toMap());
        CoreApiResult res = api_->exec(transaction);
        lmWarning() << logvariant(res.toMap());
        api_->loadCashboxStatus();
        return qMakePair(transaction, res);
    };
    execTask(task);
}

void ReportsCoreHandler::fsReport(bool brief)
{
    emit showBusy(true);
    std::function<QPair<CoreTransaction, CoreApiResult>()> task = [this, brief]() -> QPair<CoreTransaction, CoreApiResult> {
        CoreTransaction transaction;
        transaction.setCashierLogin(session_->cashier().login());
        transaction.setCashierPassword(session_->cashier().password());
        transaction.setOperation(CoreTransaction::Operation::FS_REPORT);
        transaction.setBriefReceipt(brief);
        //        transaction.setParams(map);
        transaction.genUid();
        lmWarning() << logvariant(transaction.toMap());
        CoreApiResult res = api_->exec(transaction);
        lmWarning() << logvariant(res.toMap());
        api_->loadCashboxStatus();
        return qMakePair(transaction, res);
    };
    execTask(task);
}

void ReportsCoreHandler::processResult(const CoreTransaction &source, const CoreApiResult &res)
{
    lmWarning() << res.isOk();
    if(!res.isOk())emit error(res.descr());
    emit showBusy(false);
    switch (source.operation()) {
    case CoreApiConst::RemoteCommand::OPEN_CYCLE:
    {
        if(res.isOk())
        {
            emit cycleOpened();
        }
    }break;
    case CoreApiConst::RemoteCommand::CLOSE_CYCLE:
    {
        if(res.isOk())
        {
            emit cycleClosed();
        }
    }break;
    case CoreApiConst::RemoteCommand::CALC_REPORT:
    {
        if(res.isOk())
        {
            emit calcReportReady();
        }
    }break;
    case CoreApiConst::RemoteCommand::FS_CLOSE:
    {
        if(res.isOk())
        {
            emit fsClosed();
        }
    }break;
    case CoreApiConst::RemoteCommand::FIND_FS_DOC    :
    case CoreApiConst::RemoteCommand::FIND_OFD_TICKET:
    case CoreApiConst::RemoteCommand::GET_TLV_DOC    :
    {
        if(res.isOk())
        {
            emit reportIsReady();
        }
    }break;
    case CoreApiConst::RemoteCommand::GET_OFFLINE_NOTIFICATIONS:
    {
        if(res.isOk())
        {
            saveOfflineReportToSdCard(res.data().value("fileName").toString().trimmed(), res.data().value("baseName").toString().trimmed());
        }
    }break;
    case CoreApiConst::RemoteCommand::X_REPORT:
    {
        if(res.isOk())
        {
            XReport x(fdf::DocType::CycleReport);
            x.parseMap(res.data());
            emit xReportSuccess(x.toHtml());
        }
    }break;
    case CoreApiConst::RemoteCommand::FS_REPORT:
    {
        if(res.isOk())
        {
            XReport x(fdf::DocType::FsReport);
            x.parseMap(res.data());
            emit xReportSuccess(x.toHtml());
        }
    }break;
    default:
        break;
    }
}
