#include "sendmailhelper.h"
#include "supportmailer.h"

#include "androidapihelper.h"
#include "cashboxstatuswrapper.h"
#include "sessionwrapper.h"
#include "appdirectories.h"

#include <QDir>
#include <QFileInfo>
#include <QtMath>
#include <QDebug>

#ifndef SEND_MAIL
#define SEND_MAIL

#define E_MAIL_FROM QStringLiteral("supportarmax@yandex.ru")
#define E_MAIL_TO QStringLiteral("supportarmax@yandex.ru")

#define SMTP_SERVER QStringLiteral("smtp.yandex.ru")
#define USER_NAME QStringLiteral("supportarmax")
#define PORT 465

#endif



SendMailHelper::SendMailHelper(QObject *parent) : QObject(parent),
    processing_(false)
{

}

SendMailHelper::~SendMailHelper()
{
    stopThread();
}



void SendMailHelper::stopThread()
{
    if(thread_)
    {
        thread_->quit();
        while (!thread_->isFinished()) {
            QThread::msleep(100);
        }
    }
    processing_ = false;
}

void SendMailHelper::errorMessage(QString message)
{
    Q_UNUSED(message)
    stopThread();
    emit sendResult(false);
}

void SendMailHelper::messageSended()
{
    stopThread();
    emit sendResult(true);
}

QString SendMailHelper::prepareEmailPassword() const
{
    return QStringLiteral("armaarma100500");
}

QStringList SendMailHelper::logFiles()
{

    QDir logsDir(APATH.logsPath());

    QList<QFileInfo> fil = logsDir.entryInfoList(
                QStringList() << QString("*log*"), QDir::Files | QDir::NoDotAndDotDot);

    QStringList logPaths;
    for(const QFileInfo &fi: fil)
    {
        if(fi.fileName().endsWith(".zip"))continue;
        logPaths.append(fi.absoluteFilePath());
    }
    return logPaths;
}



void SendMailHelper::sendMail(bool attachDb, QString comment)
{

    CashboxStatusWrapper wrapper;
    CashboxStatus st = wrapper.status();
    QString theme = st.modelData().modelName() + ": " + st.modelData().serial();


//    ui->buttonBox->setEnabled(false);
    if(thread_) stopThread();
    SupportMailer *worker = new SupportMailer();
    worker->setSmtp(SMTP_SERVER);
    worker->setPort(PORT);
    worker->setUseSssl(true);
    worker->setUseAuth(true);
    worker->setUser(USER_NAME);
    worker->setPassword(prepareEmailPassword());
    worker->setSender(E_MAIL_FROM);
    worker->setRecievers(QStringList()<< E_MAIL_TO);
    worker->setSubject(QStringLiteral("Android KKT: ") + theme);
    worker->setSenderFrom("");
    QStringList text;

    QString sys = QStringLiteral("Android");
    SessionWrapper session;
    worker->setSenderName(QString("Кассир: %1 Логин: %2").
        arg(session.cashier().name()).arg(session.cashier().login()));
    text << QString("Serial: %1").arg(st.modelData().serial())
         << QString("RegNumber: %1").arg(st.regData().regNumber())
         << QString("VERSION: %1 - OS: %2").arg(AndroidApiHelper().versionName()).arg(sys)
         << QString() << QString();
    text.append(comment);
    worker->setText(text.join("\n"));
    worker->setHtml(st.toHtml());
    QStringList logPaths = logFiles();

    if (attachDb){
//        QFileInfo fi = dbFileInfo();
//        if (fi.exists()){
//            logPaths.append(fi.absoluteFilePath());
//        }
    }



    if (logPaths.length() > 0)
        worker->setFiles(logPaths);

    thread_ = new AbstractWorkerThread(worker, this);
    connect(thread_.data(), SIGNAL(started()), worker, SLOT(send()), Qt::QueuedConnection);
    connect(worker, SIGNAL(success()), this, SLOT(messageSended()), Qt::QueuedConnection);
    connect(worker, SIGNAL(errorMessage(QString)), this, SLOT(errorMessage(QString)), Qt::QueuedConnection);
    connect(worker, SIGNAL(progressRegion(int,int)), this,
            SIGNAL(progressRegion(int,int)), Qt::QueuedConnection);
    connect(worker, SIGNAL(progressValue(int)), this,
            SIGNAL(progressValue(int)), Qt::QueuedConnection);

    thread_->start();
}

QString SendMailHelper::getRedableSize(qint64 size)
{
    if (size == 0){
        return "";
    }
    else{
        QString suffix = ",байт";
        double dSize = size;
        if (dSize > 1024){
            dSize = dSize/1024.0;
            suffix = " кб";
        }
        if (dSize > 1024){
            dSize = dSize/1024.0;
            suffix = " мб";
        }
        if (dSize > 1024){
            dSize = dSize/1024.0;
            suffix = " Гб";
        }

        dSize = qRound(dSize*10.0)/10.0;

        return QString("%1%2").arg(dSize).arg(suffix);
    }
}

QString SendMailHelper::logsSize()
{

    qint64 size = 0;
    QStringList logPaths = logFiles();
    foreach(QString logPath, logPaths){
        QFileInfo fi(logPath);
        //qDebug() << fi.size();
        size= size + fi.size();
    }
    return getRedableSize(size);
}

bool SendMailHelper::clearLogs()
{
    bool res = false;
    QDir logsDir(APATH.logsPath());
    res = logsDir.removeRecursively();
    res = res && logsDir.mkdir(APATH.logsPath());
    return res;
}

