#include "supportmailer.h"

#include "zip.h"
#include "SmtpMime"
#include "logmanip.h"
#include "appdirectories.h"

#include <QFileInfo>
#include <QDir>
#include <QFile>


SupportMailer::SupportMailer()
    : AbstractWorker()
    , sender_()
    , senderFrom_()
    , recievers_()
    , smtp_()
    , port_(0)
    , user_()
    , password_()
    , useSssl_(true)
    , useAuth_(true)
    , subject_()
    , text_()
    , html_()
    , files_()
{
}


SupportMailer::~SupportMailer()
{

}
QString SupportMailer::sender() const
{
    return sender_;
}

void SupportMailer::setSender(const QString &sender)
{
    sender_ = sender;
}
QStringList SupportMailer::recievers() const
{
    return recievers_;
}
QString SupportMailer::senderFrom() const
{
    return senderFrom_;
}

void SupportMailer::setSenderFrom(const QString &senderFrom)
{
    senderFrom_ = senderFrom;
}

QString SupportMailer::senderName() const
{
    return senderName_;
}

void SupportMailer::setSenderName(const QString &senderName)
{
    senderName_ = senderName;
}



void SupportMailer::setRecievers(const QStringList &recievers)
{
    recievers_ = recievers;
}
QString SupportMailer::smtp() const
{
    return smtp_;
}

void SupportMailer::setSmtp(const QString &smtp)
{
    smtp_ = smtp;
}
quint16 SupportMailer::port() const
{
    return port_;
}

void SupportMailer::setPort(const quint16 &port)
{
    port_ = port;
}
QString SupportMailer::user() const
{
    return user_;
}

void SupportMailer::setUser(const QString &user)
{
    user_ = user;
}
QString SupportMailer::password() const
{
    return password_;
}

void SupportMailer::setPassword(const QString &password)
{
    password_ = password;
}
bool SupportMailer::useSssl() const
{
    return useSssl_;
}

void SupportMailer::setUseSssl(bool useSssl)
{
    useSssl_ = useSssl;
}
bool SupportMailer::useAuth() const
{
    return useAuth_;
}

void SupportMailer::setUseAuth(bool useAuth)
{
    useAuth_ = useAuth;
}
QString SupportMailer::subject() const
{
    return subject_;
}

void SupportMailer::setSubject(const QString &subject)
{
    subject_ = subject;
}
QString SupportMailer::text() const
{
    return text_;
}

void SupportMailer::setText(const QString &text)
{
    text_ = text;
}
QString SupportMailer::html() const
{
    return html_;
}

void SupportMailer::setHtml(const QString &html)
{
    html_ = html;
}
QStringList SupportMailer::files() const
{
    return files_;
}

void SupportMailer::setFiles(const QStringList &files)
{
    files_ = files;
}


void SupportMailer::send()
{
    EmailAddress *sender = new EmailAddress(sender_);

    SmtpClient smtp (smtp_, port_, useSssl_ ? SmtpClient::SslConnection : SmtpClient::TcpConnection);
    smtp.setResponseTimeout(30000);

    MimeMessage message;

    message.setSender(sender);
    message.setSenderFrom(new EmailAddress(senderFrom_, senderName_));
    message.setSubject(subject_);

    for(QString const &reciever: recievers_)
        message.addRecipient(new EmailAddress(reciever));

    if(!text_.isEmpty())
    {
        MimeText *content = new MimeText();
        content->setText(text_);
        message.addPart(content);
        QString fn = APATH.tmpLogsPath() + QStringLiteral("/body.txt");
        QFile f(fn);
        if(f.open(QIODevice::WriteOnly))
        {
            f.write(text_.toUtf8());
            f.close();
            files_ << fn;
        }
    }
    if(!html_.isEmpty())
    {
        MimeHtml *content = new MimeHtml();
        content->setHtml(html_);
        message.addPart(content);
        QString fn = APATH.tmpLogsPath() + QStringLiteral("/status.html");
        QFile f(fn);
        if(f.open(QIODevice::WriteOnly))
        {
            f.write(html_.toUtf8());
            f.close();
            files_ << fn;
        }
    }

    QStringList files = prepareAttachedFiles();
    for(QString const &fn:files)
        message.addPart(new MimeAttachment(new QFile(fn)));

    if (!smtp.connectToHost())
    {
        emit errorMessage(tr("Connection Failed"));
    }
    else if (useAuth_ && !smtp.login(user_, password_))
    {
        emit errorMessage(tr("Authentification Failed"));
    }
    else if (!smtp.sendMail(message))
    {
        emit errorMessage(tr("Mail sending failed"));
    }
    else
    {
        emit success();
    }
    smtp.quit();
    emit progressValue(files_.size() + 1);

    clean();
}

bool SupportMailer::doStart()
{
    return true;
}

bool SupportMailer::doStop()
{
    return true;
}

QStringList SupportMailer::prepareAttachedFiles()
{
    emit progressMessage(tr("Prepare attachement"));
    emit progressRegion(0, files_.size() + 1);
    qint32 val = 0;
    emit progressValue(val);
    QStringList res;
    Zip zip;
    for(QString const &s: files_)
    {
        QFileInfo fi (s);
        QString fn = QString("%1/%2.zip").arg(APATH.tmpLogsPath(), fi.fileName());
        zip.createArchive(fn);
        zip.addFile(fi.absoluteFilePath(), Zip::Deflate9);
        zip.closeArchive();
        res << fn;
        emit progressValue(++val);
    }
    return res;
}

void SupportMailer::clean()
{
    QDir dir(APATH.tmpLogsPath());
    QFileInfoList files = dir.entryInfoList(QDir::Files);
    for(QFileInfo const &fi: files)
    {
        QFile::remove(fi.absoluteFilePath());
    }
}
