#include "basecatalogmodel.h"

#include "logmanip.h"

#include <QFutureWatcher>
#include <QFuture>
#include <QtConcurrent>


BaseCatalogModel::BaseCatalogModel(QObject *parent)
    : QAbstractListModel(parent)
    , api_(new UiCashboxServiceApi(this))
    , session_(new SessionWrapper(this))

{

}

BaseCatalogModel::~BaseCatalogModel()
{

}

QVariantMap BaseCatalogModel::getItem(int index) const
{
    return prepareItem(index);
}

Cashier BaseCatalogModel::sessionUser() const
{
    return session_->cashier();
}

void BaseCatalogModel::reload()
{
    doReload();
}

void BaseCatalogModel::saveItem(BaseCatalogItemWrapper *item)
{
    if(!item)
    {
        emit itemSavingError(tr("Нулевой блок"));
    }
    else
    {
        std::function<QPair<CoreTransaction, CoreApiResult>()> task =
                [this, item]() -> QPair<CoreTransaction, CoreApiResult> {
            CoreTransaction transaction = item->toSaveTransaction();
            transaction.setCashierLogin(session_->cashier().login());
            transaction.setCashierPassword(session_->cashier().password());
            transaction.genUid();
            lmWarning() << logvariant(transaction.toMap());
            CoreApiResult res = api_->exec(transaction);
            lmWarning() << logvariant(res.toMap());
            api_->loadReceiptData();
            return qMakePair(transaction, res);
        };
        execTask(task);
    }
}

void BaseCatalogModel::execTask(std::function<QPair<CoreTransaction, CoreApiResult> ()> &task)
{
    QFutureWatcher<QPair<CoreTransaction, CoreApiResult> > *watcher = new QFutureWatcher<QPair<CoreTransaction, CoreApiResult> >();
    connect(watcher, &QFutureWatcher<CoreApiResult>::finished, this, &BaseCatalogModel::taskFinished);
    QFuture<QPair<CoreTransaction, CoreApiResult> > future = QtConcurrent::run(task);
    watcher->setFuture(future);
}

void BaseCatalogModel::taskFinished()
{
    QFutureWatcher<QPair<CoreTransaction, CoreApiResult> > *fwatcher =
            dynamic_cast<QFutureWatcher<QPair<CoreTransaction, CoreApiResult> > *>(sender());
    lmWarning() << sender() << fwatcher;
    QFuture<QPair<CoreTransaction, CoreApiResult> > future = fwatcher->future();
    QPair<CoreTransaction, CoreApiResult>  res = future;
    fwatcher->deleteLater();
    processResult(res.first, res.second);

}

void BaseCatalogModel::toastError(const QString &msg)
{
    emit error(msg);

}
