#ifndef BASECATALOGMODEL_H
#define BASECATALOGMODEL_H

#include "uicashboxserviceapi.h"
#include "sessionwrapper.h"
#include "basecatalogitemwrapper.h"

#include <QAbstractListModel>
#include <QtQml>

class BaseCatalogModel : public QAbstractListModel
{
    Q_OBJECT
    Q_DISABLE_COPY(BaseCatalogModel)
    QML_ELEMENT
public:
    explicit BaseCatalogModel(QObject *parent = nullptr);
    ~BaseCatalogModel() override;

    Q_INVOKABLE QVariantMap getItem(int index) const;

    Cashier sessionUser() const;

signals:
    void showBusy(bool busy);
    void error(QString msg);
    void itemSavingError(QString msg);
    void itemSaved();
public slots:
    void reload();
    void saveItem(BaseCatalogItemWrapper *item);
protected:
    UiCashboxServiceApi *api_;
    SessionWrapper *session_;

    virtual void execTask(std::function<QPair<CoreTransaction, CoreApiResult> ()> &task);
    virtual void processResult(const CoreTransaction & source, const CoreApiResult &res) = 0;
    virtual void doReload()  = 0;
    virtual QVariantMap prepareItem(int index ) const = 0;
protected slots:
    void taskFinished();
    void toastError(const QString &msg);
};

Q_DECLARE_METATYPE(BaseCatalogModel*)

#endif // BASECATALOGMODEL_H
