#ifndef INDUSTRYPROPSMODEL_H
#define INDUSTRYPROPSMODEL_H

#include "receiptoperation.h"
#include <QAbstractListModel>
#include <QtQml>

class IndustryPropsModel : public QAbstractListModel
{
    Q_OBJECT
    QML_ELEMENT
    Q_DISABLE_COPY(IndustryPropsModel)
public:

    enum RecRoles
    {
        RoleFedId         = Qt::UserRole + 1,
        RoleDocDate       = Qt::UserRole + 2,
        RoleDocNumber     = Qt::UserRole + 3,
        RoleValue         = Qt::UserRole + 4,
    };
    explicit IndustryPropsModel(QObject *parent = nullptr);
    ~IndustryPropsModel() override;

    QHash<int, QByteArray> roleNames() const override;
    QVariant data(const QModelIndex &idx, int role) const override;
    int rowCount(const QModelIndex &parent = QModelIndex()) const override;

    Q_INVOKABLE QVariantList props() const;
    Q_INVOKABLE int count() const;
    Q_INVOKABLE QVariantMap item(int idx);

signals:
    void saveData(QVariantList);
    void error(QString &msg);
    void finish();
public slots:
    void save();
    void clean();
    void addItem();
    void setItem(int idx, const QVariantMap &data);
    void removeItem(int idx);
    void setProps(const QVariantList &data);
private:
    QList<IndustryProperty> props_;
};

//--------------------------------------------------------------------------------------------------

class IndustryPropertyWrapper :public QObject
{
    Q_OBJECT
    QML_ELEMENT
    Q_DISABLE_COPY(IndustryPropertyWrapper)

    Q_PROPERTY(QVariantMap prop READ prop WRITE setProp NOTIFY propChanged);

    Q_PROPERTY(QString fedId READ fedId WRITE setFedId NOTIFY fedIdChanged)
    Q_PROPERTY(QString docDt READ docDt WRITE setDocDt NOTIFY docDtChanged)
    Q_PROPERTY(QString docNumber READ docNumber WRITE setDocNumber NOTIFY docNumberChanged)
    Q_PROPERTY(QString value READ value WRITE setValue NOTIFY valueChanged)
    Q_PROPERTY(bool isValid READ isValid NOTIFY isValidChanged)


public:
    explicit IndustryPropertyWrapper(QObject *parent = nullptr);
    ~IndustryPropertyWrapper() override;

    bool isValid() const;

    QVariantMap prop() const;
    void setProp(const QVariantMap &newProp);

    const QString &fedId() const;
    void setFedId(const QString &newFedId);

    QString docDt() const;
    void setDocDt(const QString &newDocDt);

    const QString &docNumber() const;
    void setDocNumber(const QString &newDocNumber);

    const QString &value() const;
    void setValue(const QString &newValue);

signals:
    void propChanged();
    void fedIdChanged();
    void docDtChanged();
    void docNumberChanged();
    void valueChanged();
    void isValidChanged();

private:
    IndustryProperty prop_;

};

#endif // INDUSTRYPROPSMODEL_H
