#include "mainmenumodel.h"
#include "androidapihelper.h"
#include "logmanip.h"
#include "cashboxbuildconfig.h"

MainMenuModel::MainMenuModel(QObject *parent)
    : QAbstractListModel(parent)
    , status_(new CashboxStatusWrapper(this))
    , session_(new SessionWrapper(this))
    , subStack_()
{
    connect(status_, &CashboxStatusWrapper::statusChanged, this, &MainMenuModel::reload);
}

MainMenuModel::~MainMenuModel()
{

}

QHash<int, QByteArray> MainMenuModel::roleNames() const
{
    return {
        {ROLE_NAME, "mmname"},
        {ROLE_ICON, "mmicon"},
        {ROLE_ACTION, "mmaction"},
        {ROLE_LABEL, "mmlabel"}
    };
}

QVariant MainMenuModel::data(const QModelIndex &idx, int role) const
{
    if(!idx.isValid() || idx.row() >= items_.size()) return QVariant();
    QVariantMap src = items_[idx.row()].toMap();
    QString nm = QString::fromLatin1(roleNames()[role]);
    return src[nm];
}

int MainMenuModel::rowCount(const QModelIndex &parent) const
{
    Q_UNUSED(parent)
    return items_.size();
}

QVariantMap MainMenuModel::get(qint32 row) const
{
    return row >= 0 && row <= items_.size() ?
                items_[row].toMap() : QVariantMap();
}

void MainMenuModel::action(qint32 act)
{
    switch (act)
    {
    case MMActionMenuBack:
    {
        if(subStack_.isEmpty()) reload();
        else
        {
            subStack_.pop();
            if(subStack_.isEmpty()) reload();
            else action(subStack_.top());
        }
    }break;
    case MMAction::MMActionCashboxOps: createCashboxOpsSub();break;
    case MMAction::MMActionCatalogs: createCatalogsSub();break;
    case MMAction::MMActionService: createServiceSub();break;
    case MMAction::MMActionCorrections: createCorrectionsOpsSub();break;
    case MMAction::MMActionFirmware: createFirmwareSub();break;
    case MMAction::MMActionEditCashiers: emit showFirstPage("CashiersView.qml");break;
    case MMAction::MMActionStatus: emit showFirstPage("HomeForm.qml");break;
    case MMAction::MMActionOpenCycle:
        action(MMActionMenuBack);
        emit showFirstPage("CycleOpenPage.qml");
        break;
    case MMAction::MMActionCloseCycle:
        action(MMActionMenuBack);
        emit showFirstPage("CycleClosePage.qml");
        break;
    case MMAction::MMActionCalcReport:
        action(MMActionMenuBack);
        emit showFirstPage("CalcReportPage.qml");
        break;
    case MMAction::MMActionFsArchiveReport:
        action(MMActionMenuBack);
        emit showFirstPage("FsArchivePage.qml");
        break;
    case MMAction::MMActionCloseArchive:
        action(MMActionMenuBack);
        emit showFirstPage("FsCloswPage.qml");
        break;
    case MMAction::MMActionSale:
        action(MMActionMenuBack);
        emit showFirstPage("SaleView.qml");
        break;
    case MMAction::MMActionRefund:
        action(MMActionMenuBack);
        emit showFirstPage("RefundView.qml");
        break;
    case MMAction::MMActionPurchase:
        action(MMActionMenuBack);
        emit showFirstPage("PurchaseView.qml");
        break;
    case MMAction::MMActionPurchaseRefund:
        action(MMActionMenuBack);
        emit showFirstPage("PurchaseReturnView.qml");
        break;
    case MMAction::MMActionDebitCorrection:
        action(MMActionMenuBack);
        emit showFirstPage("CorrectionSaleView.qml");
        break;
    case MMAction::MMActionDebitRefundCorrection:
        action(MMActionMenuBack);
        emit showFirstPage("CorrectionRefundView.qml");
        break;
    case MMAction::MMActionCreditCorrection:
        action(MMActionMenuBack);
        emit showFirstPage("CorrectionPurchaseView.qml");
        break;
    case MMAction::MMActionCreditRefundCorrection:
        action(MMActionMenuBack);
        emit showFirstPage("CorrectionPurchaseReturnView.qml");
        break;
    case MMAction::MMOfdSettings:
        action(MMActionMenuBack);
        emit showFirstPage("OfdSettingsPage.qml");
        break;
    case MMAction::MMActionReports: createReportsSub(); break;
    case MMAction::MMActionConfig: createSettingsSub(); break;
    case MMAction::MMActionFiscalize:
        action(MMActionMenuBack);
        emit showFirstPage("FiscalizationPage.qml");
        break;
    case MMAction::MMActionLogs:
        action(MMActionMenuBack);
        emit showFirstPage("LogsSettingsPage.qml");
        break;
    case MMAction::MMActionSelfTest:
        action(MMActionMenuBack);
        emit showFirstPage("SelfTestForm.qml");
        break;
    case MMAction::MMActionFirmwareTest:
        action(MMActionMenuBack);
        emit showFirstPage("FirmwareTestForm.qml");
        break;
    case MMAction::MMActionFirmwareUpdate:
        action(MMActionMenuBack);
        emit showFirstPage("FirmwareUpdateForm.qml");
        break;
    case MMAction::MMActionOfflineReport:
        action(MMActionMenuBack);
        emit showFirstPage("OfflineReportPage.qml");
        break;
    case MMAction::MMActionCleanMgm:
        action(MMActionMenuBack);
        emit showFirstPage("FsCleanPage.qml");
        break;
    case MMAction::MMActionClearAll:
        action(MMActionMenuBack);
        emit showFirstPage("FullClearPage.qml");
        break;
    case MMAction::MMActionDtSettings  :
        action(MMActionMenuBack);
        AndroidApiHelper().showDateTimeConfig();
        break;
    case MMAction::MMActionWifiSettings:
        action(MMActionMenuBack);
        AndroidApiHelper().showWifiConfig();
        break;
    case MMAction::MMActionGsmSettings :
        action(MMActionMenuBack);
        AndroidApiHelper().showGsmConfig();
        break;
    case MMAction::MMActionMobileDataSettings :
        action(MMActionMenuBack);
        AndroidApiHelper().showMobileDataConfig();
        break;
    case MMAction::MMActionXReport:
        action(MMActionMenuBack);
        emit showFirstPage("XReportPage.qml");
        break;
    case MMAction::MMActionFsReport:
        action(MMActionMenuBack);
        emit showFirstPage("FsReportPage.qml");
        break;
    case MMAction::MMActionReceiptConfig:
        action(MMActionMenuBack);
        emit showFirstPage("ReceiptConfigPage.qml");
        break;
    default:
        break;
    }
}

bool MainMenuModel::isSubMenu() const
{
    return !subStack_.isEmpty();
}

void MainMenuModel::reload()
{
    beginResetModel();
    items_.clear();
    Cashier c = session_->cashier();
    if(status_->fiscalized())
    {
        if(c.hasPermission(Cashier::Permission::Receipts) ||
                c.hasPermission(Cashier::Permission::ReturnReceipts) ||
                c.hasPermission(Cashier::Permission::CycleOpen) ||
                c.hasPermission(Cashier::Permission::CycleClose))
        {
            items_  << createItem(
                           "cashboxops", MMActionCashboxOps,
                           "qrc:/icons/cashboxops_64.png", tr("Операции"));
        }
        if(c.hasPermission(Cashier::Permission::Receipts) ||
                c.hasPermission(Cashier::Permission::Reports))
        {
            items_ << createItem("reports", MMActionReports, "qrc:/icons/reports_64.png", tr("Отчеты"));
        }
    }
    if(c.hasPermission(Cashier::Permission::Settings))
    {
        items_  << createItem("catalogs", MMActionCatalogs, "qrc:/icons/catalogs_64.png", tr("Справочники"))
                << createItem("config", MMActionConfig, "qrc:/icons/config_64.png", tr("Настройки"));
    }
    if(c.hasPermission(Cashier::Permission::Fiscalization) || c.hasPermission(Cashier::Permission::OfflineNotifications))
    {
        items_  << createItem("service", MMActionService, "qrc:/icons/service_64.png", tr("Служебные"));
    }
    items_  << createItem("status", MMActionStatus, "qrc:/icons/info_64.png", tr("О ККТ"));
    items_ << createItem("logs", MMActionLogs, "qrc:/icons/logs_64.png", tr("Логи"));
    if(c.isSuperUser())
    {
        items_ << createItem("firmware", MMActionFirmware, "qrc:/icons/firmware_64.png", tr("ПО"));
    }
//    items_.append(QVector<QVariant>(11 - items_.size(), createEmpty()).toList());
//    items_ <<  createItem("exit", MMActionExit, "qrc:/icons/exit_64.png", tr("Выход"));
    endResetModel();
    emit isSubMenuChanged();
}

void MainMenuModel::goBack()
{
    action(MMActionMenuBack);
}


QVariantMap MainMenuModel::createItem(const QString &name, MMAction action,
                                      const QString &icon, const QString &label)
{
    static const QString MMNAME = "mmname";
    static const QString MMICON = "mmicon";
    static const QString MMACTION = "mmaction";
    static const QString MMLABEL = "mmlabel";

    QVariantMap res;
    res.insert(MMNAME, name);
    res.insert(MMACTION, action);
    res.insert(MMICON, icon);
    res.insert(MMLABEL, label);
    return res;
}

QVariantMap MainMenuModel::createEmpty()
{
    return createItem(QString(), MMAction::MMNoAction, QString(), QString());
}

//items << createItem("introduction", MMActionIntroduction,
//                    "qrc:/icons/introduction_64.png", tr("Внесение"));
//<< createItem("introduction", MMActionIntroduction,
//              "qrc:/icons/introduction_64.png", tr("Внесение"))
//<< createItem("introduction", MMActionPayout,
//              "qrc:/icons/payout_64.png", tr("Выплата"))

void MainMenuModel::createCashboxOpsSub()
{
    Cashier c = session_->cashier();
    QVariantList items;

    if(!status_->cycleIsOpen() && c.hasPermission(Cashier::Permission::Receipts))
    {
        items << createItem("openCycle", MMActionOpenCycle,
                            "qrc:/icons/open_cycle_64.png", tr("Открыть\nсмену"));
    }
    if(status_->cycleIsOpen())
    {
        if(c.hasPermission(Cashier::Permission::Receipts))
        {
            if(!status_->cycleIsOver())
            {
                items << createItem("sale", MMActionSale,
                                    "qrc:/icons/sale_64.png", tr("Приход"))
                      << createItem("purchase", MMActionPurchase,
                                    "qrc:/icons/purchase_64.png", tr("Расход"));

            }
            items << createItem("closeCycle", MMActionCloseCycle,
                                "qrc:/icons/close_cycle_64.png", tr("Закрыть\nсмену"));
        }
        if(c.hasPermission(Cashier::Permission::ReturnReceipts) && !status_->cycleIsOver())
        {
            items << createItem("refund", MMActionRefund, "qrc:/icons/refund_64.png", tr("Возврат\nприхода"))
                  << createItem("purchaseRefund", MMActionPurchaseRefund, "qrc:/icons/purchase_refund_64.png", tr("Возврат\nрасхода"));
        }
        if(c.hasOneCorrectionPermission() && !status_->cycleIsOver())
        {
            items << createItem("corrections", MMActionCorrections, "qrc:/icons/corrections_64.png", tr("Чеки\nкоррекции"));
        }
    }

//    items.append(QVector<QVariant>(11 - items.size(), createEmpty()).toList());
//    items << createItem("back", MMActionMenuBack, "qrc:/icons/back_64.png", tr("Назад"));
    createSubMenu(MMActionCashboxOps, items);
}

void MainMenuModel::createCorrectionsOpsSub()
{
    Cashier c = session_->cashier();
    QVariantList items;

    if(status_->cycleIsOpen())
    {
        if(c.hasPermission(Cashier::Permission::Corrections))
        {
            if(!status_->cycleIsOver())
            {
                items << createItem("saleCorrection", MMActionDebitCorrection,
                                    "qrc:/icons/debit_corr_64.png", tr("Коррекция\nприхода"))
                      << createItem("purchaseCorrection", MMActionCreditCorrection,
                                    "qrc:/icons/credit_corr_64.png", tr("Коррекция\nрасхода"));
            }
        }
        if(c.hasPermission(Cashier::Permission::ReturnCorrections))
        {
            items << createItem("refund", MMActionDebitRefundCorrection, "qrc:/icons/debit_refund_corr_64.png", tr("Коррекция\nвозвр. прихода"))
                  << createItem("purchaseRefund", MMActionCreditRefundCorrection, "qrc:/icons/credit_refund_corr_64.png", tr("Коррекция\nвозвр. расхода"));
        }

    }

//    items.append(QVector<QVariant>(11 - items.size(), createEmpty()).toList());
//    items << createItem("back", MMActionMenuBack, "qrc:/icons/back_64.png", tr("Назад"));
    createSubMenu(MMActionCorrections, items);
}

void MainMenuModel::createCatalogsSub()
{
    QVariantList items;
    items   << createItem("editCashiers", MMActionEditCashiers, "qrc:/icons/cashiers_64.png", tr("Кассиры"));


//    items.append(QVector<QVariant>(8 - items.size(),  createEmpty()).toList());
//    items << createItem("back", MMActionMenuBack, "qrc:/icons/back_64.png", tr("Назад"));
    createSubMenu(MMActionCatalogs, items);
}

void MainMenuModel::createServiceSub()
{
    QVariantList items;
    Cashier c = session_->cashier();
    if(c.hasPermission(Cashier::Permission::Fiscalization))
    {
        items   << createItem("reFiscalization", MMActionFiscalize, "qrc:/icons/fiscalize_64.png", tr("Пере\nрегистрация"))
                << createItem("closeArchive", MMActionCloseArchive, "qrc:/icons/close_archive_64.png", tr("Закрытие\nархива"));
        if(!status_->status().fs().isRelease())
        {
            items<< createItem("cleanFs", MMActionCleanMgm, "qrc:/icons/clean_mgm_64.png", tr("Очистить\nМГМ"));
        }
    }

    if(c.hasPermission(Cashier::Permission::OfflineNotifications) &&
            status_->status().regData().getOfflineMode())
    {
        items<< createItem("offlineReport", MMActionOfflineReport, "qrc:/icons/offline_report_64.png", tr("Выгрузка\nуведомлений"));
    }
#ifndef DISALE_FULL_CLEANING
    items<< createItem("clearAll", MMActionClearAll, "qrc:/icons/total_reset_64.png", tr("Тех.\nобнуление"));
#endif
    createSubMenu(MMActionCatalogs, items);
}

void MainMenuModel::createReportsSub()
{
    QVariantList items;

    items << createItem("calcReport", MMActionCalcReport, "qrc:/icons/calc_report_64.png", tr("О расчетах"))
          << createItem("archive", MMActionFsArchiveReport, "qrc:/icons/archive_docs_64.png",
                        tr("Документы\nиз архива"));
    if(status_ && status_->cycleIsOpen())
    {
        items<< createItem("xReport", MMActionXReport, "qrc:/icons/xreport_64.png", tr("Статистика\nза смену"));
    }
    items<< createItem("fsReport", MMActionFsReport, "qrc:/icons/fsreport_64.png", tr("Статистика\nпо ФН"));

    createSubMenu(MMActionCatalogs, items);
}

void MainMenuModel::createSettingsSub()
{
    Cashier c = session_->cashier();
    QVariantList items;
    if(c.hasPermission(Cashier::Permission::Settings))
    {
        items << createItem("receiptConfig", MMActionReceiptConfig, "qrc:/icons/receipt_config_64.png", tr("Настройки\nчека"))
              << createItem("ofdSettings", MMOfdSettings, "qrc:/icons/ofd_settings_64.png", tr("Настройки\nОФД"))
              << createItem("dtSettings", MMActionDtSettings, "qrc:/icons/dt_settings_64.png", tr("Настройки\nдаты/времени"))
              << createItem("wifiSettings", MMActionWifiSettings, "qrc:/icons/wifi_settings_64.png", tr("Настройки\nWiFi"))
              << createItem("gsmSettings", MMActionGsmSettings, "qrc:/icons/gsm_settings_64.png", tr("Настройки\nGSM"))
              << createItem("mobileDataSettings", MMActionMobileDataSettings, "qrc:/icons/mobiledata_settings_64.png", tr("Передача\nданных"));
    }

//    items.append(QVector<QVariant>(11 - items.size(),  createEmpty()).toList());
//    items << createItem("back", MMActionMenuBack, "qrc:/icons/back_64.png", tr("Назад"));
    createSubMenu(MMActionConfig, items);

}

void MainMenuModel::createFirmwareSub()
{
    QVariantList items;
    items << createItem("selfTest", MMActionSelfTest, "qrc:/icons/test_64.png", tr("Тестирование"));
    items << createItem("firmwareTest", MMActionFirmwareTest, "qrc:/icons/test_firmware_64.png", tr("Тестирование\nпрошивки"));
    Cashier c = session_->cashier();

    if(c.hasPermission(Cashier::Permission::Settings) && ENABLE_SELF_APK_UPDATE_MENU)
    {
        items << createItem("firmwareUpdate", MMActionFirmwareUpdate,
                            "qrc:/icons/update_firmware_64.png", tr("Обновление\nпрошивки"));
    }

//    items.append(QVector<QVariant>(11 - items.size(),  createEmpty()).toList());
//    items << createItem("back", MMActionMenuBack, "qrc:/icons/back_64.png", tr("Назад"));
    createSubMenu(MMActionConfig, items);
}

void MainMenuModel::createSubMenu(MainMenuModel::MMAction parent, const QVariantList &items)
{
    if(items.isEmpty())
    {

    }
    else
    {
        if(subStack_.isEmpty() || subStack_.top() != parent) subStack_.push(parent);
        beginResetModel();
        items_ = items;
        endResetModel();
    }
    emit isSubMenuChanged();
}
