#ifndef MAINMENUMODEL_H
#define MAINMENUMODEL_H

#include "cashboxstatuswrapper.h"
#include "sessionwrapper.h"

#include <QAbstractListModel>
#include <QStack>

class MainMenuModel : public QAbstractListModel
{
    Q_OBJECT
    QML_ELEMENT
    Q_DISABLE_COPY(MainMenuModel)

    Q_PROPERTY(bool isSubMenu READ isSubMenu NOTIFY isSubMenuChanged)
public:
    enum MMRole
    {
        ROLE_NAME                       = Qt::UserRole + 1,
        ROLE_ICON                       = Qt::UserRole + 2,
        ROLE_ACTION                     = Qt::UserRole + 3,
        ROLE_LABEL                      = Qt::UserRole + 4
    };
    Q_ENUM(MMRole)
    enum MMAction
    {
        MMNoAction                      = 0x00000000,
        MMActionExit                    = 0x00000001,
        MMActionConfig                  = 0x00000002,
        MMActionCatalogs                = 0x00000003,
        MMActionReports                 = 0x00000004,
        MMActionCashboxOps              = 0x00000005,
        MMActionMenuBack                = 0x00000006,
        MMActionSale                    = 0x00000007,
        MMActionRefund                  = 0x00000008,
        MMActionPurchase                = 0x00000009,
        MMActionPurchaseRefund          = 0x0000000A,
        MMActionDebitCorrection         = 0x0000000B,
        MMActionCreditCorrection        = 0x0000000C,
        MMActionOpenCycle               = 0x0000000D,
        MMActionCloseCycle              = 0x0000000E,
        MMActionEditCashiers            = 0x0000000F,
        MMActionStatus                  = 0x00000010,
        MMActionIntroduction            = 0x00000011,
        MMActionPayout                  = 0x00000012,
        MMActionService                 = 0x00000013,
        MMActionFiscalize               = 0x00000014,
        MMActionCloseArchive            = 0x00000015,
        MMActionCleanCashbox            = 0x00000016,
        MMActionCalcReport              = 0x00000017,
        MMActionFsArchiveReport         = 0x00000018,
        MMOfdSettings                   = 0x00000019,
        MMActionLogs                    = 0x0000001A,
        MMActionCorrections             = 0x0000001B,
        MMActionDebitRefundCorrection   = 0x0000001C,
        MMActionCreditRefundCorrection  = 0x0000001D,
        MMActionDtSettings              = 0x0000001E,
        MMActionWifiSettings            = 0x0000001F,
        MMActionGsmSettings             = 0x00000020,
        MMActionMobileDataSettings      = 0x00000021,
        MMActionFirmware                = 0x00000022,
        MMActionSelfTest                = 0x00000023,
        MMActionFirmwareTest            = 0x00000024,
        MMActionFirmwareUpdate          = 0x00000025,
        MMActionOfflineReport           = 0x00000026,
        MMActionCleanMgm                = 0x00000027,
        MMActionClearAll                = 0x00000028,
        MMActionXReport                 = 0x00000029,
        MMActionFsReport                = 0x0000002A,
        MMActionReceiptConfig           = 0x0000002B,
    };
    Q_ENUM(MMAction)

    explicit MainMenuModel(QObject *parent = nullptr);
    ~MainMenuModel() override;

    QHash<int, QByteArray> roleNames() const override;
    QVariant data(const QModelIndex &idx, int role) const override;
    int rowCount(const QModelIndex &parent = QModelIndex()) const override;

    Q_INVOKABLE QVariantMap get(qint32 row) const;
    Q_INVOKABLE void action(qint32 act);

    bool isSubMenu() const;

signals:
    void showPage(QString);
    void showFirstPage(QString);
    void isSubMenuChanged();
public slots:
    void reload();
    void goBack();
private:
    CashboxStatusWrapper *status_;
    SessionWrapper *session_;
    QVariantList items_;
    QStack<MMAction> subStack_;

    QVariantMap createItem(const QString &name, MMAction action, const QString &icon, const QString &label);
    QVariantMap createEmpty();

    void createCashboxOpsSub();
    void createCorrectionsOpsSub();
    void createCatalogsSub();
    void createServiceSub();
    void createReportsSub();
    void createSettingsSub();
    void createFirmwareSub();
    void createSubMenu(MMAction parent, const QVariantList &items);

};

#endif // MAINMENUMODEL_H
