#ifndef RECEIPTMODEL_H
#define RECEIPTMODEL_H


#include "receipt.h"
#include "cashboxstatuswrapper.h"
#include "sessionwrapper.h"

#include <QAbstractListModel>

class ReceiptModel :public QAbstractListModel
{
    Q_OBJECT
    Q_DISABLE_COPY(ReceiptModel)
    QML_ELEMENT

    Q_PROPERTY(ReceiptType receiptType READ receiptType WRITE setReceiptType NOTIFY receiptTypeChanged)
    Q_PROPERTY(bool isCorrection READ isCorrection WRITE setCorrection NOTIFY correctionChanged)
    Q_PROPERTY(bool supportLabels READ supportLabels  NOTIFY supportLabelsChanged)
    Q_PROPERTY(bool containsLabels READ containsLabels  NOTIFY containsLabelsChanged)
    Q_PROPERTY(QString totalAmount READ totalAmount NOTIFY totalAmountChanged)
    Q_PROPERTY(QString receiptName READ receiptName NOTIFY receiptNameChanged)
    Q_PROPERTY(QVariantList taxSystemModel READ taxSystemModel NOTIFY taxSystemModelChanged)

    Q_PROPERTY(bool canChangeOwnerName READ canChangeOwnerName NOTIFY canChangeOwnerNameChanged)
    Q_PROPERTY(bool canChangeAddress READ canChangeAddress NOTIFY canChangeAddressChanged)
    Q_PROPERTY(bool canChangePlace READ canChangePlace NOTIFY canChangePlaceChanged)
    Q_PROPERTY(bool canChangeBoxId READ canChangeBoxId NOTIFY canChangeBoxIdChanged)


    Q_PROPERTY(int taxIndex READ taxIndex WRITE setTaxIndex NOTIFY taxIndexChanged)
    Q_PROPERTY(QString cashier READ cashier WRITE setCashier NOTIFY cashierChanged)
    Q_PROPERTY(QString cashierInn READ cashierInn WRITE setCashierInn NOTIFY cashierInnChanged)
    Q_PROPERTY(QString ownerName READ ownerName WRITE setOwnerName NOTIFY ownerNameChanged)
    Q_PROPERTY(QString address READ address WRITE setAddress NOTIFY addressChanged)
    Q_PROPERTY(QString place READ place WRITE setPlace NOTIFY placeChanged)
    Q_PROPERTY(QString buyerPhone READ buyerPhone WRITE setBuyerPhone NOTIFY buyerPhoneChanged)
    Q_PROPERTY(QString boxId READ boxId WRITE setBoxId NOTIFY boxIdChanged)

    Q_PROPERTY(QVariantList industryProps READ industryProps WRITE setIndustryProps NOTIFY industryPropsChanged)

    Q_PROPERTY(QString additionalParam READ additionalParam WRITE setAdditionalParam NOTIFY additionalParamChanged)
    Q_PROPERTY(qint32 operationCode READ operationCode WRITE setOperationCode NOTIFY operationCodeChanged)
    Q_PROPERTY(QString operationDt READ operationDt WRITE setOperationDt NOTIFY operationDtChanged)
    Q_PROPERTY(QString operationParamData READ operationParamData WRITE setOperationParamData NOTIFY operationParamDataChanged)
    Q_PROPERTY(QString additionalUserParamName READ additionalUserParamName WRITE setAdditionalUserParamName NOTIFY additionalUserParamNameChanged)
    Q_PROPERTY(QString additionalUserParamVal READ additionalUserParamVal WRITE setAdditionalUserParamVal NOTIFY additionalUserParamValChanged)

    Q_PROPERTY(QString buyerName READ buyerName WRITE setBuyerName NOTIFY buyerNameChanged)
    Q_PROPERTY(QString buyerInn READ buyerInn WRITE setBuyerInn NOTIFY buyerInnChanged)
    Q_PROPERTY(QString buyerBirthDay READ buyerBirthDay WRITE setBuyerBirthDay NOTIFY buyerBirthDayChanged)
    Q_PROPERTY(QString buyerCitizenship READ buyerCitizenship WRITE setBuyerCitizenship NOTIFY buyerCitizenshipChanged)
    Q_PROPERTY(QString buyerDocCode READ buyerDocCode WRITE setBuyerDocCode NOTIFY buyerDocCodeChanged)
    Q_PROPERTY(int buyerDocCodeIndex READ buyerDocCodeIndex NOTIFY buyerDocCodeIndexChanged)
    Q_PROPERTY(QString buyerDocData READ buyerDocData WRITE setBuyerDocData NOTIFY buyerDocDataChanged)
    Q_PROPERTY(QString buyerAddress READ buyerAddress WRITE setBuyerAddress NOTIFY buyerAddressChanged)

    Q_PROPERTY(QString correctionDate READ correctionDate WRITE setCorrectionDate NOTIFY correctionDateChanged)
    Q_PROPERTY(QString correctionDocNumber READ correctionDocNumber WRITE setCorrectionDocNumber NOTIFY correctionDocNumberChanged)
    Q_PROPERTY(bool correctionIsIndependent READ correctionIsIndependent WRITE setCorrectionIsIndependent NOTIFY correctionIsIndependentChanged)
public:
    enum ReceiptType
    {
        Debit			= 0x01u,
        DebitReturn		= 0x02u,
        Credit			= 0x03u,
        CreditReturn	= 0x04u
    };
    Q_ENUM(ReceiptType)
    Q_ENUMS(ReceiptType)

    enum RecRoles
    {
        RoleIcon          = Qt::UserRole + 1,
        RoleName          = Qt::UserRole + 2,
        RolePrice         = Qt::UserRole + 3,
        RoleQuantity      = Qt::UserRole + 4,
        RoleType          = Qt::UserRole + 5,
    };

    explicit ReceiptModel(QObject *parent = nullptr);
    ~ReceiptModel()override;

    QHash<int, QByteArray> roleNames() const override;
    QVariant data(const QModelIndex &idx, int role) const override;
    int rowCount(const QModelIndex &parent = QModelIndex()) const override;

    ReceiptType  receiptType() const;
    void setReceiptType(ReceiptType type);

    bool isCorrection() const;
    void setCorrection(bool set);
    bool supportLabels() const;
    bool containsLabels() const;

    QString totalAmount() const;
    QString receiptName() const;

    bool canChangeOwnerName() const;
    bool canChangeAddress() const;
    bool canChangePlace() const;
    bool canChangeBoxId() const;

    Q_INVOKABLE QVariantMap operationData(int index) const;
    Q_INVOKABLE bool operationIsLabled(int index) const;
    Q_INVOKABLE double receiptAmount() const;
    Q_INVOKABLE double cash() const;
    Q_INVOKABLE double card() const;
    Q_INVOKABLE double prepayment() const;
    Q_INVOKABLE double postpayment() const;
    Q_INVOKABLE double barter() const;
    Q_INVOKABLE QVariantMap receiptData() const;

    Q_INVOKABLE QVariantList taxSystemModel() const;

    int taxIndex() const;
    void setTaxIndex(int idx);

    QString cashier() const;
    void setCashier(const QString &c);

    QString cashierInn() const;
    void setCashierInn(const QString &c);

    QString ownerName() const;
    void setOwnerName(const QString &newUserName);

    QString address() const;
    void setAddress(const QString &newAddress);

    QString place() const;
    void setPlace(const QString &newPlace);
    QString buyerPhone() const;
    void setBuyerPhone(const QString &phone);

    QString boxId() const;
    void setBoxId(const QString &id);


    QVariantList industryProps() const;


    const QString &additionalParam() const;
    void setAdditionalParam(const QString &newAdditionalParam);

    qint32 operationCode() const;
    void setOperationCode(qint32 newOperationCode);

    QString operationDt() const;
    void setOperationDt(const QString &newOperationDt);

    QString operationParamData() const;
    void setOperationParamData(const QString &newOperationParamData);


    QString additionalUserParamName() const;
    void setAdditionalUserParamName(const QString &newAdditionalUserParamName);

    QString additionalUserParamVal() const;
    void setAdditionalUserParamVal(const QString &newAdditionalUserParamVal);

    QString buyerName() const;
    void setBuyerName(const QString &newBuyerName);

    QString buyerInn() const;
    void setBuyerInn(const QString &newBuyerInn);

    QString buyerBirthDay() const;
    void setBuyerBirthDay(const QString &newBuyerBirthDay);

    QString buyerCitizenship() const;
    void setBuyerCitizenship(const QString &newBuyerCitizenship);

    QString buyerDocCode() const;
    void setBuyerDocCode(const QString &newBuyerDocCode);
    int buyerDocCodeIndex() const;

    QString buyerDocData() const;
    void setBuyerDocData(const QString &newBuyerDocData);

    QString buyerAddress() const;
    void setBuyerAddress(const QString &newBuyerAddress);

    QString correctionDate() const;
    void setCorrectionDate(const QString &newCorrectionDate);

    QString correctionDocNumber() const;
    void setCorrectionDocNumber(const QString &newCorrectionDocNumber);

    bool correctionIsIndependent() const;
    void setCorrectionIsIndependent(bool independent);

signals:
    void receiptTypeChanged();
    void correctionChanged();
    void supportLabelsChanged();
    void containsLabelsChanged();
    void totalAmountChanged();
    void receiptNameChanged();
    void canChangeOwnerNameChanged();
    void canChangeAddressChanged();
    void canChangePlaceChanged();
    void canChangeBoxIdChanged();
    void taxSystemModelChanged();

    void taxIndexChanged();
    void cashierChanged();
    void cashierInnChanged();
    void ownerNameChanged();
    void addressChanged();
    void placeChanged();
    void buyerPhoneChanged();
    void boxIdChanged();
    void industryPropsChanged();
    void additionalParamChanged();
    void operationCodeChanged();
    void operationDtChanged();
    void operationParamDataChanged();
    void additionalUserParamNameChanged();
    void additionalUserParamValChanged();
    void buyerNameChanged();
    void buyerInnChanged();
    void buyerBirthDayChanged();
    void buyerCitizenshipChanged();
    void buyerDocCodeChanged();
    void buyerDocCodeIndexChanged();
    void buyerDocDataChanged();
    void buyerAddressChanged();
    void correctionDateChanged();
    void correctionDocNumberChanged();
    void correctionIsIndependentChanged();

public slots:
    void addOperation(int index, const QVariantMap &map);
    void addLabledOperation(int index, const QVariantMap &map);
    void removeOperation(int index);
    void clean();
    void calcModels();
    void setIndustryProps(const QVariantList &props);
    void cleanOperationalParam();
    void cleanUserParam();
    void cleanBuyerInfo();
    void cleanCorrectionReason();

    void addQuantity(qint32 index);
    void subQuantity(qint32 index);
private:
    QVariantList taxes_;
    Receipt receipt_;
    CashboxStatusWrapper *status_;
    SessionWrapper *session_;
};

#endif // RECEIPTMODEL_H
