#include "hexstringvalidator.h"
#include "logmanip.h"

#include <QSet>

HexStringValidator::HexStringValidator(QObject *parent)
    : QValidator(parent)
    , maxSize_(64)
    , fixedSize_(false)
    , canBeEmpty_(true)
{

}

HexStringValidator::~HexStringValidator()
{

}

void HexStringValidator::fixup(QString &input) const
{
    QValidator::fixup(input);
}

QValidator::State HexStringValidator::validate(QString &input, int &pos) const
{
    static QSet<QChar> SUPPORTED = {
        QLatin1Char('0'),QLatin1Char('1'),QLatin1Char('2'),QLatin1Char('3'),
        QLatin1Char('4'),QLatin1Char('5'),QLatin1Char('6'),QLatin1Char('7'),
        QLatin1Char('8'),QLatin1Char('9'),QLatin1Char('a'),QLatin1Char('b'),
        QLatin1Char('c'),QLatin1Char('d'),QLatin1Char('e'),QLatin1Char('f')
    };

    lmWarning() << input << pos << input.size() << maxSize_;
    if(input.isEmpty())return canBeEmpty_ ? QValidator::Acceptable : QValidator::Intermediate;

    if(input.size() > maxSize_)
    {
        pos = maxSize_ - 1;
        return QValidator::Invalid;
    }
    pos = input.size() - 1;
    if(!SUPPORTED.contains(input[pos].toLower()))
    {
        --pos;
        return QValidator::Invalid;
    }

    return (!fixedSize_ || input.size() == maxSize_) ? QValidator::Acceptable :
                                                       QValidator::Intermediate;

}

qint32 HexStringValidator::maxSize() const
{
    return maxSize_;
}

void HexStringValidator::setMaxSize(qint32 newMaxSize)
{
    if (maxSize_ == newMaxSize)
        return;
    maxSize_ = newMaxSize;
    emit maxSizeChanged();
}

bool HexStringValidator::fixedSize() const
{
    return fixedSize_;
}

void HexStringValidator::setFixedSize(bool newFixedSize)
{
    if (fixedSize_ == newFixedSize)
        return;
    fixedSize_ = newFixedSize;
    emit fixedSizeChanged();
}

bool HexStringValidator::canBeEmpty() const
{
    return canBeEmpty_;
}

void HexStringValidator::setCanBeEmpty(bool newCanBeEmpty)
{
    if (canBeEmpty_ == newCanBeEmpty)
        return;
    canBeEmpty_ = newCanBeEmpty;
    emit canBeEmptyChanged();
}
