#include "runtimeuisettings.h"
#include "logmanip.h"

#include <QCoreApplication>

const QString RuntimeUiSettings::RUNTIME_SECTION = "UI_RUNTIME";

RuntimeUiSettings::RuntimeUiSettings(QObject *parent)
    : QObject(parent)
    , store_(new QSettings(QSettings::UserScope,
                           QCoreApplication::organizationName(),
                           QCoreApplication::applicationName(),
                           this))
{
}

RuntimeUiSettings::~RuntimeUiSettings()
{

}

fdf::Tax RuntimeUiSettings::lastTaxSystem() const
{
    fdf::Tax res = static_cast<fdf::Tax>(getValue(RUNTIME_SECTION, "lastTaxSys",
                                                  static_cast<qint32>(fdf::Tax::Invalid)).toInt());
    if(!fdf::taxIsValid(res))res = fdf::Tax::Invalid;
    return res;
}

void RuntimeUiSettings::setLastTaxSystem(fdf::Tax tax)
{
    setValue(RUNTIME_SECTION, "lastTaxSys", static_cast<qint32>(tax));
}

QVariant RuntimeUiSettings::getValue(const QString &section, const QString &key, const QVariant &defVal) const
{
    store_->beginGroup(section);
    QVariant res = store_->value(key, defVal);
    store_->endGroup();
    return res;
}

void RuntimeUiSettings::setValue(const QString &section, const QString &key, const QVariant &val)
{
    store_->beginGroup(section);
    store_->setValue(key, val);
    store_->endGroup();
}
