#include "sbarcodefilter.h"

#include "logmanip.h"
#include "androidapihelper.h"

#include <QOpenGLContext>
#include <QOpenGLFunctions>
#include <QBuffer>

#include <QMutex>

SBarcodeFilter::SBarcodeFilter(QObject *parent)
    : QAbstractVideoFilter(parent)
    , resultCode_()
{

}

SBarcodeFilter::~SBarcodeFilter()
{

}

QVideoFilterRunnable *SBarcodeFilter::createFilterRunnable()
{
    return new SBarcodeFilterRunnable(this);
}

void SBarcodeFilter::captureBarcode(const QString &text)
{
    if(!text.isEmpty())
    {
        bool changed = false;
        codeMutex_.lock();
        changed = text != resultCode_;
        resultCode_ = text;
        codeMutex_.unlock();
        if(changed)
        {
            setActive(false);
            emit barcodeParsed(text);
        }
    }
}

//--------------------------------------------------------------------------------------------------

SBarcodeFilterRunnable::SBarcodeFilterRunnable(SBarcodeFilter *parent)
    : parent_(parent)
    , helper_(new AndroidApiHelper(nullptr))
{

}

SBarcodeFilterRunnable::~SBarcodeFilterRunnable()
{
    delete helper_;
}

QVideoFrame SBarcodeFilterRunnable::run(QVideoFrame *input, const QVideoSurfaceFormat &surfaceFormat,
                                        QVideoFilterRunnable::RunFlags flags)
{
    Q_UNUSED(surfaceFormat);
    Q_UNUSED(flags);
    const QVideoFrame &videoFrame = *input;
    QImage outImage;
    QRect captureRect = surfaceFormat.viewport();
//    captureRect.setRect(272, 224, 752, 544);
//    lmWarning() << videoFrame.handleType() << videoFrame.pixelFormat() << QVideoFrame::imageFormatFromPixelFormat(videoFrame.pixelFormat()) << captureRect;
    if (videoFrame.handleType() == QAbstractVideoBuffer::NoHandle)
    {
        const QImage::Format imageFormat = QVideoFrame::imageFormatFromPixelFormat(videoFrame.pixelFormat());
        QImage image(videoFrame.bits(),
                videoFrame.width(),
                videoFrame.height(),
                videoFrame.bytesPerLine(),
                imageFormat);
        if (image.isNull()) {
        return QImage();
        }
        if ( image.format() != QImage::Format_RGB32) {
        image = image.convertToFormat(QImage::Format_RGB32);
        }
        outImage = image.copy(captureRect);
    }    

    if (videoFrame.handleType() == QAbstractVideoBuffer::GLTextureHandle)
    {
        QImage image(videoFrame.width(), videoFrame.height(), QImage::Format_RGB32);
        GLuint textureId = static_cast<GLuint>(videoFrame.handle().toInt());
        QOpenGLContext* ctx = QOpenGLContext::currentContext();
        QOpenGLFunctions* f = ctx->functions();
        GLuint fbo;
        f->glGenFramebuffers( 1, &fbo);
        GLint prevFbo;
        f->glGetIntegerv(GL_FRAMEBUFFER_BINDING, &prevFbo);
        f->glBindFramebuffer(GL_FRAMEBUFFER, fbo);
        f->glFramebufferTexture2D(GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0, GL_TEXTURE_2D, textureId, 0);
        f->glReadPixels(0, 0,  videoFrame.width(),  videoFrame.height(), GL_RGBA, GL_UNSIGNED_BYTE, image.bits());
        f->glBindFramebuffer( GL_FRAMEBUFFER, static_cast<GLuint>( prevFbo ) );
        outImage = image.rgbSwapped().copy(captureRect);
    }
    QString qr;
    if(!outImage.isNull())
    {
        QTransform transform;
        transform.rotate(90);
        outImage = outImage.transformed(transform, Qt::SmoothTransformation);
//        outImage.save("/storage/emulated/0/manul/logs/barcode1 .png");
        qr = parseImage(outImage);
        if(qr.isEmpty())
        {
            outImage.invertPixels(QImage::InvertRgb);
            qr = parseImage(outImage);
        }
    }

//    lmWarning() << "BARCODE_FILTER_RUN " << outImage.size() << qr <<outImage.isNull();
    if(!qr.isEmpty() && parent_)parent_->captureBarcode(qr);
    return *input;

}

QString SBarcodeFilterRunnable::parseImage(const QImage &img)
{
    QBuffer buffer;
    img.save(&buffer, "BMP", -1);
    return helper_->parseBarcode(buffer.data());
}
