#include "serialcheker.h"

#include "androidapihelper.h"
#include "cashboxbuildconfig.h"
#include "logmanip.h"

#include <QUrlQuery>
#include <QUrl>
#include <QNetworkRequest>
#include <QNetworkReply>
#include <QSslError>
#include <QJsonDocument>
#include <QJsonParseError>

const QString SerialCheker::CHECK_SERIAL_PATH = "/kkm-trade/api/ManufacturerAskSn";

SerialCheker::SerialCheker(QObject *parent)
    : QObject(parent)
    , serial_()
    , http_(new QNetworkAccessManager(this))
{
    connect(http_, &QNetworkAccessManager::sslErrors, this, &SerialCheker::sslErrors);
    connect(http_, &QNetworkAccessManager::finished, this, &SerialCheker::finished);
}

SerialCheker::~SerialCheker()
{

}

void SerialCheker::checkSerial(const QString &serial, const ModelData &model)
{
    serial_ = serial;

    QUrlQuery query;
    query.addQueryItem("idHardSoft", QString::number(model.srvModel()));
    query.addQueryItem("serialNo", serial);
    query.addQueryItem("version", AndroidApiHelper().versionName());
    query.addQueryItem("guidHardrawe", AndroidApiHelper().getHardwareHash());
    QUrl url;
    url.setHost(UMKASERVER_HOST_URL);
    url.setPath(CHECK_SERIAL_PATH);
    url.setPort(UMKASERVER_PORT);
    url.setScheme(UMKASERVER_SCHEME);
    url.setQuery(query);
    QNetworkRequest req(url);
    lmWarning() << req.url().toString();
    http_->get(req);
}

void SerialCheker::finished(QNetworkReply *reply)
{
    bool success = false;
    QString error;
    if(!reply)
    {
        if(sender())
        {
            if(sender() == http_) http_ = nullptr;
            sender()->blockSignals(true);
            sender()->deleteLater();
        }
        onError(tr("Не получен ответ от сервера"));
        return;
    }

    if(!reply->isFinished())
    {
        qWarning() << tr("Не получен ответ от сервера") ;
        onError(tr("Не получен ответ от сервера"));
        return;
    }

    if(reply->error() != QNetworkReply::NoError)
    {
        error = reply->errorString();
    }
    else
    {
        QJsonParseError jerr;
        QByteArray ansrepl = reply->readAll();

//        if(reply->rawHeader("Content-Encoding").contains("gzip"))
//        {
//            ansrepl = GZSW::gzuncompress(ansrepl);
//        }

        QJsonDocument jdoc = QJsonDocument::fromJson(ansrepl, &jerr);
        if(jerr.error != QJsonParseError::NoError)
        {
            error = jerr.errorString();
        }
        else
        {
            ansrepl.clear();
            QVariant data = jdoc.toVariant();
            if(data.type() == QVariant::Map && data.toMap().contains("result"))
            {
                error = data.toMap()["resultDescription"].toString();
            }
            else
            {
                lmWarning() << logvariant(data);
                QString res = data.toMap().value("data").toString();

                if ((res != "ok") && (res != "ok2")){
                     error =
                        (res == "failSerial") ? tr("Unknown Serial Number") :
                        (res == "failGUID") ? tr("This serial number is already registered at another box office") :
                        (res == "failHardSoft") ? tr("This device is not supported") : tr("Какая то ошибка");
                }
                else success = true;
            }

        }
    }

    QNetworkAccessManager* mgr = reply->manager();
    if(mgr)
    {

        mgr->blockSignals(true);
        if(mgr == http_) http_ = nullptr;
        if(reply)
        {
            reply->close();
            reply->deleteLater();
        }
        mgr->deleteLater();
    }
    else if(reply)
    {
        reply->close();
        reply->deleteLater();
    }
    if(success) emit serialIsValid(serial_);
    else emit serialChekingError(error);
}

void SerialCheker::sslErrors(QNetworkReply *reply, const QList<QSslError> &errors)
{
    for(QSslError err: errors) lmWarning() << err.errorString();
    if(reply)reply->ignoreSslErrors(errors);
}

void SerialCheker::onError(const QString &error)
{
    emit serialChekingError(error);
}
