#include "agentdatawrapper.h"

AgentDataWrapper::AgentDataWrapper(QObject *parent)
    : QObject(parent)
    , agentFlagModel_()
    , agentFlag_(NotAgent)
    , providerInn_()
    , providerData_()
    , agentData_()
{
    calcModels();
}

AgentDataWrapper::~AgentDataWrapper()
{

}

QVariantList AgentDataWrapper::agentFlagModel() const
{
    return agentFlagModel_;
}

int AgentDataWrapper::agentFlagIndex() const
{
    if(!fdf::itemPayAgentFlagIsValid(static_cast<fdf::ItemPayAgentFlag>(agentFlag_))) return 0;
    for(int i = 1; i < agentFlagModel_.size(); i++)
    {
        if(agentFlagModel_[i].toMap()["code"].toInt() == agentFlag_) return i;
    }
    return 0;
}

void AgentDataWrapper::setAgentFlagIndex(int idx)
{
    ItemPayAgentFlag f = ItemPayAgentFlag::NotAgent;
    if(idx > 0 && idx < agentFlagModel_.size())
    {
        f = static_cast<ItemPayAgentFlag>(agentFlagModel_[idx].toMap()["code"].toInt());
    }
    setAgentFlag(f);
}

AgentDataWrapper::ItemPayAgentFlag AgentDataWrapper::agentFlag() const
{
    return agentFlag_;
}

void AgentDataWrapper::setAgentFlag(ItemPayAgentFlag f)
{
    if(f != agentFlag_)
    {
        agentFlag_ = f;
        emit agentFlagIndexChanged();
        emit agentFlagChanged();
        emit isPayAgentChanged();
        emit isBankAgentChanged();
    }
}

bool AgentDataWrapper::isPayAgent() const
{
    return agentFlag_ & (PayAgent    |
                         PaySubAgent );
}

bool AgentDataWrapper::isBankAgent() const
{
    return agentFlag_ & (BankAgent   |
                         BankSubAgent);
}

const QString &AgentDataWrapper::providerInn() const
{
    return providerInn_;
}

void AgentDataWrapper::setProviderInn(const QString &newProviderInn)
{
    if (providerInn_ == newProviderInn)
        return;
    providerInn_ = newProviderInn;
    emit providerInnChanged();
}

QString AgentDataWrapper::providerName() const
{
    return providerData_.name();
}

void AgentDataWrapper::setProviderName(const QString &name)
{
    if(name.trimmed() != providerData_.name())
    {
        providerData_.setName(name.trimmed());
        emit providerNameChanged();
    }
}

QString AgentDataWrapper::providerPhones() const
{
    return providerData_.phones().join(",");
}

void AgentDataWrapper::setProviderPhones(const QString &phones)
{
    QStringList sl = phonesFromString(phones);
    if(providerData_.phones() != sl)
    {
        providerData_.setPhones(sl);
        emit providerPhonesChanged();
    }
}

QString AgentDataWrapper::operatorName() const
{
    return agentData_.operatorName();
}

void AgentDataWrapper::setOperatorName(const QString &name)
{
    if(agentData_.operatorName() != name)
    {
        agentData_.setOperatorName(name);
    }
}

QString AgentDataWrapper::operatorInn() const
{
    return agentData_.operatorInn();
}

void AgentDataWrapper::setOperatorInn(const QString &inn)
{
    if(agentData_.operatorInn() != inn)
    {
        agentData_.setOperatorInn(inn);
        emit operatorInnChanged();
    }
}

QString AgentDataWrapper::operatorAddress() const
{
    return agentData_.operatorAddress();
}

void AgentDataWrapper::setOperatorAddress(const QString &address)
{
    if(agentData_.operatorAddress() != address)
    {
        agentData_.setOperatorAddress(address);
        emit operatorAddressChanged();
    }
}

QString AgentDataWrapper::agentPhones() const
{
    return agentData_.agentPhones().join(",");
}

void AgentDataWrapper::setAgentPhones(const QString &phones)
{
    QStringList sl = phonesFromString(phones);
    if(agentData_.agentPhones() != sl)
    {
        agentData_.setAgentPhones(sl);
        emit agentPhonesChanged();
    }
}

QString AgentDataWrapper::payOperatorPhones() const
{
    return agentData_.paysOperatorPhones().join(",");
}

void AgentDataWrapper::setPayOperatorPhones(const QString &phones)
{
    QStringList sl = phonesFromString(phones);
    if(agentData_.paysOperatorPhones() != sl)
    {
        agentData_.setPaysOperatorPhones(sl);
        emit payOperatorPhonesChanged();
    }
}

QString AgentDataWrapper::operatorPhones() const
{
    return agentData_.operatorPhones().join(",");
}

void AgentDataWrapper::setOperatorPhones(const QString &phones)
{
    QStringList sl = phonesFromString(phones);
    if(agentData_.operatorPhones() != sl)
    {
        agentData_.setOperatorPhones(sl);
        emit operatorPhonesChanged();
    }
}

QString AgentDataWrapper::bankOperation() const
{
    return agentData_.bankOperation();
}

void AgentDataWrapper::setBankOperation(const QString &bankOperation)
{
    if(agentData_.bankOperation() != bankOperation)
    {
        agentData_.setBankOperation(bankOperation);
        emit bankOperationChanged();
    }
}

void AgentDataWrapper::save()
{
    if(agentFlag_ == NotAgent)
    {
        emit saveData(QVariantMap());
        emit finish();
    }
    else
    {
        Inn pInn(providerInn_);
        if(!pInn.isValid())
        {
            emit error(tr("Некорректный ИНН поставщика"));
            return;
        }
        CoreApiConst::ErrorCode err = CoreApiConst::ErrorCode::Ok;
        QString msg;
        if(!providerData_.isValid(false, err, msg))
        {
#if CASHBOX_MODEL != 2000
            emit error(msg);
            return;
#endif
        }
        QVariantMap res = {
            {"agentFlag", static_cast<qint32>(agentFlag_)},
            {"providerInn", providerInn_.trimmed()},
            {"providerData", providerData_.toMap()}
        };
        if(!isBankAgent() && !isPayAgent())
        {
            emit saveData(res);
            emit finish();
        }
        else
        {
            if(!agentData_.isValid(static_cast<fdf::ItemPayAgentFlag>(agentFlag_), err, msg))
            {
                emit error(msg);
                return;
            }
            res ["agentData"] = agentData_.toMap();
            emit saveData(res);
            emit finish();
        }
    }

}
void AgentDataWrapper::clean()
{
    agentFlag_ = NotAgent;
    providerInn_.clear();
    providerData_ = ProviderData();
    agentData_ = AgentData();

    emit agentFlagIndexChanged();
    emit agentFlagChanged();
    emit isPayAgentChanged();
    emit isBankAgentChanged();
    emit providerInnChanged();
    emit providerNameChanged();
    emit providerPhonesChanged();
    emit operatorNameChanged();
    emit operatorInnChanged();
    emit operatorAddressChanged();
    emit agentPhonesChanged();
    emit payOperatorPhonesChanged();
    emit operatorPhonesChanged();
}

void AgentDataWrapper::setData(const QVariantMap &map)
{
    clean();
    if(!map.isEmpty())
    {
        ItemPayAgentFlag f = static_cast<ItemPayAgentFlag>(map["agentFlag"].toInt());
        if(fdf::itemPayAgentFlagIsValid(static_cast<fdf::ItemPayAgentFlag>(f)))
        {
            agentFlag_ = f;
            providerInn_ = map["providerInn"].toString();
            providerData_.parseMap(map["providerData"].toMap());
            if(f <= PaySubAgent)
            {
                agentData_.parseMap(map["agentData"].toMap());
            }
        }
    }
    emit agentFlagIndexChanged();
    emit agentFlagChanged();
    emit isPayAgentChanged();
    emit isBankAgentChanged();
    emit providerInnChanged();
    emit providerNameChanged();
    emit providerPhonesChanged();
    emit operatorNameChanged();
    emit operatorInnChanged();
    emit operatorAddressChanged();
    emit agentPhonesChanged();
    emit payOperatorPhonesChanged();
    emit operatorPhonesChanged();
}

void AgentDataWrapper::calcModels()
{
    QVariantList res;
    QVariantMap m;
    m["code"] = NotAgent;
    m["name"] = tr("НЕ АГЕНТ");
    res << m;
    auto add = [&res](ItemPayAgentFlag f) {
        res << QVariantMap {
            {"code", f},
            {"name", fdf::itemPayAgentFlagToString(static_cast<fdf::ItemPayAgentFlag>(f))}
        };
    };
    add(BankAgent   );
    add(BankSubAgent);
    add(PayAgent    );
    add(PaySubAgent );
    add(Attorney    );
    add(Broker      );
    add(Agent       );
    agentFlagModel_ = res;
    emit agentFlagModelChanged();
}

QStringList AgentDataWrapper::phonesFromString(const QString &phones) const
{
    QStringList sl = phones.split(",");
    std::for_each(sl.begin(), sl.end(),[](QString &s){s = s.trimmed();if(!s.startsWith("+")) s = "+" + s;});
    return sl;
}
