#ifndef AGENTDATAWRAPPER_H
#define AGENTDATAWRAPPER_H

#include "receiptoperation.h"

#include <QObject>
#include <QtQml>

class AgentDataWrapper : public QObject
{
    Q_OBJECT
    QML_ELEMENT
    Q_DISABLE_COPY(AgentDataWrapper)

    Q_PROPERTY(QVariantList agentFlagModel READ agentFlagModel NOTIFY agentFlagModelChanged)
    Q_PROPERTY(int agentFlagIndex READ agentFlagIndex WRITE setAgentFlagIndex NOTIFY agentFlagIndexChanged)

    Q_PROPERTY(ItemPayAgentFlag agentFlag READ agentFlag WRITE setAgentFlag NOTIFY agentFlagChanged)
    Q_PROPERTY(bool isPayAgent READ isPayAgent NOTIFY isPayAgentChanged)
    Q_PROPERTY(bool isBankAgent READ isBankAgent NOTIFY isBankAgentChanged)

    Q_PROPERTY(QString providerInn   READ providerInn WRITE setProviderInn NOTIFY providerInnChanged)
    Q_PROPERTY(QString providerName  READ providerName WRITE setProviderName NOTIFY providerNameChanged)
    Q_PROPERTY(QString providerPhones READ providerPhones WRITE setProviderPhones NOTIFY providerPhonesChanged)

    Q_PROPERTY(QString operatorName      READ operatorName WRITE setOperatorName NOTIFY operatorNameChanged)
    Q_PROPERTY(QString operatorInn      READ operatorInn WRITE setOperatorInn NOTIFY operatorInnChanged)
    Q_PROPERTY(QString operatorAddress  READ operatorAddress WRITE setOperatorAddress NOTIFY operatorAddressChanged)
    Q_PROPERTY(QString agentPhones      READ agentPhones WRITE setAgentPhones NOTIFY agentPhonesChanged)
    Q_PROPERTY(QString payOperatorPhones READ payOperatorPhones WRITE setPayOperatorPhones NOTIFY payOperatorPhonesChanged)
    Q_PROPERTY(QString operatorPhones    READ operatorPhones WRITE setOperatorPhones NOTIFY operatorPhonesChanged)
    Q_PROPERTY(QString bankOperation    READ bankOperation WRITE setBankOperation NOTIFY bankOperationChanged)


public:
    enum ItemPayAgentFlag
    {
        NotAgent                            = 0x00,
        BankAgent                           = 0x01,
        BankSubAgent                        = 0x02,
        PayAgent                            = 0x04,
        PaySubAgent                         = 0x08,
        Attorney                            = 0x10,
        Broker                              = 0x20,
        Agent                               = 0x40
    };
    Q_ENUM(ItemPayAgentFlag)
    Q_ENUMS(ItemPayAgentFlag)

    explicit AgentDataWrapper(QObject *parent = nullptr);
    ~AgentDataWrapper() override;

    QVariantList agentFlagModel() const;

    int agentFlagIndex() const;
    void setAgentFlagIndex(int idx);

    ItemPayAgentFlag agentFlag() const;
    void setAgentFlag(ItemPayAgentFlag f);

    bool isPayAgent() const;
    bool isBankAgent() const;

    const QString &providerInn() const;
    void setProviderInn(const QString &newProviderInn);

    QString providerName() const;
    void setProviderName(const QString &name);

    QString providerPhones() const;
    void setProviderPhones(const QString &phones);

    QString operatorName() const;
    void setOperatorName(const QString &name);

    QString operatorInn() const;
    void setOperatorInn(const QString &inn);

    QString operatorAddress() const;
    void setOperatorAddress(const QString &address);

    QString agentPhones() const;
    void setAgentPhones(const QString &phones);

    QString payOperatorPhones() const;
    void setPayOperatorPhones(const QString &phones);

    QString operatorPhones() const;
    void setOperatorPhones(const QString &phones);

    QString bankOperation() const;
    void setBankOperation(const QString &bankOperation);


signals:
    void saveData(QVariantMap);
    void finish();
    void error(QString msg);
    void agentFlagModelChanged();
    void agentFlagIndexChanged();
    void agentFlagChanged();
    void isPayAgentChanged();
    void isBankAgentChanged();

    void providerInnChanged();
    void providerNameChanged();
    void providerPhonesChanged();
    void operatorNameChanged();
    void operatorInnChanged();
    void operatorAddressChanged();
    void agentPhonesChanged();
    void payOperatorPhonesChanged();
    void operatorPhonesChanged();
    void bankOperationChanged();
public slots:
    void save();
    void clean();
    void setData(const QVariantMap &map);
private:
    QVariantList agentFlagModel_;
    ItemPayAgentFlag agentFlag_;
    QString providerInn_;
    ProviderData providerData_;
    AgentData agentData_;

    void calcModels();
    QStringList phonesFromString(const QString &phones) const;
};

#endif // AGENTDATAWRAPPER_H
