#include "basecatalogitemwrapper.h"

#include "basecatalogmodel.h"

BaseCatalogItemWrapper::BaseCatalogItemWrapper(QObject *parent)
    : QObject(parent)
    , model_(nullptr)
{

}

BaseCatalogItemWrapper::~BaseCatalogItemWrapper()
{

}

void BaseCatalogItemWrapper::setData(const QVariantMap &data)
{
    parseMap(data);
}

void BaseCatalogItemWrapper::save()
{
    if(!model_)
    {
        emit error(tr("Нет модели данных"));
    }
    else
    {
        emit showBusy(true);
        model_->saveItem(this);
    }
}

void BaseCatalogItemWrapper::onSavingError(const QString &msg)
{
    emit showBusy(false);
    emit error(msg);
}

void BaseCatalogItemWrapper::onItemSaved()
{
    emit showBusy(false);
    emit saved();
}

BaseCatalogModel *BaseCatalogItemWrapper::model() const
{
    return model_;
}

void BaseCatalogItemWrapper::setModel(BaseCatalogModel *model)
{
    model_ = model;
    if(model)
    {
        connect(model, &BaseCatalogModel::itemSaved,
                this, &BaseCatalogItemWrapper::onItemSaved);
        connect(model, &BaseCatalogModel::itemSavingError,
                this, &BaseCatalogItemWrapper::onSavingError);
    }
    processModel();
}
