#include "baseworkincomingwrapper.h"

BaseWorkIncomingWrapper::BaseWorkIncomingWrapper(BaseWorkIncoming *data, QObject *parent)
    : QObject(parent)
    , data_(data)
    , status_(new CashboxStatusWrapper(this))
    , session_(new SessionWrapper(this))
{
    connect(status_, &CashboxStatusWrapper::statusChanged,
            this, &BaseWorkIncomingWrapper::onStatusChanged);
    onStatusChanged();
    reset();
}

BaseWorkIncomingWrapper::~BaseWorkIncomingWrapper()
{
    if(data_) delete data_;
}

QString BaseWorkIncomingWrapper::cashier() const
{
    return data_ ? data_->cashier() : QString();
}

void BaseWorkIncomingWrapper::setCashier(const QString &newCashier)
{
    if(data_ && data_->cashier() != newCashier)
    {
        data_->setCashier(newCashier);
        emit cashierChanged();
    }
}

QString BaseWorkIncomingWrapper::cashierInn() const
{
    return data_ ? data_->cashierInn() : QString();
}

void BaseWorkIncomingWrapper::setCashierInn(const QString &newCashierInn)
{
    if(data_ && data_->cashierInn() != newCashierInn)
    {
        data_->setCashierInn(newCashierInn);
        emit cashierInnChanged();
    }
}

QString BaseWorkIncomingWrapper::additionalParam() const
{
    return data_ ? data_->additionalParam() : QString();

}

void BaseWorkIncomingWrapper::setAdditionalParam(const QString &newAdditionalParam)
{
    if(data_ && data_->additionalParam() != newAdditionalParam)
    {
        data_->setAdditionalParam(newAdditionalParam);
        emit additionalParamChanged();
    }
}

QString BaseWorkIncomingWrapper::additionalData() const
{
    return data_ ? QString::fromLatin1(data_->additionalData().toHex()) : QString();
}

void BaseWorkIncomingWrapper::setAdditionalData(const QString &newAdditionalData)
{
    if(data_ && data_->additionalData().toHex() != newAdditionalData.toLatin1())
    {
        data_->setAdditionalData(QByteArray::fromHex(newAdditionalData.toLatin1()));
        emit additionalDataChanged();
    }
}

QString BaseWorkIncomingWrapper::ownerName() const
{
    return data_ ? data_->ownerName() : QString();
}

void BaseWorkIncomingWrapper::setOwnerName(const QString &newUserName)
{
    if(data_ && canChangeOwnerName() && data_->ownerName() != newUserName)
    {
        data_->setOwnerName(newUserName);
        emit ownerNameChanged();
    }
}

QString BaseWorkIncomingWrapper::address() const
{
    return data_ ? data_->address() : QString();
}

void BaseWorkIncomingWrapper::setAddress(const QString &newAddress)
{
    if(data_ && canChangeAddress() && data_->address() != newAddress)
    {
        data_->setAddress(newAddress);
        emit addressChanged();
    }
}

QString BaseWorkIncomingWrapper::place() const
{
    return data_ ? data_->place() : QString();
}

void BaseWorkIncomingWrapper::setPlace(const QString &newPlace)
{
    if(data_ && canChangePlace() && data_->place() != newPlace)
    {
        data_->setPlace(newPlace);
        emit placeChanged();
    }
}

bool BaseWorkIncomingWrapper::canChangeOwnerName() const
{
    return false;
}

bool BaseWorkIncomingWrapper::canChangeAddress() const
{
    return true;//status_->status().regData().address().isEmpty();
}

bool BaseWorkIncomingWrapper::canChangePlace() const
{
    return true;//status_->status().regData().place().isEmpty();
}

QVariantMap BaseWorkIncomingWrapper::toMap() const
{
    return data_ ? data_->toMap() : QVariantMap();
}

void BaseWorkIncomingWrapper::reset()
{
    if(data_)
    {
        data_->clean();
        data_->setCashier(session_->cashierName());
        data_->setCashierInn(session_->cashier().inn());
        RegData rd = status_->status().regData();
        data_->setAddress(rd.address());
        data_->setPlace(rd.place());
        data_->setOwnerName(rd.ownerName());
        emit cashierChanged();
        emit cashierInnChanged();
        emit additionalParamChanged();
        emit additionalDataChanged();
        emit ownerNameChanged();
        emit addressChanged();
        emit placeChanged();
    }
}

void BaseWorkIncomingWrapper::onStatusChanged()
{
    emit canChangeOwnerNameChanged();
    emit canChangeAddressChanged();
    emit canChangePlaceChanged();
}

//--------------------------------------------------------------------------------------------------
CycleOpenIncomingWrapper::CycleOpenIncomingWrapper(QObject *parent)
    : BaseWorkIncomingWrapper(new CycleOpenIncoming, parent)
{

}

CycleOpenIncomingWrapper::~CycleOpenIncomingWrapper()
{

}

CycleOpenIncoming *CycleOpenIncomingWrapper::incoming() const
{
    return dynamic_cast<CycleOpenIncoming *>(data_);
}

//--------------------------------------------------------------------------------------------------

CycleCloseIncomingWrapper::CycleCloseIncomingWrapper(QObject *parent)
    : BaseWorkIncomingWrapper(new CycleCloseIncoming, parent)
{

}

CycleCloseIncomingWrapper::~CycleCloseIncomingWrapper()
{

}

CycleCloseIncoming *CycleCloseIncomingWrapper::incoming() const
{
    return dynamic_cast<CycleCloseIncoming *>(data_);
}
//--------------------------------------------------------------------------------------------------
CalcReportIncomingWrapper::CalcReportIncomingWrapper(QObject *parent)
    : BaseWorkIncomingWrapper(new CalcReportIncoming, parent)
{

}

CalcReportIncomingWrapper::~CalcReportIncomingWrapper()
{

}

CalcReportIncoming *CalcReportIncomingWrapper::incoming() const
{
    return dynamic_cast<CalcReportIncoming *>(data_);
}
//--------------------------------------------------------------------------------------------------
FsCloseIncomingWrapper::FsCloseIncomingWrapper(QObject *parent)
    : BaseWorkIncomingWrapper(new FsCloseIncoming, parent)
{

}

FsCloseIncomingWrapper::~FsCloseIncomingWrapper()
{

}

FsCloseIncoming *FsCloseIncomingWrapper::incoming() const
{
    return dynamic_cast<FsCloseIncoming *>(data_);
}
