#ifndef BASEWORKINCOMINGWRAPPER_H
#define BASEWORKINCOMINGWRAPPER_H

#include "cycleopendoc.h"
#include "cycleclosedoc.h"
#include "calcreportdoc.h"
#include "fsclosedoc.h"
#include "cashboxstatuswrapper.h"
#include "sessionwrapper.h"

#include <QObject>
#include <QtQml>

class BaseWorkIncomingWrapper : public QObject
{
    Q_OBJECT
    Q_DISABLE_COPY(BaseWorkIncomingWrapper)
    QML_ELEMENT

    Q_PROPERTY(QString cashier READ cashier WRITE setCashier NOTIFY cashierChanged)
    Q_PROPERTY(QString cashierInn READ cashierInn WRITE setCashierInn NOTIFY cashierInnChanged)
    Q_PROPERTY(QString additionalParam READ additionalParam WRITE setAdditionalParam NOTIFY additionalParamChanged)
    Q_PROPERTY(QString additionalData READ additionalData WRITE setAdditionalData NOTIFY additionalDataChanged)
    Q_PROPERTY(QString ownerName READ ownerName WRITE setOwnerName NOTIFY ownerNameChanged)
    Q_PROPERTY(QString address READ address WRITE setAddress NOTIFY addressChanged)
    Q_PROPERTY(QString place READ place WRITE setPlace NOTIFY placeChanged)

    Q_PROPERTY(bool canChangeOwnerName READ canChangeOwnerName NOTIFY canChangeOwnerNameChanged)
    Q_PROPERTY(bool canChangeAddress READ canChangeAddress NOTIFY canChangeAddressChanged)
    Q_PROPERTY(bool canChangePlace READ canChangePlace NOTIFY canChangePlaceChanged)

public:
    explicit BaseWorkIncomingWrapper(BaseWorkIncoming *data, QObject *parent = nullptr);
    ~BaseWorkIncomingWrapper() override;

    QString cashier() const;
    void setCashier(const QString &newCashier);

    QString cashierInn() const;
    void setCashierInn(const QString &newCashierInn);

    QString additionalParam() const;
    void setAdditionalParam(const QString &newAdditionalParam);

    QString additionalData() const;
    void setAdditionalData(const QString &newAdditionalData);

    QString ownerName() const;
    void setOwnerName(const QString &newUserName);

    QString address() const;
    void setAddress(const QString &newAddress);

    QString place() const;
    void setPlace(const QString &newPlace);

    bool canChangeOwnerName() const;
    bool canChangeAddress() const;
    bool canChangePlace() const;

    Q_INVOKABLE QVariantMap toMap() const;

public slots:
    void reset();
signals:
    void cashierChanged();
    void cashierInnChanged();
    void additionalParamChanged();
    void additionalDataChanged();
    void ownerNameChanged();
    void addressChanged();
    void placeChanged();

    void canChangeOwnerNameChanged();
    void canChangeAddressChanged();
    void canChangePlaceChanged();
    void error(QString);

protected:
    BaseWorkIncoming *data_;
    CashboxStatusWrapper *status_;
    SessionWrapper *session_;

private slots:
    void onStatusChanged();
};


//--------------------------------------------------------------------------------------------------

class CycleOpenIncomingWrapper :public BaseWorkIncomingWrapper
{
    Q_OBJECT
    QML_ELEMENT
    Q_DISABLE_COPY(CycleOpenIncomingWrapper)
public:
    explicit CycleOpenIncomingWrapper(QObject *parent = nullptr);
    ~CycleOpenIncomingWrapper() override;
private:
    CycleOpenIncoming *incoming() const;
};

//--------------------------------------------------------------------------------------------------

class CycleCloseIncomingWrapper :public BaseWorkIncomingWrapper
{
    Q_OBJECT
    QML_ELEMENT
    Q_DISABLE_COPY(CycleCloseIncomingWrapper)
public:
    explicit CycleCloseIncomingWrapper(QObject *parent = nullptr);
    ~CycleCloseIncomingWrapper() override;
private:
    CycleCloseIncoming *incoming() const;
};
//--------------------------------------------------------------------------------------------------

class CalcReportIncomingWrapper :public BaseWorkIncomingWrapper
{
    Q_OBJECT
    QML_ELEMENT
    Q_DISABLE_COPY(CalcReportIncomingWrapper)
public:
    explicit CalcReportIncomingWrapper(QObject *parent = nullptr);
    ~CalcReportIncomingWrapper() override;
private:
    CalcReportIncoming *incoming() const;
};
//--------------------------------------------------------------------------------------------------

class FsCloseIncomingWrapper :public BaseWorkIncomingWrapper
{
    Q_OBJECT
    QML_ELEMENT
    Q_DISABLE_COPY(FsCloseIncomingWrapper)
public:
    explicit FsCloseIncomingWrapper(QObject *parent = nullptr);
    ~FsCloseIncomingWrapper() override;
private:
    FsCloseIncoming *incoming() const;
};
#endif // BASEWORKINCOMINGWRAPPER_H
