#ifndef CASHBOXSTATUSWRAPPER_H
#define CASHBOXSTATUSWRAPPER_H

#include "cashboxstatus.h"

#include <QObject>
#include <QPointer>
#include <QMutex>
#include <QtQml>

class CashboxStatusWrapper : public QObject
{
    Q_OBJECT
    QML_ELEMENT
    QML_NAMED_ELEMENT(CashboxStatus)
    Q_DISABLE_COPY(CashboxStatusWrapper)
    friend class UiCashboxServiceApi;

    Q_PROPERTY(QString statusInfo READ statusInfo NOTIFY statusInfoChanged)
    Q_PROPERTY(bool hasSerial READ hasSerial NOTIFY hasSerialChanged)
    Q_PROPERTY(bool fiscalized READ fiscalized NOTIFY fiscalizedChanged)
    Q_PROPERTY(bool cycleIsOpen READ cycleIsOpen NOTIFY cycleIsOpenChanged)
    Q_PROPERTY(bool cycleIsOver READ cycleIsOver NOTIFY cycleIsOverChanged)
    Q_PROPERTY(bool archiveIsClosed READ archiveIsClosed NOTIFY archiveIsClosedChanged)
    Q_PROPERTY(bool isDebug READ isDebug NOTIFY isDebugChanged)
    Q_PROPERTY(bool inAutoMode READ inAutoMode NOTIFY inAutoModeChanged)
    Q_PROPERTY(bool inOfflineMode READ inOfflineMode NOTIFY inOfflineModeChanged)
    Q_PROPERTY(bool hasNotPrinted READ hasNotPrinted NOTIFY hasNotPrintedChanged)
    Q_PROPERTY(QString serial READ serial NOTIFY serialChanged)
    Q_PROPERTY(QString orgName READ orgName NOTIFY orgNameChanged)
    Q_PROPERTY(QString orgAddress READ orgAddress NOTIFY orgAddressChanged)
    Q_PROPERTY(QString place READ place NOTIFY placeChanged)
    Q_PROPERTY(QString cycleOpenedHours READ cycleOpenedHours NOTIFY cycleOpenedHoursChanged)
    Q_PROPERTY(QString cycleNumber READ cycleNumber NOTIFY cycleNumberChanged)
    Q_PROPERTY(qint32 lastDocNumber READ lastDocNumber NOTIFY lastDocNumberChanged)
    Q_PROPERTY(qint32 regsCount READ regsCount NOTIFY regsCountChanged)
    Q_PROPERTY(QString ipAddress READ ipAddress NOTIFY ipAddressChanged)
    Q_PROPERTY(QString dsn READ dsn NOTIFY statusChanged)
public:
    explicit CashboxStatusWrapper(QObject *parent = nullptr);
    ~CashboxStatusWrapper() override;

    CashboxStatus status() const;

    QString statusInfo() const;

    bool hasSerial() const;
    bool fiscalized() const;
    bool cycleIsOpen()const;
    bool cycleIsOver()const;
    bool archiveIsClosed() const;
    bool isDebug() const;
    bool inAutoMode() const;
    bool inOfflineMode() const;
    bool hasNotPrinted() const;

    QString dsn() const;
    QString serial() const;
    QString orgName() const;
    QString orgAddress() const;
    QString place() const;
    QString cycleOpenedHours() const;
    QString cycleNumber() const;
    qint32 lastDocNumber() const;
    qint32 regsCount() const;
    QString ipAddress() const;



signals:
    void statusInfoChanged();
    void statusChanged();
    void hasSerialChanged();
    bool fiscalizedChanged();
    void cycleIsOpenChanged();
    void cycleIsOverChanged();
    void archiveIsClosedChanged();
    void serialChanged();
    void orgNameChanged();
    void orgAddressChanged();
    void placeChanged();
    void cycleOpenedHoursChanged();
    void cycleNumberChanged();
    void isDebugChanged();
    bool inAutoModeChanged();
    bool inOfflineModeChanged();
    bool hasNotPrintedChanged();
    void lastDocNumberChanged();
    void regsCountChanged();
    void ipAddressChanged();
private:
    static QPointer<CashboxStatusWrapper> _globalStatus;
    static QMutex _statusMutex;
    static CashboxStatus _status;

    friend void updateCashboxStatus(const QVariantMap &data);

    explicit CashboxStatusWrapper(bool destroyStatic);

    static void setGlobalStatus(const CashboxStatus &status);
};

#endif // CASHBOXSTATUSWRAPPER_H
