#include "cashierwrapper.h"
#include "logmanip.h"
#include "basecatalogmodel.h"


CashierWrapper::CashierWrapper(QObject *parent)
    : BaseCatalogItemWrapper(parent)
    , cashier_()
{
    lmWarning() << logvariant(cashier_.toMap());
}

CashierWrapper::~CashierWrapper()
{

}

Cashier CashierWrapper::cashier() const
{
    return cashier_;
}

void CashierWrapper::setCashier(const Cashier &cashier)
{
    cashier_ = cashier;
    lmWarning() << logvariant(cashier.toMap());
    emit cashierDataChanged();
    emit idChanged();
    emit serverIdChanged();
    emit loginChanged();
    emit nameChanged();
    emit passwordChanged();
    emit innChanged();
    emit permissionsChanged();
    emit permReceiptsChanged();
    emit permReportsChanged();
    emit permSettingsChanged();
    emit permFiscalizationChanged();
    emit permReturnReceiptsChanged();
    emit permWriteSerialChanged();
    emit permCycleOpenChanged();
    emit permCycleCloseChanged();
    emit permCorrectionsChanged();
    emit permReturnCorrectionsChanged();
    emit permOfflineNotificationsChanged();
    emit isSuChanged();
    emit cashierIsEnabledChanged();
}

CoreTransaction CashierWrapper::toSaveTransaction() const
{
    CoreTransaction trans;
    trans.setOperation(CoreTransaction::Operation::SAVE_CASHIER);
    if(model_)
    {
        trans.setCashierLogin(model_->sessionUser().login());
        trans.setCashierPassword(model_->sessionUser().password());
    }
    trans.setParams(cashier_.toMap());
    trans.genUid();
    return trans;
}

QVariantMap CashierWrapper::toMap() const
{
    return cashier_.toMap();
}

void CashierWrapper::parseMap(const QVariantMap &map)
{
    lmWarning() << logvariant(cashier_.toMap());
    setCashierData(map);
}

QVariantMap CashierWrapper::cashierData() const
{
    return cashier_.toMap();
}

void CashierWrapper::setCashierData(const QVariantMap &newCashierData)
{
    lmWarning() << logvariant(cashier_.toMap());
    setCashier(Cashier(newCashierData));
}

qint32 CashierWrapper::id() const
{
    return cashier_.id();
}

void CashierWrapper::setId(qint32 newId)
{
    lmWarning() << logvariant(cashier_.toMap());
    if (cashier_.id() == newId)
        return;
    cashier_.setId(newId);
    emit idChanged();
}

qint64 CashierWrapper::serverId() const
{
    return cashier_.serverId();
}

void CashierWrapper::setServerId(qint64 newServerId)
{
    lmWarning() << logvariant(cashier_.toMap());
    if (cashier_.serverId() == newServerId)
        return;
    cashier_.setServerId(newServerId);
    emit serverIdChanged();
}

QString CashierWrapper::login() const
{
    return cashier_.login();
}

void CashierWrapper::setLogin(const QString &newLogin)
{
    lmWarning() << logvariant(cashier_.toMap());
    if (cashier_.login() == newLogin)
        return;
    cashier_.setLogin(newLogin);
    emit loginChanged();
}

QString CashierWrapper::name() const
{
    return cashier_.name();
}

void CashierWrapper::setName(const QString &newName)
{
    lmWarning() << logvariant(cashier_.toMap());
    if (cashier_.name() == newName)
        return;
    cashier_.setName(newName);
    emit nameChanged();
}

QString CashierWrapper::password() const
{
    return cashier_.password();
}

void CashierWrapper::setPassword(const QString &newPassword)
{
    lmWarning() << logvariant(cashier_.toMap());
    if (cashier_.password() == newPassword)
        return;
    cashier_.setPassword(newPassword);
    emit passwordChanged();
}

QString CashierWrapper::inn() const
{
    return cashier_.inn();
}

void CashierWrapper::setInn(const QString &newInn)
{
    lmWarning() << logvariant(cashier_.toMap());
    if (cashier_.inn() == newInn)
        return;
    cashier_.setInn(newInn);
    emit innChanged();
}

qint32 CashierWrapper::permissions() const
{
    return static_cast<qint32>(cashier_.permissions());
}

void CashierWrapper::setPermissions(qint32 newPermissions)
{
    lmWarning() << logvariant(cashier_.toMap());
    if (cashier_.permissions() == static_cast<quint32>(newPermissions))
        return;
    cashier_.setPermissions(Cashier::Permissions(newPermissions));
    emit permissionsChanged();
    emit permReceiptsChanged();
    emit permReportsChanged();
    emit permSettingsChanged();
    emit permFiscalizationChanged();
    emit permReturnReceiptsChanged();
    emit permWriteSerialChanged();
    emit permCycleOpenChanged();
    emit permCycleCloseChanged();
    emit permCorrectionsChanged();
    emit permReturnCorrectionsChanged();
    emit permOfflineNotificationsChanged();
}

bool CashierWrapper::permReceipts() const
{
    return cashier_.hasPermission(Cashier::Permission::Receipts);
}

void CashierWrapper::setPermReceipts(bool newPermReceipts)
{
    lmWarning() << logvariant(cashier_.toMap());
    if (cashier_.hasPermission(Cashier::Permission::Receipts) == newPermReceipts)
        return;
    cashier_.setPermission(Cashier::Permission::Receipts, newPermReceipts);
    emit permReceiptsChanged();
    emit permissionsChanged();
}

bool CashierWrapper::permReports() const
{
    return cashier_.hasPermission(Cashier::Permission::Reports);
}

void CashierWrapper::setPermReports(bool newPermReports)
{
    lmWarning() << logvariant(cashier_.toMap());
    if (cashier_.hasPermission(Cashier::Permission::Reports) == newPermReports)
        return;
    cashier_.setPermission(Cashier::Permission::Reports, newPermReports);
    emit permReportsChanged();
    emit permissionsChanged();
}

bool CashierWrapper::permSettings() const
{
    return cashier_.hasPermission(Cashier::Permission::Settings);
}

void CashierWrapper::setPermSettings(bool newPermSettings)
{
    lmWarning() << logvariant(cashier_.toMap());
    if (cashier_.hasPermission(Cashier::Permission::Settings) == newPermSettings)
        return;
    cashier_.setPermission(Cashier::Permission::Settings, newPermSettings);
    emit permSettingsChanged();
    emit permissionsChanged();
}

bool CashierWrapper::permFiscalization() const
{
    return cashier_.hasPermission(Cashier::Permission::Fiscalization);
}

void CashierWrapper::setPermFiscalization(bool newPermFiscalization)
{
    lmWarning() << logvariant(cashier_.toMap());
    if (cashier_.hasPermission(Cashier::Permission::Fiscalization) == newPermFiscalization)
        return;
    cashier_.setPermission(Cashier::Permission::Fiscalization, newPermFiscalization);
    emit permFiscalizationChanged();
    emit permissionsChanged();
}

bool CashierWrapper::permReturnReceipts() const
{
    return cashier_.hasPermission(Cashier::Permission::ReturnReceipts);
}

void CashierWrapper::setPermReturnReceipts(bool newPermReturnReceipts)
{
    lmWarning() << logvariant(cashier_.toMap());
    if (cashier_.hasPermission(Cashier::Permission::ReturnReceipts) == newPermReturnReceipts)
        return;
    cashier_.setPermission(Cashier::Permission::ReturnReceipts, newPermReturnReceipts);
    emit permReturnReceiptsChanged();
    emit permissionsChanged();
}

bool CashierWrapper::permWriteSerial() const
{
    return cashier_.hasPermission(Cashier::Permission::WriteSerial);
}

void CashierWrapper::setPermWriteSerial(bool newPermWriteSerial)
{
    lmWarning() << logvariant(cashier_.toMap());
    if (cashier_.hasPermission(Cashier::Permission::WriteSerial) == newPermWriteSerial)
        return;
    cashier_.setPermission(Cashier::Permission::WriteSerial, newPermWriteSerial);
    emit permWriteSerialChanged();
    emit permissionsChanged();
}

bool CashierWrapper::permCycleOpen() const
{
    return cashier_.hasPermission(Cashier::Permission::CycleOpen);
}

void CashierWrapper::setPermCycleOpen(bool newPermCycleOpen)
{
    lmWarning() << logvariant(cashier_.toMap());
    if (cashier_.hasPermission(Cashier::Permission::CycleOpen) == newPermCycleOpen)
        return;
    cashier_.setPermission(Cashier::Permission::CycleOpen, newPermCycleOpen);
    emit permCycleOpenChanged();
    emit permissionsChanged();
}

bool CashierWrapper::permCycleClose() const
{
    return cashier_.hasPermission(Cashier::Permission::CycleClose);
}

void CashierWrapper::setPermCycleClose(bool newPermCycleClose)
{
    lmWarning() << logvariant(cashier_.toMap());
    if (cashier_.hasPermission(Cashier::Permission::CycleClose) == newPermCycleClose)
        return;
    cashier_.setPermission(Cashier::Permission::CycleClose, newPermCycleClose);
    emit permCycleCloseChanged();
    emit permissionsChanged();
}

bool CashierWrapper::permCorrections() const
{
    return cashier_.hasPermission(Cashier::Permission::Corrections);
}

void CashierWrapper::setPermCorrections(bool newPermCorrections)
{
    lmWarning() << logvariant(cashier_.toMap());
    if (cashier_.hasPermission(Cashier::Permission::Corrections) == newPermCorrections)
        return;
    cashier_.setPermission(Cashier::Permission::Corrections, newPermCorrections);
    emit permCorrectionsChanged();
    emit permissionsChanged();
}

bool CashierWrapper::permReturnCorrections() const
{
    return cashier_.hasPermission(Cashier::Permission::ReturnCorrections);
}

void CashierWrapper::setPermReturnCorrections(bool newPermReturnCorrections)
{
    lmWarning() << logvariant(cashier_.toMap());
    if (cashier_.hasPermission(Cashier::Permission::ReturnCorrections) == newPermReturnCorrections)
        return;
    cashier_.setPermission(Cashier::Permission::ReturnCorrections, newPermReturnCorrections);
    emit permReturnCorrectionsChanged();
    emit permissionsChanged();
}

bool CashierWrapper::permOfflineNotifications() const
{
    return cashier_.hasPermission(Cashier::Permission::OfflineNotifications);
}

void CashierWrapper::setPermOfflineNotifications(bool newPermOfflineNotifications)
{
    lmWarning() << logvariant(cashier_.toMap());
    if (cashier_.hasPermission(Cashier::Permission::OfflineNotifications) == newPermOfflineNotifications)
        return;
    cashier_.setPermission(Cashier::Permission::OfflineNotifications, newPermOfflineNotifications);
    emit permOfflineNotificationsChanged();
    emit permissionsChanged();
}

bool CashierWrapper::isSu() const
{
    return cashier_.isSuperUser();
}

bool CashierWrapper::cashierIsEnabled() const
{
    if(!model_) return cashier_.serverId() <= 0;
    return cashier_.serverId() <= 0 && cashier_.login() != model_->sessionUser().login();
}

void CashierWrapper::processModel()
{
    emit cashierIsEnabledChanged();
}


