#ifndef CASHIERWRAPPER_H
#define CASHIERWRAPPER_H

#include "basecatalogitemwrapper.h"
#include "cashier.h"

#include <QtQml>
//TODO: привести в соответствие

class CashierWrapper : public BaseCatalogItemWrapper
{
    Q_OBJECT
    QML_ELEMENT
    Q_DISABLE_COPY(CashierWrapper)

    Q_PROPERTY(QVariantMap cashierData READ cashierData WRITE setCashierData NOTIFY cashierDataChanged)
    Q_PROPERTY(qint32 id READ id WRITE setId NOTIFY idChanged)
    Q_PROPERTY(qint64 serverId READ serverId WRITE setServerId NOTIFY serverIdChanged)
    Q_PROPERTY(QString login READ login WRITE setLogin NOTIFY loginChanged)
    Q_PROPERTY(QString name READ name WRITE setName NOTIFY nameChanged)
    Q_PROPERTY(QString password READ password WRITE setPassword NOTIFY passwordChanged)
    Q_PROPERTY(QString inn READ inn WRITE setInn NOTIFY innChanged)
    Q_PROPERTY(qint32 permissions READ permissions WRITE setPermissions NOTIFY permissionsChanged)
    Q_PROPERTY(bool permReceipts READ permReceipts WRITE setPermReceipts NOTIFY permReceiptsChanged)
    Q_PROPERTY(bool permReports READ permReports WRITE setPermReports NOTIFY permReportsChanged)
    Q_PROPERTY(bool permSettings READ permSettings WRITE setPermSettings NOTIFY permSettingsChanged)
    Q_PROPERTY(bool permFiscalization READ permFiscalization WRITE setPermFiscalization NOTIFY permFiscalizationChanged)
    Q_PROPERTY(bool permReturnReceipts READ permReturnReceipts WRITE setPermReturnReceipts NOTIFY permReturnReceiptsChanged)
    Q_PROPERTY(bool permWriteSerial READ permWriteSerial WRITE setPermWriteSerial NOTIFY permWriteSerialChanged)
    Q_PROPERTY(bool permCycleOpen READ permCycleOpen WRITE setPermCycleOpen NOTIFY permCycleOpenChanged)
    Q_PROPERTY(bool permCycleClose READ permCycleClose WRITE setPermCycleClose NOTIFY permCycleCloseChanged)
    Q_PROPERTY(bool permCorrections READ permCorrections WRITE setPermCorrections NOTIFY permCorrectionsChanged)
    Q_PROPERTY(bool permReturnCorrections READ permReturnCorrections WRITE setPermReturnCorrections NOTIFY permReturnCorrectionsChanged)
    Q_PROPERTY(bool permOfflineNotifications READ permOfflineNotifications WRITE setPermOfflineNotifications NOTIFY permOfflineNotificationsChanged)
    Q_PROPERTY(bool isSu READ isSu NOTIFY isSuChanged)
    Q_PROPERTY(bool cashierIsEnabled READ cashierIsEnabled NOTIFY cashierIsEnabledChanged)
public:
    explicit CashierWrapper(QObject *parent = nullptr);
    ~CashierWrapper() override;

    enum Permission
    {
        PermReceipts                = 0x00000001u,
        PermReports                 = 0x00000002u,
        PermSettings                = 0x00000004u,
        PermFiscalization           = 0x00000008u,
        PermReturnReceipts          = 0x00000010u,
        PermWriteSerial             = 0x00000020u,
        PermCycleOpen               = 0x00000040u,
        PermCycleClose              = 0x00000080u,
        PermCorrections             = 0x00000100u,
        PermReturnCorrections       = 0x00000200u,
        PermOfflineNotifications    = 0x00000400u,
    };
    Q_ENUM(Permission)
    Q_ENUMS(Permission)

    Cashier cashier() const;
    void setCashier(const Cashier &cashier);

    CoreTransaction toSaveTransaction() const override;
    QVariantMap toMap() const override;
    void parseMap(const QVariantMap &map) override;


    QVariantMap cashierData() const;
    void setCashierData(const QVariantMap &newCashierData);

    qint32 id() const;
    void setId(qint32 newId);

    qint64 serverId() const;
    void setServerId(qint64 newServerId);

    QString login() const;
    void setLogin(const QString &newLogin);

    QString name() const;
    void setName(const QString &newName);

    QString password() const;
    void setPassword(const QString &newPassword);

    QString inn() const;
    void setInn(const QString &newInn);

    qint32 permissions() const;
    void setPermissions(qint32 newPermissions);

    bool permReceipts() const;
    void setPermReceipts(bool newPermReceipts);

    bool permReports() const;
    void setPermReports(bool newPermReports);

    bool permSettings() const;
    void setPermSettings(bool newPermSettings);

    bool permFiscalization() const;
    void setPermFiscalization(bool newPermFiscalization);

    bool permReturnReceipts() const;
    void setPermReturnReceipts(bool newPermReturnReceipts);

    bool permWriteSerial() const;
    void setPermWriteSerial(bool newPermWriteSerial);

    bool permCycleOpen() const;
    void setPermCycleOpen(bool newPermCycleOpen);

    bool permCycleClose() const;
    void setPermCycleClose(bool newPermCycleClose);

    bool permCorrections() const;
    void setPermCorrections(bool newPermCorrections);

    bool permReturnCorrections() const;
    void setPermReturnCorrections(bool newPermReturnCorrections);

    bool permOfflineNotifications() const;
    void setPermOfflineNotifications(bool newPermOfflineNotifications);

    bool isSu() const;

    bool cashierIsEnabled() const;

signals:
    void cashierDataChanged();
    void idChanged();
    void serverIdChanged();
    void loginChanged();
    void nameChanged();
    void passwordChanged();
    void innChanged();
    void permissionsChanged();
    void permReceiptsChanged();
    void permReportsChanged();
    void permSettingsChanged();
    void permFiscalizationChanged();
    void permReturnReceiptsChanged();
    void permWriteSerialChanged();
    void permCycleOpenChanged();
    void permCycleCloseChanged();
    void permCorrectionsChanged();
    void permReturnCorrectionsChanged();
    void permOfflineNotificationsChanged();
    void isSuChanged();

    void cashierIsEnabledChanged();

private:
    Cashier cashier_;
    void processModel() override;
};


#endif // CASHIERWRAPPER_H
