#include "incomingregwrapper.h"
#include "regdatavalidator.h"
#include "ofdsettings.h"

#include "logmanip.h"

IncomingRegWrapper::IncomingRegWrapper(QObject *parent)
    : QObject(parent)
    , currentReg_()
    , newReg_()
    , status_(new CashboxStatusWrapper(this))
    , session_(new SessionWrapper(this))
    , operationsModel_()
{
    connect(status_, &CashboxStatusWrapper::statusChanged,
            this, &IncomingRegWrapper::onStatusChanged);
    onStatusChanged();
    reset();
}

IncomingRegWrapper::~IncomingRegWrapper()
{

}

bool IncomingRegWrapper::getAutoMode() const
{
    return newReg_.getAutoMode();
}

void IncomingRegWrapper::setAutoMode(bool set)
{
    if(set != newReg_.getAutoMode())
    {
        newReg_.setAutoMode(set);
        emit autoModeChanged();
    }
}

bool IncomingRegWrapper::getOfflineMode() const
{
    return newReg_.getOfflineMode();
}

void IncomingRegWrapper::setOfflineMode(bool set)
{
    if(newReg_.getOfflineMode() != set)
    {
        newReg_.setOfflineMode(set);
        emit offlineModeChanged();
    }
}

bool IncomingRegWrapper::getEncriptionMode() const
{
    return newReg_.getEncriptionMode();
}

void IncomingRegWrapper::setEncriptionMode(bool set)
{
    if(newReg_.getEncriptionMode() != set)
    {
        newReg_.setEncriptionMode(set);
        emit encriptionModeChanged();
    }
}

const QString &IncomingRegWrapper::address() const
{
    return newReg_.address();
}

void IncomingRegWrapper::setAddress(const QString &newAddress)
{
    if(newReg_.address() != newAddress)
    {
        newReg_.setAddress(newAddress);
        emit addressChanged();
    }
}

const QString &IncomingRegWrapper::ofdInn() const
{
    return newReg_.ofdInn();
}

void IncomingRegWrapper::setOfdInn(const QString &newOfdInn)
{
    if(newReg_.ofdInn() != newOfdInn)
    {
        newReg_.setOfdInn(newOfdInn);
        emit ofdInnChanged();
        Inn inn(newOfdInn);
        if(inn.isValid(false) && OfdSettings::SUPPORTED_OFD.contains(newOfdInn))
        {
            setOfdName(OfdSettings::SUPPORTED_OFD[newOfdInn].shortName());
        }
    }
}

const QString &IncomingRegWrapper::userInn() const
{
    return newReg_.ownerInn();
}

void IncomingRegWrapper::setUserInn(const QString &newUserInn)
{
    if(newReg_.ownerInn() != newUserInn)
    {
        newReg_.setOwnerInn(newUserInn);
        emit userInnChanged();
    }
}

const QString &IncomingRegWrapper::cashier() const
{
    return newReg_.cashier();
}

void IncomingRegWrapper::setCashier(const QString &newCashier)
{
    if(newReg_.cashier() != newCashier)
    {
        newReg_.setCashier(newCashier);
        emit cashierChanged();
    }
}

const QString &IncomingRegWrapper::boxId() const
{
    return newReg_.boxId();
}

void IncomingRegWrapper::setBoxId(const QString &newBoxId)
{
    if(newReg_.boxId() != newBoxId)
    {
        newReg_.setBoxId(newBoxId);
        emit boxIdChanged();
    }
}

const QString &IncomingRegWrapper::regNumber() const
{
    return newReg_.regNumber();
}

void IncomingRegWrapper::setRegNumber(const QString &newRegNumber)
{
    if(newReg_.regNumber() != newRegNumber)
    {
        newReg_.setRegNumber(newRegNumber);
        emit regNumberChanged();
    }
}

const QString &IncomingRegWrapper::ofdName() const
{
    return newReg_.ofdName();
}

void IncomingRegWrapper::setOfdName(const QString &newOfdName)
{
    if(newReg_.ofdName() != newOfdName)
    {
        newReg_.setOfdName(newOfdName);
        emit ofdNameChanged();
    }
}

const QString &IncomingRegWrapper::userName() const
{
    return newReg_.ownerName();
}

void IncomingRegWrapper::setUserName(const QString &newUserName)
{
    if(newReg_.ownerName() != newUserName)
    {
        newReg_.setOwnerName(newUserName);
        emit userNameChanged();
    }
}

const QString &IncomingRegWrapper::ftsSite() const
{
    return newReg_.ftsSite();
}

void IncomingRegWrapper::setFtsSite(const QString &newFtsSite)
{
    if(newReg_.ftsSite() != newFtsSite)
    {
        newReg_.setFtsSite(newFtsSite);
        emit ftsSiteChanged();
    }
}

bool IncomingRegWrapper::allowBasic() const
{
    return newReg_.taxes().testFlag(fdf::Tax::Basic);
}

void IncomingRegWrapper::setAllowBasic(bool set)
{
    if(allowBasic() != set)
    {
        newReg_.setTax(fdf::Tax::Basic, set);
        emit allowBasicChanged();
    }
}

bool IncomingRegWrapper::allowSimpleProfit() const
{
    return newReg_.taxes().testFlag(fdf::Tax::SimpleProfit);
}

void IncomingRegWrapper::setAllowSimpleProfit(bool set)
{
    if(allowSimpleProfit() != set)
    {
        newReg_.setTax(fdf::Tax::SimpleProfit, set);
        emit allowSimpleProfitChanged();
    }
}

bool IncomingRegWrapper::allowSimpleProfitExpense() const
{
    return newReg_.taxes().testFlag(fdf::Tax::SimpleProfitExpense);
}

void IncomingRegWrapper::setAllowSimpleProfitExpense(bool set)
{
    if(allowSimpleProfitExpense() != set)
    {
        newReg_.setTax(fdf::Tax::SimpleProfitExpense, set);
        emit allowSimpleProfitExpenseChanged();
    }
}

bool IncomingRegWrapper::allowSingleOnImputedProfit() const
{
    return newReg_.taxes().testFlag(fdf::Tax::SingleOnImputedProfit);
}

void IncomingRegWrapper::setAllowSingleOnImputedProfit(bool set)
{
    if(allowSingleOnImputedProfit() != set)
    {
        newReg_.setTax(fdf::Tax::SingleOnImputedProfit, set);
        emit allowSingleOnImputedProfitChanged();
    }
}

bool IncomingRegWrapper::allowSingleAgriculturalTax() const
{
    return newReg_.taxes().testFlag(fdf::Tax::SingleAgriculturalTax);
}

void IncomingRegWrapper::setAllowSingleAgriculturalTax(bool set)
{
    if(allowSingleAgriculturalTax() != set)
    {
        newReg_.setTax(fdf::Tax::SingleAgriculturalTax, set);
        emit allowSingleAgriculturalTaxChanged();
    }
}

bool IncomingRegWrapper::allowPatentSystemOfTaxation() const
{
    return newReg_.taxes().testFlag(fdf::Tax::PatentSystemOfTaxation);
}

void IncomingRegWrapper::setAllowPatentSystemOfTaxation(bool set)
{
    if(allowPatentSystemOfTaxation() != set)
    {
        newReg_.setTax(fdf::Tax::PatentSystemOfTaxation, set);
        emit allowPatentSystemOfTaxationChanged();
    }
}

const QString &IncomingRegWrapper::email() const
{
    return newReg_.email();
}

void IncomingRegWrapper::setEmail(const QString &newEmail)
{
    if(newReg_.email() != newEmail)
    {
        newReg_.setEmail(newEmail);
        emit emailChanged();
    }
}

const QString &IncomingRegWrapper::place() const
{
    return newReg_.place();
}

void IncomingRegWrapper::setPlace(const QString &newPlace)
{
    if(newReg_.place() != newPlace)
    {
        newReg_.setPlace(newPlace);
        emit placeChanged();
    }
}

const QString &IncomingRegWrapper::cashierInn() const
{
    return newReg_.cashierInn();
}

void IncomingRegWrapper::setCashierInn(const QString &newCashierInn)
{
    if(newReg_.cashierInn() != newCashierInn)
    {
        newReg_.setCashierInn(newCashierInn);
        emit cashierInnChanged();
    }
}

const QString &IncomingRegWrapper::additionalParam() const
{
    return newReg_.additionalParam();
}

void IncomingRegWrapper::setAdditionalParam(const QString &newAdditionalParam)
{
    if(newReg_.additionalParam() != newAdditionalParam)
    {
        newReg_.setAdditionalParam(newAdditionalParam);
        emit additionalParamChanged();
    }
}

QString IncomingRegWrapper::additionalParamData() const
{
    return QString::fromLatin1(newReg_.additionalParamData().toHex());
}

void IncomingRegWrapper::setAdditionalParamData(const QString &newAdditionalParamData)
{
    QByteArray param = QByteArray::fromHex(newAdditionalParamData.toLatin1());
    if(newReg_.additionalParamData() != param)
    {
        newReg_.setAdditionalParamData(param);
        emit additionalParamDataChanged();
    }
}

bool IncomingRegWrapper::getExternalPrinter() const
{
    return newReg_.extFlags().getExternalPrinter();
}

void IncomingRegWrapper::setExternalPrinter(bool set)
{
    if(getExternalPrinter() != set)
    {
        newReg_.setExtFlag(fdf::RegFlag1290::ExternalPrinter, set);
        emit allowExternalPrinterChanged();
    }
}

bool IncomingRegWrapper::getFosaCashbox() const
{
    return newReg_.extFlags().getFosaCashbox();
}

void IncomingRegWrapper::setFosaCashbox(bool set)
{
    if(getFosaCashbox() != set)
    {
        newReg_.setExtFlag(fdf::RegFlag1290::FosaCashbox, set);
        emit allowFosaCashboxChanged();
    }
    if(set)
    {
        newReg_.setExtFlag(fdf::RegFlag1290::ServicesOnly, true);
        emit allowServicesOnlyChanged();
    }
}

bool IncomingRegWrapper::getInternetOnly() const
{
    return newReg_.extFlags().getInternetOnly();
}

void IncomingRegWrapper::setInternetOnly(bool set)
{
    if(getInternetOnly() != set)
    {
        newReg_.setExtFlag(fdf::RegFlag1290::InternetOnly, set);
        emit allowInternetOnlyChanged();
    }

}

bool IncomingRegWrapper::getExcisableProducts() const
{
    return newReg_.extFlags().getExcisableProducts();
}

void IncomingRegWrapper::setExcisableProducts(bool set)
{
    if(getExcisableProducts() != set)
    {
        newReg_.setExtFlag(fdf::RegFlag1290::ExcisableProducts, set);
        emit allowExcisableProductsChanged();
    }

}

bool IncomingRegWrapper::getLabledProducts() const
{
    return newReg_.extFlags().getLabledProducts();
}

void IncomingRegWrapper::setLabledProducts(bool set)
{
    if(getLabledProducts() != set)
    {
        newReg_.setExtFlag(fdf::RegFlag1290::LabledProducts, set);
        emit allowLabledProductsChanged();
    }

}

bool IncomingRegWrapper::getServicesOnly() const
{
    return newReg_.extFlags().getServicesOnly();
}

void IncomingRegWrapper::setServicesOnly(bool set)
{
    if(getServicesOnly() != set)
    {
        newReg_.setExtFlag(fdf::RegFlag1290::ServicesOnly, set);
        emit allowServicesOnlyChanged();
    }

}

bool IncomingRegWrapper::getGamesCashbox() const
{
    return newReg_.extFlags().getGamesCashbox();
}

void IncomingRegWrapper::setGamesCashbox(bool set)
{
    if(getGamesCashbox() != set)
    {
        newReg_.setExtFlag(fdf::RegFlag1290::GamesCashbox, set);
        emit allowGamesCashboxChanged();
    }
}

bool IncomingRegWrapper::getLotteryCashbox() const
{
    return newReg_.extFlags().getLotteryCashbox();
}

void IncomingRegWrapper::setLotteryCashbox(bool set)
{
    if(getLotteryCashbox() != set)
    {
        newReg_.setExtFlag(fdf::RegFlag1290::LotteryCashbox, set);
        emit allowLotteryCashboxChanged();
    }

}

bool IncomingRegWrapper::getPawnshop() const
{
    return newReg_.extFlags().getPawnshop();
}

void IncomingRegWrapper::setPawnshop(bool set)
{
    if(getPawnshop() != set)
    {
        newReg_.setExtFlag(fdf::RegFlag1290::Pawnshop, set);
        emit allowPawnshopChanged();
    }

}

bool IncomingRegWrapper::getInsurance() const
{
    return newReg_.extFlags().getInsurance();
}

void IncomingRegWrapper::setInsurance(bool set)
{
    if(getInsurance() != set)
    {
        newReg_.setExtFlag(fdf::RegFlag1290::Insurance, set);
        emit allowInsuranceChanged();
    }

}

bool IncomingRegWrapper::getAutoCashboxForLabled() const
{
    return newReg_.extFlags().getAutoCashboxForLabled();
}

void IncomingRegWrapper::setAutoCashboxForLabled(bool set)
{
    if(getAutoCashboxForLabled() != set)
    {
        newReg_.setExtFlag(fdf::RegFlag1290::AutoCashboxForLabled, set);
        emit allowAutoCashboxForLabledCashboxChanged();
    }
}

bool IncomingRegWrapper::getRestaurant() const
{
    return newReg_.extFlags().getRestaurant();
}

void IncomingRegWrapper::setRestaurant(bool set)
{
    if(getRestaurant() != set)
    {
        newReg_.setExtFlag(fdf::RegFlag1290::Restaurant, set);
        emit allowRestaurantChanged();
    }
}

bool IncomingRegWrapper::getWholesale() const
{
    return newReg_.extFlags().getWholesale();
}

void IncomingRegWrapper::setWholesale(bool set)
{
    if(getWholesale() != set)
    {
        newReg_.setExtFlag(fdf::RegFlag1290::Wholesale, set);
        emit allowWholesaleChanged();
    }
}

bool IncomingRegWrapper::getFsChange() const
{
    return newReg_.reasons().getFsChange();
}

void IncomingRegWrapper::setFsChange(bool set)
{
    if(getFsChange() != set)
    {
        newReg_.setReason(fdf::Reason::FsChange, set);
        emit reasonFsChangeChanged();
    }
}

bool IncomingRegWrapper::getOfdChange() const
{
    return newReg_.reasons().getOfdChange();
}

void IncomingRegWrapper::setOfdChange(bool set)
{
    if(getOfdChange() != set)
    {
        newReg_.setReason(fdf::Reason::OfdChange, set);
        emit reasonOfdChangeChanged();
    }

}

bool IncomingRegWrapper::getUserNameChange() const
{
    return newReg_.reasons().getUserNameChange();
}

void IncomingRegWrapper::setUserNameChange(bool set)
{
    if(getUserNameChange() != set)
    {
        newReg_.setReason(fdf::Reason::UserNameChange, set);
        emit reasonUserNameChangeChanged();
    }

}

bool IncomingRegWrapper::getAddressChange() const
{
    return newReg_.reasons().getAddressChange();
}

void IncomingRegWrapper::setAddressChange(bool set)
{
    if(getAddressChange() != set)
    {
        newReg_.setReason(fdf::Reason::AddressChange, set);
        emit reasonAddressChangeChanged();
    }

}

bool IncomingRegWrapper::getOfflineToOnline() const
{
    return newReg_.reasons().getOfflineToOnline();
}

void IncomingRegWrapper::setOfflineToOnline(bool set)
{
    if(getOfflineToOnline() != set)
    {
        newReg_.setReason(fdf::Reason::OfflineToOnline, set);
        emit reasonOfflineToOnlineChanged();
    }

}

bool IncomingRegWrapper::getOnlineToOffline() const
{
    return newReg_.reasons().getOnlineToOffline();
}

void IncomingRegWrapper::setOnlineToOffline(bool set)
{
    if(getOnlineToOffline() != set)
    {
        newReg_.setReason(fdf::Reason::OnlineToOffline, set);
        emit reasonOnlineToOfflineChanged();
    }

}

bool IncomingRegWrapper::getCashboxVersionChange() const
{
    return newReg_.reasons().getCashboxVersionChange();
}

void IncomingRegWrapper::setCashboxVersionChange(bool set)
{
    if(getCashboxVersionChange() != set)
    {
        newReg_.setReason(fdf::Reason::CashboxVersionChange, set);
        emit reasonCashboxVersionChangeChanged();
    }

}

bool IncomingRegWrapper::getTaxesChange() const
{
    return newReg_.reasons().getTaxesChange();
}

void IncomingRegWrapper::setTaxesChange(bool set)
{
    if(getTaxesChange() != set)
    {
        newReg_.setReason(fdf::Reason::TaxesChange, set);
        emit reasonTaxesChangeChanged();
    }

}

bool IncomingRegWrapper::getBoxIdChange() const
{
    return newReg_.reasons().getBoxIdChange();
}

void IncomingRegWrapper::setBoxIdChange(bool set)
{
    if(getBoxIdChange() != set)
    {
        newReg_.setReason(fdf::Reason::BoxIdChange, set);
        emit reasonBoxIdChangeChanged();
    }

}

bool IncomingRegWrapper::getAutoToSingle() const
{
    return newReg_.reasons().getAutoToSingle();
}

void IncomingRegWrapper::setAutoToSingle(bool set)
{
    if(getAutoToSingle() != set)
    {
        newReg_.setReason(fdf::Reason::AutoToSingle, set);
        emit reasonAutoToSingleChanged();
    }

}

bool IncomingRegWrapper::getSingleToAuto() const
{
    return newReg_.reasons().getSingleToAuto();
}

void IncomingRegWrapper::setSingleToAuto(bool set)
{
    if(getSingleToAuto() != set)
    {
        newReg_.setReason(fdf::Reason::SingleToAuto, set);
        emit reasonSingleToAutoChanged();
    }

}

bool IncomingRegWrapper::getBasicToFosa() const
{
    return newReg_.reasons().getBasicToFosa();
}

void IncomingRegWrapper::setBasicToFosa(bool set)
{
    if(getBasicToFosa() != set)
    {
        newReg_.setReason(fdf::Reason::BasicToFosa, set);
        emit reasonBasicToFosaChanged();
    }

}

bool IncomingRegWrapper::getFosaToBasic() const
{
    return newReg_.reasons().getFosaToBasic();
}

void IncomingRegWrapper::setFosaToBasic(bool set)
{
    if(getFosaToBasic() != set)
    {
        newReg_.setReason(fdf::Reason::FosaToBasic, set);
        emit reasonFosaToBasicChanged();
    }

}

bool IncomingRegWrapper::getInternetToBasic() const
{
    return newReg_.reasons().getInternetToBasic();
}

void IncomingRegWrapper::setInternetToBasic(bool set)
{
    if(getInternetToBasic() != set)
    {
        newReg_.setReason(fdf::Reason::InternetToBasic, set);
        emit reasonInternetToBasicChanged();
    }

}

bool IncomingRegWrapper::getBasicToInternet() const
{
    return newReg_.reasons().getBasicToInternet();
}

void IncomingRegWrapper::setBasicToInternet(bool set)
{
    if(getBasicToInternet() != set)
    {
        newReg_.setReason(fdf::Reason::BasicToInternet, set);
        emit reasonBasicToInternetChanged();
    }

}

bool IncomingRegWrapper::getGamesToBasic() const
{
    return newReg_.reasons().getGamesToBasic();
}

void IncomingRegWrapper::setGamesToBasic(bool set)
{
    if(getGamesToBasic() != set)
    {
        newReg_.setReason(fdf::Reason::GamesToBasic, set);
        emit reasonGamesToBasicChanged();
    }

}

bool IncomingRegWrapper::getBasicToGames() const
{
    return newReg_.reasons().getBasicToGames();
}

void IncomingRegWrapper::setBasicToGames(bool set)
{
    if(getBasicToGames() != set)
    {
        newReg_.setReason(fdf::Reason::BasicToGames, set);
        emit reasonBasicToGamesChanged();
    }

}

bool IncomingRegWrapper::getLotteryToBasic() const
{
    return newReg_.reasons().getLotteryToBasic();
}

void IncomingRegWrapper::setLotteryToBasic(bool set)
{
    if(getLotteryToBasic() != set)
    {
        newReg_.setReason(fdf::Reason::LotteryToBasic, set);
        emit reasonLotteryToBasicChanged();
    }

}

bool IncomingRegWrapper::getBasicToLottery() const
{
    return newReg_.reasons().getBasicToLottery();
}

void IncomingRegWrapper::setBasicToLottery(bool set)
{
    if(getBasicToLottery() != set)
    {
        newReg_.setReason(fdf::Reason::BasicToLottery, set);
        emit reasonAnotherReasonsChanged();
    }

}

bool IncomingRegWrapper::getFFDChange() const
{
    return newReg_.reasons().getFFDChange();
}

void IncomingRegWrapper::setFFDChange(bool set)
{
    if(getFFDChange() != set)
    {
        newReg_.setReason(fdf::Reason::FFDChange, set);
        emit reasonFFDChangeChanged();
    }

}

bool IncomingRegWrapper::getAnotherReasons() const
{
    return newReg_.reasons().getAnotherReasons();
}

void IncomingRegWrapper::setAnotherReasons(bool set)
{
    if(getAnotherReasons() != set)
    {
        newReg_.setReason(fdf::Reason::AnotherReasons, set);
        emit reasonAnotherReasonsChanged();
    }
}

QVariantMap IncomingRegWrapper::getRegData() const
{
    return newReg_.toMap();
}

QVariantList IncomingRegWrapper::operationsModel() const
{
    return operationsModel_;
}

int IncomingRegWrapper::codeOfOperation(int index) const
{
    if(index < 0 || index > operationsModel_.size()) return 0;
    return operationsModel_[index].toMap()["code"].toInt();
}


void IncomingRegWrapper::setRegData(const QVariantMap &data)
{
    currentReg_.parseMap(data);
    newReg_.parseMap(data);
}

void IncomingRegWrapper::reset()
{
    newReg_ = currentReg_;
    newReg_.setCashier(session_->cashierName());
    newReg_.setCashierInn(session_->cashier().inn());
}

void IncomingRegWrapper::preprint()
{
    if(validate())
    {
        emit needPreprint(newReg_.toMap());
    }
}

void IncomingRegWrapper::fiscalize()
{
    if(validate())
    {
        emit needFiscalize(newReg_.toMap());
    }
}

void IncomingRegWrapper::genRegNumber()
{
    if(!status_->isDebug())
    {
        emit error(tr("Доступно только для МГМ"));
    }
    else if(status_->fiscalized())
    {
        emit error(tr("МГМ уже активирован"));
    }
    else if(!status_->hasSerial())
    {
        emit error(tr("Не задан заводской номер ККТ"));
    }
    else
    {
        Inn inn(userInn());
        if(!inn.isValid())
        {
            emit error(tr("Укажите корректный инн пользователя"));
        }
        else
        {
            RegNumber rnm(status_->serial(), userInn());
            setRegNumber(rnm.value());
        }

    }
}

bool IncomingRegWrapper::validate()
{
    RegDataValidator validator;
    CashboxStatus st  =status_->status();
    validator.setFs(st.fs());
    validator.setModel(st.modelData());
    validator.setOldReg(st.regData());
    bool res = validator.validate(newReg_);
    if(!res)
    {
        emit error(validator.lastErrorMsg());
    }
    return res;
}

void IncomingRegWrapper::onStatusChanged()
{
    currentReg_ = status_->status().regData();


    QVariantMap m;
    operationsModel_.clear();
    if(!status_->fiscalized())
    {
        operationsModel_.clear();
        m["name"] = "Первичная регистрация";
        m["code"] = 0;
        operationsModel_ << m;
        m["name"] = "Замена ФН";
        m["code"] = 1;
        operationsModel_ << m;
    }
    else if(status_->fiscalized() && !status_->cycleIsOpen())
    {
        m["name"] = "Перерегистрация";
        m["code"] = 2;
        operationsModel_ << m;
    }
    emit operationsModelChanged();
}
