#include "ofdsettingswrapper.h"
#include "inn.h"
#include "logmanip.h"


OfdSettingsWrapper::OfdSettingsWrapper(QObject *parent)
    : BaseCoreHandler(parent)
    , status_(new CashboxStatusWrapper(this))
    , ofd_()
    , supportedOfd_(OfdSettings::SUPPORTED_OFD)
{
    connect(status_, &CashboxStatusWrapper::statusChanged,
            this, &OfdSettingsWrapper::innEnabledChanged);
    QTimer::singleShot(50, this, &OfdSettingsWrapper::reload);
}

OfdSettingsWrapper::~OfdSettingsWrapper()
{

}

bool OfdSettingsWrapper::innEnabled() const
{
    return !status_->status().isFiscalized();
}



qint32 OfdSettingsWrapper::keysTimeout() const
{
    return ofd_.keysTimeout();
}

void OfdSettingsWrapper::setKeysTimeout(qint32 newKeysTimeout)
{
    if (ofd_.keysTimeout() == newKeysTimeout)
        return;
    ofd_.setKeysTimeout(newKeysTimeout);
    emit keysTimeoutChanged();
}

void OfdSettingsWrapper::reload()
{
    emit showBusy(true);
    std::function<QPair<CoreTransaction, CoreApiResult>()> task =
            [this]() -> QPair<CoreTransaction, CoreApiResult> {
        CoreTransaction transaction;
        transaction.setCashierLogin(session_->cashier().login());
        transaction.setCashierPassword(session_->cashier().password());
        transaction.setOperation(CoreTransaction::Operation::LOAD_OFD_SETTINGS);
        transaction.genUid();
        lmWarning() << logvariant(transaction.toMap());
        CoreApiResult res = api_->exec(transaction);
        lmWarning() << logvariant(res.toMap());
        return qMakePair(transaction, res);
    };
    execTask(task);
}

void OfdSettingsWrapper::reloadSupported()
{
    emit showBusy(true);
    std::function<QPair<CoreTransaction, CoreApiResult>()> task =
            [this]() -> QPair<CoreTransaction, CoreApiResult> {
        CoreTransaction transaction;
        transaction.setCashierLogin(session_->cashier().login());
        transaction.setCashierPassword(session_->cashier().password());
        transaction.setOperation(CoreTransaction::Operation::LOAD_OFD_SETTINGS);
        transaction.genUid();
        lmWarning() << logvariant(transaction.toMap());
        CoreApiResult res = api_->exec(transaction);
        lmWarning() << logvariant(res.toMap());
        return qMakePair(transaction, res);
    };
    execTask(task);
}

void OfdSettingsWrapper::save()
{
    emit showBusy(true);
    std::function<QPair<CoreTransaction, CoreApiResult>()> task =
            [this]() -> QPair<CoreTransaction, CoreApiResult> {
        CoreTransaction transaction;
        transaction.setCashierLogin(session_->cashier().login());
        transaction.setCashierPassword(session_->cashier().password());
        transaction.setOperation(CoreTransaction::Operation::SAVE_OFD_SETTINGS);
        transaction.setParams(ofd_.toMap());
        transaction.genUid();
        lmWarning() << logvariant(transaction.toMap());
        CoreApiResult res = api_->exec(transaction);
        lmWarning() << logvariant(res.toMap());
        return qMakePair(transaction, res);
    };
    execTask(task);
}

void OfdSettingsWrapper::setData(const QVariantMap &map)
{
    ofd_.setMap(map);
    emit addressChanged();
    emit portChanged();
    emit dnsChanged();
    emit queryIntervalChanged();
    emit urlChanged();
    emit nameChanged();
    emit shortNameChanged();
    emit labelsAddressChanged();
    emit labelsPortChanged();
    emit keysAddressChanged();
    emit keysPortChanged();
    emit testTimeoutChanged();
    emit keysTimeoutChanged();
    emit innChanged();
    emit innEnabledChanged();
}

qint32 OfdSettingsWrapper::testTimeout() const
{
    return ofd_.testTimeout();
}

void OfdSettingsWrapper::setTestTimeout(qint32 newTestTimeout)
{
    if (ofd_.testTimeout() == newTestTimeout)
        return;
    ofd_.setTestTimeout(newTestTimeout);
    emit testTimeoutChanged();
}

quint16 OfdSettingsWrapper::keysPort() const
{
    return ofd_.keysPort();
}

void OfdSettingsWrapper::setKeysPort(quint16 newKeysPort)
{
    if (ofd_.keysPort() == newKeysPort)
        return;
    ofd_.setKeysPort(newKeysPort);
    emit keysPortChanged();
}

QString OfdSettingsWrapper::keysAddress() const
{
    return ofd_.keysAddress();
}

void OfdSettingsWrapper::setKeysAddress(const QString &newKeysAddress)
{
    if (ofd_.keysAddress() == newKeysAddress)
        return;
    ofd_.setKeysAddress(newKeysAddress);
    emit keysAddressChanged();
}

quint16 OfdSettingsWrapper::labelsPort() const
{
    return ofd_.labelsPort();
}

void OfdSettingsWrapper::setLabelsPort(quint16 newLabelsPort)
{
    if (ofd_.labelsPort() == newLabelsPort)
        return;
    ofd_.setLabelsPort(newLabelsPort);
    emit labelsPortChanged();
}

QString OfdSettingsWrapper::labelsAddress() const
{
    return ofd_.labelsAddress();
}

void OfdSettingsWrapper::setLabelsAddress(const QString &newLabelsAddress)
{
    if (ofd_.labelsAddress() == newLabelsAddress)
        return;
    ofd_.setLabelsAddress(newLabelsAddress);
    emit labelsAddressChanged();
}

QString OfdSettingsWrapper::shortName() const
{
    return ofd_.shortName();
}

void OfdSettingsWrapper::setShortName(const QString &newShortName)
{
    if (ofd_.shortName() == newShortName)
        return;
    ofd_.setShortName(newShortName);
    emit shortNameChanged();
}

QString OfdSettingsWrapper::name() const
{
    return ofd_.name();
}

void OfdSettingsWrapper::setName(const QString &newName)
{
    if (ofd_.name() == newName)
        return;
    ofd_.setName(newName);
    emit nameChanged();
}

QString OfdSettingsWrapper::url() const
{
    return ofd_.url();
}

void OfdSettingsWrapper::setUrl(const QString &newUrl)
{
    if (ofd_.url() == newUrl)
        return;
    ofd_.setUrl(newUrl);
    emit urlChanged();
}

quint32 OfdSettingsWrapper::queryInterval() const
{
    return ofd_.queryInterval();
}

void OfdSettingsWrapper::setQueryInterval(quint32 newQueryInterval)
{
    if (ofd_.queryInterval() == newQueryInterval)
        return;
    ofd_.setQueryInterval(newQueryInterval);
    emit queryIntervalChanged();
}

QString OfdSettingsWrapper::dns() const
{
    return ofd_.dns();
}

void OfdSettingsWrapper::setDns(const QString &newDns)
{
    if (ofd_.dns() == newDns)
        return;
    ofd_.setDns(newDns);
    emit dnsChanged();
}

quint16 OfdSettingsWrapper::port() const
{
    return ofd_.port();
}

void OfdSettingsWrapper::setPort(quint16 newPort)
{
    if (ofd_.port() == newPort)
        return;
    ofd_.setPort(newPort);
    emit portChanged();
}

QString OfdSettingsWrapper::address() const
{
    return ofd_.address();
}

void OfdSettingsWrapper::setAddress(const QString &newAddress)
{
    if (ofd_.address() == newAddress)
        return;
    ofd_.setAddress(newAddress);
    emit addressChanged();
}

QString OfdSettingsWrapper::inn() const
{
    return ofd_.inn();
}

void OfdSettingsWrapper::setInn(const QString &newInn)
{
    if (ofd_.inn() == newInn)
        return;
    ofd_.setInn(newInn);
    Inn inn(newInn);
    if(inn.isValid() && supportedOfd_.contains(newInn))
    {
        ofd_ = supportedOfd_[newInn];
        emit addressChanged();
        emit portChanged();
        emit dnsChanged();
        emit queryIntervalChanged();
        emit urlChanged();
        emit nameChanged();
        emit shortNameChanged();
        emit labelsAddressChanged();
        emit labelsPortChanged();
        emit keysAddressChanged();
        emit keysPortChanged();
        emit testTimeoutChanged();
        emit keysTimeoutChanged();
    }
    emit innChanged();
}

void OfdSettingsWrapper::processResult(const CoreTransaction &source, const CoreApiResult &res)
{
    lmWarning() << res.isOk();
    if(!res.isOk())emit error(res.descr());
    emit showBusy(false);
    switch (source.operation()) {
    case CoreApiConst::RemoteCommand::LOAD_OFD_SETTINGS:
    {
        if(res.isOk())
        {
            setData(res.data());
        }
    }break;
    case CoreApiConst::RemoteCommand::SAVE_OFD_SETTINGS:
    {
        if(res.isOk())
        {
            setData(res.data());
            emit success();
        }
    }break;
    default:
        break;
    }
}
