#ifndef OFDSETTINGSWRAPPER_H
#define OFDSETTINGSWRAPPER_H

#include "ofdsettings.h"
#include "basecorehandler.h"
#include <QtQml>

class OfdSettingsWrapper: public BaseCoreHandler
{
    Q_OBJECT
    QML_ELEMENT
    Q_DISABLE_COPY(OfdSettingsWrapper)

    Q_PROPERTY(QString innEnabled READ innEnabled NOTIFY innEnabledChanged)
    Q_PROPERTY(QString inn READ inn WRITE setInn NOTIFY innChanged)
    Q_PROPERTY(QString address READ address WRITE setAddress NOTIFY addressChanged)
    Q_PROPERTY(quint16 port READ port WRITE setPort NOTIFY portChanged)
    Q_PROPERTY(QString dns READ dns WRITE setDns NOTIFY dnsChanged)
    Q_PROPERTY(quint32 queryInterval READ queryInterval WRITE setQueryInterval NOTIFY queryIntervalChanged)
    Q_PROPERTY(QString url READ url WRITE setUrl NOTIFY urlChanged)
    Q_PROPERTY(QString name READ name WRITE setName NOTIFY nameChanged)
    Q_PROPERTY(QString shortName READ shortName WRITE setShortName NOTIFY shortNameChanged)
    Q_PROPERTY(QString labelsAddress READ labelsAddress WRITE setLabelsAddress NOTIFY labelsAddressChanged)
    Q_PROPERTY(quint16 labelsPort READ labelsPort WRITE setLabelsPort NOTIFY labelsPortChanged)
    Q_PROPERTY(QString keysAddress READ keysAddress WRITE setKeysAddress NOTIFY keysAddressChanged)
    Q_PROPERTY(quint16 keysPort READ keysPort WRITE setKeysPort NOTIFY keysPortChanged)
    Q_PROPERTY(qint32 testTimeout READ testTimeout WRITE setTestTimeout NOTIFY testTimeoutChanged)
    Q_PROPERTY(qint32 keysTimeout READ keysTimeout WRITE setKeysTimeout NOTIFY keysTimeoutChanged)

public:
    explicit OfdSettingsWrapper(QObject *parent = nullptr);
    ~OfdSettingsWrapper() override;


    bool innEnabled() const;

    QString inn() const;
    void setInn(const QString &newInn);

    QString address() const;
    void setAddress(const QString &newAddress);

    quint16 port() const;
    void setPort(quint16 newPort);

    QString dns() const;
    void setDns(const QString &newDns);

    quint32 queryInterval() const;
    void setQueryInterval(quint32 newQueryInterval);

    QString url() const;
    void setUrl(const QString &newUrl);

    QString name() const;
    void setName(const QString &newName);

    QString shortName() const;
    void setShortName(const QString &newShortName);

    QString labelsAddress() const;
    void setLabelsAddress(const QString &newLabelsAddress);

    quint16 labelsPort() const;
    void setLabelsPort(quint16 newLabelsPort);

    QString keysAddress() const;
    void setKeysAddress(const QString &newKeysAddress);

    quint16 keysPort() const;
    void setKeysPort(quint16 newKeysPort);

    qint32 testTimeout() const;
    void setTestTimeout(qint32 newTestTimeout);

    qint32 keysTimeout() const;
    void setKeysTimeout(qint32 newKeysTimeout);
public slots:
    void reload();
    void reloadSupported();
    void save();
    void setData(const QVariantMap &map);
signals:
    void success();
    void innEnabledChanged();
    void innChanged();
    void addressChanged();
    void portChanged();
    void dnsChanged();
    void queryIntervalChanged();
    void urlChanged();
    void nameChanged();
    void shortNameChanged();
    void labelsAddressChanged();
    void labelsPortChanged();
    void keysAddressChanged();
    void keysPortChanged();
    void testTimeoutChanged();
    void keysTimeoutChanged();

private:
    CashboxStatusWrapper *status_;
    OfdSettings ofd_;
    QMap<QString, OfdSettings> supportedOfd_;

    void processResult(const CoreTransaction & source, const CoreApiResult &res) override;


};

#endif // OFDSETTINGSWRAPPER_H
