#ifndef RECEIPTOPERATIONWRAPPER_H
#define RECEIPTOPERATIONWRAPPER_H

#include "basecorehandler.h"
#include "receiptoperation.h"
#include "cashboxstatuswrapper.h"

class ReceiptOperationWrapper : public BaseCoreHandler
{
    Q_OBJECT
    Q_DISABLE_COPY(ReceiptOperationWrapper)
    QML_ELEMENT
    Q_PROPERTY(QVariantMap data READ data WRITE setData NOTIFY dataChanged)
    Q_PROPERTY(int index READ index WRITE setIndex NOTIFY indexChanged)
    Q_PROPERTY(QVariantList opTypeModel READ opTypeModel NOTIFY opTypeModelChanged)
    Q_PROPERTY(QVariantList itemUnitModel READ itemUnitModel NOTIFY itemUnitModelChanged)
    Q_PROPERTY(QVariantList vatModel READ vatModel NOTIFY vatModelChanged)
    Q_PROPERTY(QVariantList payTypeModel READ payTypeModel NOTIFY payTypeModelChanged)
    Q_PROPERTY(bool isExcise READ isExcise NOTIFY isExciseChanged)


    Q_PROPERTY(ROperationType opType READ opType WRITE setOpType NOTIFY opTypeChanged)
    Q_PROPERTY(int opTypeIndex READ opTypeIndex WRITE setOpTypeIndex NOTIFY opTypeIndexChanged)
    Q_PROPERTY(int itemUnit READ itemUnit WRITE setItemUnit NOTIFY itemUnitChanged)
    Q_PROPERTY(QString name READ name WRITE setName NOTIFY nameChanged)
    Q_PROPERTY(VatRate vatRate READ vatRate WRITE setVatRate NOTIFY vatRateChanged)
    Q_PROPERTY(ItemPaymentType payType READ payType WRITE setPayType NOTIFY payTypeChanged)
    Q_PROPERTY(QString price READ price WRITE setPrice NOTIFY priceChanged)
    Q_PROPERTY(QString priceForUnit READ priceForUnit WRITE setPriceForUnit NOTIFY priceForUnitChanged)
    Q_PROPERTY(QString quantity READ quantity WRITE setQuantity NOTIFY quantityChanged)
    Q_PROPERTY(QString additionalParam READ additionalParam WRITE setAdditionalParam NOTIFY additionalParamChanged)
    Q_PROPERTY(QString exciseTax READ exciseTax WRITE setExciseTax NOTIFY exciseTaxChanged)
    Q_PROPERTY(QString countryCode READ countryCode WRITE setCountryCode NOTIFY countryCodeChanged)
    Q_PROPERTY(QString declarationNumber READ declarationNumber WRITE setDeclarationNumber NOTIFY declarationNumberChanged)

    Q_PROPERTY(QVariantList industryProps READ industryProps WRITE setIndustryProps NOTIFY industryPropsChanged)

    Q_PROPERTY(bool isSale READ isSale WRITE setIsSale NOTIFY isSaleChanged)
    Q_PROPERTY(int promisStatus READ promisStatus WRITE setPromisStatus NOTIFY promisStatusChanged)
    Q_PROPERTY(bool hasFractial READ hasFractial WRITE setHasFractial NOTIFY hasFractialChanged)

    Q_PROPERTY(qint32 nominator READ nominator WRITE setNominator NOTIFY nominatorChanged)
    Q_PROPERTY(qint32 denominator READ denominator WRITE setDenominator NOTIFY denominatorChanged)
    Q_PROPERTY(QString label READ label WRITE setLabel NOTIFY labelChanged)
    Q_PROPERTY(bool canSkipB5 READ canSkipB5 NOTIFY canSkipB5Changed FINAL)

public:
    enum ROperationType
    {
        ROTInvalid                          = 0,
        Product                             = 1,
        ExcisableProduct                    = 2,
        Job                                 = 3,
        Service                             = 4,
        GameRate                            = 5,
        GamePrize                           = 6,
        Lottery                             = 7,
        LotteryPrize                        = 8,
        RiaRights                           = 9,
        PrepaymentOp                        = 10,
        AgentCommission                     = 11,
        Fine                                = 12,
        Another                             = 13,
        PropertyRight                       = 14,
        NonReleaseIncome                    = 15,
        AnotherPayments                     = 16,
        TradeFee                            = 17,
        SpaFee                              = 18,
        Deposit                             = 19,
        Cconsumption                        = 20,
        PensionContribution                 = 21,
        PensionContribution2                = 22,
        MedicalContribution                 = 23,
        MedicalContribution2                = 24,
        SocialContribution                  = 25,
        CasinoPayment                       = 26,
        PaymentOfMoney                      = 27,
        ExcisableLabledWitoutCodeProduct    = 30,
        ExcisableLabledProduct              = 31,
        LabledWitoutCodeProduct             = 32,
        LabledProduct                       = 33
    };
    Q_ENUM(ROperationType)
    Q_ENUMS(ROperationType)

    enum VatRate
    {
        Vat20                   = 0x01,
        Vat10                   = 0x02,
        Vat20_120               = 0x03,
        Vat10_110               = 0x04,
        Vat0                    = 0x05,
        VatNone                 = 0x06
    };
    Q_ENUM(VatRate)
    Q_ENUMS(VatRate)

    enum ItemPaymentType
    {
        FullPrepayment                  = 1,
        Prepayment                      = 2,
        Advance                         = 3,
        FullPayment                     = 4,
        PartialPayment                  = 5,
        Credit                          = 6,
        CrediPayment                    = 7
    };
    Q_ENUM(ItemPaymentType)
    Q_ENUMS(ItemPaymentType)

    enum PromisStatus
    {
        Piece               = 0,
        Measured            = 1,
        OtherPromisStatus   = 2
    };
    Q_ENUM(PromisStatus)
    Q_ENUMS(PromisStatus)

    explicit ReceiptOperationWrapper(QObject *parent = nullptr);
    ~ReceiptOperationWrapper() override;

    qint32 index() const;
    void setIndex(qint32 newIndex);

    QVariantMap data() const;
    void setData(const QVariantMap &map);

    bool fractionQuantity() const;

    QVariantList opTypeModel() const;
    QVariantList vatModel() const;
    QVariantList payTypeModel() const;

    Q_INVOKABLE QVariantMap agentData() const;


    int opTypeIndex() const;
    void setOpTypeIndex(int idx);

    ROperationType opType() const;
    void setOpType(ROperationType tp);
    bool isExcise() const;

    QString name() const;
    void setName(const QString &n);

    VatRate vatRate() const;
    void setVatRate(VatRate rate);

    ItemPaymentType payType() const;
    void setPayType(ItemPaymentType tp);

    QString price() const;
    void setPrice(const QString &price);

    QString priceForUnit() const;
    void setPriceForUnit(const QString &price);


    QString quantity() const;
    void setQuantity(const QString &q);

    QVariantList industryProps() const;


    QString additionalParam() const;
    void setAdditionalParam(const QString &newAdditionalParam);

    QString exciseTax() const;
    void setExciseTax(const QString &newExciseTax);

    QString countryCode() const;
    void setCountryCode(const QString &newCountryCode);

    QString declarationNumber() const;
    void setDeclarationNumber(const QString &newDeclarationNumber);

    bool isSale() const;
    void setIsSale(bool newIsSale);

    int promisStatus() const;
    void setPromisStatus(int newPromisStatus);

    const QVariantList &itemUnitModel() const;

    int itemUnit() const;
    void setItemUnit(int idx);

    bool hasFractial() const;
    void setHasFractial(bool newHasFractial);

    qint32 nominator() const;
    void setNominator(qint32 newNominator);

    qint32 denominator() const;
    void setDenominator(qint32 newDenominator);

    QString label() const;
    void setLabel(QString newLabel);

    Q_INVOKABLE QString labelText() const;

    bool canSkipB5() const;

signals:
    void finish();
    void saveData(int, QVariantMap);
    void indexChanged();
    void dataChanged();
    void opTypeModelChanged();
    void vatModelChanged();
    void payTypeModelChanged();

    void opTypeChanged();
    void opTypeIndexChanged();
    void nameChanged();
    void vatRateChanged();
    void payTypeChanged();
    void priceChanged();
    void priceForUnitChanged();
    void quantityChanged();
    void industryPropsChanged();

    void additionalParamChanged();

    void exciseTaxChanged();

    void countryCodeChanged();

    void declarationNumberChanged();
    void isExciseChanged();


    void isSaleChanged();

    void promisStatusChanged();
    void itemUnitModelChanged();
    void itemUnitChanged();

    void hasFractialChanged();

    void nominatorChanged();

    void denominatorChanged();

    void labelChanged();

    bool success();
    void needCheckLabel();

    void canSkipB5Changed();

public slots:
    void parseBarcode(const QString &code);
    void save();
    void setAgentData(const QVariantMap &map);
    void setIndustryProps(const QVariantList &props);
    void checkLabel(int flags);
private:
    qint32 index_;
    QVariantList opTypeModel_;
    QVariantList itemUnitModel_;
    ReceiptOperation operation_;
    CashboxStatusWrapper *status_;
    bool isSale_;
    int promisStatus_;
    LabelCode label_;

    void createOpTypeModel();
    void createItemUnitModel();

    void processResult(const CoreTransaction & source, const CoreApiResult &res) override;

};


/*
    +QString name_;
    +fdf::ReceiptOperationType type_;
    +fdf::VatRate vatRate_;
    +fdf::ItemPaymentType paymentType_;

    +FixNumber quantity_;
    +-std::optional<FixNumber> price_;       //Цена позиции std::optional<FixNumber> priceForPack_;//Цена пачки, если маркированный товар с дробью std::optional<FixNumber> priceForUnit_;//Цена 1/знаменатель товара, если товар с дробью

    fdf::ItemUnit unit_;
    std::optional<QuantityFraction> fraction_;

    QString additionalParam_;
    std::optional<FixNumber> exciseTax_;
    QString countryCode_;
    QString declarationNumber_;




    +fdf::ItemPayAgentFlag agentFlag_;
    +std::optional<AgentData> agentData_;
    +std::optional<ProviderData> providerData_;
    +QString providerInn_;

    QList<IndustryProperty> industryProperties_;

*/

#endif // RECEIPTOPERATIONWRAPPER_H
