#include "sessionwrapper.h"

QPointer<SessionWrapper> SessionWrapper::_globalSession;
QMutex SessionWrapper::_sessionMutex;
Cashier SessionWrapper::_cashier;


SessionWrapper::SessionWrapper(QObject *parent)
    : QObject(parent)
{
    _sessionMutex.lock();
    if(!_globalSession)
    {
        _globalSession = new SessionWrapper(false);
    }
    connect(_globalSession, &SessionWrapper::cashierChanged,
            this, &SessionWrapper::cashierChanged);
    _sessionMutex.unlock();
}

SessionWrapper::~SessionWrapper()
{

}

Cashier SessionWrapper::cashier() const
{
    _sessionMutex.lock();
    Cashier c = _cashier;
    _sessionMutex.unlock();;
    return c;
}

QString SessionWrapper::cashierName() const
{
    return cashier().name();
}

SessionWrapper::SessionWrapper(bool destroyStatic)
{
    if(destroyStatic && _globalSession) delete _globalSession;
    _globalSession = this;
}
void SessionWrapper::setGlobalCashier(const Cashier &cashier)
{
    _sessionMutex.lock();
    if(!_globalSession)
    {
        _globalSession = new SessionWrapper(false);
    }
    Cashier oldCashier = _cashier;
    _cashier = cashier;
    _sessionMutex.unlock();
    if(oldCashier != cashier)
    {
        emit _globalSession->cashierChanged();
    }
}

