#ifndef SIMPLERECEIPTOPERATIONWRAPPER_H
#define SIMPLERECEIPTOPERATIONWRAPPER_H

#include "simplereceiptoperation.h"
#include "cashboxstatuswrapper.h"
#include "labelcode.h"

#include <QObject>
#include <QtQml>

//TODO: в Receipt два списка: простые операции и маркированные товары

class SimpleReceiptOperationWrapper : public QObject
{
    Q_OBJECT
    QML_ELEMENT
    Q_DISABLE_COPY(SimpleReceiptOperationWrapper)

    Q_PROPERTY(QVariantMap data READ data WRITE setData NOTIFY dataChanged)
    Q_PROPERTY(int index READ index WRITE setIndex NOTIFY indexChanged)
    Q_PROPERTY(QVariantList opTypeModel READ opTypeModel NOTIFY opTypeModelChanged)
    Q_PROPERTY(QVariantList itemUnitModel READ itemUnitModel NOTIFY itemUnitModelChanged)

    Q_PROPERTY(QVariantList vatModel READ vatModel NOTIFY vatModelChanged)
    Q_PROPERTY(QVariantList payTypeModel READ payTypeModel NOTIFY payTypeModelChanged)

    Q_PROPERTY(ROperationType opType READ opType WRITE setOpType NOTIFY opTypeChanged)
    Q_PROPERTY(int opTypeIndex READ opTypeIndex WRITE setOpTypeIndex NOTIFY opTypeIndexChanged)
    Q_PROPERTY(int itemUnit READ itemUnit WRITE setItemUnit NOTIFY itemUnitChanged)
    Q_PROPERTY(QString name READ name WRITE setName NOTIFY nameChanged)
    Q_PROPERTY(VatRate vatRate READ vatRate WRITE setVatRate NOTIFY vatRateChanged)
    Q_PROPERTY(ItemPaymentType payType READ payType WRITE setPayType NOTIFY payTypeChanged)
    Q_PROPERTY(QString price READ price WRITE setPrice NOTIFY priceChanged)
    Q_PROPERTY(QString quantity READ quantity WRITE setQuantity NOTIFY quantityChanged)
    Q_PROPERTY(QString additionalParam READ additionalParam WRITE setAdditionalParam NOTIFY additionalParamChanged)
    Q_PROPERTY(QString exciseTax READ exciseTax WRITE setExciseTax NOTIFY exciseTaxChanged)
    Q_PROPERTY(QString countryCode READ countryCode WRITE setCountryCode NOTIFY countryCodeChanged)
    Q_PROPERTY(QString declarationNumber READ declarationNumber WRITE setDeclarationNumber NOTIFY declarationNumberChanged)
    Q_PROPERTY(QVariantList industryProps READ industryProps WRITE setIndustryProps NOTIFY industryPropsChanged)

public:
    enum ROperationType
    {
        ROTInvalid                          = 0,
        Product                             = 1,
        ExcisableProduct                    = 2,
        Job                                 = 3,
        Service                             = 4,
        GameRate                            = 5,
        GamePrize                           = 6,
        Lottery                             = 7,
        LotteryPrize                        = 8,
        RiaRights                           = 9,
        PrepaymentOp                        = 10,
        AgentCommission                     = 11,
        Fine                                = 12,
        Another                             = 13,
        PropertyRight                       = 14,
        NonReleaseIncome                    = 15,
        AnotherPayments                     = 16,
        TradeFee                            = 17,
        SpaFee                              = 18,
        Deposit                             = 19,
        Cconsumption                        = 20,
        PensionContribution                 = 21,
        PensionContribution2                = 22,
        MedicalContribution                 = 23,
        MedicalContribution2                = 24,
        SocialContribution                  = 25,
        CasinoPayment                       = 26,
        PaymentOfMoney                      = 27,
        ExcisableLabledWitoutCodeProduct    = 30,
        ExcisableLabledProduct              = 31,
        LabledWitoutCodeProduct             = 32,
        LabledProduct                       = 33
    };
    Q_ENUM(ROperationType)
    Q_ENUMS(ROperationType)

    enum VatRate
    {
        Vat20                   = 0x01,
        Vat10                   = 0x02,
        Vat20_120               = 0x03,
        Vat10_110               = 0x04,
        Vat0                    = 0x05,
        VatNone                 = 0x06
    };
    Q_ENUM(VatRate)
    Q_ENUMS(VatRate)

    enum ItemPaymentType
    {
        FullPrepayment                  = 1,
        Prepayment                      = 2,
        Advance                         = 3,
        FullPayment                     = 4,
        PartialPayment                  = 5,
        Credit                          = 6,
        CrediPayment                    = 7
    };
    Q_ENUM(ItemPaymentType)
    Q_ENUMS(ItemPaymentType)

    explicit SimpleReceiptOperationWrapper(QObject *parent = nullptr);
    ~SimpleReceiptOperationWrapper() override;

    qint32 index() const;
    void setIndex(qint32 newIndex);

    QVariantMap data() const;
    void setData(const QVariantMap &map);

    QVariantList opTypeModel() const;
    QVariantList vatModel() const;
    QVariantList payTypeModel() const;

    QVariantList itemUnitModel() const;

    Q_INVOKABLE QVariantMap agentData() const;


    int opTypeIndex() const;
    void setOpTypeIndex(int idx);

    int itemUnit() const;
    void setItemUnit(int idx);

    ROperationType opType() const;
    void setOpType(ROperationType tp);


    QString name() const;
    void setName(const QString &n);

    VatRate vatRate() const;
    void setVatRate(VatRate rate);

    ItemPaymentType payType() const;
    void setPayType(ItemPaymentType tp);

    QString price() const;
    void setPrice(const QString &price);

    QString quantity() const;
    void setQuantity(const QString &q);

    QString additionalParam() const;
    void setAdditionalParam(const QString &newAdditionalParam);

    QString exciseTax() const;
    void setExciseTax(const QString &newExciseTax);

    QString countryCode() const;
    void setCountryCode(const QString &newCountryCode);

    QString declarationNumber() const;
    void setDeclarationNumber(const QString &newDeclarationNumber);

    QVariantList industryProps() const;

    Q_INVOKABLE QString labelText() const;
signals:
    void finish();
    void error(QString);
    bool showBusy(bool);
    void saveData(int, QVariantMap);
    void indexChanged();
    void dataChanged();
    void opTypeModelChanged();
    void vatModelChanged();
    void payTypeModelChanged();
    void itemUnitModelChanged();
    void itemUnitChanged();


    void opTypeChanged();
    void opTypeIndexChanged();
    void nameChanged();
    void vatRateChanged();
    void payTypeChanged();
    void priceChanged();
    void quantityChanged();
    void additionalParamChanged();
    void exciseTaxChanged();
    void countryCodeChanged();
    void declarationNumberChanged();
    void barcodeChanged();
    void industryPropsChanged();
public slots:
    void parseBarcode(const QString &code);
    void save();
    void setAgentData(const QVariantMap &map);
    void setIndustryProps(const QVariantList &props);
private:
    qint32 index_;
    QVariantList opTypeModel_;
    QVariantList itemUnitModel_;

    SimpleReceiptOperation operation_;
    CashboxStatusWrapper *status_;
    LabelCode label_;

    void createOpTypeModel();
    void createItemUnitModel();
};

#endif // SIMPLERECEIPTOPERATIONWRAPPER_H
